#include <Python.h>

#include <Numeric/arrayobject.h>

#ifndef included_ASE_Compiler_Python_Function
#include <ASE/Compiler/Python/Function_IOR.h>
#endif

#ifndef included_ASE_Loader
#include <ASE/Loader_IOR.h>
#endif

#ifndef included_ASE_BaseException
#include <ASE/BaseException_IOR.h>
#endif

static const struct ASE_Compiler_Python_Function__external *_epv = NULL;

enum ReferenceType {STOLEN_REF, WEAK_REF, NEW_REF};

struct ASE_Compiler_Python_Function_PythonObject {
  PyObject_HEAD;
  struct ASE_Compiler_Python_Function__object *ior;
  enum ReferenceType refType;
  PyObject *weakRefList;
};

/* Need to forward declare the type structure */
static PyTypeObject SIDLPythonObjectType;

static PyObject *sidl_new(PyTypeObject *type, struct ASE_BaseInterface__object *ior, enum ReferenceType refType)
{
  PyObject *return_value = NULL;
  
  if (ior)
  {
    struct ASE_Compiler_Python_Function_PythonObject *self = (struct ASE_Compiler_Python_Function_PythonObject *) type->tp_alloc(type, 0);
    
    self->weakRefList = NULL;
    if (self)
    {
      struct ASE_BaseClass__object *baseIor = (*(ior->d_epv->castObject))(ior->d_object, "ASE.BaseClass", NULL);
      
      self->ior = ((*(ior->d_epv->castObject)))(ior->d_object, type->tp_name, NULL);
      self->refType = refType;
      /* Allow objects without an attached implementation (used in the skeleton constructor) */
      if (baseIor->d_data)
      {
        if (refType == NEW_REF)
        {
          (*(baseIor->d_epv->addReference))(baseIor->d_object, NULL);
        }
      }
      return_value = (PyObject *) self;
    }
    else
    {
      PyErr_SetString(PyExc_MemoryError, (char *) "Unable to allocate interface object");
    }
  }
  else
  {
    return_value = Py_None;
    Py_INCREF(return_value);
  }
  return return_value;
}

static PyObject *ASE_Compiler_Python_Function__wrap_ior_weak(struct ASE_Compiler_Python_Function__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Function");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Function");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Function");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Function");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, WEAK_REF);
}

static PyObject *ASE_BaseException__wrap_ior(struct ASE_BaseException__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.BaseException");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "BaseException");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.BaseException");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.BaseException");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_CompilerException__wrap_ior(struct ASE_Compiler_CompilerException__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "CompilerException");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.CompilerException");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.CompilerException");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Attribute__wrap_ior(struct ASE_Compiler_Attribute__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Attribute");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Attribute");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Attribute");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Attribute");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Function__wrap_ior(struct ASE_Compiler_Python_Function__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Function");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Function");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Function");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Function");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Vertex__wrap_ior(struct ASE_Compiler_Vertex__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Vertex");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Vertex");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Vertex");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Vertex");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Comment__wrap_ior(struct ASE_Compiler_Comment__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Comment");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Comment");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Comment");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Comment");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Vertex__wrap_ior(struct ASE_Compiler_Python_Vertex__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Vertex");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Vertex");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Vertex");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Vertex");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_BaseInterface__wrap_ior(struct ASE_BaseInterface__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.BaseInterface");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "BaseInterface");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.BaseInterface");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.BaseInterface");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static long ASE_Compiler_Comment__extract_ior(struct ASE_Compiler_Python_Function_PythonObject *obj, struct ASE_Compiler_Comment__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Comment", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Comment");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Vertex__extract_ior(struct ASE_Compiler_Python_Function_PythonObject *obj, struct ASE_Compiler_Vertex__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Vertex", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Vertex");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Function__extract_ior(struct ASE_Compiler_Python_Function_PythonObject *obj, struct ASE_Compiler_Python_Function__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Function", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Function");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Visitor__extract_ior(struct ASE_Compiler_Python_Function_PythonObject *obj, struct ASE_Compiler_Python_Visitor__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Visitor", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Visitor");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Attribute__extract_ior(struct ASE_Compiler_Python_Function_PythonObject *obj, struct ASE_Compiler_Attribute__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Attribute", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Attribute");
    return 0;
  }
  return 1;
}

static long ASE_BaseInterface__extract_ior(struct ASE_Compiler_Python_Function_PythonObject *obj, struct ASE_BaseInterface__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.BaseInterface", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.BaseInterface");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Vertex__extract_ior(struct ASE_Compiler_Python_Function_PythonObject *obj, struct ASE_Compiler_Python_Vertex__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Vertex", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Vertex");
    return 0;
  }
  return 1;
}

#ifndef included_ASE_Compiler_Vertex
#include <ASE/Compiler/Vertex_IOR.h>
#endif

static PyObject *ASE_Compiler_Vertex__wrap_array_copy(struct ASE_Compiler_Vertex__array *array)
{
  PyArrayObject *_return_value = NULL;
  
  if (array)
  {
    const struct ASE_Compiler_Vertex__external *(*_externals)(void) = (const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL);
    const struct ASE_Compiler_Vertex__external *_extFunc = (*_externals)();
    long dimen = (*_extFunc->getDimen)(array, NULL);
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {0};
    int32_t indStride[1] = {0};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array numElem = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    long i = 0;
    
    for(i = 0; i < dimen; i++)
    {
      lower.d_firstElement[i] = (*_extFunc->getLower)(array, i, NULL);
      upper.d_firstElement[i] = (*_extFunc->getUpper)(array, i, NULL);
      numElem.d_firstElement[i] = upper.d_firstElement[i] - lower.d_firstElement[i] + 1;
    }
    _return_value = (PyArrayObject *) PyArray_FromDims(dimen, numElem.d_firstElement, PyArray_OBJECT);
    if (_return_value)
    {
      long size = 1;
      char *data = _return_value->data;
      
      for(i = 0; i < dimen; i++)
      {
        size *= numElem.d_firstElement[i];
      }
      while(size--)
      {
        struct ASE_Compiler_Vertex__object *elem = (*_extFunc->getElement)(array, &lower, NULL);
        
        (*(elem->d_epv->addReference))(elem->d_object, NULL);
        *(PyObject **) data = ASE_Compiler_Vertex__wrap_ior(elem);
        if (!(*(PyObject **) data))
        {
          Py_DECREF(_return_value);
          _return_value = NULL;
          goto error;
        }
        for(i = 0; i < dimen; i++)
        {
          data += _return_value->strides[i];
          if (++(lower.d_firstElement[i]) <= upper.d_firstElement[i])
          {
            break;
          }
          else
          {
            data -= _return_value->strides[i] * numElem.d_firstElement[i];
            lower.d_firstElement[i] -= numElem.d_firstElement[i];
          }
        }
      }
    }
    error: free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(numElem.d_firstElement);
  }
  return (PyObject *) _return_value;
}

#ifndef included_string
#include <string_IOR.h>
#endif

static PyObject *string__wrap_array_copy(struct string__array *array)
{
  PyArrayObject *_return_value = NULL;
  
  if (array)
  {
    const struct string__external *_extFunc = string__externals();
    long dimen = (*_extFunc->getDimen)(array, NULL);
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {0};
    int32_t indStride[1] = {0};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array numElem = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    long i = 0;
    
    for(i = 0; i < dimen; i++)
    {
      lower.d_firstElement[i] = (*_extFunc->getLower)(array, i, NULL);
      upper.d_firstElement[i] = (*_extFunc->getUpper)(array, i, NULL);
      numElem.d_firstElement[i] = upper.d_firstElement[i] - lower.d_firstElement[i] + 1;
    }
    _return_value = (PyArrayObject *) PyArray_FromDims(dimen, numElem.d_firstElement, PyArray_OBJECT);
    if (_return_value)
    {
      long size = 1;
      char *data = _return_value->data;
      
      for(i = 0; i < dimen; i++)
      {
        size *= numElem.d_firstElement[i];
      }
      while(size--)
      {
        const char *elem = (*_extFunc->getElement)(array, &lower, NULL);
        
        if (elem)
        {
          *(PyObject **) data = PyString_FromString(elem);
          free((char *) elem);
        }
        else
        {
          Py_INCREF(Py_None);
          *(PyObject **) data = Py_None;
        }
        for(i = 0; i < dimen; i++)
        {
          data += _return_value->strides[i];
          if (++(lower.d_firstElement[i]) <= upper.d_firstElement[i])
          {
            break;
          }
          else
          {
            data -= _return_value->strides[i] * numElem.d_firstElement[i];
            lower.d_firstElement[i] -= numElem.d_firstElement[i];
          }
        }
      }
    }
    error: free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(numElem.d_firstElement);
  }
  return (PyObject *) _return_value;
}

#ifndef included_ASE_Compiler_Attribute
#include <ASE/Compiler/Attribute_IOR.h>
#endif

static PyObject *ASE_Compiler_Attribute__wrap_array_copy(struct ASE_Compiler_Attribute__array *array)
{
  PyArrayObject *_return_value = NULL;
  
  if (array)
  {
    const struct ASE_Compiler_Attribute__external *(*_externals)(void) = (const struct ASE_Compiler_Attribute__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Attribute", NULL);
    const struct ASE_Compiler_Attribute__external *_extFunc = (*_externals)();
    long dimen = (*_extFunc->getDimen)(array, NULL);
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {0};
    int32_t indStride[1] = {0};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array numElem = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    long i = 0;
    
    for(i = 0; i < dimen; i++)
    {
      lower.d_firstElement[i] = (*_extFunc->getLower)(array, i, NULL);
      upper.d_firstElement[i] = (*_extFunc->getUpper)(array, i, NULL);
      numElem.d_firstElement[i] = upper.d_firstElement[i] - lower.d_firstElement[i] + 1;
    }
    _return_value = (PyArrayObject *) PyArray_FromDims(dimen, numElem.d_firstElement, PyArray_OBJECT);
    if (_return_value)
    {
      long size = 1;
      char *data = _return_value->data;
      
      for(i = 0; i < dimen; i++)
      {
        size *= numElem.d_firstElement[i];
      }
      while(size--)
      {
        struct ASE_Compiler_Attribute__object *elem = (*_extFunc->getElement)(array, &lower, NULL);
        
        (*(elem->d_epv->addReference))(elem->d_object, NULL);
        *(PyObject **) data = ASE_Compiler_Attribute__wrap_ior(elem);
        if (!(*(PyObject **) data))
        {
          Py_DECREF(_return_value);
          _return_value = NULL;
          goto error;
        }
        for(i = 0; i < dimen; i++)
        {
          data += _return_value->strides[i];
          if (++(lower.d_firstElement[i]) <= upper.d_firstElement[i])
          {
            break;
          }
          else
          {
            data -= _return_value->strides[i] * numElem.d_firstElement[i];
            lower.d_firstElement[i] -= numElem.d_firstElement[i];
          }
        }
      }
    }
    error: free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(numElem.d_firstElement);
  }
  return (PyObject *) _return_value;
}

#ifndef included_ASE_Compiler_Python_Vertex
#include <ASE/Compiler/Python/Vertex_IOR.h>
#endif

static PyObject *ASE_Compiler_Python_Vertex__wrap_array_copy(struct ASE_Compiler_Python_Vertex__array *array)
{
  PyArrayObject *_return_value = NULL;
  
  if (array)
  {
    const struct ASE_Compiler_Python_Vertex__external *(*_externals)(void) = (const struct ASE_Compiler_Python_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Python.Vertex", NULL);
    const struct ASE_Compiler_Python_Vertex__external *_extFunc = (*_externals)();
    long dimen = (*_extFunc->getDimen)(array, NULL);
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {0};
    int32_t indStride[1] = {0};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array numElem = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    long i = 0;
    
    for(i = 0; i < dimen; i++)
    {
      lower.d_firstElement[i] = (*_extFunc->getLower)(array, i, NULL);
      upper.d_firstElement[i] = (*_extFunc->getUpper)(array, i, NULL);
      numElem.d_firstElement[i] = upper.d_firstElement[i] - lower.d_firstElement[i] + 1;
    }
    _return_value = (PyArrayObject *) PyArray_FromDims(dimen, numElem.d_firstElement, PyArray_OBJECT);
    if (_return_value)
    {
      long size = 1;
      char *data = _return_value->data;
      
      for(i = 0; i < dimen; i++)
      {
        size *= numElem.d_firstElement[i];
      }
      while(size--)
      {
        struct ASE_Compiler_Python_Vertex__object *elem = (*_extFunc->getElement)(array, &lower, NULL);
        
        (*(elem->d_epv->addReference))(elem->d_object, NULL);
        *(PyObject **) data = ASE_Compiler_Python_Vertex__wrap_ior(elem);
        if (!(*(PyObject **) data))
        {
          Py_DECREF(_return_value);
          _return_value = NULL;
          goto error;
        }
        for(i = 0; i < dimen; i++)
        {
          data += _return_value->strides[i];
          if (++(lower.d_firstElement[i]) <= upper.d_firstElement[i])
          {
            break;
          }
          else
          {
            data -= _return_value->strides[i] * numElem.d_firstElement[i];
            lower.d_firstElement[i] -= numElem.d_firstElement[i];
          }
        }
      }
    }
    error: free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(numElem.d_firstElement);
  }
  return (PyObject *) _return_value;
}

#ifndef included_ASE_Compiler_Comment
#include <ASE/Compiler/Comment_IOR.h>
#endif

static PyObject *ASE_Compiler_Comment__wrap_array_copy(struct ASE_Compiler_Comment__array *array)
{
  PyArrayObject *_return_value = NULL;
  
  if (array)
  {
    const struct ASE_Compiler_Comment__external *(*_externals)(void) = (const struct ASE_Compiler_Comment__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Comment", NULL);
    const struct ASE_Compiler_Comment__external *_extFunc = (*_externals)();
    long dimen = (*_extFunc->getDimen)(array, NULL);
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {0};
    int32_t indStride[1] = {0};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array numElem = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    long i = 0;
    
    for(i = 0; i < dimen; i++)
    {
      lower.d_firstElement[i] = (*_extFunc->getLower)(array, i, NULL);
      upper.d_firstElement[i] = (*_extFunc->getUpper)(array, i, NULL);
      numElem.d_firstElement[i] = upper.d_firstElement[i] - lower.d_firstElement[i] + 1;
    }
    _return_value = (PyArrayObject *) PyArray_FromDims(dimen, numElem.d_firstElement, PyArray_OBJECT);
    if (_return_value)
    {
      long size = 1;
      char *data = _return_value->data;
      
      for(i = 0; i < dimen; i++)
      {
        size *= numElem.d_firstElement[i];
      }
      while(size--)
      {
        struct ASE_Compiler_Comment__object *elem = (*_extFunc->getElement)(array, &lower, NULL);
        
        (*(elem->d_epv->addReference))(elem->d_object, NULL);
        *(PyObject **) data = ASE_Compiler_Comment__wrap_ior(elem);
        if (!(*(PyObject **) data))
        {
          Py_DECREF(_return_value);
          _return_value = NULL;
          goto error;
        }
        for(i = 0; i < dimen; i++)
        {
          data += _return_value->strides[i];
          if (++(lower.d_firstElement[i]) <= upper.d_firstElement[i])
          {
            break;
          }
          else
          {
            data -= _return_value->strides[i] * numElem.d_firstElement[i];
            lower.d_firstElement[i] -= numElem.d_firstElement[i];
          }
        }
      }
    }
    error: free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(numElem.d_firstElement);
  }
  return (PyObject *) _return_value;
}

#ifndef included_ASE_Compiler_Vertex
#include <ASE/Compiler/Vertex_IOR.h>
#endif

static long ASE_Compiler_Vertex__extract_array(PyObject *obj, struct ASE_Compiler_Vertex__array **array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_OBJECT, 0, 0);
  long _return_value = 0;
  
  *array = NULL;
  if (pyArray)
  {
    int32_t dimen = pyArray->nd;
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {dimen - 1};
    int32_t indStride[1] = {1};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array stride = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    int32_t d = 0;
    const struct ASE_Compiler_Vertex__external *(*_externals)(void) = (const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL);
    const struct ASE_Compiler_Vertex__external *_extFunc = (*_externals)();
    long size = PyArray_Size((PyObject *) pyArray);
    char *data = pyArray->data;
    long i;
    struct int__array indices = {
      malloc(sizeof(long) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    
    for(d = 0; d < dimen; d++)
    {
      lower.d_firstElement[d] = 0;
      upper.d_firstElement[d] = pyArray->dimensions[d] - 1;
      stride.d_firstElement[d] = pyArray->strides[d] / pyArray->descr->elsize;
      if (pyArray->strides[d] % pyArray->descr->elsize)
      {
        free(lower.d_firstElement);
        free(upper.d_firstElement);
        free(stride.d_firstElement);
        return 0;
      }
    }
    *array = (*_extFunc->createArray)(dimen, &lower, &upper, NULL);
    free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(stride.d_firstElement);
    memset(indices.d_firstElement, 0, sizeof(long) * (dimen ? dimen : 1));
    for(i = 0; i < size; i++)
    {
      struct ASE_Compiler_Vertex__object *elem;
      
      if (!ASE_Compiler_Vertex__extract_ior((struct ASE_Compiler_Python_Function_PythonObject *) *(PyObject **) data, &elem))
      {
        (*_extFunc->destroyArray)(*array, NULL);
        *array = NULL;
        goto error;
      }
      (*_extFunc->setElement)(*array, &indices, elem, NULL);
      for(d = 0; d < dimen; d++)
      {
        data += pyArray->strides[d];
        if (++(indices.d_firstElement[d]) < pyArray->dimensions[d])
        {
          break;
        }
        else
        {
          indices.d_firstElement[d] = 0;
          data -= pyArray->dimensions[d] * pyArray->strides[d];
        }
      }
    }
    _return_value = 1;
    error: free(indices.d_firstElement);
    Py_DECREF(pyArray);
  }
  return _return_value;
}

#ifndef included_string
#include <string_IOR.h>
#endif

static long string__extract_array(PyObject *obj, struct string__array **array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_OBJECT, 0, 0);
  long _return_value = 0;
  
  *array = NULL;
  if (pyArray)
  {
    int32_t dimen = pyArray->nd;
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {dimen - 1};
    int32_t indStride[1] = {1};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array stride = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    int32_t d = 0;
    const struct string__external *_extFunc = string__externals();
    long size = PyArray_Size((PyObject *) pyArray);
    char *data = pyArray->data;
    long i;
    struct int__array indices = {
      malloc(sizeof(long) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    
    for(d = 0; d < dimen; d++)
    {
      lower.d_firstElement[d] = 0;
      upper.d_firstElement[d] = pyArray->dimensions[d] - 1;
      stride.d_firstElement[d] = pyArray->strides[d] / pyArray->descr->elsize;
      if (pyArray->strides[d] % pyArray->descr->elsize)
      {
        free(lower.d_firstElement);
        free(upper.d_firstElement);
        free(stride.d_firstElement);
        return 0;
      }
    }
    *array = (*_extFunc->createArray)(dimen, &lower, &upper, NULL);
    free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(stride.d_firstElement);
    memset(indices.d_firstElement, 0, sizeof(long) * (dimen ? dimen : 1));
    for(i = 0; i < size; i++)
    {
      const char *elem;
      
      if (!(elem = PyString_AsString(*(PyObject **) data)))
      {
        (*_extFunc->destroyArray)(*array, NULL);
        *array = NULL;
        goto error;
      }
      /* Copying the Python string to IOR controlled memory */
      elem = elem ? strcpy(malloc(strlen(elem) + 1), elem) : NULL;
      (*_extFunc->setElement)(*array, &indices, elem, NULL);
      for(d = 0; d < dimen; d++)
      {
        data += pyArray->strides[d];
        if (++(indices.d_firstElement[d]) < pyArray->dimensions[d])
        {
          break;
        }
        else
        {
          indices.d_firstElement[d] = 0;
          data -= pyArray->dimensions[d] * pyArray->strides[d];
        }
      }
    }
    _return_value = 1;
    error: free(indices.d_firstElement);
    Py_DECREF(pyArray);
  }
  return _return_value;
}

#ifndef included_ASE_Compiler_Attribute
#include <ASE/Compiler/Attribute_IOR.h>
#endif

static long ASE_Compiler_Attribute__extract_array(PyObject *obj, struct ASE_Compiler_Attribute__array **array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_OBJECT, 0, 0);
  long _return_value = 0;
  
  *array = NULL;
  if (pyArray)
  {
    int32_t dimen = pyArray->nd;
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {dimen - 1};
    int32_t indStride[1] = {1};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array stride = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    int32_t d = 0;
    const struct ASE_Compiler_Attribute__external *(*_externals)(void) = (const struct ASE_Compiler_Attribute__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Attribute", NULL);
    const struct ASE_Compiler_Attribute__external *_extFunc = (*_externals)();
    long size = PyArray_Size((PyObject *) pyArray);
    char *data = pyArray->data;
    long i;
    struct int__array indices = {
      malloc(sizeof(long) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    
    for(d = 0; d < dimen; d++)
    {
      lower.d_firstElement[d] = 0;
      upper.d_firstElement[d] = pyArray->dimensions[d] - 1;
      stride.d_firstElement[d] = pyArray->strides[d] / pyArray->descr->elsize;
      if (pyArray->strides[d] % pyArray->descr->elsize)
      {
        free(lower.d_firstElement);
        free(upper.d_firstElement);
        free(stride.d_firstElement);
        return 0;
      }
    }
    *array = (*_extFunc->createArray)(dimen, &lower, &upper, NULL);
    free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(stride.d_firstElement);
    memset(indices.d_firstElement, 0, sizeof(long) * (dimen ? dimen : 1));
    for(i = 0; i < size; i++)
    {
      struct ASE_Compiler_Attribute__object *elem;
      
      if (!ASE_Compiler_Attribute__extract_ior((struct ASE_Compiler_Python_Function_PythonObject *) *(PyObject **) data, &elem))
      {
        (*_extFunc->destroyArray)(*array, NULL);
        *array = NULL;
        goto error;
      }
      (*_extFunc->setElement)(*array, &indices, elem, NULL);
      for(d = 0; d < dimen; d++)
      {
        data += pyArray->strides[d];
        if (++(indices.d_firstElement[d]) < pyArray->dimensions[d])
        {
          break;
        }
        else
        {
          indices.d_firstElement[d] = 0;
          data -= pyArray->dimensions[d] * pyArray->strides[d];
        }
      }
    }
    _return_value = 1;
    error: free(indices.d_firstElement);
    Py_DECREF(pyArray);
  }
  return _return_value;
}

#ifndef included_ASE_Compiler_Python_Vertex
#include <ASE/Compiler/Python/Vertex_IOR.h>
#endif

static long ASE_Compiler_Python_Vertex__extract_array(PyObject *obj, struct ASE_Compiler_Python_Vertex__array **array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_OBJECT, 0, 0);
  long _return_value = 0;
  
  *array = NULL;
  if (pyArray)
  {
    int32_t dimen = pyArray->nd;
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {dimen - 1};
    int32_t indStride[1] = {1};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array stride = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    int32_t d = 0;
    const struct ASE_Compiler_Python_Vertex__external *(*_externals)(void) = (const struct ASE_Compiler_Python_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Python.Vertex", NULL);
    const struct ASE_Compiler_Python_Vertex__external *_extFunc = (*_externals)();
    long size = PyArray_Size((PyObject *) pyArray);
    char *data = pyArray->data;
    long i;
    struct int__array indices = {
      malloc(sizeof(long) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    
    for(d = 0; d < dimen; d++)
    {
      lower.d_firstElement[d] = 0;
      upper.d_firstElement[d] = pyArray->dimensions[d] - 1;
      stride.d_firstElement[d] = pyArray->strides[d] / pyArray->descr->elsize;
      if (pyArray->strides[d] % pyArray->descr->elsize)
      {
        free(lower.d_firstElement);
        free(upper.d_firstElement);
        free(stride.d_firstElement);
        return 0;
      }
    }
    *array = (*_extFunc->createArray)(dimen, &lower, &upper, NULL);
    free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(stride.d_firstElement);
    memset(indices.d_firstElement, 0, sizeof(long) * (dimen ? dimen : 1));
    for(i = 0; i < size; i++)
    {
      struct ASE_Compiler_Python_Vertex__object *elem;
      
      if (!ASE_Compiler_Python_Vertex__extract_ior((struct ASE_Compiler_Python_Function_PythonObject *) *(PyObject **) data, &elem))
      {
        (*_extFunc->destroyArray)(*array, NULL);
        *array = NULL;
        goto error;
      }
      (*_extFunc->setElement)(*array, &indices, elem, NULL);
      for(d = 0; d < dimen; d++)
      {
        data += pyArray->strides[d];
        if (++(indices.d_firstElement[d]) < pyArray->dimensions[d])
        {
          break;
        }
        else
        {
          indices.d_firstElement[d] = 0;
          data -= pyArray->dimensions[d] * pyArray->strides[d];
        }
      }
    }
    _return_value = 1;
    error: free(indices.d_firstElement);
    Py_DECREF(pyArray);
  }
  return _return_value;
}

#ifndef included_ASE_Compiler_Comment
#include <ASE/Compiler/Comment_IOR.h>
#endif

static long ASE_Compiler_Comment__extract_array(PyObject *obj, struct ASE_Compiler_Comment__array **array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_OBJECT, 0, 0);
  long _return_value = 0;
  
  *array = NULL;
  if (pyArray)
  {
    int32_t dimen = pyArray->nd;
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {dimen - 1};
    int32_t indStride[1] = {1};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array stride = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    int32_t d = 0;
    const struct ASE_Compiler_Comment__external *(*_externals)(void) = (const struct ASE_Compiler_Comment__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Comment", NULL);
    const struct ASE_Compiler_Comment__external *_extFunc = (*_externals)();
    long size = PyArray_Size((PyObject *) pyArray);
    char *data = pyArray->data;
    long i;
    struct int__array indices = {
      malloc(sizeof(long) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    
    for(d = 0; d < dimen; d++)
    {
      lower.d_firstElement[d] = 0;
      upper.d_firstElement[d] = pyArray->dimensions[d] - 1;
      stride.d_firstElement[d] = pyArray->strides[d] / pyArray->descr->elsize;
      if (pyArray->strides[d] % pyArray->descr->elsize)
      {
        free(lower.d_firstElement);
        free(upper.d_firstElement);
        free(stride.d_firstElement);
        return 0;
      }
    }
    *array = (*_extFunc->createArray)(dimen, &lower, &upper, NULL);
    free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(stride.d_firstElement);
    memset(indices.d_firstElement, 0, sizeof(long) * (dimen ? dimen : 1));
    for(i = 0; i < size; i++)
    {
      struct ASE_Compiler_Comment__object *elem;
      
      if (!ASE_Compiler_Comment__extract_ior((struct ASE_Compiler_Python_Function_PythonObject *) *(PyObject **) data, &elem))
      {
        (*_extFunc->destroyArray)(*array, NULL);
        *array = NULL;
        goto error;
      }
      (*_extFunc->setElement)(*array, &indices, elem, NULL);
      for(d = 0; d < dimen; d++)
      {
        data += pyArray->strides[d];
        if (++(indices.d_firstElement[d]) < pyArray->dimensions[d])
        {
          break;
        }
        else
        {
          indices.d_firstElement[d] = 0;
          data -= pyArray->dimensions[d] * pyArray->strides[d];
        }
      }
    }
    _return_value = 1;
    error: free(indices.d_firstElement);
    Py_DECREF(pyArray);
  }
  return _return_value;
}

static PyObject *ASE_Compiler_Python_Flags_getFlags(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getFlags");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getFlags");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getFlags))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Flags_setFlags(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "flags",
    NULL};
  long _okay = 0;
  const char *flags = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setFlags");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setFlags");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &flags);
  /* Copying the Python string to IOR controlled memory */
  flags = flags ? strcpy(malloc(strlen(flags) + 1), flags) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setFlags))(_self_ior, flags, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) flags);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_clone(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  struct ASE_Compiler_Vertex__object *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for clone");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in clone");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->clone))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Vertex__wrap_ior, _return_value);
  if (_return_value)
  {
    (*((((struct ASE_BaseInterface__object *) _return_value))->d_epv->addReference))(((struct ASE_BaseInterface__object *) _return_value)->d_object, NULL);
  }
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_getIdentifier(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getIdentifier");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getIdentifier");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getIdentifier))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_setIdentifier(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "identifier",
    NULL};
  long _okay = 0;
  const char *identifier = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setIdentifier");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setIdentifier");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &identifier);
  /* Copying the Python string to IOR controlled memory */
  identifier = identifier ? strcpy(malloc(strlen(identifier) + 1), identifier) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setIdentifier))(_self_ior, identifier, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) identifier);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_getAttributes(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  struct ASE_Compiler_Attribute__array *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getAttributes");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getAttributes");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getAttributes))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Attribute__wrap_array_copy, _return_value);
  (*((*((const struct ASE_Compiler_Attribute__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Attribute", NULL)))()->destroyArray))(_return_value, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_setAttributes(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "attributes",
    NULL};
  long _okay = 0;
  struct ASE_Compiler_Attribute__array *attributes = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setAttributes");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setAttributes");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Attribute__extract_array, &attributes);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setAttributes))(_self_ior, attributes, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_hasAttribute(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *name = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for hasAttribute");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in hasAttribute");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &name);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->hasAttribute))(_self_ior, name, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_addAttribute(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "attribute",
    NULL};
  long _okay = 0;
  struct ASE_Compiler_Attribute__object *attribute = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for addAttribute");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in addAttribute");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Attribute__extract_ior, &attribute);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->addAttribute))(_self_ior, attribute, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_deleteAttribute(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *name = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for deleteAttribute");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in deleteAttribute");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &name);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->deleteAttribute))(_self_ior, name, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *_obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, _obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_getAttribute(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *name = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  struct ASE_Compiler_Attribute__object *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getAttribute");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getAttribute");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &name);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getAttribute))(_self_ior, name, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *_obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, _obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Attribute__wrap_ior, _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  if (_return_value)
  {
    (*((((struct ASE_BaseInterface__object *) _return_value))->d_epv->addReference))(((struct ASE_BaseInterface__object *) _return_value)->d_object, NULL);
  }
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_getParent(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  struct ASE_Compiler_Vertex__object *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getParent");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getParent");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getParent))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Vertex__wrap_ior, _return_value);
  if (_return_value)
  {
    (*((((struct ASE_BaseInterface__object *) _return_value))->d_epv->addReference))(((struct ASE_BaseInterface__object *) _return_value)->d_object, NULL);
  }
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_setParent(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "parent",
    NULL};
  long _okay = 0;
  struct ASE_Compiler_Vertex__object *parent = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setParent");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setParent");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Vertex__extract_ior, &parent);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setParent))(_self_ior, parent, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_getChildren(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  struct ASE_Compiler_Vertex__array *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getChildren");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getChildren");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getChildren))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Vertex__wrap_array_copy, _return_value);
  (*((*((const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL)))()->destroyArray))(_return_value, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_setChildren(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "children",
    NULL};
  long _okay = 0;
  struct ASE_Compiler_Vertex__array *children = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setChildren");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setChildren");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Vertex__extract_array, &children);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setChildren))(_self_ior, children, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_addChildren(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "children",
    NULL};
  long _okay = 0;
  struct ASE_Compiler_Vertex__array *children = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for addChildren");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in addChildren");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Vertex__extract_array, &children);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->addChildren))(_self_ior, children, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_removeChildren(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "children",
    NULL};
  long _okay = 0;
  struct ASE_Compiler_Vertex__array *children = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for removeChildren");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in removeChildren");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Vertex__extract_array, &children);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->removeChildren))(_self_ior, children, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *_obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, _obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_getLineNumber(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getLineNumber");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getLineNumber");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getLineNumber))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_setLineNumber(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "lineNumber",
    NULL};
  long _okay = 0;
  int32_t lineNumber = 0;
  long _local_lineNumber = 0;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setLineNumber");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setLineNumber");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "l", _kwlist, &_local_lineNumber);
  /* Convert long to int32_t */
  lineNumber = _local_lineNumber;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setLineNumber))(_self_ior, lineNumber, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_getComments(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  struct ASE_Compiler_Comment__array *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getComments");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getComments");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getComments))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Comment__wrap_array_copy, _return_value);
  (*((*((const struct ASE_Compiler_Comment__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Comment", NULL)))()->destroyArray))(_return_value, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_setComments(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "comments",
    NULL};
  long _okay = 0;
  struct ASE_Compiler_Comment__array *comments = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setComments");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setComments");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Comment__extract_array, &comments);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setComments))(_self_ior, comments, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_addComments(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "comments",
    NULL};
  long _okay = 0;
  struct ASE_Compiler_Comment__array *comments = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for addComments");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in addComments");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Comment__extract_array, &comments);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->addComments))(_self_ior, comments, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Vertex_removeComments(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "comments",
    NULL};
  long _okay = 0;
  struct ASE_Compiler_Comment__array *comments = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for removeComments");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in removeComments");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Comment__extract_array, &comments);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->removeComments))(_self_ior, comments, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *_obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, _obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getReferenceCount(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getReferenceCount");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getReferenceCount");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getReferenceCount))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_addReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for addReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in addReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->addReference))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_deleteReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for deleteReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in deleteReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->deleteReference))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_isSame(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  long _okay = 0;
  struct ASE_BaseInterface__object *obj = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isSame");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in isSame");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_BaseInterface__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->isSame))(_self_ior, obj, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getClassName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getClassName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getClassName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getClassName))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getName))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_setName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *name = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &name);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setName))(_self_ior, name, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_isInstanceOf(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  long _okay = 0;
  const char *typeName = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isInstanceOf");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in isInstanceOf");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->isInstanceOf))(_self_ior, typeName, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) typeName);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getSuper(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  struct ASE_BaseInterface__object *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getSuper");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getSuper");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getSuper))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_BaseInterface__wrap_ior, _return_value);
  if (_return_value)
  {
    (*((((struct ASE_BaseInterface__object *) _return_value))->d_epv->addReference))(((struct ASE_BaseInterface__object *) _return_value)->d_object, NULL);
  }
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Function_accept(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "visitor",
    NULL};
  long _okay = 0;
  struct ASE_Compiler_Python_Visitor__object *visitor = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for accept");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in accept");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Visitor__extract_ior, &visitor);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->accept))(_self_ior, visitor, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *_obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, _obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Function_copy(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  struct ASE_Compiler_Python_Vertex__object *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for copy");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in copy");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->copy))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Python_Vertex__wrap_ior, _return_value);
  if (_return_value)
  {
    (*((((struct ASE_BaseInterface__object *) _return_value))->d_epv->addReference))(((struct ASE_BaseInterface__object *) _return_value)->d_object, NULL);
  }
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Function_getArgumentNames(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  struct string__array *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getArgumentNames");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getArgumentNames");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getArgumentNames))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) string__wrap_array_copy, _return_value);
  (*(string__externals()->destroyArray))(_return_value, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Function_setArgumentNames(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "argumentNames",
    NULL};
  long _okay = 0;
  struct string__array *argumentNames = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setArgumentNames");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setArgumentNames");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) string__extract_array, &argumentNames);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setArgumentNames))(_self_ior, argumentNames, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Function_getDefaults(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  struct ASE_Compiler_Python_Vertex__array *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getDefaults");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getDefaults");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getDefaults))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Python_Vertex__wrap_array_copy, _return_value);
  (*((*((const struct ASE_Compiler_Python_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Python.Vertex", NULL)))()->destroyArray))(_return_value, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Function_setDefaults(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "defaults",
    NULL};
  long _okay = 0;
  struct ASE_Compiler_Python_Vertex__array *defaults = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setDefaults");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setDefaults");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Vertex__extract_array, &defaults);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setDefaults))(_self_ior, defaults, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *_obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, _obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Function_weakReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  struct ASE_Compiler_Python_Function__object *_self_ior = ((struct ASE_Compiler_Python_Function_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  struct ASE_Compiler_Python_Function__object *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for weakReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in weakReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = _self_ior;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Python_Function__wrap_ior_weak, _return_value);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Function_Function(PyObject *_type, PyObject *_args, PyObject *_kwdict)
{
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  long _okay = 0;
  struct ASE_Compiler_Python_Function__object *obj = NULL;
  int created = 0;
  struct ASE_Compiler_Python_Function__object *_return_value = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "|O&", _kwlist, (void *) ASE_Compiler_Python_Function__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  if (obj)
  {
    created = 0;
  }
  else
  {
    created = 1;
    obj = (*(_epv->createObject))(NULL);
  }
  _return_value = obj;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Python_Function__wrap_ior, _return_value);
  if (_return_value)
  {
    (*((((struct ASE_BaseInterface__object *) _return_value))->d_epv->addReference))(((struct ASE_BaseInterface__object *) _return_value)->d_object, NULL);
  }
  if (created)
  {
    (*(obj->d_epv->deleteReference))(obj->d_object, NULL);
  }
  return _pyReturn;
}

static struct PyMethodDef _FunctionObjectMethods[] = {
  {(char *) "getFlags", (PyCFunction) ASE_Compiler_Python_Flags_getFlags, (METH_VARARGS | METH_KEYWORDS), (char *) "getFlags()\n\
RETURNS (return string _return_value)\n\
 Retrieve the flags"},
  {(char *) "setFlags", (PyCFunction) ASE_Compiler_Python_Flags_setFlags, (METH_VARARGS | METH_KEYWORDS), (char *) "setFlags(in string flags)\n\
RETURNS None\n\
\n\
 Set the flags"},
  {(char *) "clone", (PyCFunction) ASE_Compiler_Vertex_clone, (METH_VARARGS | METH_KEYWORDS), (char *) "clone()\n\
RETURNS (return ASE.Compiler.Vertex _return_value)\n\
 Return an empty object of the same type"},
  {(char *) "getIdentifier", (PyCFunction) ASE_Compiler_Vertex_getIdentifier, (METH_VARARGS | METH_KEYWORDS), (char *) "getIdentifier()\n\
RETURNS (return string _return_value)\n\
 Retrieve the identifier associated with this vertex"},
  {(char *) "setIdentifier", (PyCFunction) ASE_Compiler_Vertex_setIdentifier, (METH_VARARGS | METH_KEYWORDS), (char *) "setIdentifier(in string identifier)\n\
RETURNS None\n\
\n\
 Assign the identifier associated with this vertex"},
  {(char *) "getAttributes", (PyCFunction) ASE_Compiler_Vertex_getAttributes, (METH_VARARGS | METH_KEYWORDS), (char *) "getAttributes()\n\
RETURNS (return array _return_value)\n\
 Get attribute list for this element"},
  {(char *) "setAttributes", (PyCFunction) ASE_Compiler_Vertex_setAttributes, (METH_VARARGS | METH_KEYWORDS), (char *) "setAttributes(in array attributes)\n\
RETURNS None\n\
\n\
 Set attribute list for this element"},
  {(char *) "hasAttribute", (PyCFunction) ASE_Compiler_Vertex_hasAttribute, (METH_VARARGS | METH_KEYWORDS), (char *) "hasAttribute(in string name)\n\
RETURNS (return bool _return_value)\n\
 Determine whether an attribute is present on this vertex"},
  {(char *) "addAttribute", (PyCFunction) ASE_Compiler_Vertex_addAttribute, (METH_VARARGS | METH_KEYWORDS), (char *) "addAttribute(in ASE.Compiler.Attribute attribute)\n\
RETURNS None\n\
\n\
 Add attribute to this vertex"},
  {(char *) "deleteAttribute", (PyCFunction) ASE_Compiler_Vertex_deleteAttribute, (METH_VARARGS | METH_KEYWORDS), (char *) "deleteAttribute(in string name)\n\
RETURNS None\n\
\n\
 Remove an attribute on this vertex   Throws an exception if the attribute does not exist"},
  {(char *) "getAttribute", (PyCFunction) ASE_Compiler_Vertex_getAttribute, (METH_VARARGS | METH_KEYWORDS), (char *) "getAttribute(in string name)\n\
RETURNS (return ASE.Compiler.Attribute _return_value)\n\
 Get an attribute on this vertex   Throws an exception if the attribute does not exist"},
  {(char *) "getParent", (PyCFunction) ASE_Compiler_Vertex_getParent, (METH_VARARGS | METH_KEYWORDS), (char *) "getParent()\n\
RETURNS (return ASE.Compiler.Vertex _return_value)\n\
 Retrieve the parent of this vertex"},
  {(char *) "setParent", (PyCFunction) ASE_Compiler_Vertex_setParent, (METH_VARARGS | METH_KEYWORDS), (char *) "setParent(in ASE.Compiler.Vertex parent)\n\
RETURNS None\n\
\n\
 Assign the parent of this vertex"},
  {(char *) "getChildren", (PyCFunction) ASE_Compiler_Vertex_getChildren, (METH_VARARGS | METH_KEYWORDS), (char *) "getChildren()\n\
RETURNS (return array _return_value)\n\
 Retrieve the children of this vertex"},
  {(char *) "setChildren", (PyCFunction) ASE_Compiler_Vertex_setChildren, (METH_VARARGS | METH_KEYWORDS), (char *) "setChildren(in array children)\n\
RETURNS None\n\
\n\
 Set the children of this vertex"},
  {(char *) "addChildren", (PyCFunction) ASE_Compiler_Vertex_addChildren, (METH_VARARGS | METH_KEYWORDS), (char *) "addChildren(in array children)\n\
RETURNS None\n\
\n\
 Add children to this vertex (duplicates allowed)"},
  {(char *) "removeChildren", (PyCFunction) ASE_Compiler_Vertex_removeChildren, (METH_VARARGS | METH_KEYWORDS), (char *) "removeChildren(in array children)\n\
RETURNS None\n\
\n\
 Remove children from this vertex   Throws an exception if any child does not exist"},
  {(char *) "getLineNumber", (PyCFunction) ASE_Compiler_Vertex_getLineNumber, (METH_VARARGS | METH_KEYWORDS), (char *) "getLineNumber()\n\
RETURNS (return int _return_value)\n\
 Retrieve the line number associated with this vertex"},
  {(char *) "setLineNumber", (PyCFunction) ASE_Compiler_Vertex_setLineNumber, (METH_VARARGS | METH_KEYWORDS), (char *) "setLineNumber(in int lineNumber)\n\
RETURNS None\n\
\n\
 Set the line number associated with this vertex"},
  {(char *) "getComments", (PyCFunction) ASE_Compiler_Vertex_getComments, (METH_VARARGS | METH_KEYWORDS), (char *) "getComments()\n\
RETURNS (return array _return_value)\n\
 Retrieve the comments of this vertex"},
  {(char *) "setComments", (PyCFunction) ASE_Compiler_Vertex_setComments, (METH_VARARGS | METH_KEYWORDS), (char *) "setComments(in array comments)\n\
RETURNS None\n\
\n\
 Set the children of this vertex"},
  {(char *) "addComments", (PyCFunction) ASE_Compiler_Vertex_addComments, (METH_VARARGS | METH_KEYWORDS), (char *) "addComments(in array comments)\n\
RETURNS None\n\
\n\
 Add comments to this vertex (duplicates allowed)"},
  {(char *) "removeComments", (PyCFunction) ASE_Compiler_Vertex_removeComments, (METH_VARARGS | METH_KEYWORDS), (char *) "removeComments(in array comments)\n\
RETURNS None\n\
\n\
 Remove comments from this vertex   Throws an exception if a comment does not exist"},
  {(char *) "getReferenceCount", (PyCFunction) ASE_BaseInterface_getReferenceCount, (METH_VARARGS | METH_KEYWORDS), (char *) "getReferenceCount()\n\
RETURNS (return int _return_value)\n\
 Retrieve the intrinsic reference count of the underlying object"},
  {(char *) "addReference", (PyCFunction) ASE_BaseInterface_addReference, (METH_VARARGS | METH_KEYWORDS), (char *) "addReference()\n\
RETURNS None\n\
\n\
 Add one to the intrinsic reference count of the underlying object - Objects continue to exist as long as the reference count is positive - Clients should call this method whenever they create another ongoing reference to an object or interface"},
  {(char *) "deleteReference", (PyCFunction) ASE_BaseInterface_deleteReference, (METH_VARARGS | METH_KEYWORDS), (char *) "deleteReference()\n\
RETURNS None\n\
\n\
 Decrease by one the intrinsic reference count of the underlying object - Delete the object if the reference count is non-positive - Clients should call this method whenever they remove a reference to an object or interface"},
  {(char *) "isSame", (PyCFunction) ASE_BaseInterface_isSame, (METH_VARARGS | METH_KEYWORDS), (char *) "isSame(in ASE.BaseInterface obj)\n\
RETURNS (return bool _return_value)\n\
 Return true if and only if <code>obj</code> refers to the same underlying object as this object"},
  {(char *) "getClassName", (PyCFunction) ASE_BaseInterface_getClassName, (METH_VARARGS | METH_KEYWORDS), (char *) "getClassName()\n\
RETURNS (return string _return_value)\n\
 Return the type name of the underlying class object"},
  {(char *) "getName", (PyCFunction) ASE_BaseInterface_getName, (METH_VARARGS | METH_KEYWORDS), (char *) "getName()\n\
RETURNS (return string _return_value)\n\
 Retrieve the object name"},
  {(char *) "setName", (PyCFunction) ASE_BaseInterface_setName, (METH_VARARGS | METH_KEYWORDS), (char *) "setName(in string name)\n\
RETURNS None\n\
\n\
 Set the object name"},
  {(char *) "isInstanceOf", (PyCFunction) ASE_BaseInterface_isInstanceOf, (METH_VARARGS | METH_KEYWORDS), (char *) "isInstanceOf(in string typeName)\n\
RETURNS (return bool _return_value)\n\
 Return <code>true</code> if this object is an instance of the specified type - This routine will return <code>true</code> if and only if a cast to the specified type would succeed"},
  {(char *) "getSuper", (PyCFunction) ASE_BaseInterface_getSuper, (METH_VARARGS | METH_KEYWORDS), (char *) "getSuper()\n\
RETURNS (return ASE.BaseInterface _return_value)\n\
 Retrieve an object with the parent interface IMPL: Obviously, this must be implemented in the IOR of each interface/class"},
  {(char *) "accept", (PyCFunction) ASE_Compiler_Python_Function_accept, (METH_VARARGS | METH_KEYWORDS), (char *) "accept(in ASE.Compiler.Python.Visitor visitor)\n\
RETURNS None\n\
\n\
 Overridden to dispatch correctly"},
  {(char *) "copy", (PyCFunction) ASE_Compiler_Python_Function_copy, (METH_VARARGS | METH_KEYWORDS), (char *) "copy()\n\
RETURNS (return ASE.Compiler.Python.Vertex _return_value)\n\
 Need to override this"},
  {(char *) "getArgumentNames", (PyCFunction) ASE_Compiler_Python_Function_getArgumentNames, (METH_VARARGS | METH_KEYWORDS), (char *) "getArgumentNames()\n\
RETURNS (return array _return_value)\n\
 Retrieve the argument names"},
  {(char *) "setArgumentNames", (PyCFunction) ASE_Compiler_Python_Function_setArgumentNames, (METH_VARARGS | METH_KEYWORDS), (char *) "setArgumentNames(in array argumentNames)\n\
RETURNS None\n\
\n\
 Set the argument names"},
  {(char *) "getDefaults", (PyCFunction) ASE_Compiler_Python_Function_getDefaults, (METH_VARARGS | METH_KEYWORDS), (char *) "getDefaults()\n\
RETURNS (return array _return_value)\n\
 Retrieve the default argument values"},
  {(char *) "setDefaults", (PyCFunction) ASE_Compiler_Python_Function_setDefaults, (METH_VARARGS | METH_KEYWORDS), (char *) "setDefaults(in array defaults)\n\
RETURNS None\n\
\n\
 Set the default argument values"},
  {(char *) "weakReference", (PyCFunction) ASE_Compiler_Python_Function_weakReference, (METH_VARARGS | METH_KEYWORDS), (char *) "weakReference()\n\
RETURNS (return ASE.Compiler.Python.Function _return_value)\n\
"},
  {NULL, NULL}};

static struct PyMethodDef _FunctionModuleMethods[] = {{NULL, NULL}};

static void sidl_dealloc(struct ASE_Compiler_Python_Function_PythonObject *self)
{
  if (self->weakRefList)
  {
    PyObject_ClearWeakRefs((PyObject *) self);
  }
  if ((self->refType != WEAK_REF) && (self->ior->d_data))
  {
    (*(((struct ASE_BaseInterface__object *) self->ior)->d_epv->deleteReference))(self->ior->d_object, NULL);
  }
  self->ob_type->tp_free(self);
}

static int sidl_cmp(struct ASE_Compiler_Python_Function_PythonObject *self, PyObject *arg)
{
  struct ASE_Compiler_Python_Function__object *ior1 = self->ior;
  struct ASE_BaseInterface__object *ior2 = (struct ASE_BaseInterface__object *) ((struct ASE_Compiler_Python_Function_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (arg->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for sidl_cmp");
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1->d_object);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2->d_object);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  _return_value = PyObject_Compare(o1, o2);
  Py_DECREF(o1);
  Py_DECREF(o2);
  return _return_value;
}

static long sidl_hash(struct ASE_Compiler_Python_Function_PythonObject *self)
{
  struct ASE_Compiler_Python_Function__object *ior = self->ior;
  PyObject *_hash_arg = NULL;
  long _return_value = 0;
  
  if (!ior)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_hash()");
    return _return_value;
  }
  _hash_arg = Py_BuildValue((char *) "i", ior->d_object);
  if (!_hash_arg)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not hash IOR pointer in sidl_hash()");
    return _return_value;
  }
  _return_value = PyObject_Hash(_hash_arg);
  Py_DECREF(_hash_arg);
  return _return_value;
}

static PyObject *sidl_str(struct ASE_Compiler_Python_Function_PythonObject *self)
{
  struct ASE_Compiler_Python_Function__object *ior = self->ior;
  const char *_str = NULL;
  PyObject *_return_value = NULL;
  
  _str = (*(ior->d_epv->strObject))(ior, NULL);
  _return_value = PyString_FromString(_str);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyObject *sidl_richcmp(struct ASE_Compiler_Python_Function_PythonObject *self, PyObject *arg, int op)
{
  struct ASE_Compiler_Python_Function__object *ior1 = self->ior;
  struct ASE_BaseInterface__object *ior2 = (struct ASE_BaseInterface__object *) ((struct ASE_Compiler_Python_Function_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  PyObject *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (!PyObject_HasAttrString(arg, (char *) "getClassName"))
  {
    switch(op) {
      case Py_EQ:
        _return_value = Py_False;
        Py_INCREF(_return_value);
        break;
      case Py_NE:
        _return_value = Py_True;
        Py_INCREF(_return_value);
        break;
      case Py_LT:
      case Py_LE:
      case Py_GT:
      case Py_GE:
        PyErr_SetString(PyExc_TypeError, (char *) "sidl_richcmp: Cannot order non-SIDL object");
    }
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1->d_object);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2->d_object);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  _return_value = PyObject_RichCompare(o1, o2, op);
  Py_DECREF(o1);
  Py_DECREF(o2);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyObject *sidl_getiter(struct ASE_Compiler_Python_Function_PythonObject *self)
{
  Py_INCREF(self);
  return (PyObject *) self;
}

static PyObject *sidl_iternext(struct ASE_Compiler_Python_Function_PythonObject *self)
{
  struct ASE_Compiler_Python_Function__iterator *ior = self->ior;
  struct ASE_Compiler_Python_Function__object *nextIorObj = (*(((struct ASE_BaseInterface__object *) ior)->d_epv->iterNext))(ior, NULL);
  static struct ASE_Compiler_Python_Function_PythonObject *nextPyObj = NULL;
  
  if (!nextIorObj)
  {
    if (nextPyObj)
    {
      Py_DECREF(nextPyObj);
      nextPyObj = NULL;
      (*(_epv->destroyIterator))(ior, NULL);
      self->ior = NULL;
    }
    return NULL;
  }
  if (!ior->objWrapper)
  {
    ior->objWrapper = sidl_new(&SIDLPythonObjectType, (struct ASE_BaseInterface__object *) nextIorObj, WEAK_REF);
    nextPyObj = (struct ASE_Compiler_Python_Function_PythonObject *) ior->objWrapper;
  }
  else
  {
    nextPyObj = (struct ASE_Compiler_Python_Function_PythonObject *) ior->objWrapper;
    nextPyObj->ior = nextIorObj;
  }
  Py_INCREF(nextPyObj);
  return (PyObject *) nextPyObj;
}

static PyTypeObject SIDLPythonObjectType = {
  PyObject_HEAD_INIT(NULL)
  0,
  (char *) "ASE.Compiler.Python.Function",
  sizeof(struct ASE_Compiler_Python_Function_PythonObject),
  0,
  (destructor) sidl_dealloc,
  (printfunc) 0,
  (getattrfunc) 0,
  (setattrfunc) 0,
  (cmpfunc) sidl_cmp,
  (reprfunc) 0,
  (PyNumberMethods *) 0,
  (PySequenceMethods *) 0,
  (PyMappingMethods *) 0,
  (hashfunc) sidl_hash,
  (ternaryfunc) 0,
  (reprfunc) sidl_str,
  (getattrofunc) 0,
  (setattrofunc) 0,
  (PyBufferProcs *) 0,
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_WEAKREFS,
  (char *) "This is a Python wrapper for a SIDL object of type ASE.Compiler.Python.Function",
  (traverseproc) 0,
  (inquiry) 0,
  (richcmpfunc) sidl_richcmp,
  (int) &((struct ASE_Compiler_Python_Function_PythonObject *) 0)->weakRefList,
  (getiterfunc) sidl_getiter,
  (iternextfunc) sidl_iternext,
  _FunctionObjectMethods,
  (struct PyMemberDef *) 0,
  (struct PyGetSetDef *) 0,
  0,
  0,
  (descrgetfunc) 0,
  (descrsetfunc) 0,
  0,
  (initproc) 0,
  (allocfunc) 0,
  (newfunc) ASE_Compiler_Python_Function_Function,
  (freefunc) 0,
  (inquiry) 0};

void initFunction(void)
{
  PyObject *module = NULL;
  const struct ASE_Compiler_Python_Function__external *(*_extFunc)(void);
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Setup the python type */
  if (PyType_Ready(&SIDLPythonObjectType) < 0)
  {
    return;
  }
  /* Initialize the module */
  module = Py_InitModule3((char *) "Function", _FunctionModuleMethods, (char *) " A Python function, bound or unbound   The children of this node constitue the body of the function   Default values are not yet supported");
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Could not initialize module ASE.Compiler.Python.Function");
    return;
  }
  /* Add the type to the module and increment the ref count */
  Py_INCREF(&SIDLPythonObjectType);
  PyModule_AddObject(module, (char *) "Function", (PyObject *) &SIDLPythonObjectType);
  /* Set externals function */
  _extFunc = (const struct ASE_Compiler_Python_Function__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(&_exception)->getClassMethodTable("ASE.Compiler.Python.Function", &_exception);
  /* Check exception */
  if (_exception)
  {
    _exception = _exception->d_epv->castObject(_exception, "ASE.BaseException", NULL);
    PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    return;
  }
  /* Set epv pointer */
  if (_extFunc)
  {
    _epv = (*_extFunc)();
  }
  if (!_epv)
  {
    PyErr_SetString(PyExc_ImportError, "Cannot load SIDL implementation for ASE.Compiler.Python.Function");
    return;
  }
  /* Import Numeric */
  import_array();
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Cannot import Numeric Python array module");
    return;
  }
}
