#!/usr/bin/env python
import user
import importer
import script

import os

class Tarballer(script.Script):
  def setupHelp(self, help):
    import nargs

    help = script.Script.setupHelp(self, help)
    help.addArgument('Tarballer', '-createTarball', nargs.ArgBool(None, 1, 'Create the bootstrap tarball', isTemporary = 1))
    help.addArgument('Tarballer', '-tarballName', nargs.Arg(None, 'Compiler.tgz', 'Filename for the bootstrap tarball', isTemporary = 1))
    help.addArgument('Tarballer', '-pushTarball', nargs.ArgBool(None, 0, 'Replace the bootstrap tarball on the PETSc site', isTemporary = 1))
    return help

  def setup(self):
    script.Script.setup(self)
    self.compilerDir = os.path.abspath(os.getcwd())
    return

  def getTarballName(self):
    if not hasattr(self, '_tarballName'):
      self._tarballName = self.argDB['tarballName']
    return self._tarballName
  def setTarballName(self, tarballName):
    self._tarballName = tarballName
    return
  tarballName = property(getTarballName, setTarballName, doc = 'The tarball filename')

  def prepareRDict(self):
    '''Retain a copy of RDict.db, and remove system specific items'''
    import RDict
    import shutil

    self.oldRDict = os.path.join(self.compilerDir, 'RDict.db')
    self.tmpRDict = os.tempnam(os.path.dirname(self.compilerDir), 'RDict.db')
    shutil.copyfile(self.oldRDict, self.tmpRDict)
    self.logPrint('Copied '+self.oldRDict+' to '+self.tmpRDict)

    d = RDict.RDict(autoShutdown = 0)
    del d['SCANDAL_DIR']
    self.logPrint('Removed SCANDAL_DIR from RDict')
    d.shutdown()
    return

  def createTarball(self):
    import os

    if os.path.isfile(self.tarballName):
      self.logPrint('Removed old tarball: '+self.tarballName)
      os.remove(self.tarballName)
    filename = os.tempnam(os.path.dirname(self.compilerDir), 'Compiler')
    excludes = ['*.pyc', '*.o', '*.so', '*~', '*.log', '*.bkp', 'parsetab.py']
    flags = ['-cvz']
    cmd = ['tar']
    cmd.extend(['--exclude "'+exclude+'"' for exclude in excludes])
    cmd.extend(flags)
    cmd.extend(['-f', filename])
    cmd.extend(['-C', os.path.dirname(self.compilerDir), 'Compiler'])
    self.logPrint('Making tarball: '+filename)
    self.executeShellCommand(' '.join(cmd))
    os.rename(filename, self.tarballName)
    return

  def restoreRDict(self):
    '''Restore the unmodified Rdict.db'''
    import shutil

    shutil.copyfile(self.tmpRDict, self.oldRDict)
    self.logPrint('Copied '+self.tmpRDict+' to '+self.oldRDict)
    os.remove(self.tmpRDict)
    self.logPrint('Removed '+self.tmpRDict)
    return

  def pushTarball(self):
    '''Copy the new tarball to the PETSc site'''
    self.logPrint('Pushing new tarball to ANL')
    cmd = ['scp', self.tarballName, 'harley.mcs.anl.gov:/mcs/ftp/pub/petsc/ase']
    self.executeShellCommand(' '.join(cmd))
    return

  def run(self):
    self.setup()
    if self.argDB['createTarball']:
      self.prepareRDict()
      self.createTarball()
      self.restoreRDict()
    if self.argDB['pushTarball']:
      self.pushTarball()
    return

if __name__ == '__main__':
  Tarballer().run()
