#!/usr/bin/env python
from __future__ import generators
import user
import script

import os

class Constructor(script.Script):
  def __init__(self, argDB = None):
    import sys
    script.Script.__init__(self, sys.argv[1:])
    return

  def setupHelp(self, help):
    import nargs

    help = script.Script.setupHelp(self, help)
    help.addArgument('Constructor', 'rootUrl',       nargs.Arg(None, None, 'The URL for the head project', isTemporary = 1))
    help.addArgument('Constructor', 'bootstrapRoot', nargs.Arg(None, None, 'The path for the root of the bootstrap install', isTemporary = 1))
    help.addArgument('Constructor', 'defaultRoot',   nargs.Arg(None, '/PETSc3', 'The root directory for all projects'))
    help.addArgument('Constructor', 'clear',         nargs.ArgBool(None, 0, 'Clear installed projects', isTemporary = 1))
    help.addArgument('Constructor', 'install',       nargs.ArgBool(None, 0, 'Install all dependent projects', isTemporary = 1))
    help.addArgument('Constructor', 'show',          nargs.ArgBool(None, 0, 'Show all dependent projects', isTemporary = 1))
    help.addArgument('Constructor', 'showInstalled', nargs.ArgBool(None, 0, 'Show all installed projects', isTemporary = 1))
    help.addArgument('Constructor', 'ASE1',          nargs.ArgBool(None, 0, 'Defaults for ASE 1', isTemporary = 1))
    help.addArgument('Constructor', 'ASE2',          nargs.ArgBool(None, 0, 'Defaults for ASE 2', isTemporary = 1))
    return help

  def getRootUrl(self):
    if not hasattr(self, '_rootUrl'):
      self._rootUrl = self.argDB['rootUrl']
      if not isinstance(self._rootUrl, list):
        self._rootUrl = [self._rootUrl]
    return self._rootUrl

  def setRootUrl(self, rootUrl):
    self._rootUrl = rootUrl
    return
  rootUrl = property(getRootUrl, setRootUrl, doc = 'The list of URLs for the head projects')

  def getMake(self, path):
    oldDir = os.getcwd()
    os.chdir(path)
    mod = self.getModule(path, 'make')
    if hasattr(mod, 'Make'):
      make = mod.Make()
    elif hasattr(mod, 'PetscMake'):
      make = mod.PetscMake()
    else:
      raise RuntimeError('Cannot instantiate a make object')
    os.chdir(oldDir)
    return make

  def getDependencies(self, rootUrls, urlSeen = []):
    import install.urlMapping
    for rootUrl in rootUrls:
      if rootUrl in urlSeen:
        return
      rootMake = self.getMake(os.path.join(self.argDB['defaultRoot'], install.urlMapping.UrlMappingNew.getRepositoryPath(rootUrl)))
      urlSeen.append(rootUrl)
      if hasattr(rootMake, 't_getDependencies'):
        dependencies = rootMake.t_getDependencies()
      elif hasattr(rootMake, 'dependencies'):
        rootMake.setup()
        dependencies = rootMake.dependencies.keys()
      else:
        raise RuntimeError('Cannot get dependencies for make')
      for make in self.getDependencies(dependencies, urlSeen):
        yield make
      yield rootMake
    return

  def installProject(self, make):
    if hasattr(make, 'mainBuild'):
      make.mainBuild(['activate', 'install'])
    elif hasattr(make, 'install'):
      oldDir = os.getcwd()
      os.chdir(make.root)
      make.configure(make.builder)
      if hasattr(make, 'buildSetup'):
        make.buildSetup(make.builder)
      make.install(make.builder, self.argDB)
      os.chdir(oldDir)
    else:
      raise RuntimeError('Could not install project at '+make.getRoot())
    return

  def showProject(self, project):
    print 'url',project.url
    print '  paths',project.paths
    print '  pkgs',project.packages
    print '  impls',project.implementations
    return

  def run(self):
    self.setup()
    if self.argDB['ASE1']:
      self.argDB['clear'] = 1
      self.argDB['rootUrl'] = ['bk://sidl.bkbits.net/Compiler', 'bk://ase.bkbits.net/Compiler', 'bk://ase.bkbits.net/Runtime']
      self.argDB['install'] = 1
      self.argDB['showInstalled'] = 1
    elif self.argDB['ASE2']:
      self.argDB['clear'] = 1
      self.argDB['rootUrl'] = ['bk://ply.bkbits.net/ply-dev', 'bk://ase.bkbits.net/Compiler-new']
      self.argDB['install'] = 1
      self.argDB['showInstalled'] = 1
    if self.argDB['clear']:
      self.argDB['installedprojects'] = []
    if self.argDB['install']:
      if 'bootstrapRoot' in self.argDB:
        import install.urlMapping
        if not 'installedprojects' in self.argDB:
          self.argDB['installedprojects'] = []
        self.installProject(self.getMake(os.path.join(self.argDB['defaultRoot'], install.urlMapping.UrlMappingNew.getRepositoryPath('bk://ply.bkbits.net/ply-dev'))))
        for make in [self.getMake(os.path.join(self.argDB['bootstrapRoot'], repo)) for repo in ['Runtime', 'Compiler']]:
          for sidlFile in make.sidl:
            baseName = os.path.splitext(os.path.basename(sidlFile))[0]
            dir = os.path.join(make.root, 'client-bootstrap')
            if not os.path.isdir(dir):
              os.mkdir(dir)
            make.project.appendPath('Python', dir)
            for language in make.serverLanguages:
              make.project.appendPath(language, os.path.join(make.root, 'server-'+language.lower()+'-'+baseName))
            for language in make.clientLanguages:
              dir = os.path.join(make.root, 'client-'+language.lower())
              if not os.path.isdir(dir):
                os.mkdir(dir)
              make.project.appendPath(language, dir)
          projects = filter(lambda project: not project.getUrl() == make.project.getUrl(), self.argDB['installedprojects'])
          self.argDB['installedprojects'] = projects+[make.project]
      else:
        for make in self.getDependencies(self.rootUrl):
          self.installProject(make)
    if self.argDB['show']:
      print 'Dependent Projects:'
      print '-------------------'
      for make in self.getDependencies(self.rootUrl):
        self.showProject(make.project)
    if self.argDB['showInstalled']:
      print 'Installed Projects:'
      print '-------------------'
      for project in self.argDB['installedprojects']:
        self.showProject(project)
    return

if __name__ == '__main__':
  Constructor().run()
