#!/usr/bin/env python
import user
import importer
import unittest
import os
import sys

def checkMemory(printClass = 0):
  import ASE.BaseInterface
  import ASE.BaseClass_impl
  printClass = printClass and hasattr(ASE.BaseInterface.BaseInterface, 'getIORClassName')
  for addr in ASE.BaseClass_impl.objects:
    if ASE.BaseClass_impl.objects[addr] == 1:
      if printClass:
        if not ASE.BaseInterface.BaseInterface.getIORClassName(addr) == 'ASE.DLL':
          print 'Failed to destroy object',hex(addr),ASE.BaseInterface.BaseInterface.getIORClassName(addr)
      else:
        print 'Failed to destroy object',hex(addr),'(probably ASE.DLL)'
##    elif ASE.BaseClass_impl.objects[addr] == 2:
##      print 'Object',hex(addr),'successfully destroyed'
  return

class CxxPreprocessor (unittest.TestCase):
  '''Tests the Cxx parser implementation'''
  def __init__(self, methodName):
    unittest.TestCase.__init__(self, methodName)
    self.baseDirectory = os.path.dirname(os.path.dirname(os.getcwd()))
    self.setupPaths()
    return

  def setupPaths(self):
    '''If Scandal was called as a script, setup both the python include path and the loader search path'''
    if not __name__ == '__main__':
      return
    dirs = [os.path.join(self.baseDirectory, 'ase', 'Runtime'),
            os.path.join(self.baseDirectory, 'ase', 'Compiler')]
    serverDirs = [os.path.join(dirs[0], 'server-python-ase'),
                  os.path.join(dirs[1], 'server-python-compiler'),
                  os.path.join(dirs[1], 'server-python-sidl'),
                  os.path.join(dirs[1], 'server-python-sidlVisitor'),
                  os.path.join(dirs[1], 'server-python-pythonI'),
                  os.path.join(dirs[1], 'server-python-pythonII'),
                  os.path.join(dirs[1], 'server-python-pythonIII'),
                  os.path.join(dirs[1], 'server-python-pythonVisitor'),
                  os.path.join(dirs[1], 'server-python-bootstrap'),
                  os.path.join(dirs[1], 'server-python-cxxI'),
                  os.path.join(dirs[1], 'server-python-cxxII'),
                  os.path.join(dirs[1], 'server-python-cxxIII'),
                  os.path.join(dirs[1], 'server-python-cxxIV'),
                  os.path.join(dirs[1], 'server-python-cxxV'),
                  os.path.join(dirs[1], 'server-python-cxxVisitor')]
    sys.path.extend([os.path.join(self.baseDirectory, 'ply', 'ply-dev')])
    sys.path.extend([os.path.join(d, 'client-bootstrap') for d in dirs])
    sys.path.extend([os.path.join(d, 'client-python') for d in dirs])
    sys.path.extend([d for d in serverDirs])
    import ASE.Loader
    ASE.Loader.Loader.setSearchPath([os.path.join(d, 'lib') for d in dirs])
    import atexit
    atexit.register(checkMemory)
    atexit.register(ASE.Loader.Loader.setLibraries, [])
    return

  def setUp(self):
    import ASE.Compiler.Cxx.cIncludeExpansion
    import ASE.Compiler.Cxx.cparse
    import ASE.Compiler.Cxx.Printer
    import cStringIO

    self.oldStdout = sys.stdout
    sys.stdout     = cStringIO.StringIO()
    self.expander  = ASE.Compiler.Cxx.cIncludeExpansion.IncludeExpander()
    self.parser    = ASE.Compiler.Cxx.cparse.setup()
    self.printer   = ASE.Compiler.Cxx.Printer.Printer()
    ASE.Compiler.Cxx.cparse.lexer = self.expander
    return

  def tearDown(self):
    import ASE.Compiler.Cxx.cparse

    ASE.Compiler.Cxx.cparse.typeDefs.clear()
    self.parser.restart()
    self.expander.typeDefs.clear()
    self.expander = None
    self.parser   = None
    self.printer  = None
    sys.stdout.close()
    sys.stdout = self.oldStdout
    import gc
    gc.collect()
    checkMemory(1)
    return

  def testPreprocessorConditionals(self):
    '''Verify that preprocessor conditionals are correct handled and preserved'''
    import ASE.Compiler.Cxx.Define

    code = '''\
#ifndef MATT
  #define MATT 1
#endif /* MATT */
'''
    ast = self.parser.parse(code, lexer = self.expander)
    self.assertEqual(len(ast), 1, 'Should return a single directive')
    self.assertEqual(len(ast[0].getChildren()), 1, 'Should return a single declaration')
    self.assertEqual(ASE.Compiler.Cxx.Define.Define(ast[0].getChildren()[0]).getReplacementText(), '1')
    self.expander.clearDefines()
    code = '''\
#define MATT
#ifdef MATT
  #define KNEPLEY 2
#endif /* MATT */
'''
    ast = self.parser.parse(code, lexer = self.expander)
    self.assertEqual(len(ast), 2, 'Should return both directives')
    self.assertEqual(len(ast[1].getChildren()), 1, 'Should return a single declaration')
    self.assertEqual(ASE.Compiler.Cxx.Define.Define(ast[1].getChildren()[0]).getReplacementText(), '2')
    self.expander.clearDefines()
    code = '''\
#ifndef MATT
  #define MATT 1
#else
  #define KNEPLEY 2
#endif /* MATT */
'''
    ast = self.parser.parse(code, lexer = self.expander)
    self.assertEqual(len(ast), 1)
    self.assertEqual(len(ast[0].getChildren()), 1)
    self.assertEqual(ASE.Compiler.Cxx.Define.Define(ast[0].getChildren()[0]).getReplacementText(), '1')
    self.assertEqual(ast[0].getInactiveCode(), '# define KNEPLEY 2')
    self.expander.clearDefines()
    code = '''\
#ifdef MATT
  #define MATT 1
#else
  #define KNEPLEY 2
#endif /* MATT */
'''
    ast = self.parser.parse(code, lexer = self.expander)
    self.assertEqual(len(ast), 1)
    self.assertEqual(len(ast[0].getChildren()), 1)
    self.assertEqual(ASE.Compiler.Cxx.Define.Define(ast[0].getChildren()[0]).getReplacementText(), '2')
    self.assertEqual(ast[0].getInactiveCode(), '# define MATT 1')
    self.expander.clearDefines()
    return

  def testPreprocessorNestedConditionals(self):
    '''Verify that nested preprocessor conditionals are correct handled and preserved'''
    import ASE.Compiler.Cxx.Define

    code = '''\
#ifndef MATT
  #define MATT 1
  #ifdef MATT
    #define KNEPLEY 2
  #endif
#endif /* MATT */
'''
    ast = self.parser.parse(code, lexer = self.expander)
    self.assertEqual(len(ast), 1, 'Should return a single directive')
    self.assertEqual(len(ast[0].getChildren()), 2, 'Should return two declarations')
    self.assertEqual(ASE.Compiler.Cxx.Define.Define(ast[0].getChildren()[0]).getReplacementText(), '1')
    self.assertEqual(len(ast[0].getChildren()[1].getChildren()), 1, 'Should return a single declaration')
    self.assertEqual(ASE.Compiler.Cxx.Define.Define(ast[0].getChildren()[1].getChildren()[0]).getReplacementText(), '2')
    return

  def testBasicDeclaration(self):
    '''Verify that declarations of basic types are valid'''
    code = '''\
char mattChar;
double mattDouble;
float mattFloat;
int mattInt;
long mattLong;
short mattShort;
'''
    ast = self.parser.parse(code, lexer = self.expander)
    [tree.accept(self.printer) for tree in ast]
    self.assertEqual(sys.stdout.getvalue(), code)
    return

  def testBasicPointerDeclaration(self):
    '''Verify that declarations of pointers to basic types are valid'''
    code = '''\
char *mattChar;
double *mattDouble;
float *mattFloat;
int *mattInt;
long *mattLong;
short *mattShort;
void *mattVoid1;
void **mattVoid2;
void ***mattVoid3;
void ****mattVoid4;
'''
    ast = self.parser.parse(code, lexer = self.expander)
    [tree.accept(self.printer) for tree in ast]
    self.assertEqual(sys.stdout.getvalue(), code)
    return

  def testMultipleDeclarations(self):
    '''Verify that multiple declarations of the same type are normalized to separate declarations'''
    inputCode = '''\
char mattChar1, mattChar2, mattChar3;
int mattInt, *mattIntP, **mattIntPP;
'''
    outputCode = '''\
char mattChar1;
char mattChar2;
char mattChar3;
int mattInt;
int *mattIntP;
int **mattIntPP;
'''
    ast = self.parser.parse(inputCode, lexer = self.expander)
    [tree.accept(self.printer) for tree in ast]
    self.assertEqual(sys.stdout.getvalue(), outputCode)
    return

  def testStructDeclaration(self):
    '''Verify that we can declare structs'''
    code = '''\
struct _mattBasicStruct {
  char mattChar;
  double mattDouble;
  float mattFloat;
  int mattInt;
  long mattLong;
  short mattShort;
};
struct _mattAdvancedStruct {
  char ***mattChar;
  double mattDouble;
  double *mattDoubleP;
  double **mattDoublePP;
};
'''
    ast = self.parser.parse(code, lexer = self.expander)
    [tree.accept(self.printer) for tree in ast]
    self.assertEqual(sys.stdout.getvalue(), code)
    return

  def testBasicTypedef(self):
    '''Verify that we can typedef basic types'''
    code = '''\
typedef char mattChar;
typedef double mattDouble;
typedef float mattFloat;
typedef int mattInt;
typedef long mattLong;
typedef short mattShort;
const mattChar c;
const mattDouble d;
const mattFloat f;
const mattInt i;
const mattLong l;
const mattShort s;
'''
    ast = self.parser.parse(code, lexer = self.expander)
    [tree.accept(self.printer) for tree in ast]
    self.assertEqual(sys.stdout.getvalue(), code)
    return

  def testBasicPointerTypedef(self):
    '''Verify that we can typedef pointers to basic types'''
    code = '''\
typedef char *mattChar;
typedef double *mattDouble;
typedef float *mattFloat;
typedef int *mattInt;
typedef long *mattLong;
typedef short *mattShort;
typedef void *mattVoid1;
typedef void **mattVoid2;
typedef void ***mattVoid3;
typedef void ****mattVoid4;
const mattChar c;
const mattDouble d;
const mattFloat f;
const mattInt i;
const mattLong l;
const mattShort s;
const mattVoid1 v1;
const mattVoid2 v2;
const mattVoid3 v3;
const mattVoid4 v4;
'''
    ast = self.parser.parse(code, lexer = self.expander)
    [tree.accept(self.printer) for tree in ast]
    self.assertEqual(sys.stdout.getvalue(), code)
    return

  def testStructTypedef(self):
    '''Verify that we can typedef structs'''
    code = '''\
typedef struct {
  char mattChar;
  double mattDouble;
  float mattFloat;
  int mattInt;
  long mattLong;
  short mattShort;
} mattStruct;
const mattStruct s;
'''
    ast = self.parser.parse(code, lexer = self.expander)
    [tree.accept(self.printer) for tree in ast]
    self.assertEqual(sys.stdout.getvalue(), code)
    return

if __name__ == '__main__':
  unittest.main()
