# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.DepthFirstVisitor
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Iterator
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.StubGenerator
import ASE.Compiler.Vertex
# Ending imports section
pass
import ASE.Compiler.Python.Add
import ASE.Compiler.Python.Assign
import ASE.Compiler.Python.AssAttr
import ASE.Compiler.Python.AssName
import ASE.Compiler.Python.CallFunc
import ASE.Compiler.Python.Class
import ASE.Compiler.Python.Comment
import ASE.Compiler.Python.Compare
import ASE.Compiler.Python.Const
import ASE.Compiler.Python.Function
import ASE.Compiler.Python.GetAttr
import ASE.Compiler.Python.If
import ASE.Compiler.Python.Import
import ASE.Compiler.Python.Keyword
import ASE.Compiler.Python.List
import ASE.Compiler.Python.Module
import ASE.Compiler.Python.Name
import ASE.Compiler.Python.Raise
import ASE.Compiler.Python.Return
import ASE.Compiler.Python.Slice
import ASE.Compiler.Python.Subscript
import ASE.Compiler.Python.Stmt
import ASE.Compiler.Python.Tuple
import ASE.Compiler.Python.While
# Starting static methods section
pass
# Ending static methods section
pass

class BootstrapStubGenerator:
  '''
  This visitor transforms a SIDL AST into a set of Python Bootstrap Stub ASTs
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.source = {}

    self.packages = [[]]
    self.classes = [[]]
    self.sourceLanguages = ['Python']
    import ASE.Compiler.Python.SIDL
    self.SIDL = ASE.Compiler.Python.SIDL.SIDL()
  
  def getSourceLanguages(self):
    '''
    Retrieve the list of the languages in which ASTs are produced
    '''
    return self.sourceLanguages
  
  def setSourceLanguages(self, outputLanguages):
    '''
    Set the list of the languages in which ASTs are produced
    '''
    self.sourceLanguages = sourceLanguages
  
  def getSource(self, language):
    '''
    Retrieve the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.getSourceLanguages()):
      self.raiseException('Invalid source language: ' + str(language))
    if not (language in self.source):
      self.source[language] = []
    return self.source[language]
  
  def setSource(self, language, vertices):
    '''
    Set the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.getSourceLanguages()):
      self.raiseException('Invalid source language: ' + str(language))
    self.source[language] = list(vertices)
  
  def addSource(self, language, vertices):
    '''
    Add to the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.getSourceLanguages()):
      self.raiseException('Invalid source language: ' + str(language))
    if not (language in self.source):
      self.source[language] = []
    self.source[language].extend(vertices)
  
  def removeSource(self, language, vertices):
    '''
    Remove to the list of source ASTs for the given language
    Throw an exception if a vertex or language does not exist
    '''
    if not (language in self.getSourceLanguages()):
      self.raiseException('Invalid source language: ' + str(language))
    if not (language in self.source):
      self.source[language] = []
    for vertex in vertices:
      if vertex in self.source[language]:
        self.source[language].remove(vertex)
      else:
        e = ASE.Compiler.CompilerException.CompilerException()
        e.setMessage('Source tree does not exist: ' + vertex.getIdentifier())
        raise e
  
  def visitPackage(self, node):
    '''
    Each package has an associated __init__.py module
    '''
    name = ASE.Compiler.Python.Const.Const()
    name.setValue_string('\'' + node.getIdentifier() + '\'')
    self.packageAppend(name)
    module = ASE.Compiler.Python.Module.Module()
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    module.setIdentifier(node.getFullIdentifier() + '.__init__')
    module.setComments(node.getComments())
    module.setChildren([stmt])
    assign = ASE.Compiler.Python.Assign.Assign()
    target = ASE.Compiler.Python.AssName.AssName()
    value = ASE.Compiler.Python.List.List()
    target.setIdentifier('all')
    assign.setValue(value)
    assign.setChildren([target])
    stmt.setChildren([assign])
    self.pushPackage()
    self.__IORself.visitVertex(node)
    value.setChildren(self.popPackage())
    self.addSource('Python', [module])
  
  def visitInterface(self, node):
    '''
    Interfaces merely wrap up classes
    '''
    import ASE.Compiler.SIDL.Method
    name = ASE.Compiler.Python.Const.Const()
    name.setValue_string('\'' + node.getIdentifier() + '\'')
    self.packageAppend(name)
    module = ASE.Compiler.Python.Module.Module()
    module.setIdentifier(node.getFullIdentifier())
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    module.setChildren([stmt])
    klass = ASE.Compiler.Python.Class.Class()
    klass.setIdentifier(node.getIdentifier())
    klass.setComments(node.getComments())
    stmt.addChildren([klass])
    body = ASE.Compiler.Python.Stmt.Stmt()
    klass.setChildren([body])
    body.addChildren(self.createInterfaceConstructor(node))
    body.addChildren(self.createEquality(node))
    body.addChildren(self.createInterfaceGetattr(node))
    self.pushClass()
    self.m_weakReference(node)
    body.addChildren(self.popClass())
    if not (self.SIDL.getBaseException(node) is None):
      assign = ASE.Compiler.Python.Assign.Assign()
      target = ASE.Compiler.Python.AssName.AssName()
      target.setIdentifier('Exception')
      value = ASE.Compiler.Python.Name.Name()
      value.setIdentifier(node.getIdentifier())
      assign.setValue(value)
      assign.setChildren([target])
      module.addChildren([assign])
    self.addSource('Python', [module])
  
  def visitClass(self, node):
    '''
    Classes wrap up implementation classes
    '''
    import ASE.Compiler.SIDL.Method
    name = ASE.Compiler.Python.Const.Const()
    name.setValue_string('\'' + node.getIdentifier() + '\'')
    self.packageAppend(name)
    module = ASE.Compiler.Python.Module.Module()
    module.setIdentifier(node.getFullIdentifier())
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    module.setChildren([stmt])
    if self.hasStaticMethod(node):
      importImpl = ASE.Compiler.Python.Import.Import()
      importImpl.setModuleNames([node.getFullIdentifier() + '_impl'])
      importImpl.setAlias(node.getFullIdentifier() + '_impl', 'implMod')
      stmt.addChildren([importImpl])
    klass = ASE.Compiler.Python.Class.Class()
    klass.setIdentifier(node.getIdentifier())
    klass.setComments(node.getComments())
    stmt.addChildren([klass])
    body = ASE.Compiler.Python.Stmt.Stmt()
    klass.setChildren([body])
    body.addChildren(self.createClassConstructor(node))
    body.addChildren(self.createClassGetattr(node))
    body.addChildren(self.createGetBaseObject(node))
    body.addChildren(self.createEquality(node))
    body.addChildren(self.createHash(node))
    body.addChildren(self.createNonzero(node))
    body.addChildren(self.createRepr(node))
    body.addChildren(self.createStr(node))
    self.pushClass()
    for method in node.getMethods():
      if hasattr(self, 'm_' + method.getIdentifier()):
        getattr(self, 'm_' + method.getIdentifier())(node, method)
    for method in node.getImplementedMethods():
      if (method.getStatic() and not (hasattr(self, 'm_' + method.getIdentifier()))):
        method.accept(self.__IORself)
    self.m_weakReference(node)
    body.addChildren(self.popClass())
    if not (self.SIDL.getBaseException(node) is None):
      assign = ASE.Compiler.Python.Assign.Assign()
      target = ASE.Compiler.Python.AssName.AssName()
      target.setIdentifier('Exception')
      value = ASE.Compiler.Python.Name.Name()
      value.setIdentifier(node.getIdentifier())
      assign.setValue(value)
      assign.setChildren([target])
      module.addChildren([assign])
    self.addSource('Python', [module])
  
  def visitMethod(self, node):
    '''
    Methods delegate directly to the contained object
    '''
    func = self.createSignature(node)
    func.setChildren([self.createMethodBody(node)])
    self.classAppend(func)
    if node.getStatic():
      self.classAppend(self.setStaticMethod(node))
  # Ending class methods section
  pass
  
  def createSignature(self, method):
    func = ASE.Compiler.Python.Function.Function()
    func.setIdentifier(method.getIdentifier())
    argNames = [parameter.getIdentifier() for parameter in method.getInParameters()]
    if not (method.getStatic()):
      argNames.insert(0, 'self')
    func.setArgumentNames(argNames)
    func.setComments(method.getComments())
    return func
  
  def createMethodBody(self, method):
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    if method.getStatic():
      impl = ASE.Compiler.Python.Name.Name()
      impl.setIdentifier('implMod')
    else:
      selfObj = ASE.Compiler.Python.Name.Name()
      selfObj.setIdentifier('self')
      impl = ASE.Compiler.Python.GetAttr.GetAttr()
      impl.setName('obj')
      impl.setChildren([selfObj])
    implFunc = ASE.Compiler.Python.GetAttr.GetAttr()
    implFunc.setName(method.getIdentifier())
    implFunc.setChildren([impl])
    args = []
    for parameter in method.getInParameters():
      arg = ASE.Compiler.Python.Name.Name()
      arg.setIdentifier(parameter.getIdentifier())
      args.append(arg)
    dispatch = ASE.Compiler.Python.CallFunc.CallFunc()
    dispatch.setArguments(args)
    dispatch.setChildren([implFunc])
    returnVal = ASE.Compiler.Python.Return.Return()
    returnVal.setChildren([dispatch])
    stmt.addChildren([returnVal])
    return stmt
  
  def setStaticMethod(self, method, name = None):
    if name is None:
      name = method.getIdentifier()
    assign = ASE.Compiler.Python.Assign.Assign()
    target = ASE.Compiler.Python.AssName.AssName()
    target.setIdentifier(name)
    func = ASE.Compiler.Python.Name.Name()
    func.setIdentifier(name)
    makeStatic = ASE.Compiler.Python.Name.Name()
    makeStatic.setIdentifier('staticmethod')
    value = ASE.Compiler.Python.CallFunc.CallFunc()
    value.setArguments([func])
    value.setChildren([makeStatic])
    assign.setValue(value)
    assign.setChildren([target])
    return assign
  
  def createInterfaceConstructor(self, interface):
    func = ASE.Compiler.Python.Function.Function()
    func.setIdentifier('__init__')
    func.setArgumentNames(['self', 'obj'])
    func.setComments(interface.getComments())
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    assign = ASE.Compiler.Python.Assign.Assign()
    target = ASE.Compiler.Python.AssAttr.AssAttr()
    selfObj = ASE.Compiler.Python.Name.Name()
    selfObj.setIdentifier('self')
    obj = ASE.Compiler.Python.Name.Name()
    obj.setIdentifier('obj')
    target.setIdentifier('obj')
    target.setChildren([selfObj])
    assign.setValue(obj)
    assign.setChildren([target])
    stmt.setChildren([assign])
    func.setChildren([stmt])
    return [func]
  
  def createClassConstructor(self, interface):
    func = ASE.Compiler.Python.Function.Function()
    func.setIdentifier('__init__')
    func.setArgumentNames(['self', 'obj', 'subClasses'])
    func.setComments(interface.getComments())
    func.setDefaults([ASE.Compiler.Python.PyNone.PyNone(), ASE.Compiler.Python.PyNone.PyNone()])
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    pySelf = ASE.Compiler.Python.Name.Name()
    pySelf.setIdentifier('self')
    obj = ASE.Compiler.Python.Name.Name()
    obj.setIdentifier('obj')
    subClasses = ASE.Compiler.Python.Name.Name()
    subClasses.setIdentifier('subClasses')
    classes = ASE.Compiler.Python.GetAttr.GetAttr()
    classes.setName('classes')
    classes.setChildren([pySelf])
    selfObj = ASE.Compiler.Python.GetAttr.GetAttr()
    selfObj.setName('obj')
    selfObj.setChildren([pySelf])
    initClasses = ASE.Compiler.Python.Assign.Assign()
    target = ASE.Compiler.Python.AssAttr.AssAttr()
    target.setIdentifier('classes')
    target.setChildren([pySelf])
    initClasses.setValue(ASE.Compiler.Python.List.List())
    initClasses.setChildren([target])
    stmt.addChildren([initClasses])
    noSubClasses = ASE.Compiler.Python.Compare.Compare()
    noSubClasses.setTestExpr('is', ASE.Compiler.Python.PyNone.PyNone())
    noSubClasses.setChildren([subClasses])
    hasSubClasses = ASE.Compiler.Python.Not.Not()
    hasSubClasses.setChildren([noSubClasses])
    appendFunc = ASE.Compiler.Python.GetAttr.GetAttr()
    appendFunc.setName('extend')
    appendFunc.setChildren([classes])
    appendSubClasses = ASE.Compiler.Python.CallFunc.CallFunc()
    appendSubClasses.setChildren([appendFunc])
    appendSubClasses.setArguments([subClasses])
    discard = ASE.Compiler.Python.Discard.Discard()
    discard.setChildren([appendSubClasses])
    checkSubclasses = ASE.Compiler.Python.If.If()
    checkSubclasses.setChildren([hasSubClasses, discard])
    createObj = ASE.Compiler.Python.Stmt.Stmt()
    createObj.addChildren([checkSubclasses])
    if len(interface.getParents()):
      parent = interface.getParents()[0]
      importParent = ASE.Compiler.Python.Import.Import()
      importParent.setModuleNames([parent.getFullIdentifier()])
      createObj.addChildren([importParent])
      target = ASE.Compiler.Python.AssAttr.AssAttr()
      target.setIdentifier('parent')
      target.setChildren([pySelf])
      parentClass = ASE.Compiler.Python.Name.Name()
      parentClass.setIdentifier(parent.getFullIdentifier())
      parentConstructor = ASE.Compiler.Python.GetAttr.GetAttr()
      parentConstructor.setName(parent.getIdentifier())
      parentConstructor.setChildren([parentClass])
      subClasses = ASE.Compiler.Python.Keyword.Keyword()
      subClasses.setIdentifier('subClasses')
      subClasses.setChildren([classes])
      parentObj = ASE.Compiler.Python.CallFunc.CallFunc()
      parentObj.setChildren([parentConstructor])
      parentObj.setArguments([subClasses])
      initParent = ASE.Compiler.Python.Assign.Assign()
      initParent.setValue(parentObj)
      initParent.setChildren([target])
    else:
      initParent = ASE.Compiler.Python.Assign.Assign()
      target = ASE.Compiler.Python.AssAttr.AssAttr()
      target.setIdentifier('parent')
      target.setChildren([pySelf])
      initParent.setValue(ASE.Compiler.Python.PyNone.PyNone())
      initParent.setChildren([target])
    createObj.addChildren([initParent])
    if not (self.hasStaticMethod(interface)):
      importImpl = ASE.Compiler.Python.Import.Import()
      importImpl.setModuleNames([interface.getFullIdentifier() + '_impl'])
      createObj.addChildren([importImpl])
      constructor = ASE.Compiler.Python.Name.Name()
      constructor.setIdentifier(interface.getFullIdentifier() + '_impl.' + interface.getIdentifier())
    else:
      constructor = ASE.Compiler.Python.Name.Name()
      constructor.setIdentifier('implMod.' + interface.getIdentifier())
    newObj = ASE.Compiler.Python.CallFunc.CallFunc()
    newObj.setChildren([constructor])
    newObj.setArguments([pySelf])
    target = ASE.Compiler.Python.AssAttr.AssAttr()
    target.setIdentifier('obj')
    target.setChildren([pySelf])
    setObj = ASE.Compiler.Python.Assign.Assign()
    setObj.setValue(newObj)
    setObj.setChildren([target])
    createObj.addChildren([setObj])
    appendFunc = ASE.Compiler.Python.GetAttr.GetAttr()
    appendFunc.setName('append')
    appendFunc.setChildren([classes])
    appendObj = ASE.Compiler.Python.CallFunc.CallFunc()
    appendObj.setChildren([appendFunc])
    appendObj.setArguments([selfObj])
    discard = ASE.Compiler.Python.Discard.Discard()
    discard.setChildren([appendObj])
    createObj.addChildren([discard])
    if len(interface.getParents()):
      parent = ASE.Compiler.Python.GetAttr.GetAttr()
      parent.setName('parent')
      parent.setChildren([pySelf])
      parentClasses = ASE.Compiler.Python.GetAttr.GetAttr()
      parentClasses.setName('classes')
      parentClasses.setChildren([parent])
      appendFunc = ASE.Compiler.Python.GetAttr.GetAttr()
      appendFunc.setName('insert')
      appendFunc.setChildren([parentClasses])
      minusOne = ASE.Compiler.Python.Const.Const()
      minusOne.setValue_int(-(1))
      appendParent = ASE.Compiler.Python.CallFunc.CallFunc()
      appendParent.setChildren([appendFunc])
      appendParent.setArguments([minusOne, selfObj])
      discard = ASE.Compiler.Python.Discard.Discard()
      discard.setChildren([appendParent])
      createObj.addChildren([discard])
    initObj = ASE.Compiler.Python.Assign.Assign()
    target = ASE.Compiler.Python.AssAttr.AssAttr()
    target.setIdentifier('obj')
    target.setChildren([pySelf])
    initObj.setValue(obj)
    initObj.setChildren([target])
    initParent = ASE.Compiler.Python.Assign.Assign()
    target = ASE.Compiler.Python.AssAttr.AssAttr()
    target.setIdentifier('parent')
    target.setChildren([pySelf])
    obj = ASE.Compiler.Python.Name.Name()
    obj.setIdentifier('obj')
    initParent.setValue(obj)
    initParent.setChildren([target])
    noObjStmt = ASE.Compiler.Python.Stmt.Stmt()
    noObjStmt.setChildren([initObj, initParent])
    obj = ASE.Compiler.Python.Name.Name()
    obj.setIdentifier('obj')
    comp = ASE.Compiler.Python.Compare.Compare()
    comp.setTestExpr('is', ASE.Compiler.Python.PyNone.PyNone())
    comp.setChildren([obj])
    checkObj = ASE.Compiler.Python.If.If()
    checkObj.setChildren([comp, createObj, noObjStmt])
    stmt.addChildren([checkObj])
    func.setChildren([stmt])
    return [func]
  
  def createEquality(self, interface):
    func = ASE.Compiler.Python.Function.Function()
    func.setIdentifier('__eq__')
    func.setArgumentNames(['self', 'obj'])
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    selfObj = ASE.Compiler.Python.Name.Name()
    selfObj.setIdentifier('self')
    obj = ASE.Compiler.Python.Name.Name()
    obj.setIdentifier('obj')
    objStr = ASE.Compiler.Python.Const.Const()
    objStr.setValue_string('\'obj\'')
    zero = ASE.Compiler.Python.Const.Const()
    zero.setValue_int(0)
    hasAttr = ASE.Compiler.Python.Name.Name()
    hasAttr.setIdentifier('hasattr')
    hasObj = ASE.Compiler.Python.CallFunc.CallFunc()
    hasObj.setArguments([obj, objStr])
    hasObj.setChildren([hasAttr])
    noObj = ASE.Compiler.Python.Not.Not()
    noObj.setChildren([hasObj])
    returnFalse = ASE.Compiler.Python.Return.Return()
    returnFalse.setChildren([zero])
    checkObj = ASE.Compiler.Python.If.If()
    checkObj.addChildren([noObj, returnFalse])
    isSame = ASE.Compiler.Python.GetAttr.GetAttr()
    isSame.setName('isSame')
    isSame.setChildren([selfObj])
    dispatch = ASE.Compiler.Python.CallFunc.CallFunc()
    dispatch.setArguments([obj])
    dispatch.setChildren([isSame])
    returnVal = ASE.Compiler.Python.Return.Return()
    returnVal.setChildren([dispatch])
    stmt.addChildren([checkObj, returnVal])
    func.setChildren([stmt])
    return [func]
  
  def createInterfaceGetattr(self, interface):
    comment = ASE.Compiler.Python.Comment.Comment()
    comment.setComment('Every call to the interface is dispatched to the underlying class')
    func = ASE.Compiler.Python.Function.Function()
    func.setIdentifier('__getattr__')
    func.setArgumentNames(['self', 'attr'])
    func.setComments([comment])
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    selfObj = ASE.Compiler.Python.Name.Name()
    selfObj.setIdentifier('self')
    attr = ASE.Compiler.Python.Name.Name()
    attr.setIdentifier('attr')
    getAttr = ASE.Compiler.Python.Name.Name()
    getAttr.setIdentifier('getattr')
    obj = ASE.Compiler.Python.GetAttr.GetAttr()
    obj.setName('obj')
    obj.setChildren([selfObj])
    dispatch = ASE.Compiler.Python.CallFunc.CallFunc()
    dispatch.setArguments([obj, attr])
    dispatch.setChildren([getAttr])
    returnVal = ASE.Compiler.Python.Return.Return()
    returnVal.setChildren([dispatch])
    stmt.addChildren([returnVal])
    func.setChildren([stmt])
    return [func]
  
  def createClassGetattr(self, interface):
    comment = ASE.Compiler.Python.Comment.Comment()
    comment.setComment('The virtual dispatch is encoded here')
    func = ASE.Compiler.Python.Function.Function()
    func.setIdentifier('__getattr__')
    func.setArgumentNames(['self', 'attr'])
    func.setComments([comment])
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    selfObj = ASE.Compiler.Python.Name.Name()
    selfObj.setIdentifier('self')
    attr = ASE.Compiler.Python.Name.Name()
    attr.setIdentifier('attr')
    klass = ASE.Compiler.Python.Name.Name()
    klass.setIdentifier('klass')
    getAttr = ASE.Compiler.Python.Name.Name()
    getAttr.setIdentifier('getattr')
    hasAttr = ASE.Compiler.Python.Name.Name()
    hasAttr.setIdentifier('hasattr')
    classes = ASE.Compiler.Python.GetAttr.GetAttr()
    classes.setName('classes')
    classes.setChildren([selfObj])
    testKlass = ASE.Compiler.Python.CallFunc.CallFunc()
    testKlass.setArguments([klass, attr])
    testKlass.setChildren([hasAttr])
    dispatch = ASE.Compiler.Python.CallFunc.CallFunc()
    dispatch.setArguments([klass, attr])
    dispatch.setChildren([getAttr])
    returnKlass = ASE.Compiler.Python.Return.Return()
    returnKlass.setChildren([dispatch])
    checkKlass = ASE.Compiler.Python.If.If()
    checkKlass.setChildren([testKlass, returnKlass])
    klassLoop = ASE.Compiler.Python.For.For()
    klassLoop.setVar(klass)
    klassLoop.setList(classes)
    klassLoop.setChildren([checkKlass])
    parent = ASE.Compiler.Python.GetAttr.GetAttr()
    parent.setName('parent')
    parent.setChildren([selfObj])
    testParent = ASE.Compiler.Python.Compare.Compare()
    testParent.setTestExpr('is', ASE.Compiler.Python.PyNone.PyNone())
    testParent.setChildren([parent])
    noParent = ASE.Compiler.Python.Not.Not()
    noParent.setChildren([testParent])
    dispatch = ASE.Compiler.Python.CallFunc.CallFunc()
    dispatch.setArguments([parent, attr])
    dispatch.setChildren([getAttr])
    returnParent = ASE.Compiler.Python.Return.Return()
    returnParent.setChildren([dispatch])
    checkParent = ASE.Compiler.Python.If.If()
    checkParent.setChildren([noParent, returnParent])
    excType = ASE.Compiler.Python.Name.Name()
    excType.setIdentifier('AttributeError')
    raiseError = ASE.Compiler.Python.Raise.Raise()
    raiseError.setChildren([excType, attr])
    stmt.addChildren([klassLoop, checkParent, raiseError])
    func.setChildren([stmt])
    return [func]
  
  def createGetBaseObject(self, interface):
    comment = ASE.Compiler.Python.Comment.Comment()
    comment.setComment('Get the underlying object which might have been cast several times')
    func = ASE.Compiler.Python.Function.Function()
    func.setIdentifier('getBaseObject')
    func.setArgumentNames(['obj'])
    func.setComments([comment])
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    obj = ASE.Compiler.Python.Name.Name()
    obj.setIdentifier('obj')
    objObj = ASE.Compiler.Python.GetAttr.GetAttr()
    objObj.setName('obj')
    objObj.setChildren([obj])
    objStr = ASE.Compiler.Python.Const.Const()
    objStr.setValue_string('\'obj\'')
    hasAttr = ASE.Compiler.Python.Name.Name()
    hasAttr.setIdentifier('hasattr')
    checkObj = ASE.Compiler.Python.CallFunc.CallFunc()
    checkObj.setArguments([obj, objStr])
    checkObj.setChildren([hasAttr])
    assign = ASE.Compiler.Python.Assign.Assign()
    target = ASE.Compiler.Python.AssName.AssName()
    target.setIdentifier('obj')
    assign.setValue(objObj)
    assign.setChildren([target])
    objLoop = ASE.Compiler.Python.While.While()
    objLoop.setTest(checkObj)
    objLoop.setChildren([assign])
    returnObj = ASE.Compiler.Python.Return.Return()
    returnObj.setChildren([obj])
    stmt.addChildren([objLoop, returnObj])
    func.setChildren([stmt])
    return [func, self.setStaticMethod(None, 'getBaseObject')]
  
  def createHash(self, interface):
    comment = ASE.Compiler.Python.Comment.Comment()
    comment.setComment('Hashes IOR pointer')
    func = ASE.Compiler.Python.Function.Function()
    func.setIdentifier('__hash__')
    func.setArgumentNames(['self'])
    func.setComments([comment])
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    selfObj = ASE.Compiler.Python.Name.Name()
    selfObj.setIdentifier('self')
    id = ASE.Compiler.Python.Name.Name()
    id.setIdentifier('id')
    hash = ASE.Compiler.Python.Name.Name()
    hash.setIdentifier('hash')
    baseObj = ASE.Compiler.Python.GetAttr.GetAttr()
    baseObj.setName('getBaseObject')
    baseObj.setChildren([selfObj])
    objCall = ASE.Compiler.Python.CallFunc.CallFunc()
    objCall.setArguments([selfObj])
    objCall.setChildren([baseObj])
    idCall = ASE.Compiler.Python.CallFunc.CallFunc()
    idCall.setArguments([objCall])
    idCall.setChildren([id])
    hashCall = ASE.Compiler.Python.CallFunc.CallFunc()
    hashCall.setArguments([idCall])
    hashCall.setChildren([hash])
    returnVal = ASE.Compiler.Python.Return.Return()
    returnVal.setChildren([hashCall])
    stmt.addChildren([returnVal])
    func.setChildren([stmt])
    return [func]
  
  def createNonzero(self, interface):
    comment = ASE.Compiler.Python.Comment.Comment()
    comment.setComment('Always return true')
    func = ASE.Compiler.Python.Function.Function()
    func.setIdentifier('__nonzero__')
    func.setArgumentNames(['self'])
    func.setComments([comment])
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    one = ASE.Compiler.Python.Const.Const()
    one.setValue_int(1)
    returnVal = ASE.Compiler.Python.Return.Return()
    returnVal.setChildren([one])
    stmt.addChildren([returnVal])
    func.setChildren([stmt])
    return [func]
  
  def createRepr(self, interface):
    func = self.createStr(interface)[0]
    func.setIdentifier('__repr__')
    return [func]
  
  def createStr(self, interface):
    func = ASE.Compiler.Python.Function.Function()
    func.setIdentifier('__str__')
    func.setArgumentNames(['self'])
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    selfObj = ASE.Compiler.Python.Name.Name()
    selfObj.setIdentifier('self')
    id = ASE.Compiler.Python.Name.Name()
    id.setIdentifier('id')
    hex = ASE.Compiler.Python.Name.Name()
    hex.setIdentifier('hex')
    className = ASE.Compiler.Python.GetAttr.GetAttr()
    className.setName('getClassName')
    className.setChildren([selfObj])
    nameCall = ASE.Compiler.Python.CallFunc.CallFunc()
    nameCall.setChildren([className])
    idCall = ASE.Compiler.Python.CallFunc.CallFunc()
    idCall.setArguments([selfObj])
    idCall.setChildren([id])
    hexCall = ASE.Compiler.Python.CallFunc.CallFunc()
    hexCall.setArguments([idCall])
    hexCall.setChildren([hex])
    startStr = ASE.Compiler.Python.Const.Const()
    startStr.setValue_string('\'<class \'')
    midStr = ASE.Compiler.Python.Const.Const()
    midStr.setValue_string('\' at \'')
    endStr = ASE.Compiler.Python.Const.Const()
    endStr.setValue_string('\'>\'')
    concat = ASE.Compiler.Python.Add.Add()
    concat.setChildren([startStr, nameCall, midStr, hexCall, endStr])
    returnVal = ASE.Compiler.Python.Return.Return()
    returnVal.setChildren([concat])
    stmt.addChildren([returnVal])
    func.setChildren([stmt])
    return [func]
  
  def m_getClassName(self, interface, method):
    func = self.createSignature(method)
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    className = ASE.Compiler.Python.Const.Const()
    className.setValue_string('\'' + interface.getFullIdentifier() + '\'')
    returnVal = ASE.Compiler.Python.Return.Return()
    returnVal.setChildren([className])
    stmt.addChildren([returnVal])
    func.setChildren([stmt])
    self.classAppend(func)
    if method.getStatic():
      self.classAppend(self.setStaticMethod(method))
    return 
  
  def m_isInstanceOf(self, interface, method):
    func = self.createSignature(method)
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    typeName = ASE.Compiler.Python.Name.Name()
    typeName.setIdentifier(method.getChildren()[0].getIdentifier())
    pySelf = ASE.Compiler.Python.Name.Name()
    pySelf.setIdentifier('self')
    len = ASE.Compiler.Python.Name.Name()
    len.setIdentifier('len')
    classes = ASE.Compiler.Python.GetAttr.GetAttr()
    classes.setName('classes')
    classes.setChildren([pySelf])
    obj = ASE.Compiler.Python.GetAttr.GetAttr()
    obj.setName('obj')
    obj.setChildren([pySelf])
    one = ASE.Compiler.Python.Const.Const()
    one.setValue_int(1)
    lenClasses = ASE.Compiler.Python.CallFunc.CallFunc()
    lenClasses.setArguments([classes])
    lenClasses.setChildren([len])
    checkClasses = ASE.Compiler.Python.Compare.Compare()
    checkClasses.setTestExpr('>', one)
    checkClasses.setChildren([lenClasses])
    importLoader = ASE.Compiler.Python.Import.Import()
    importLoader.setModuleNames(['ASE.Loader'])
    aseMod = ASE.Compiler.Python.Name.Name()
    aseMod.setIdentifier('ASE')
    loaderMod = ASE.Compiler.Python.GetAttr.GetAttr()
    loaderMod.setName('Loader')
    loaderMod.setChildren([aseMod])
    loaderClass = ASE.Compiler.Python.GetAttr.GetAttr()
    loaderClass.setName('Loader')
    loaderClass.setChildren([loaderMod])
    createFunc = ASE.Compiler.Python.GetAttr.GetAttr()
    createFunc.setName('createClass')
    createFunc.setChildren([loaderClass])
    zero = ASE.Compiler.Python.Const.Const()
    zero.setValue_int(0)
    firstClass = ASE.Compiler.Python.Subscript.Subscript()
    firstClass.setSubscripts([zero])
    firstClass.setChildren([classes])
    module = ASE.Compiler.Python.GetAttr.GetAttr()
    module.setName('__module__')
    module.setChildren([firstClass])
    minusFive = ASE.Compiler.Python.Const.Const()
    minusFive.setValue_int(-(5))
    nameSlice = ASE.Compiler.Python.Slice.Slice()
    nameSlice.setUpper(minusFive)
    nameSlice.setChildren([module])
    superObj = ASE.Compiler.Python.CallFunc.CallFunc()
    superObj.setArguments([nameSlice])
    superObj.setChildren([createFunc])
    superFunc = ASE.Compiler.Python.GetAttr.GetAttr()
    superFunc.setName('isInstanceOf')
    superFunc.setChildren([superObj])
    superCall = ASE.Compiler.Python.CallFunc.CallFunc()
    superCall.setArguments([typeName])
    superCall.setChildren([superFunc])
    returnSuper = ASE.Compiler.Python.Return.Return()
    returnSuper.setChildren([superCall])
    createStmt = ASE.Compiler.Python.Stmt.Stmt()
    createStmt.setChildren([importLoader, returnSuper])
    emptyClasses = ASE.Compiler.Python.Compare.Compare()
    emptyClasses.setTestExpr('==', zero)
    emptyClasses.setChildren([lenClasses])
    noObj = ASE.Compiler.Python.Compare.Compare()
    noObj.setTestExpr('is', ASE.Compiler.Python.PyNone.PyNone())
    noObj.setChildren([obj])
    hasObj = ASE.Compiler.Python.Not.Not()
    hasObj.setChildren([noObj])
    checkObj = ASE.Compiler.Python.And.And()
    checkObj.setChildren([emptyClasses, hasObj])
    objFunc = ASE.Compiler.Python.GetAttr.GetAttr()
    objFunc.setName('isInstanceOf')
    objFunc.setChildren([obj])
    objCall = ASE.Compiler.Python.CallFunc.CallFunc()
    objCall.setArguments([typeName])
    objCall.setChildren([objFunc])
    returnSuper = ASE.Compiler.Python.Return.Return()
    returnSuper.setChildren([objCall])
    typeNames = ASE.Compiler.Python.Tuple.Tuple()
    for inf in self.getInterfaces(interface):
      pyName = ASE.Compiler.Python.Const.Const()
      pyName.setValue_string('\'' + inf.getFullIdentifier() + '\'')
      typeNames.addChildren([pyName])
    testType = ASE.Compiler.Python.Compare.Compare()
    testType.setTestExpr('in', typeNames)
    testType.setChildren([typeName])
    true = ASE.Compiler.Python.Const.Const()
    true.setValue_int(1)
    returnTrue = ASE.Compiler.Python.Return.Return()
    returnTrue.setChildren([true])
    checkType = ASE.Compiler.Python.If.If()
    checkType.setChildren([checkClasses, createStmt, checkObj, returnSuper, testType, returnTrue])
    false = ASE.Compiler.Python.Const.Const()
    false.setValue_int(0)
    returnFalse = ASE.Compiler.Python.Return.Return()
    returnFalse.setChildren([false])
    stmt.addChildren([checkType, returnFalse])
    func.setChildren([stmt])
    self.classAppend(func)
    if method.getStatic():
      self.classAppend(self.setStaticMethod(method))
    return 
  
  def m_isSameOld(self, interface, method):
    func = self.createSignature(method)
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    pySelf = ASE.Compiler.Python.Name.Name()
    pySelf.setIdentifier('self')
    obj = ASE.Compiler.Python.Name.Name()
    obj.setIdentifier('obj')
    objStr = ASE.Compiler.Python.Const.Const()
    objStr.setValue_string('\'obj\'')
    obj1 = ASE.Compiler.Python.Name.Name()
    obj1.setIdentifier('obj1')
    obj1Ass = ASE.Compiler.Python.AssName.AssName()
    obj1Ass.setIdentifier('obj1')
    obj2 = ASE.Compiler.Python.Name.Name()
    obj2.setIdentifier('obj2')
    obj2Ass = ASE.Compiler.Python.AssName.AssName()
    obj2Ass.setIdentifier('obj2')
    hasAttr = ASE.Compiler.Python.Name.Name()
    hasAttr.setIdentifier('hasattr')
    id = ASE.Compiler.Python.Name.Name()
    id.setIdentifier('id')
    selfObj = ASE.Compiler.Python.GetAttr.GetAttr()
    selfObj.setName('obj')
    selfObj.setChildren([pySelf])
    initObj1 = ASE.Compiler.Python.Assign.Assign()
    initObj1.setValue(selfObj)
    initObj1.setChildren(obj1Ass)
    checkObj1 = ASE.Compiler.Python.CallFunc.CallFunc()
    checkObj1.setArguments([obj1, objStr])
    checkObj1.setChildren([hasAttr])
    obj1Obj = ASE.Compiler.Python.GetAttr.GetAttr()
    obj1Obj.setName('obj')
    obj1Obj.setChildren([obj1])
    updateObj1 = ASE.Compiler.Python.Assign.Assign()
    updateObj1.setValue(obj1Obj)
    updateObj1.setChildren(obj1Ass)
    whileObj1 = ASE.Compiler.Python.While.While()
    whileObj1.setTest(checkObj1)
    whileObj1.setChildren(updateObj1)
    checkObj2 = ASE.Compiler.Python.CallFunc.CallFunc()
    checkObj2.setArguments([obj2, objStr])
    checkObj2.setChildren([hasAttr])
    objObj = ASE.Compiler.Python.GetAttr.GetAttr()
    objObj.setName('obj')
    objObj.setChildren([obj])
    initObj2 = ASE.Compiler.Python.Assign.Assign()
    initObj2.setValue(objObj)
    initObj2.setChildren(obj2Ass)
    obj2Obj = ASE.Compiler.Python.GetAttr.GetAttr()
    obj2Obj.setName('obj')
    obj2Obj.setChildren([obj2])
    updateObj2 = ASE.Compiler.Python.Assign.Assign()
    updateObj2.setValue(obj2Obj)
    updateObj2.setChildren(obj2Ass)
    whileObj2 = ASE.Compiler.Python.While.While()
    whileObj2.setTest(checkObj2)
    whileObj2.setChildren(updateObj2)
    obj1Id = ASE.Compiler.Python.CallFunc.CallFunc()
    obj1Id.setArguments([obj1])
    obj1Id.setChildren([id])
    obj2Id = ASE.Compiler.Python.CallFunc.CallFunc()
    obj2Id.setArguments([obj2])
    obj2Id.setChildren([id])
    objEqual = ASE.Compiler.Python.Compare.Compare()
    objEqual.setTestExpr('==', obj2Id)
    objEqual.setChildren([obj1Id])
    returnVal = ASE.Compiler.Python.Return.Return()
    returnVal.setChildren([objEqual])
    stmt.addChildren([initObj1, whileObj1, initObj2, whileObj2, returnVal])
    func.setChildren([stmt])
    self.classAppend(func)
    if method.getStatic():
      self.classAppend(self.setStaticMethod(method))
    return 
  
  def m_isSame(self, interface, method):
    func = self.createSignature(method)
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    pySelf = ASE.Compiler.Python.Name.Name()
    pySelf.setIdentifier('self')
    obj = ASE.Compiler.Python.Name.Name()
    obj.setIdentifier('obj')
    id = ASE.Compiler.Python.Name.Name()
    id.setIdentifier('id')
    baseObj = ASE.Compiler.Python.GetAttr.GetAttr()
    baseObj.setName('getBaseObject')
    baseObj.setChildren([pySelf])
    baseObjSelf = ASE.Compiler.Python.CallFunc.CallFunc()
    baseObjSelf.setArguments([pySelf])
    baseObjSelf.setChildren([baseObj])
    idSelf = ASE.Compiler.Python.CallFunc.CallFunc()
    idSelf.setArguments([baseObjSelf])
    idSelf.setChildren([id])
    baseObjObj = ASE.Compiler.Python.CallFunc.CallFunc()
    baseObjObj.setArguments([obj])
    baseObjObj.setChildren([baseObj])
    idObj = ASE.Compiler.Python.CallFunc.CallFunc()
    idObj.setArguments([baseObjObj])
    idObj.setChildren([id])
    objEqual = ASE.Compiler.Python.Compare.Compare()
    objEqual.setTestExpr('==', idObj)
    objEqual.setChildren([idSelf])
    returnVal = ASE.Compiler.Python.Return.Return()
    returnVal.setChildren([objEqual])
    stmt.addChildren([returnVal])
    func.setChildren([stmt])
    self.classAppend(func)
    if method.getStatic():
      self.classAppend(self.setStaticMethod(method))
    return 
  
  def m_createClass(self, interface, method):
    func = self.createSignature(method)
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    className = ASE.Compiler.Python.Name.Name()
    className.setIdentifier('className')
    mod = ASE.Compiler.Python.Name.Name()
    mod.setIdentifier('mod')
    modAss = ASE.Compiler.Python.AssName.AssName()
    modAss.setIdentifier('mod')
    components = ASE.Compiler.Python.Name.Name()
    components.setIdentifier('components')
    componentsAss = ASE.Compiler.Python.AssName.AssName()
    componentsAss.setIdentifier('components')
    comp = ASE.Compiler.Python.Name.Name()
    comp.setIdentifier('comp')
    pyImport = ASE.Compiler.Python.Name.Name()
    pyImport.setIdentifier('__import__')
    getAttr = ASE.Compiler.Python.Name.Name()
    getAttr.setIdentifier('getattr')
    dot = ASE.Compiler.Python.Const.Const()
    dot.setValue_string('\'.\'')
    one = ASE.Compiler.Python.Const.Const()
    one.setValue_int(1)
    minusOne = ASE.Compiler.Python.Const.Const()
    minusOne.setValue_int(-(1))
    importMod = ASE.Compiler.Python.CallFunc.CallFunc()
    importMod.setArguments([className])
    importMod.setChildren([pyImport])
    setMod = ASE.Compiler.Python.Assign.Assign()
    setMod.setChildren([modAss])
    setMod.setValue(importMod)
    splitFunc = ASE.Compiler.Python.GetAttr.GetAttr()
    splitFunc.setName('split')
    splitFunc.setChildren([className])
    splitName = ASE.Compiler.Python.CallFunc.CallFunc()
    splitName.setArguments([dot])
    splitName.setChildren([splitFunc])
    setComp = ASE.Compiler.Python.Assign.Assign()
    setComp.setChildren([componentsAss])
    setComp.setValue(splitName)
    tailComp = ASE.Compiler.Python.Slice.Slice()
    tailComp.setChildren([components])
    tailComp.setLower(one)
    getComp = ASE.Compiler.Python.CallFunc.CallFunc()
    getComp.setArguments([mod, comp])
    getComp.setChildren([getAttr])
    updateMod = ASE.Compiler.Python.Assign.Assign()
    updateMod.setChildren([modAss])
    updateMod.setValue(getComp)
    compLoop = ASE.Compiler.Python.For.For()
    compLoop.setVar(comp)
    compLoop.setList(tailComp)
    compLoop.setChildren([updateMod])
    lastComp = ASE.Compiler.Python.Subscript.Subscript()
    lastComp.setChildren([components])
    lastComp.setSubscripts([minusOne])
    getConstructor = ASE.Compiler.Python.CallFunc.CallFunc()
    getConstructor.setArguments([mod, lastComp])
    getConstructor.setChildren([getAttr])
    createObj = ASE.Compiler.Python.CallFunc.CallFunc()
    createObj.setChildren([getConstructor])
    returnVal = ASE.Compiler.Python.Return.Return()
    returnVal.setChildren([createObj])
    stmt.addChildren([setMod, setComp, compLoop, returnVal])
    func.setChildren([stmt])
    self.classAppend(func)
    if method.getStatic():
      self.classAppend(self.setStaticMethod(method))
    return 
  
  def m_weakReference(self, interface):
    func = ASE.Compiler.Python.Function.Function()
    func.setIdentifier('weakReference')
    func.setArgumentNames(['self'])
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    selfName = ASE.Compiler.Python.Name.Name()
    selfName.setIdentifier('self')
    returnVal = ASE.Compiler.Python.Return.Return()
    returnVal.setChildren([selfName])
    stmt.addChildren([returnVal])
    func.setChildren([stmt])
    self.classAppend(func)
    return 
  
  def getInterfaces(self, interface, seen = None):
    if seen is None:
      seen = []
    for parent in interface.getParents():
      for inf in self.getInterfaces(parent, seen):
        yield inf
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      import ASE.Compiler.SIDL.Class
      for parent in ASE.Compiler.SIDL.Class.Class(interface).getInterfaces():
        for inf in self.getInterfaces(parent, seen):
          yield inf
    if not (interface.getFullIdentifier() in seen):
      seen.append(interface.getFullIdentifier())
      yield interface
    return 
  
  def hasStaticMethod(self, type):
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      for method in type.getMethods():
        if method.getStatic():
          return 1
    return 0
  
  def pushPackage(self):
    return self.packages.append([])
  
  def popPackage(self):
    return self.packages.pop()
  
  def packageAppend(self, name):
    return self.packages[-(1)].append(name)
  
  def pushClass(self):
    return self.classes.append([])
  
  def popClass(self):
    return self.classes.pop()
  
  def classAppend(self, name):
    return self.classes[-(1)].append(name)
