#include <stdlib.h>

#include <string.h>

#ifndef included_ASE_Compiler_Search
#include <ASE/Compiler/Search_IOR.h>
#endif

/* Initialization routine defined in the skeleton */
extern void ASE_Compiler_Search__set_epv(struct ASE_Compiler_Search__epv *epv, struct ASE_BaseException__object **_exception);

#ifndef included_ASE_ASEException
#include <ASE/ASEException_IOR.h>
#endif

static struct ASE_Compiler_Search__array *createArray(/*in*/ int32_t dimen, /*in*/ struct int__array *lower, /*in*/ struct int__array *upper, /*out*/ struct ASE_BaseException__object **_exception)
{
  static int32_t arraySize = sizeof(struct ASE_Compiler_Search__array) + (sizeof(int32_t) - (sizeof(struct ASE_Compiler_Search__array)) % sizeof(int32_t)) % sizeof(int32_t);
  struct ASE_Compiler_Search__array *_return_value = (struct ASE_Compiler_Search__array *) malloc(arraySize + 3 * sizeof(int32_t) * (dimen ? dimen : 1));
  int32_t size = 1;
  int32_t d = 0;
  
  /* Initialize array structure */
  _return_value->d_dimen = dimen;
  _return_value->d_borrowed = 0;
  _return_value->d_lower = (int32_t *) ((char *) _return_value + arraySize);
  _return_value->d_upper = _return_value->d_lower + (dimen ? dimen : 1);
  _return_value->d_stride = _return_value->d_upper + (dimen ? dimen : 1);
  _return_value->d_lower[0] = 0;
  _return_value->d_upper[0] = 0;
  _return_value->d_stride[0] = 1;
  /* Determine array size and set bounds */
  for(d = 0; d < dimen; d++)
  {
    _return_value->d_lower[d] = lower->d_firstElement[d];
    _return_value->d_upper[d] = upper->d_firstElement[d];
    _return_value->d_stride[d] = size;
    size *= upper->d_firstElement[d] - lower->d_firstElement[d] + 1;
  }
  size *= sizeof(struct ASE_Compiler_Search__object *);
  /* Allocate and initialize data */
  _return_value->d_firstElement = (struct ASE_Compiler_Search__object **) malloc(size);
  memset(_return_value->d_firstElement, 0, size);
  return _return_value;
}

static struct ASE_Compiler_Search__array *borrowArray(/*in*/ int32_t dimen, /*in*/ struct int__array *lower, /*in*/ struct int__array *upper, /*in*/ struct int__array *stride, /*in*/ void *first, /*out*/ struct ASE_BaseException__object **_exception)
{
  static int32_t arraySize = sizeof(struct ASE_Compiler_Search__array) + (sizeof(int32_t) - (sizeof(struct ASE_Compiler_Search__array)) % sizeof(int32_t)) % sizeof(int32_t);
  struct ASE_Compiler_Search__array *_return_value = (struct ASE_Compiler_Search__array *) malloc(arraySize + 3 * sizeof(int32_t) * (dimen ? dimen : 1));
  int32_t size = 1;
  int32_t d = 0;
  
  /* Initialize array structure */
  _return_value->d_dimen = dimen;
  _return_value->d_borrowed = 1;
  _return_value->d_lower = (int32_t *) ((char *) _return_value + arraySize);
  _return_value->d_upper = _return_value->d_lower + (dimen ? dimen : 1);
  _return_value->d_stride = _return_value->d_upper + (dimen ? dimen : 1);
  _return_value->d_lower[0] = 0;
  _return_value->d_upper[0] = 0;
  _return_value->d_stride[0] = 1;
  /* Determine array size and set bounds */
  for(d = 0; d < dimen; d++)
  {
    _return_value->d_lower[d] = lower->d_firstElement[d];
    _return_value->d_upper[d] = upper->d_firstElement[d];
    _return_value->d_stride[d] = stride->d_firstElement[d];
    size *= upper->d_firstElement[d] - lower->d_firstElement[d] + 1;
  }
  size *= sizeof(struct ASE_Compiler_Search__object *);
  _return_value->d_firstElement = (struct ASE_Compiler_Search__object **) first;
  return _return_value;
}

static void destroyArray(/*in*/ struct ASE_Compiler_Search__array *array, /*out*/ struct ASE_BaseException__object **_exception)
{
  if (!array->d_borrowed)
  {
    free(array->d_firstElement);
  }
  free(array);
}

static int32_t getDimen(/*in*/ struct ASE_Compiler_Search__array *array, /*out*/ struct ASE_BaseException__object **_exception)
{
  return array->d_dimen;
}

static int32_t getLower(/*in*/ struct ASE_Compiler_Search__array *array, /*in*/ int32_t dimen, /*out*/ struct ASE_BaseException__object **_exception)
{
  return array->d_lower[dimen];
}

static int32_t getUpper(/*in*/ struct ASE_Compiler_Search__array *array, /*in*/ int32_t dimen, /*out*/ struct ASE_BaseException__object **_exception)
{
  return array->d_upper[dimen];
}

static struct ASE_Compiler_Search__object *getElement(/*in*/ struct ASE_Compiler_Search__array *array, /*in*/ struct int__array *indices, /*out*/ struct ASE_BaseException__object **_exception)
{
  struct ASE_Compiler_Search__object *_result = NULL;
  struct ASE_Compiler_Search__object **_element = NULL;
  int32_t d = 0;
  
  _element = array->d_firstElement;
  for(d = 0; d < array->d_dimen; d++)
  {
    if (indices->d_firstElement[d] < array->d_lower[d] || indices->d_firstElement[d] > array->d_upper[d])
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Invalid indices", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return _result;
    }
    _element += array->d_stride[d] * (indices->d_firstElement[d] - array->d_lower[d]);
  }
  if (*_element)
  {
    (*(((struct ASE_BaseInterface__object *) (*_element)->d_object)->d_epv->deleteReference))((*_element)->d_object, (void **) _exception);
  }
  return *_element;
}

static struct ASE_Compiler_Search__object *getElement4(/*in*/ struct ASE_Compiler_Search__array *array, /*in*/ int32_t i1, /*in*/ int32_t i2, /*in*/ int32_t i3, /*in*/ int32_t i4, /*out*/ struct ASE_BaseException__object **_exception)
{
  struct ASE_Compiler_Search__object *_result = NULL;
  
  return _result;
}

static void setElement(/*in*/ struct ASE_Compiler_Search__array *array, /*in*/ struct int__array *indices, /*in*/ struct ASE_Compiler_Search__object *value, /*out*/ struct ASE_BaseException__object **_exception)
{
  struct ASE_Compiler_Search__object **_element = NULL;
  int32_t d = 0;
  
  _element = array->d_firstElement;
  for(d = 0; d < array->d_dimen; d++)
  {
    if (indices->d_firstElement[d] < array->d_lower[d] || indices->d_firstElement[d] > array->d_upper[d])
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Invalid indices", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    _element += array->d_stride[d] * (indices->d_firstElement[d] - array->d_lower[d]);
  }
  if (value)
  {
    (*(((struct ASE_BaseInterface__object *) (value)->d_object)->d_epv->addReference))((value)->d_object, (void **) _exception);
  }
  if (*_element)
  {
    (*(((struct ASE_BaseInterface__object *) (*_element)->d_object)->d_epv->deleteReference))((*_element)->d_object, (void **) _exception);
  }
  *_element = value;
}

static void setElement4(/*in*/ struct ASE_Compiler_Search__array *array, /*in*/ int32_t i1, /*in*/ int32_t i2, /*in*/ int32_t i3, /*in*/ int32_t i4, /*in*/ struct ASE_Compiler_Search__object *value, /*out*/ struct ASE_BaseException__object **_exception)
{
}

static void *castIterator(struct ASE_Compiler_Search__object *self, /*in*/ const char *name, /*out*/ struct ASE_BaseException__object **_exception)
{
  /* We are guaranteed here that the first member in any IOR is a ASE.BaseInterface object */
  struct ASE_BaseClass__object *obj = ((struct ASE_BaseInterface__object *) self)->d_object;
  void *cast = NULL;
  
  if (!strcmp(name, "ASE.Compiler.Search"))
  {
    cast = self;
  }
  else
  if (!strcmp(name, "ASE.BaseClass"))
  {
    cast = obj;
  }
  else
  if (!strcmp(name, "ASE.BaseInterface"))
  {
    cast = (&obj->ase_baseinterface);
  }
  return cast;
}

static struct ASE_Compiler_Search__iterator *createIterator(/*in*/ void *iterator, /*in*/ void *iterNext, /*out*/ struct ASE_BaseException__object **_exception)
{
  struct ASE_Compiler_Search__iterator *self = (struct ASE_Compiler_Search__iterator *) malloc(sizeof(struct ASE_Compiler_Search__iterator));
  struct ASE_BaseClass__object *parent = &self->ase_baseclass;
  struct ASE_BaseClass__epv *ASE_BaseClass_epv = (struct ASE_BaseClass__epv *) malloc(sizeof(struct ASE_BaseClass__epv));
  struct ASE_BaseInterface__epv *ASE_BaseInterface_epv = (struct ASE_BaseInterface__epv *) malloc(sizeof(struct ASE_BaseInterface__epv));
  
  (*(ASE_BaseClass__externals()->initObject))(parent, self, _exception);
  ASE_BaseClass_epv->getReferenceCount = self->ase_baseclass.d_epv->getReferenceCount;
  ASE_BaseClass_epv->addReference = self->ase_baseclass.d_epv->addReference;
  ASE_BaseClass_epv->deleteReference = self->ase_baseclass.d_epv->deleteReference;
  ASE_BaseClass_epv->isSame = self->ase_baseclass.d_epv->isSame;
  ASE_BaseClass_epv->getClassName = self->ase_baseclass.d_epv->getClassName;
  ASE_BaseClass_epv->getName = self->ase_baseclass.d_epv->getName;
  ASE_BaseClass_epv->setName = self->ase_baseclass.d_epv->setName;
  ASE_BaseClass_epv->isInstanceOf = self->ase_baseclass.d_epv->isInstanceOf;
  ASE_BaseClass_epv->getSuper = self->ase_baseclass.d_epv->getSuper;
  ASE_BaseInterface_epv->getReferenceCount = self->ase_baseclass.ase_baseinterface.d_epv->getReferenceCount;
  ASE_BaseInterface_epv->addReference = self->ase_baseclass.ase_baseinterface.d_epv->addReference;
  ASE_BaseInterface_epv->deleteReference = self->ase_baseclass.ase_baseinterface.d_epv->deleteReference;
  ASE_BaseInterface_epv->isSame = self->ase_baseclass.ase_baseinterface.d_epv->isSame;
  ASE_BaseInterface_epv->getClassName = self->ase_baseclass.ase_baseinterface.d_epv->getClassName;
  ASE_BaseInterface_epv->getName = self->ase_baseclass.ase_baseinterface.d_epv->getName;
  ASE_BaseInterface_epv->setName = self->ase_baseclass.ase_baseinterface.d_epv->setName;
  ASE_BaseInterface_epv->isInstanceOf = self->ase_baseclass.ase_baseinterface.d_epv->isInstanceOf;
  ASE_BaseInterface_epv->getSuper = self->ase_baseclass.ase_baseinterface.d_epv->getSuper;
  ASE_BaseClass_epv->castObject = castIterator;
  ASE_BaseClass_epv->iterNext = iterNext;
  ASE_BaseInterface_epv->castObject = castIterator;
  ASE_BaseInterface_epv->iterNext = iterNext;
  self->ase_baseclass.ase_baseinterface.d_epv = ASE_BaseInterface_epv;
  self->ase_baseclass.d_epv = ASE_BaseClass_epv;
  self->iterator = iterator;
  self->objWrapper = NULL;
  return self;
}

static void destroyIterator(/*in*/ struct ASE_Compiler_Search__iterator *self, /*out*/ struct ASE_BaseException__object **_exception)
{
  struct ASE_BaseClass__object *parent = &self->ase_baseclass;
  
  self->iterator = NULL;
  self->objWrapper = NULL;
  free(self->ase_baseclass.ase_baseinterface.d_epv);
  free(self->ase_baseclass.d_epv);
  self->ase_baseclass.d_epv = self->ase_baseclass.d_oldEpv;
  (*(ASE_BaseClass__externals()->finiObject))(parent, _exception);
  memset(self, 0, sizeof(struct ASE_Compiler_Search__iterator));
  free(self);
}

/* The entry points into the DLL containing this type */
static struct ASE_Compiler_Search__external externalEntryPoints = {
  createArray,
  borrowArray,
  destroyArray,
  getDimen,
  getLower,
  getUpper,
  getElement,
  getElement4,
  setElement,
  setElement4,
  createIterator,
  destroyIterator};

const struct ASE_Compiler_Search__external *ASE_Compiler_Search__externals(void)
{
  return &externalEntryPoints;
}
