#!/usr/bin/env python
'''
Parser for expression in preprocessor directives
'''
import user
import importer
import clex
import yacc

tokens = clex.tokens

# Parsing rules
##precedence = (
##  ('left','PLUS','MINUS','TIMES','DIVIDE','AND'),
##  ('left','LAND','LOR','LT','LE','GT','GE','EQ','NE')
##  )

debug  = 0
parser = None

def p_exp_and(t):
  '''exp : exp LAND term'''
  if debug: print t[1],'and',t[3]
  t[0] = t[1] and t[3]

def p_exp_or(t):
  '''exp : exp LOR term'''
  if debug: print t[1],'or',t[3]
  t[0] = t[1] or t[3]

def p_exp_lt(t):
  '''exp : exp LT term'''
  if debug: print t[1],'<',t[3]
  t[0] = t[1] < t[3]

def p_exp_le(t):
  '''exp : exp LE term'''
  if debug: print t[1],'<=',t[3]
  t[0] = t[1] <= t[3]

def p_exp_gt(t):
  '''exp : exp GT term'''
  if debug: print t[1],'>',t[3]
  t[0] = t[1] > t[3]

def p_exp_ge(t):
  '''exp : exp GE term'''
  if debug: print t[1],'>=',t[3]
  t[0] = t[1] >= t[3]

def p_exp_eq(t):
  '''exp : exp EQ term'''
  if debug: print t[1],'==',t[3]
  t[0] = t[1] == t[3]

def p_exp_ne(t):
  '''exp : exp NE term'''
  if debug: print t[1],'!=',t[3]
  t[0] = not t[1] == t[3]

def p_exp_conditional(t):
  '''exp : exp CONDOP exp COLON exp'''
  if debug: print 'if',t[1],'then',t[3],'else',t[5]
  if t[1]:
    t[0] = t[3]
  else:
    t[0] = t[5]

def p_exp_term(t):
  '''exp : term'''
  t[0] = t[1]

def p_term_not(t):
  '''term : LNOT term'''
  if debug: print '!',t[2]
  t[0] = not t[2]

def p_term_plus(t):
  '''term : term PLUS factor'''
  if debug: print t[1],'+',t[3]
  t[0] = t[1] + t[3]

def p_term_minus(t):
  '''term : term MINUS factor'''
  if debug: print t[1],'-',t[3]
  t[0] = t[1] - t[3]

def p_term_times(t):
  '''term : term TIMES factor'''
  if debug: print t[1],'*',t[3]
  t[0] = t[1] * t[3]

def p_term_divide(t):
  '''term : term DIVIDE factor'''
  if debug: print t[1],'/',t[3]
  t[0] = long(t[1] / t[3])

def p_term_and(t):
  '''term : term AND factor'''
  if debug: print t[1],'&',t[3]
  t[0] = t[1] & t[3]

def p_term_factor(t):
  '''term : factor'''
  t[0] = t[1]

def p_factor_exp(t):
  '''factor : LPAREN exp RPAREN'''
  if debug: print '(',t[2],')'
  t[0] = t[2]

def p_factor_const_1(t):
  '''factor : ICONST'''
  if debug: print 'Constant ',t[1]
  t[0] = long(t[1])

def p_factor_const_2(t):
  '''factor : HCONST'''
  # This sucks, why doesn't int() handle this?
  if debug: print 'Constant ',t[1]
  t[0] = eval(t[1])

##def p_constant(t):
##  '''constant : ICONST'''
##  t[0] = long(t[1])

def p_error(t):
  print 'ERROR:',t
  print parser.symstack
  print parser.statestack
  raise RuntimeError('Expression parse error: '+str(t))

def getDriver(debug = 0):
  global parser

  if parser is None:
    parser = yacc.yacc(debug = debug, tabmodule = 'expParsetab')
  return parser

if __name__ == '__main__':
  import sys

  parser = getDriver()
  f = file(sys.argv[1])
  s = f.read()
  f.close()
  print parser.parse(s)
