# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.IORGenerator
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.DepthFirstVisitor
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Iterator
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class IORGenerator:
  '''
  This visitor transforms a SIDL AST into a set of C IOR ASTs
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    import ASE.Compiler.Python.Cxx
    import ASE.Compiler.Python.IOR
    import ASE.Compiler.Python.SIDL
    self.source = {}

    self.sourceLanguages = ['Cxx']
    self.IOR = ASE.Compiler.Python.IOR.IOR()
    self.SIDL = ASE.Compiler.Python.SIDL.SIDL()
    self.Cxx = ASE.Compiler.Python.Cxx.Cxx()
    self.templateMethods = {}

  
  def getSourceLanguages(self):
    '''
    Retrieve the list of the languages in which ASTs are produced
    '''
    return self.sourceLanguages
  
  def setSourceLanguages(self, outputLanguages):
    '''
    Set the list of the languages in which ASTs are produced
    '''
    self.sourceLanguages = sourceLanguages
  
  def getSource(self, language):
    '''
    Retrieve the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      raise RuntimeError('Invalid source language: ' + str(language))
    if not (language in self.source):
      self.source[language] = []
    return self.source[language]
  
  def setSource(self, language, vertices):
    '''
    Set the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      raise RuntimeError('Invalid source language: ' + str(language))
    self.source[language] = list(vertices)
  
  def addSource(self, language, vertices):
    '''
    Add to the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      raise RuntimeError('Invalid source language: ' + str(language))
    if not (language in self.source):
      self.source[language] = []
    self.source[language].extend(vertices)
  
  def removeSource(self, language, vertices):
    '''
    Remove to the list of source ASTs for the given language
    Throw an exception if a vertex or language does not exist
    '''
    if not (language in self.sourceLanguages):
      raise RuntimeError('Invalid source language: ' + str(language))
    if not (language in self.source):
      self.source[language] = []
    for vertex in vertices:
      if vertex in self.source[language]:
        self.source[language].remove(vertex)
      else:
        import ASE.Compiler.CompilerException
        e = ASE.Compiler.CompilerException.CompilerException()
        e.setMessage('Source tree does not exist: ' + vertex.getIdentifier())
        raise e
  
  def visitInterface(self, node):
    if node.getFullIdentifier() == 'ASE.BaseInterface':
      self.createBaseTypes()
    decls = []
    defns = []
    
    def extend(d):
      decls.extend(d[0])
      defns.extend(d[1])
      return 
    if node.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      [self.createTemplateMethod(method) for method in node.getMethods()]
      if node.isInstanceOf('ASE.Compiler.SIDL.Class'):
        extend(([], [self.IOR.getGuardedInclude(self.SIDL.getLoader(node))]))
    extend(self.getEPV(node))
    extend(self.getIORImplementations(node))
    extend(self.getSEPV(node))
    extend(self.getObject(node))
    extend(self.getArray(node))
    extend(self.getIterator(node))
    extend(self.getExternal(node))
    extend(self.getExternals(node))
    self.addSource('Cxx', [self.getHeaderFile(node, decls)])
    self.addSource('Cxx', [self.getSourceFile(node, defns)])
  
  def visitClass(self, node):
    self.visitInterface(node)
  
  def visitEnumeration(self, node):
    import ASE.Compiler.Cxx.Enumeration
    import ASE.Compiler.Cxx.Enumerator
    import ASE.Compiler.SIDL.Enumerator
    enumType = ASE.Compiler.Cxx.Enumeration.Enumeration()
    enumType.setIdentifier(node.getFullIdentifier().replace('.', '_') + '__enum')
    enumType.setInitialized(1)
    for child in node.getChildren():
      enumerator = ASE.Compiler.SIDL.Enumerator.Enumerator(child)
      entry = ASE.Compiler.Cxx.Enumerator.Enumerator()
      entry.setIdentifier(enumerator.getIdentifier())
      entry.setInitialized(1)
      entry.setValue(enumerator.getValue())
      enumType.addChildren([entry])
    enumDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    enumDecl.setType(enumType)
    self.addSource('Cxx', [self.getHeaderFile(node, [self.Cxx.getDecl(enumDecl)])])
  # Ending class methods section
  pass
  
  def hasStaticMethod(self, type):
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      for method in type.getMethods():
        if method.getStatic():
          return 1
    return 0
  
  def getForwardDeclarations(self, type, noExceptions = 0):
    import ASE.Compiler.SIDL.Array
    import ASE.Compiler.SIDL.Interface
    import ASE.Compiler.SIDL.Iterator
    import ASE.Compiler.SIDL.Type
    import ASE.Compiler.Cxx.Struct
    if (ASE.Compiler.SIDL.Type.Type(type).getBaseType() or type.isInstanceOf('ASE.Compiler.SIDL.Enumeration') or type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse')):
      return ([], [])
    interface = ASE.Compiler.SIDL.Interface.Interface(type)
    decls = []
    objStruct = self.IOR.getType(interface).getChildren()[0]
    decls.append(self.Cxx.getDecl(objStruct, 'These structures are used in this header'))
    iterType = ASE.Compiler.SIDL.Iterator.Iterator()
    iterType.setType(type)
    decls.append(self.Cxx.getDecl(self.IOR.getType(iterType).getChildren()[0]))
    if self.hasStaticMethod(interface):
      sepvStruct = ASE.Compiler.Cxx.Struct.Struct()
      sepvStruct.setIdentifier(self.IOR.getStaticExternalType(interface).getChildren()[0].getIdentifier())
      decls.append(self.Cxx.getDecl(sepvStruct))
    objArray = ASE.Compiler.SIDL.Array.Array()
    objArray.setType(interface)
    arrayStruct = self.IOR.getType(objArray).getChildren()[0]
    decls.append(self.Cxx.getDecl(arrayStruct))
    if not (noExceptions):
      if (len(interface.getMethods()) and not (interface.getFullIdentifier() == 'ASE.BaseInterface')):
        baseExc = self.SIDL.getBaseException(interface.getMethods()[0].getExceptions()[0])
        excStruct = self.IOR.getType(baseExc).getChildren()[0]
        decls.append(self.Cxx.getDecl(excStruct))
        excArray = ASE.Compiler.SIDL.Array.Array()
        excArray.setType(baseExc)
        arrayStruct = self.IOR.getType(excArray).getChildren()[0]
        decls.append(self.Cxx.getDecl(arrayStruct))
    return (decls, [])
  
  def getEPV(self, interface):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    if interface.getBaseType():
      return ([], [])
    epvStruct = ASE.Compiler.Cxx.Struct.Struct()
    epvStruct.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__epv')
    epvStruct.setComments([self.Cxx.getComment('The entry point vector (EPV) is the virtual function table for this interface')])
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Comment
    import ASE.Compiler.SIDL.Interface
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    epvStruct.addChildren([self.IOR.getFunctionDeclaration(interface, method, 1) for method in self.getBuiltinMethods(interface)])
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      epvStruct.addChildren([self.IOR.getFunctionDeclaration(interface, method, 1) for method in [self.getImplConstructor(interface), self.getImplDestructor(interface)]])
    epvStruct.addChildren([self.IOR.getFunctionDeclaration(interface, method, 1) for method in self.getTemplateMethods(interface.getMethods()) if not (method.getStatic())])
    epvDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    epvDecl.setType(epvStruct)
    epvParam = ASE.Compiler.Cxx.Parameter.Parameter()
    epvParam.setIdentifier('epv')
    epvParam.setType(self.IOR.getEpvType(interface))
    excParam = ASE.Compiler.Cxx.Parameter.Parameter()
    excParam.setIdentifier('_exception')
    excParam.setType(self.Cxx.getPointer(self.IOR.getType(self.SIDL.getBaseException(self.SIDL.getASEException(interface)))))
    setEPVName = ASE.Compiler.Cxx.Declarator.Declarator()
    setEPVName.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__set_epv')
    setEPVDecl = ASE.Compiler.Cxx.Function.Function()
    setEPVDecl.setChildren([setEPVName])
    setEPVDecl.setType(self.Cxx.typeMap['void'])
    setEPVDecl.setParameters([epvParam, excParam])
    setEPVDecl.setExtern(1)
    return ([self.Cxx.getDecl(epvDecl)], [self.Cxx.getDecl(setEPVDecl, 'Initialization routine defined in the skeleton')])
  
  def getSEPVFunc(self, klass):
    '''
    This is a hack
    '''
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Type
    sepvType = ASE.Compiler.SIDL.Type.Type()
    sepvType.setIdentifier('struct ' + klass.getFullIdentifier().replace('.', '_') + '__sepv *')
    getStaticEPV = ASE.Compiler.SIDL.Method.Method()
    getStaticEPV.setIdentifier('getStaticEPV')
    getStaticEPV.setReturnParameter(self.SIDL.getParameter('_return_value', sepvType, 'return'))
    getStaticEPV.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
    getStaticEPV.setStatic(1)
    return getStaticEPV
  
  def getSEPV(self, klass):
    import ASE.Compiler.SIDL.Interface
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Negation
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.Return
    if not (self.hasStaticMethod(klass)):
      return ([], [])
    sepvType = self.IOR.getStaticExternalType(klass)
    sepvStruct = ASE.Compiler.Cxx.Struct.Struct()
    sepvStruct.setIdentifier(sepvType.getChildren()[0].getIdentifier())
    sepvStruct.addChildren([self.IOR.getFunctionDeclaration(klass, method, 1) for method in self.getTemplateMethods(klass.getMethods()) if method.getStatic()])
    sepvDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    sepvDecl.setType(sepvStruct)
    sepvStruct = ASE.Compiler.Cxx.Struct.Struct()
    sepvStruct.setIdentifier(sepvType.getChildren()[0].getIdentifier())
    sepvVar = self.Cxx.getVar('sepv')
    sepvInit = ASE.Compiler.Cxx.Initializer.Initializer()
    sepvInit.setChildren([self.Cxx.getInteger(0) for method in self.getTemplateMethods(klass.getMethods()) if method.getStatic()])
    initVar = self.Cxx.getVar('initialized')
    setSEPVVar = self.Cxx.getVar(klass.getFullIdentifier().replace('.', '_') + '__set_sepv')
    sepvParam = ASE.Compiler.Cxx.Parameter.Parameter()
    sepvParam.setIdentifier(sepvVar.getIdentifier())
    sepvParam.setType(sepvType)
    setSEPVName = ASE.Compiler.Cxx.Declarator.Declarator()
    setSEPVName.setIdentifier(setSEPVVar.getIdentifier())
    setSEPVDecl = ASE.Compiler.Cxx.Function.Function()
    setSEPVDecl.setChildren([setSEPVName])
    setSEPVDecl.setType(self.Cxx.typeMap['void'])
    setSEPVDecl.setParameters([sepvParam])
    setSEPVDecl.setExtern(1)
    negInitVar = ASE.Compiler.Cxx.Negation.Negation()
    negInitVar.setChildren([initVar])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    initSEPV = self.Cxx.getFunctionCall(setSEPVVar, [self.Cxx.getAddress(sepvVar)])
    compound.addChildren([self.Cxx.getExpStmt(initSEPV)])
    for method in [m for m in self.getIORMethods(klass) if m.getStatic()]:
      compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(sepvVar, method.getIdentifier(), 0), self.Cxx.getVar(method.getIdentifier())))])
    compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(initVar, self.Cxx.getInteger(1)))])
    checkInit = ASE.Compiler.Cxx.If.If()
    checkInit.setBranch(negInitVar)
    checkInit.setChildren([compound])
    returnSEPV = ASE.Compiler.Cxx.Return.Return()
    returnSEPV.setChildren([self.Cxx.getAddress(sepvVar)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setDeclarations([self.Cxx.getDeclaration(sepvVar, sepvStruct, sepvInit, 'Static methods for this class', 1), self.Cxx.getDeclaration(initVar, self.IOR.typeMap['bool'], isStatic = 1)])
    compound.setChildren([checkInit, returnSEPV])
    sepvFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(klass, self.getSEPVFunc(klass)).getChildren()[0])
    sepvFunc.setBody(compound)
    sepvFunc.setStatic(1)
    return ([self.Cxx.getDecl(sepvDecl, 'The static entry point vector (SEPV) is the static function table for this class')], [self.Cxx.getDecl(setSEPVDecl, 'Initialization routine defined in the skeleton'), sepvFunc])
  
  def getConstructor(self, klass):
    import ASE.Compiler.SIDL.Method
    constructor = ASE.Compiler.SIDL.Method.Method()
    constructor.setIdentifier('createObject')
    constructor.setReturnParameter(self.SIDL.getParameter('_return_value', klass, 'return'))
    constructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
    constructor.setParent(klass)
    constructor.setStatic(1)
    constructor.setComments([self.SIDL.getComment('Builtin method')])
    return constructor
  
  def getImplConstructor(self, klass):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Comment
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Builtin method')
    objParam = ASE.Compiler.SIDL.Parameter.Parameter()
    objParam.setIdentifier('obj')
    objParam.setType(klass)
    objParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    constructor = ASE.Compiler.SIDL.Method.Method()
    constructor.setIdentifier(klass.getIdentifier())
    constructor.setChildren([objParam])
    constructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
    constructor.setParent(klass)
    constructor.setStatic(1)
    constructor.setComments([comment])
    return constructor
  
  def getInit(self, klass):
    import ASE.Compiler.SIDL.Method
    init = ASE.Compiler.SIDL.Method.Method()
    init.setIdentifier('initObject')
    init.setChildren([self.SIDL.getParameter('self', klass), self.SIDL.getParameter('obj', self.SIDL.typeMap['opaque'])])
    init.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
    init.setStatic(1)
    return init
  
  def getFini(self, klass):
    import ASE.Compiler.SIDL.Method
    fini = ASE.Compiler.SIDL.Method.Method()
    fini.setIdentifier('finiObject')
    fini.setChildren([self.SIDL.getParameter('self', klass)])
    fini.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
    fini.setStatic(1)
    return fini
  
  def getDestructor(self, interface):
    import ASE.Compiler.SIDL.Method
    destructor = ASE.Compiler.SIDL.Method.Method()
    destructor.setIdentifier('destroyObject')
    destructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(interface))])
    destructor.setParent(interface)
    destructor.setComments([self.SIDL.getComment('Builtin method')])
    return destructor
  
  def getImplDestructor(self, klass):
    import ASE.Compiler.SIDL.Method
    destructor = ASE.Compiler.SIDL.Method.Method()
    destructor.setIdentifier(klass.getIdentifier() + '_')
    destructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
    destructor.setParent(klass)
    destructor.setComments([self.SIDL.getComment('Builtin method')])
    return destructor
  
  def getCast(self, interface):
    import ASE.Compiler.SIDL.Method
    cast = ASE.Compiler.SIDL.Method.Method()
    cast.setIdentifier('castObject')
    cast.setReturnParameter(self.SIDL.getParameter('ior', self.SIDL.typeMap['opaque'], 'return'))
    cast.setChildren([self.SIDL.getParameter('name', self.SIDL.typeMap['string'])])
    cast.setParent(interface)
    if interface.isInstanceOf('ASE.Compiler.SIDL.Iterator'):
      import ASE.Compiler.SIDL.Iterator
      interface = ASE.Compiler.SIDL.Iterator.Iterator(interface).getType()
      cast.setIdentifier('castIterator')
    cast.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(interface))])
    cast.setComments([self.SIDL.getComment('Builtin method')])
    return cast
  
  def getStr(self, interface):
    import ASE.Compiler.SIDL.Method
    string = ASE.Compiler.SIDL.Method.Method()
    string.setIdentifier('strObject')
    string.setReturnParameter(self.SIDL.getParameter('_return_value', self.SIDL.typeMap['string'], 'return'))
    string.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(interface))])
    string.setParent(interface)
    string.setComments([self.SIDL.getComment('Builtin method')])
    return string
  
  def getIterNext(self, interface):
    '''
    The iterator looks like an entry object, but conceals an iterator underneath
    '''
    import ASE.Compiler.SIDL.Method
    iterNext = ASE.Compiler.SIDL.Method.Method()
    iterNext.setIdentifier('iterNext')
    iterNext.setReturnParameter(self.SIDL.getParameter('_return_value', interface, 'return'))
    iterNext.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(interface))])
    iterNext.setParent(interface)
    iterNext.setComments([self.SIDL.getComment('Builtin method')])
    return iterNext
  
  def getBuiltinMethods(self, interface):
    return [self.getCast(interface), self.getIterNext(interface), self.getDestructor(interface), self.getStr(interface)]
  
  def initInterfaceEpv(self, interface, implementor, epvVar, body, seen):
    if interface in seen:
      return 
    seen.append(interface)
    comment = 'Initialize interface ' + interface.getFullIdentifier() + ' methods with those of class ' + implementor.getFullIdentifier()
    for (m, methodName) in [(method, method.getIdentifier()) for method in self.getBuiltinMethods(interface) + list(self.getTemplateMethods(interface.getMethods()))]:
      infEpvVar = self.Cxx.getVar(interface.getFullIdentifier().replace('.', '_') + '_epv')
      infMethod = self.Cxx.getStructRef(infEpvVar, methodName, 0)
      objMethod = self.Cxx.getStructRef(epvVar, methodName, 0)
      setMethod = self.Cxx.getAssignment(infMethod, self.IOR.castToPrototype(objMethod, interface, m))
      body.addChildren([self.Cxx.getExpStmt(setMethod, comment)])
      comment = None
    for parent in interface.getParents():
      self.initInterfaceEpv(parent, implementor, epvVar, body, seen)
    return 
  
  def initClassEpv(self, klass, obj, body, isPointer = 0, interfacesSeen = None, implMethods = None):
    if interfacesSeen is None:
      interfacesSeen = []
    epvVar = self.Cxx.getVar(klass.getFullIdentifier().replace('.', '_') + '_epv')
    for parent in klass.getParents():
      epvVar = self.Cxx.getVar(klass.getFullIdentifier().replace('.', '_') + '_epv')
      parentObj = self.Cxx.getStructRef(obj, parent.getFullIdentifier().replace('.', '_').lower(), isPointer)
      parentEpvVar = self.Cxx.getStructRef(parentObj, 'd_epv', 0)
      comment = 'Override class ' + klass.getFullIdentifier() + ' methods with those of superclass ' + parent.getFullIdentifier()
      if implMethods is None:
        implMethods = []
      implMethods.extend([m.getIdentifier() for m in self.getTemplateMethods(list(klass.getImplementedMethods()) + self.getIORMethods(klass))])
      for method in [m for m in self.getTemplateMethods(parent.getMethods()) if not (m.getIdentifier() in implMethods)]:
        classMethod = self.Cxx.getStructRef(epvVar, method.getIdentifier(), 0)
        parentMethod = self.Cxx.getStructRef(parentEpvVar, method.getIdentifier())
        setMethod = self.Cxx.getAssignment(classMethod, self.IOR.castToPrototype(parentMethod, klass, method))
        body.addChildren([self.Cxx.getExpStmt(setMethod, comment)])
        comment = None
      comment = 'Virtualize superclass ' + parent.getFullIdentifier() + ' methods with those of class ' + klass.getFullIdentifier()
      parentEpvVar = self.Cxx.getVar(parent.getFullIdentifier().replace('.', '_') + '_epv')
      for method in self.getBuiltinMethods(klass) + list(self.getTemplateMethods(parent.getMethods())):
        classMethod = self.Cxx.getStructRef(epvVar, method.getIdentifier(), 0)
        parentMethod = self.Cxx.getStructRef(parentEpvVar, method.getIdentifier(), 0)
        setMethod = self.Cxx.getAssignment(parentMethod, self.IOR.castToPrototype(classMethod, parent, method))
        body.addChildren([self.Cxx.getExpStmt(setMethod, comment)])
        comment = None
    for interface in klass.getInterfaces():
      self.initInterfaceEpv(interface, klass, epvVar, body, interfacesSeen)
    for parent in klass.getParents():
      import ASE.Compiler.SIDL.Class
      parentObj = self.Cxx.getStructRef(obj, parent.getFullIdentifier().replace('.', '_').lower(), isPointer)
      self.initClassEpv(ASE.Compiler.SIDL.Class.Class(parent), parentObj, body, interfacesSeen = interfacesSeen, implMethods = implMethods)
    return 
  
  def initEpv(self, klass, body):
    '''
    Initialize the static EPV structures for klass
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Negation
    epvInitVar = self.Cxx.getVar('epvInitialized')
    negEpvInit = ASE.Compiler.Cxx.Negation.Negation()
    negEpvInit.setChildren([epvInitVar])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    epvVar = self.Cxx.getVar(klass.getFullIdentifier().replace('.', '_') + '_epv')
    for method in self.getBuiltinMethods(klass) + [m for m in self.getTemplateMethods(self.getIORMethods(klass)) if not (m.getStatic())]:
      if method.getIdentifier() == 'iterNext':
        continue
      objMethod = self.Cxx.getStructRef(epvVar, method.getIdentifier(), 0)
      setMethod = self.Cxx.getAssignment(objMethod, self.IOR.castToPrototype(self.Cxx.getVar(method.getIdentifier()), klass, method))
      compound.addChildren([self.Cxx.getExpStmt(setMethod)])
    skelInit = self.Cxx.getFunctionCall(klass.getFullIdentifier().replace('.', '_') + '__set_epv', [self.Cxx.getAddress(epvVar), '_exception'])
    compound.addChildren([self.Cxx.getExpStmt(skelInit)])
    self.initClassEpv(klass, self.Cxx.getVar('self'), compound, isPointer = 1)
    setEpvInit = self.Cxx.getAssignment(epvInitVar, self.Cxx.getInteger(1))
    compound.addChildren([self.Cxx.getExpStmt(setEpvInit)])
    checkEpvInit = ASE.Compiler.Cxx.If.If()
    checkEpvInit.setBranch(negEpvInit)
    checkEpvInit.setChildren([compound])
    body.addDeclarations([self.Cxx.getDeclaration(epvInitVar, self.IOR.typeMap['bool'], isStatic = 1)])
    body.addChildren([checkEpvInit])
    return 
  
  def setEpv(self, interface, obj, body, seen = None, isPointer = 0):
    import ASE.Compiler.Cxx.Type
    if seen is None:
      seen = []
    epvVar = self.Cxx.getVar(interface.getFullIdentifier().replace('.', '_') + '_epv')
    if not (interface in seen):
      seen.append(interface)
      body.addDeclarations([self.Cxx.getDeclaration(epvVar, ASE.Compiler.Cxx.Type.Type(self.IOR.getEpvType(interface).getChildren()[0]), isStatic = 1)])
    for parent in interface.getParents():
      parentObj = self.Cxx.getStructRef(obj, parent.getFullIdentifier().replace('.', '_').lower(), isPointer)
      self.setEpv(parent, parentObj, body, seen)
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      import ASE.Compiler.SIDL.Class
      for inf in ASE.Compiler.SIDL.Class.Class(interface).getInterfaces():
        infObj = self.Cxx.getStructRef(obj, inf.getFullIdentifier().replace('.', '_').lower(), isPointer)
        self.setEpv(inf, infObj, body, seen)
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(obj, 'd_epv', isPointer), self.Cxx.getAddress(epvVar)))])
    return 
  
  def setInterfaceObject(self, interface, selfVar, objVar, body, useSelfPointer = 0):
    infStruct = self.Cxx.getStructRef(selfVar, interface.getFullIdentifier().replace('.', '_').lower(), useSelfPointer)
    infObj = self.IOR.getTrueIOR(infStruct, 0)
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(infObj, objVar))])
    for parent in interface.getParents():
      self.setInterfaceObject(parent, infStruct, objVar, body)
    return 
  
  def getInterfaceCastCheck(self, interface, obj):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Negation
    trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    trueBranch.setChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getVar('cast'), obj))])
    strCmp = self.Cxx.getFunctionCall('strcmp', ['name', self.Cxx.getString(interface.getFullIdentifier())])
    negStrCmp = ASE.Compiler.Cxx.Negation.Negation()
    negStrCmp.setChildren([strCmp])
    checkInterface = ASE.Compiler.Cxx.If.If()
    checkInterface.setBranch(negStrCmp)
    checkInterface.setChildren([trueBranch])
    yield (interface, checkInterface)
    for parent in interface.getParents():
      parentObj = self.Cxx.getStructRef(obj, parent.getFullIdentifier().replace('.', '_').lower())
      for check in self.getInterfaceCastCheck(parent, self.Cxx.getGroup(self.Cxx.getAddress(parentObj))):
        yield check
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      for inf in ASE.Compiler.SIDL.Class.Class(interface).getInterfaces():
        infObj = self.Cxx.getStructRef(obj, inf.getFullIdentifier().replace('.', '_').lower())
        for check in self.getInterfaceCastCheck(inf, self.Cxx.getGroup(self.Cxx.getAddress(infObj))):
          yield check
    return 
  
  def getCastCheck(self, interface, obj):
    interfaces = []
    topCheck = None
    currentCheck = None
    for (inf, check) in self.getInterfaceCastCheck(interface, obj):
      if not (inf in interfaces):
        interfaces.append(inf)
        if currentCheck is None:
          topCheck = check
        else:
          currentCheck.addChildren([check])
        currentCheck = check
    return topCheck
  
  def getObject(self, type):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    defns = []
    if type.getBaseType():
      typeName = type.getIdentifier()
      if typeName in ['dcomplex', 'fcomplex']:
        realDecl = ASE.Compiler.Cxx.Declarator.Declarator()
        realDecl.setIdentifier('real')
        imagDecl = ASE.Compiler.Cxx.Declarator.Declarator()
        imagDecl.setIdentifier('imaginary')
        if typeName == 'dcomplex':
          realDecl.setType(self.IOR.typeMap['double'])
          imagDecl.setType(self.IOR.typeMap['double'])
        if typeName == 'fcomplex':
          realDecl.setType(self.IOR.typeMap['float'])
          imagDecl.setType(self.IOR.typeMap['float'])
        objStruct = ASE.Compiler.Cxx.Struct.Struct()
        objStruct.setIdentifier('SIDL_' + typeName)
        objStruct.setChildren([self.Cxx.getDecl(realDecl), self.Cxx.getDecl(imagDecl)])
        objDecl = ASE.Compiler.Cxx.Declarator.Declarator()
        objDecl.setType(objStruct)
      else:
        return ([], [])
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      interface = ASE.Compiler.SIDL.Interface.Interface(type)
      objStruct = ASE.Compiler.Cxx.Struct.Struct()
      objStruct.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__object')
      for parent in interface.getParents():
        parentDecl = ASE.Compiler.Cxx.Declarator.Declarator()
        parentDecl.setIdentifier(parent.getFullIdentifier().replace('.', '_').lower())
        parentDecl.setType(ASE.Compiler.Cxx.Type.Type(self.IOR.getType(parent).getChildren()[0]))
        objStruct.addChildren([self.Cxx.getDecl(parentDecl)])
      if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
        for interf in ASE.Compiler.SIDL.Class.Class(interface).getInterfaces():
          interfaceDecl = ASE.Compiler.Cxx.Declarator.Declarator()
          interfaceDecl.setIdentifier(interf.getFullIdentifier().replace('.', '_').lower())
          interfaceDecl.setType(ASE.Compiler.Cxx.Type.Type(self.IOR.getType(interf).getChildren()[0]))
          objStruct.addChildren([self.Cxx.getDecl(interfaceDecl)])
      epvStruct = ASE.Compiler.Cxx.Struct.Struct()
      epvStruct.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__epv')
      epvType = ASE.Compiler.Cxx.Pointer.Pointer()
      epvType.setChildren([epvStruct])
      epvDecl = ASE.Compiler.Cxx.Declarator.Declarator()
      epvDecl.setIdentifier('d_epv')
      epvDecl.setType(epvType)
      oldEpvDecl = ASE.Compiler.Cxx.Declarator.Declarator()
      oldEpvDecl.setIdentifier('d_oldEpv')
      oldEpvDecl.setType(epvType)
      objStruct.addChildren([self.Cxx.getDecl(epvDecl), self.Cxx.getDecl(oldEpvDecl)])
      origDecl = ASE.Compiler.Cxx.Declarator.Declarator()
      origDecl.setIdentifier('d_object')
      origDecl.setType(self.IOR.typeMap['opaque'])
      objStruct.addChildren([self.Cxx.getDecl(origDecl)])
      if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
        implDecl = ASE.Compiler.Cxx.Declarator.Declarator()
        implDecl.setIdentifier('d_data')
        implDecl.setType(self.IOR.typeMap['opaque'])
        objStruct.addChildren([self.Cxx.getDecl(implDecl)])
      objDecl = ASE.Compiler.Cxx.Declarator.Declarator()
      objDecl.setType(objStruct)
      if type.isInstanceOf('ASE.Compiler.SIDL.Class'):
        selfVar = self.Cxx.getVar('self')
        objVar = self.Cxx.getVar('obj')
        selfType = self.IOR.getType(type)
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        for parent in type.getParents():
          parentVar = self.Cxx.getVar('parent')
          parentObj = self.Cxx.getStructRef(selfVar, parent.getFullIdentifier().replace('.', '_').lower())
          compound.addDeclarations([self.Cxx.getDeclaration(parentVar, self.IOR.getType(parent), self.Cxx.getInitializer(None, self.Cxx.getAddress(parentObj)))])
          extFuncVar = self.Cxx.getVar('extFunc')
          extFuncCastPtr = ASE.Compiler.Cxx.Declarator.Declarator()
          extFuncCastPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
          extFuncCastDecl = ASE.Compiler.Cxx.Function.Function()
          extFuncCastDecl.setChildren([self.Cxx.getDeclGroup(extFuncCastPtr)])
          extFuncCastDecl.setType(self.IOR.getExternalType(parent))
          extFunc = ASE.Compiler.Cxx.Cast.Cast()
          extFunc.setTypeName(extFuncCastDecl)
          extFunc.setChildren([self.IOR.getClassMethodTable(parent)])
          extFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
          extFuncPtr.setIdentifier(extFuncVar.getIdentifier())
          extFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
          extFuncDecl = ASE.Compiler.Cxx.Function.Function()
          extFuncDecl.setChildren([self.Cxx.getDeclGroup(extFuncPtr)])
          extFuncDecl.setType(self.IOR.getExternalType(parent))
          extFuncDecl.setInitializer(self.Cxx.getInitializer(None, extFunc))
          compound.addDeclarations([self.Cxx.getDecl(extFuncDecl)])
          parentInitVar = self.Cxx.getVar('parentInit')
          parentInitFunc = self.Cxx.getStructRef(self.Cxx.getGroup(self.Cxx.getFunctionCall(extFuncVar)), 'initObject')
          parentInitInit = self.Cxx.getInitializer(None, parentInitFunc)
          parentInitDecl = self.IOR.getFunctionDeclaration(parent, self.getInit(parent), isPointer = 1, onlyTypes = 1)
          ASE.Compiler.Cxx.Function.Function(parentInitDecl.getChildren()[0]).setInitializer(parentInitInit)
          parentInitDecl.getChildren()[0].getChildren()[0].getChildren()[0].setIdentifier(parentInitVar.getIdentifier())
          compound.addDeclarations([parentInitDecl])
          compound.addChildren([self.IOR.getErrorCheck(type, None, parentInitVar, 'Unable to find initializer for ' + parent.getFullIdentifier(), returnVar = self.Cxx.getVar('_result'))])
          compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall(self.Cxx.getGroup(self.Cxx.getIndirection(parentInitVar)), [parentVar, objVar, self.Cxx.getVar('_exception')]))])
        self.initEpv(type, compound)
        self.setEpv(type, selfVar, compound, isPointer = 1)
        compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(selfVar, 'd_oldEpv'), self.Cxx.getStructRef(selfVar, 'd_epv')), 'Save old EPV for getSuper()')])
        compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.IOR.getTrueIOR(selfVar), objVar), 'Initialize pointers to the original object')])
        for interface in type.getInterfaces():
          self.setInterfaceObject(interface, selfVar, objVar, compound, useSelfPointer = 1)
        constructorFunc = self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getStructRef(selfVar, 'd_epv'), type.getIdentifier()))))
        compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(selfVar, 'd_data'), self.Cxx.getNull()), 'Setup implementation data')])
        compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall(constructorFunc, [selfVar, '_exception']), 'Call the Skeleton constructor')])
        initFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, self.getInit(type)).getChildren()[0])
        initFunc.setBody(compound)
        initFunc.setStatic(1)
        mallocSelf = self.Cxx.castToType(self.Cxx.getFunctionCall('malloc', [self.Cxx.getSizeof(selfType.getChildren()[0])]), selfType)
        returnSelf = ASE.Compiler.Cxx.Return.Return()
        returnSelf.setChildren([selfVar])
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        compound.setDeclarations([self.Cxx.getDeclaration(selfVar, selfType, self.Cxx.getInitializer(None, mallocSelf))])
        compound.setChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall(initFunc.getChildren()[0], [selfVar, selfVar, '_exception'])), returnSelf])
        createFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, self.getConstructor(type)).getChildren()[0])
        createFunc.setBody(compound)
        createFunc.setStatic(1)
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        for interface in list(type.getInterfaces()) + list(type.getParents()):
          infStruct = self.Cxx.getStructRef(selfVar, interface.getFullIdentifier().replace('.', '_').lower())
          compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(infStruct, 'd_epv', 0), self.Cxx.getStructRef(infStruct, 'd_oldEpv', 0)))])
        destructorFunc = self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getStructRef(selfVar, 'd_epv'), type.getIdentifier() + '_'))))
        compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall(destructorFunc, [selfVar, '_exception']))])
        for parent in type.getParents():
          parentVar = self.Cxx.getVar('parent')
          parentObj = self.Cxx.getStructRef(selfVar, parent.getFullIdentifier().replace('.', '_').lower())
          compound.addDeclarations([self.Cxx.getDeclaration(parentVar, self.IOR.getType(parent), self.Cxx.getInitializer(None, self.Cxx.getAddress(parentObj)))])
          extFuncVar = self.Cxx.getVar('extFunc')
          extFuncCastPtr = ASE.Compiler.Cxx.Declarator.Declarator()
          extFuncCastPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
          extFuncCastDecl = ASE.Compiler.Cxx.Function.Function()
          extFuncCastDecl.setChildren([self.Cxx.getDeclGroup(extFuncCastPtr)])
          extFuncCastDecl.setType(self.IOR.getExternalType(parent))
          extFunc = ASE.Compiler.Cxx.Cast.Cast()
          extFunc.setTypeName(extFuncCastDecl)
          extFunc.setChildren([self.IOR.getClassMethodTable(parent)])
          extFuncInit = ASE.Compiler.Cxx.Initializer.Initializer()
          extFuncInit.setChildren([extFunc])
          extFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
          extFuncPtr.setIdentifier(extFuncVar.getIdentifier())
          extFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
          extFuncDecl = ASE.Compiler.Cxx.Function.Function()
          extFuncDecl.setChildren([self.Cxx.getDeclGroup(extFuncPtr)])
          extFuncDecl.setType(self.IOR.getExternalType(parent))
          extFuncDecl.setInitializer(extFuncInit)
          compound.addDeclarations([self.Cxx.getDecl(extFuncDecl)])
          parentFiniVar = self.Cxx.getVar('parentFini')
          parentFiniFunc = self.Cxx.getStructRef(self.Cxx.getGroup(self.Cxx.getFunctionCall(extFuncVar)), 'finiObject')
          parentFiniDecl = self.IOR.getFunctionDeclaration(parent, self.getFini(parent), isPointer = 1, onlyTypes = 1)
          ASE.Compiler.Cxx.Function.Function(parentFiniDecl.getChildren()[0]).setInitializer(self.Cxx.getInitializer(None, parentFiniFunc))
          parentFiniDecl.getChildren()[0].getChildren()[0].getChildren()[0].setIdentifier(parentFiniVar.getIdentifier())
          compound.addDeclarations([parentFiniDecl])
          compound.addChildren([self.IOR.getErrorCheck(type, None, parentFiniVar, 'Unable to find finalizer for ' + parent.getFullIdentifier(), returnVar = self.Cxx.getVar('_result'))])
          compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall(self.Cxx.getGroup(self.Cxx.getIndirection(parentFiniVar)), [parentVar, '_exception']))])
        finiFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, self.getFini(type)).getChildren()[0])
        finiFunc.setBody(compound)
        finiFunc.setStatic(1)
        objVar = self.IOR.getTrueIOR(selfVar)
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall(finiFunc.getChildren()[0], [objVar, '_exception']))])
        compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('memset', [objVar, 0, self.Cxx.getSizeof(selfType.getChildren()[0])]))])
        compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('free', [objVar]))])
        destroyFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, self.getDestructor(type)).getChildren()[0])
        destroyFunc.setBody(compound)
        destroyFunc.setStatic(1)
        objVar = self.Cxx.getVar('obj')
        castVar = self.Cxx.getVar('cast')
        objInit = self.Cxx.getInitializer(None, self.IOR.getTrueIOR(self.Cxx.getGroup(self.Cxx.castToType(selfVar, self.IOR.getType(self.SIDL.getBaseInterface(type))))))
        castReturn = ASE.Compiler.Cxx.Return.Return()
        castReturn.setChildren([castVar])
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        compound.setDeclarations([self.Cxx.getDeclaration(objVar, self.IOR.getType(type), objInit, 'We are guaranteed here that the first member in any IOR is a ASE.BaseInterface object'), self.Cxx.getDeclaration(castVar, self.IOR.typeMap['opaque'])])
        compound.setChildren([self.getCastCheck(type, objVar), castReturn])
        castFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, self.getCast(type)).getChildren()[0])
        castFunc.setBody(compound)
        castFunc.setStatic(1)
        strReturn = ASE.Compiler.Cxx.Return.Return()
        strReturn.setChildren([self.Cxx.getString(type.getFullIdentifier())])
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        compound.setChildren([strReturn])
        strFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, self.getStr(type)).getChildren()[0])
        strFunc.setBody(compound)
        strFunc.setStatic(1)
        defns = []
        for parent in type.getParents():
          if type.getFullIdentifier() in ['ASE.DLL', 'ASE.Loader']:
            aseExc = self.SIDL.getASEException(type)
            defns.append(self.IOR.getGuardedInclude(self.SIDL.getBaseException(aseExc)))
            defns.extend(self.getForwardDeclarations(aseExc)[0])
            defns.extend(self.getEPV(aseExc)[0])
            defns.extend(self.getObject(aseExc)[0])
            defns.extend(self.getExternal(aseExc)[0])
            defns.extend(self.getExternals(aseExc)[0])
          else:
            defns.append(self.IOR.getGuardedInclude(self.SIDL.getASEException(type)))
        defns.extend([castFunc, strFunc, finiFunc, destroyFunc, initFunc, createFunc])
    return ([self.Cxx.getDecl(objDecl, 'The IOR object structure for this type')], defns)
  
  def initArray(self, type, method, body, isBorrowed):
    import ASE.Compiler.Cxx.Addition
    import ASE.Compiler.Cxx.ArrayReference
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.For
    import ASE.Compiler.Cxx.Increment
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.LessThan
    import ASE.Compiler.Cxx.Modulo
    import ASE.Compiler.Cxx.Multiplication
    import ASE.Compiler.Cxx.MultiplicationAssignment
    import ASE.Compiler.Cxx.Sizeof
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Subtraction
    inParams = method.getInParameters()
    dimenVar = self.Cxx.getVar(inParams[0].getIdentifier())
    lowerVar = self.Cxx.getVar(inParams[1].getIdentifier())
    upperVar = self.Cxx.getVar(inParams[2].getIdentifier())
    if len(inParams) > 3:
      strideVar = self.Cxx.getVar(inParams[3].getIdentifier())
    else:
      strideVar = None
    returnVar = self.Cxx.getVar(method.getReturnParameter().getIdentifier())
    arraySizeVar = self.Cxx.getVar('arraySize')
    sizeVar = self.Cxx.getVar('size')
    dVar = self.Cxx.getVar('d')
    arrayType = self.IOR.getType(method.getReturnParameter().getType())
    arrayStruct = ASE.Compiler.Cxx.Struct.Struct(arrayType.getChildren()[0])
    intDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    intDecl.setType(self.IOR.typeMap['int'])
    sizeofInt = ASE.Compiler.Cxx.Sizeof.Sizeof()
    sizeofInt.setTypeName(intDecl)
    structDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    structDecl.setType(arrayStruct)
    sizeofStruct = ASE.Compiler.Cxx.Sizeof.Sizeof()
    sizeofStruct.setTypeName(structDecl)
    elemDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    elemDecl.setType(self.IOR.getType(type))
    sizeofElem = ASE.Compiler.Cxx.Sizeof.Sizeof()
    sizeofElem.setTypeName(elemDecl)
    structOffset = ASE.Compiler.Cxx.Modulo.Modulo()
    structOffset.setChildren([self.Cxx.getGroup(sizeofStruct), sizeofInt])
    wordDiff = ASE.Compiler.Cxx.Subtraction.Subtraction()
    wordDiff.setChildren([sizeofInt, structOffset])
    wordOffset = ASE.Compiler.Cxx.Modulo.Modulo()
    wordOffset.setChildren([self.Cxx.getGroup(wordDiff), sizeofInt])
    arraySize = ASE.Compiler.Cxx.Addition.Addition()
    arraySize.setChildren([sizeofStruct, wordOffset])
    arraySizeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    arraySizeInit.setChildren([arraySize])
    sizeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    sizeInit.setChildren([self.Cxx.getInteger(1)])
    safeDimen = ASE.Compiler.Cxx.If.If()
    safeDimen.setBranch(dimenVar)
    safeDimen.setChildren([dimenVar, self.Cxx.getInteger(1)])
    safeDimen.setTernary(1)
    indexSize = ASE.Compiler.Cxx.Multiplication.Multiplication()
    indexSize.setChildren([sizeofInt, self.Cxx.getGroup(safeDimen)])
    indicesSize = ASE.Compiler.Cxx.Multiplication.Multiplication()
    indicesSize.setChildren([self.Cxx.getInteger(3), indexSize])
    returnSize = ASE.Compiler.Cxx.Addition.Addition()
    returnSize.setChildren([arraySizeVar, indicesSize])
    returnAlloc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    returnAlloc.setChildren([self.Cxx.getVar('malloc')])
    returnAlloc.setArguments([returnSize])
    returnInit = ASE.Compiler.Cxx.Initializer.Initializer()
    returnInit.setChildren([self.Cxx.castToType(returnAlloc, arrayType)])
    body.addDeclarations([self.Cxx.getDeclaration(arraySizeVar, self.IOR.typeMap['int'], arraySizeInit, isStatic = 1), self.Cxx.getDeclaration(returnVar, arrayType, returnInit), self.Cxx.getDeclaration(sizeVar, self.IOR.typeMap['int'], sizeInit), self.Cxx.getDeclaration(dVar, self.IOR.typeMap['int'])])
    arrayDimen = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayDimen.setChildren([returnVar])
    arrayDimen.setMember('d_dimen')
    arrayDimen.setPointer(1)
    arrayBorrow = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayBorrow.setChildren([returnVar])
    arrayBorrow.setMember('d_borrowed')
    arrayBorrow.setPointer(1)
    arrayLower = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayLower.setChildren([returnVar])
    arrayLower.setMember('d_lower')
    arrayLower.setPointer(1)
    arrayUpper = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayUpper.setChildren([returnVar])
    arrayUpper.setMember('d_upper')
    arrayUpper.setPointer(1)
    arrayStride = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayStride.setChildren([returnVar])
    arrayStride.setMember('d_stride')
    arrayStride.setPointer(1)
    dArrayLower = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dArrayLower.setChildren([arrayLower])
    dArrayLower.setIndex(dVar)
    dArrayUpper = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dArrayUpper.setChildren([arrayUpper])
    dArrayUpper.setIndex(dVar)
    dArrayStride = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dArrayStride.setChildren([arrayStride])
    dArrayStride.setIndex(dVar)
    lowerFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    lowerFirst.setChildren([lowerVar])
    lowerFirst.setMember('d_firstElement')
    lowerFirst.setPointer(1)
    upperFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    upperFirst.setChildren([upperVar])
    upperFirst.setMember('d_firstElement')
    upperFirst.setPointer(1)
    dLower = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dLower.setChildren([lowerFirst])
    dLower.setIndex(dVar)
    dUpper = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dUpper.setChildren([upperFirst])
    dUpper.setIndex(dVar)
    if not (strideVar is None):
      strideFirst = ASE.Compiler.Cxx.StructReference.StructReference()
      strideFirst.setChildren([strideVar])
      strideFirst.setMember('d_firstElement')
      strideFirst.setPointer(1)
      dStride = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
      dStride.setChildren([strideFirst])
      dStride.setIndex(dVar)
    setDimen = ASE.Compiler.Cxx.Assignment.Assignment()
    setDimen.setChildren([arrayDimen, dimenVar])
    setBorrowed = ASE.Compiler.Cxx.Assignment.Assignment()
    setBorrowed.setChildren([arrayBorrow, self.Cxx.getInteger(isBorrowed)])
    incReturn = ASE.Compiler.Cxx.Addition.Addition()
    incReturn.setChildren([self.Cxx.castToType(returnVar, self.IOR.typeMap['char'], 1), arraySizeVar])
    setLower = ASE.Compiler.Cxx.Assignment.Assignment()
    setLower.setChildren([arrayLower, self.Cxx.castToType(self.Cxx.getGroup(incReturn), self.IOR.typeMap['int'], 1)])
    incLower = ASE.Compiler.Cxx.Addition.Addition()
    incLower.setChildren([arrayLower, self.Cxx.getGroup(safeDimen)])
    setUpper = ASE.Compiler.Cxx.Assignment.Assignment()
    setUpper.setChildren([arrayUpper, incLower])
    incUpper = ASE.Compiler.Cxx.Addition.Addition()
    incUpper.setChildren([arrayUpper, self.Cxx.getGroup(safeDimen)])
    setStride = ASE.Compiler.Cxx.Assignment.Assignment()
    setStride.setChildren([arrayStride, incUpper])
    lowerFirst = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    lowerFirst.setChildren([arrayLower])
    lowerFirst.setIndex(self.Cxx.getInteger(0))
    initLower = ASE.Compiler.Cxx.Assignment.Assignment()
    initLower.setChildren([lowerFirst, self.Cxx.getInteger(0)])
    upperFirst = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    upperFirst.setChildren([arrayUpper])
    upperFirst.setIndex(self.Cxx.getInteger(0))
    initUpper = ASE.Compiler.Cxx.Assignment.Assignment()
    initUpper.setChildren([upperFirst, self.Cxx.getInteger(0)])
    strideFirst = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    strideFirst.setChildren([arrayStride])
    strideFirst.setIndex(self.Cxx.getInteger(0))
    initStride = ASE.Compiler.Cxx.Assignment.Assignment()
    initStride.setChildren([strideFirst, self.Cxx.getInteger(1)])
    body.addChildren([self.Cxx.getExpStmt(setDimen, 'Initialize array structure')])
    body.addChildren(map(self.Cxx.getExpStmt, [setBorrowed, setLower, setUpper, setStride, initLower, initUpper, initStride]))
    setLower = ASE.Compiler.Cxx.Assignment.Assignment()
    setLower.setChildren([dArrayLower, dLower])
    setUpper = ASE.Compiler.Cxx.Assignment.Assignment()
    setUpper.setChildren([dArrayUpper, dUpper])
    if strideVar is None:
      setStride = ASE.Compiler.Cxx.Assignment.Assignment()
      setStride.setChildren([dArrayStride, sizeVar])
    else:
      setStride = ASE.Compiler.Cxx.Assignment.Assignment()
      setStride.setChildren([dArrayStride, dStride])
    rowDiff = ASE.Compiler.Cxx.Subtraction.Subtraction()
    rowDiff.setChildren([dUpper, dLower])
    rowSize = ASE.Compiler.Cxx.Addition.Addition()
    rowSize.setChildren([rowDiff, self.Cxx.getInteger(1)])
    setSize = ASE.Compiler.Cxx.MultiplicationAssignment.MultiplicationAssignment()
    setSize.setChildren([sizeVar, rowSize])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren(map(self.Cxx.getExpStmt, [setLower, setUpper, setStride, setSize]))
    dInit = ASE.Compiler.Cxx.Assignment.Assignment()
    dInit.setChildren([dVar, self.Cxx.getInteger(0)])
    dCmp = ASE.Compiler.Cxx.LessThan.LessThan()
    dCmp.setChildren([dVar, dimenVar])
    dInc = ASE.Compiler.Cxx.Increment.Increment()
    dInc.setChildren([dVar])
    dInc.setPostfix(1)
    sizeLoop = ASE.Compiler.Cxx.For.For()
    sizeLoop.setInitialization(dInit)
    sizeLoop.setBranch(dCmp)
    sizeLoop.setIncrement(dInc)
    sizeLoop.setChildren([compound])
    sizeLoop.setComments([self.Cxx.getComment('Determine array size and set bounds')])
    setSize = ASE.Compiler.Cxx.MultiplicationAssignment.MultiplicationAssignment()
    setSize.setChildren([sizeVar, sizeofElem])
    body.addChildren([sizeLoop, self.Cxx.getExpStmt(setSize)])
    return 
  
  def m_createArray(self, type, method, body):
    import ASE.Compiler.Cxx.Addition
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Multiplication
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.Sizeof
    import ASE.Compiler.Cxx.StructReference
    self.initArray(type, method, body, 0)
    returnVar = self.Cxx.getVar(method.getReturnParameter().getIdentifier())
    sizeVar = self.Cxx.getVar('size')
    arrayFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayFirst.setChildren([returnVar])
    arrayFirst.setMember('d_firstElement')
    arrayFirst.setPointer(1)
    dataAlloc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    dataAlloc.setChildren([self.Cxx.getVar('malloc')])
    dataAlloc.setArguments([sizeVar])
    setData = ASE.Compiler.Cxx.Assignment.Assignment()
    setData.setChildren([arrayFirst, self.Cxx.castToType(dataAlloc, self.IOR.getType(type), 1)])
    dataInit = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    dataInit.setChildren([self.Cxx.getVar('memset')])
    dataInit.setArguments([arrayFirst, self.Cxx.getInteger(0), sizeVar])
    returnArray = ASE.Compiler.Cxx.Return.Return()
    returnArray.setChildren([returnVar])
    body.addChildren([self.Cxx.getExpStmt(setData, 'Allocate and initialize data'), self.Cxx.getExpStmt(dataInit), returnArray])
    return 
  
  def m_borrowArray(self, type, method, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.Return
    self.initArray(type, method, body, 1)
    firstVar = self.Cxx.getVar(method.getInParameters()[4].getIdentifier())
    returnVar = self.Cxx.getVar(method.getReturnParameter().getIdentifier())
    arrayFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayFirst.setChildren([returnVar])
    arrayFirst.setMember('d_firstElement')
    arrayFirst.setPointer(1)
    setData = ASE.Compiler.Cxx.Assignment.Assignment()
    setData.setChildren([arrayFirst, self.Cxx.castToType(firstVar, self.IOR.getType(type), 1)])
    returnArray = ASE.Compiler.Cxx.Return.Return()
    returnArray.setChildren([returnVar])
    body.addChildren([self.Cxx.getExpStmt(setData), returnArray])
    return 
  
  def m_destroyArray(self, type, method, body):
    import ASE.Compiler.Cxx.Comment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Return
    comment = self.Cxx.getComment('Destroy the array (NULL arrays are ignored)')
    arrayVar = self.Cxx.getVar('array')
    arrayMem = self.Cxx.getStructRef(arrayVar, 'd_firstElement')
    borrowed = self.Cxx.getStructRef(arrayVar, 'd_borrowed')
    freeArray = self.Cxx.getFunctionCall('free', [arrayMem])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(freeArray)])
    notBorrowed = ASE.Compiler.Cxx.Negation.Negation()
    notBorrowed.setChildren([borrowed])
    checkBorrowed = ASE.Compiler.Cxx.If.If()
    checkBorrowed.setBranch(notBorrowed)
    checkBorrowed.setChildren([compound])
    freeStruct = self.Cxx.getFunctionCall('free', [arrayVar])
    body.addChildren([checkBorrowed, self.Cxx.getExpStmt(freeStruct)])
    return 
  
  def m_getDimen(self, type, method, body):
    import ASE.Compiler.Cxx.Return
    arrayDimen = self.Cxx.getStructRef(self.Cxx.getVar(method.getInParameters()[0].getIdentifier()), 'd_dimen')
    returnDimen = ASE.Compiler.Cxx.Return.Return()
    returnDimen.setChildren([arrayDimen])
    body.addChildren([returnDimen])
    return 
  
  def m_getLower(self, type, method, body):
    import ASE.Compiler.Cxx.Return
    arrayLower = self.Cxx.getStructRef(self.Cxx.getVar(method.getInParameters()[0].getIdentifier()), 'd_lower')
    lowerInd = self.Cxx.getArrayRef(arrayLower, self.Cxx.getVar(method.getInParameters()[1].getIdentifier()))
    returnLower = ASE.Compiler.Cxx.Return.Return()
    returnLower.setChildren([lowerInd])
    body.addChildren([returnLower])
    return 
  
  def m_getUpper(self, type, method, body):
    import ASE.Compiler.Cxx.Return
    arrayUpper = self.Cxx.getStructRef(self.Cxx.getVar(method.getInParameters()[0].getIdentifier()), 'd_upper')
    upperInd = self.Cxx.getArrayRef(arrayUpper, self.Cxx.getVar(method.getInParameters()[1].getIdentifier()))
    returnUpper = ASE.Compiler.Cxx.Return.Return()
    returnUpper.setChildren([upperInd])
    body.addChildren([returnUpper])
    return 
  
  def findElement(self, type, method, body):
    import ASE.Compiler.Cxx.AdditionAssignment
    import ASE.Compiler.Cxx.ArrayReference
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.For
    import ASE.Compiler.Cxx.GreaterThan
    import ASE.Compiler.Cxx.Increment
    import ASE.Compiler.Cxx.LessThan
    import ASE.Compiler.Cxx.LogicalOr
    import ASE.Compiler.Cxx.Multiplication
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Subtraction
    arrayVar = self.Cxx.getVar(method.getInParameters()[0].getIdentifier())
    indexVar = self.Cxx.getVar(method.getInParameters()[1].getIdentifier())
    elemVar = self.Cxx.getVar('_element')
    dVar = self.Cxx.getVar('d')
    elemType = ASE.Compiler.Cxx.Pointer.Pointer()
    elemType.setChildren([self.IOR.getType(type)])
    body.addDeclarations([self.Cxx.getDeclaration(elemVar, elemType), self.Cxx.getDeclaration(dVar, self.IOR.typeMap['int'])])
    arrayDimen = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayDimen.setChildren([arrayVar])
    arrayDimen.setMember('d_dimen')
    arrayDimen.setPointer(1)
    arrayLower = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayLower.setChildren([arrayVar])
    arrayLower.setMember('d_lower')
    arrayLower.setPointer(1)
    arrayUpper = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayUpper.setChildren([arrayVar])
    arrayUpper.setMember('d_upper')
    arrayUpper.setPointer(1)
    arrayStride = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayStride.setChildren([arrayVar])
    arrayStride.setMember('d_stride')
    arrayStride.setPointer(1)
    arrayFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayFirst.setChildren([arrayVar])
    arrayFirst.setMember('d_firstElement')
    arrayFirst.setPointer(1)
    indexFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    indexFirst.setChildren([indexVar])
    indexFirst.setMember('d_firstElement')
    indexFirst.setPointer(1)
    dIndex = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dIndex.setChildren([indexFirst])
    dIndex.setIndex(dVar)
    dLower = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dLower.setChildren([arrayLower])
    dLower.setIndex(dVar)
    dUpper = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dUpper.setChildren([arrayUpper])
    dUpper.setIndex(dVar)
    dStride = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dStride.setChildren([arrayStride])
    dStride.setIndex(dVar)
    returnInit = ASE.Compiler.Cxx.Assignment.Assignment()
    returnInit.setChildren([elemVar, arrayFirst])
    body.addChildren([self.Cxx.getExpStmt(returnInit)])
    cmpLower = ASE.Compiler.Cxx.LessThan.LessThan()
    cmpLower.setChildren([dIndex, dLower])
    cmpUpper = ASE.Compiler.Cxx.GreaterThan.GreaterThan()
    cmpUpper.setChildren([dIndex, dUpper])
    cmpIndex = ASE.Compiler.Cxx.LogicalOr.LogicalOr()
    cmpIndex.setChildren([cmpLower, cmpUpper])
    rowIndex = ASE.Compiler.Cxx.Subtraction.Subtraction()
    rowIndex.setChildren([dIndex, dLower])
    rowSize = ASE.Compiler.Cxx.Multiplication.Multiplication()
    rowSize.setChildren([dStride, self.Cxx.getGroup(rowIndex)])
    incElem = ASE.Compiler.Cxx.AdditionAssignment.AdditionAssignment()
    incElem.setChildren([elemVar, rowSize])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    if (type.isInstanceOf('ASE.Compiler.SIDL.Interface') and not (type.getFullIdentifier() in ['ASE.BaseInterface', 'ASE.BaseException', 'ASE.BaseClass'])):
      compound.addChildren([self.IOR.getErrorCheck(type, method, None, 'Invalid indices', cmpIndex, returnVar = self.Cxx.getVar('_result'))])
    compound.addChildren([self.Cxx.getExpStmt(incElem)])
    dInit = ASE.Compiler.Cxx.Assignment.Assignment()
    dInit.setChildren([dVar, self.Cxx.getInteger(0)])
    cmpDimen = ASE.Compiler.Cxx.LessThan.LessThan()
    cmpDimen.setChildren([dVar, arrayDimen])
    dInc = ASE.Compiler.Cxx.Increment.Increment()
    dInc.setChildren([dVar])
    dInc.setPostfix(1)
    dimenLoop = ASE.Compiler.Cxx.For.For()
    dimenLoop.setInitialization(dInit)
    dimenLoop.setBranch(cmpDimen)
    dimenLoop.setIncrement(dInc)
    dimenLoop.setChildren([compound])
    body.addChildren([dimenLoop])
    return 
  
  def adjustRefCount(self, type, infVar, addRef, body):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.StructReference
    infObj = self.IOR.getTrueIOR(self.Cxx.getGroup(infVar))
    infEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    infEpv.setChildren([self.Cxx.getGroup(self.Cxx.castToType(infObj, self.IOR.getType(self.SIDL.getBaseInterface(type))))])
    infEpv.setMember('d_epv')
    infEpv.setPointer(1)
    increfFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    increfFuncPtr.setChildren([infEpv])
    if addRef:
      increfFuncPtr.setMember('addReference')
    else:
      increfFuncPtr.setMember('deleteReference')
    increfFuncPtr.setPointer(1)
    increfFunc = ASE.Compiler.Cxx.Indirection.Indirection()
    increfFunc.setChildren([self.Cxx.getGroup(increfFuncPtr)])
    increfInf = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    increfInf.setChildren([self.Cxx.getGroup(increfFunc)])
    increfInf.setArguments([infObj, self.Cxx.castToType(self.Cxx.getVar('_exception'), self.IOR.typeMap['opaque'], 1)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(increfInf)])
    checkInf = ASE.Compiler.Cxx.If.If()
    checkInf.setBranch(infVar)
    checkInf.setChildren([compound])
    body.addChildren([checkInf])
    return 
  
  def m_getElement(self, type, method, body):
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Return
    elemVar = self.Cxx.getVar('_element')
    returnVar = self.Cxx.getVar('_result')
    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.IOR.getType(method.getReturnParameter().getType()))])
    self.findElement(type, method, body)
    elemInd = ASE.Compiler.Cxx.Indirection.Indirection()
    elemInd.setChildren([elemVar])
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      self.adjustRefCount(type, elemInd, 0, body)
    returnElem = ASE.Compiler.Cxx.Return.Return()
    returnElem.setChildren([elemInd])
    body.addChildren([returnElem])
    return 
  
  def m_getElement4(self, type, method, body):
    import ASE.Compiler.Cxx.Return
    returnVar = self.Cxx.getVar('_result')
    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.IOR.getType(method.getReturnParameter().getType()))])
    returnElem = ASE.Compiler.Cxx.Return.Return()
    returnElem.setChildren([returnVar])
    body.addChildren([returnElem])
    return 
  
  def m_setElement(self, type, method, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.Indirection
    valueVar = self.Cxx.getVar(method.getInParameters()[2].getIdentifier())
    elemVar = self.Cxx.getVar('_element')
    elemInd = ASE.Compiler.Cxx.Indirection.Indirection()
    elemInd.setChildren([elemVar])
    self.findElement(type, method, body)
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      self.adjustRefCount(type, valueVar, 1, body)
      self.adjustRefCount(type, elemInd, 0, body)
    setElem = ASE.Compiler.Cxx.Assignment.Assignment()
    setElem.setChildren([elemInd, valueVar])
    body.addChildren([self.Cxx.getExpStmt(setElem)])
    return 
  
  def m_setElement4(self, type, method, body):
    return 
  
  def getArray(self, type):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    if (type.getBaseType() and type.getIdentifier() == 'void'):
      return ([], [])
    arrayStruct = ASE.Compiler.Cxx.Struct.Struct()
    arrayStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__array')
    dataType = ASE.Compiler.Cxx.Pointer.Pointer()
    dataType.setChildren([self.IOR.getType(type)])
    dataDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    dataDecl.setIdentifier('d_firstElement')
    dataDecl.setType(dataType)
    intArrayType = ASE.Compiler.Cxx.Pointer.Pointer()
    intArrayType.setChildren([self.IOR.typeMap['int']])
    lowerDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    lowerDecl.setIdentifier('d_lower')
    lowerDecl.setType(intArrayType)
    upperDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    upperDecl.setIdentifier('d_upper')
    upperDecl.setType(intArrayType)
    strideDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    strideDecl.setIdentifier('d_stride')
    strideDecl.setType(intArrayType)
    borrowedDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    borrowedDecl.setIdentifier('d_borrowed')
    borrowedDecl.setType(self.IOR.typeMap['bool'])
    dimenDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    dimenDecl.setIdentifier('d_dimen')
    dimenDecl.setType(self.IOR.typeMap['int'])
    arrayStruct.addChildren(map(self.Cxx.getDecl, [dataDecl, lowerDecl, upperDecl, strideDecl, borrowedDecl, dimenDecl]))
    arrayDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    arrayDecl.setType(arrayStruct)
    decls = [self.Cxx.getDecl(arrayDecl, 'The IOR array structure for this type')]
    if type.getFullIdentifier() == 'ASE.ProjectState':
      decls.append(self.IOR.getGuardedInclude(self.SIDL.getASEException(type)))
    defns = []
    if not ((type.getBaseType() and type.getIdentifier() == 'void')):
      if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
        for parent in type.getParents():
          if not (type.getFullIdentifier() in ['ASE.DLL', 'ASE.Loader']):
            defns.extend([self.IOR.getGuardedInclude(self.SIDL.getASEException(type))])
      for method in self.getArrayMethods(type):
        import ASE.Compiler.Cxx.CompoundStatement
        import ASE.Compiler.Cxx.Function
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        funcDefn = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, method).getChildren()[0])
        funcDefn.setBody(compound)
        funcDefn.setStatic(1)
        getattr(self, 'm_' + method.getIdentifier())(type, method, compound)
        defns.append(funcDefn)
    return (decls, defns)
  
  def getArrayMethods(self, type):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Array
    import ASE.Compiler.SIDL.Comment
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    arrayMethods = []
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      exc = [self.SIDL.getBaseException(self.SIDL.getASEException(type))]
    else:
      exc = []
    intArrayType = ASE.Compiler.SIDL.Array.Array()
    intArrayType.setType(self.SIDL.typeMap['int'])
    intReturn = ASE.Compiler.SIDL.Parameter.Parameter()
    intReturn.setIdentifier('_return_value')
    intReturn.setType(self.SIDL.typeMap['int'])
    intReturn.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    arrayType = ASE.Compiler.SIDL.Array.Array()
    arrayType.setType(type)
    arrayParam = ASE.Compiler.SIDL.Parameter.Parameter()
    arrayParam.setIdentifier('array')
    arrayParam.setType(arrayType)
    arrayParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    arrayReturn = ASE.Compiler.SIDL.Parameter.Parameter()
    arrayReturn.setIdentifier('_return_value')
    arrayReturn.setType(arrayType)
    arrayReturn.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    elemParam = ASE.Compiler.SIDL.Parameter.Parameter()
    elemParam.setIdentifier('value')
    elemParam.setType(type)
    elemParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    elemReturn = ASE.Compiler.SIDL.Parameter.Parameter()
    elemReturn.setIdentifier('_return_value')
    elemReturn.setType(type)
    elemReturn.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    dimenParam = ASE.Compiler.SIDL.Parameter.Parameter()
    dimenParam.setIdentifier('dimen')
    dimenParam.setType(self.SIDL.typeMap['int'])
    dimenParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    lowerParam = ASE.Compiler.SIDL.Parameter.Parameter()
    lowerParam.setIdentifier('lower')
    lowerParam.setType(intArrayType)
    lowerParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    upperParam = ASE.Compiler.SIDL.Parameter.Parameter()
    upperParam.setIdentifier('upper')
    upperParam.setType(intArrayType)
    upperParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    strideParam = ASE.Compiler.SIDL.Parameter.Parameter()
    strideParam.setIdentifier('stride')
    strideParam.setType(intArrayType)
    strideParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    firstParam = ASE.Compiler.SIDL.Parameter.Parameter()
    firstParam.setIdentifier('first')
    firstParam.setType(self.SIDL.typeMap['opaque'])
    firstParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    indicesParam = ASE.Compiler.SIDL.Parameter.Parameter()
    indicesParam.setIdentifier('indices')
    indicesParam.setType(intArrayType)
    indicesParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    indexParam1 = ASE.Compiler.SIDL.Parameter.Parameter()
    indexParam1.setIdentifier('i1')
    indexParam1.setType(self.SIDL.typeMap['int'])
    indexParam1.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    indexParam2 = ASE.Compiler.SIDL.Parameter.Parameter()
    indexParam2.setIdentifier('i2')
    indexParam2.setType(self.SIDL.typeMap['int'])
    indexParam2.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    indexParam3 = ASE.Compiler.SIDL.Parameter.Parameter()
    indexParam3.setIdentifier('i3')
    indexParam3.setType(self.SIDL.typeMap['int'])
    indexParam3.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    indexParam4 = ASE.Compiler.SIDL.Parameter.Parameter()
    indexParam4.setIdentifier('i4')
    indexParam4.setType(self.SIDL.typeMap['int'])
    indexParam4.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Create a dense array of the given dimension with specified index bounds\nThis array owns and manages its data\nAll object pointers are initialized to NULL')
    createArray = ASE.Compiler.SIDL.Method.Method()
    createArray.setIdentifier('createArray')
    createArray.setReturnParameter(arrayReturn)
    createArray.setChildren([dimenParam, lowerParam, upperParam])
    createArray.setExceptions(exc)
    createArray.setStatic(1)
    createArray.setComments([comment])
    arrayMethods.append(createArray)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Create an array using user-provided memory')
    borrowArray = ASE.Compiler.SIDL.Method.Method()
    borrowArray.setIdentifier('borrowArray')
    borrowArray.setReturnParameter(arrayReturn)
    borrowArray.setChildren([dimenParam, lowerParam, upperParam, strideParam, firstParam])
    borrowArray.setExceptions(exc)
    borrowArray.setStatic(1)
    borrowArray.setComments([comment])
    arrayMethods.append(borrowArray)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Destroy the array (NULL arrays are ignored)')
    destroyArray = ASE.Compiler.SIDL.Method.Method()
    destroyArray.setIdentifier('destroyArray')
    destroyArray.setChildren([arrayParam])
    destroyArray.setExceptions(exc)
    destroyArray.setStatic(1)
    destroyArray.setComments([comment])
    arrayMethods.append(destroyArray)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Return the number of dimensions in the array\nIf the array pointer is NULL, zero is returned')
    getDimen = ASE.Compiler.SIDL.Method.Method()
    getDimen.setIdentifier('getDimen')
    getDimen.setReturnParameter(intReturn)
    getDimen.setChildren([arrayParam])
    getDimen.setExceptions(exc)
    getDimen.setStatic(1)
    getDimen.setComments([comment])
    arrayMethods.append(getDimen)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Return the lower bound on dimension ind\nIf the array pointer is NULL or dimen is not a valid dimension, zero is returned')
    getLower = ASE.Compiler.SIDL.Method.Method()
    getLower.setIdentifier('getLower')
    getLower.setReturnParameter(intReturn)
    getLower.setChildren([arrayParam, dimenParam])
    getLower.setExceptions(exc)
    getLower.setStatic(1)
    getLower.setComments([comment])
    arrayMethods.append(getLower)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Return the upper bound on dimension ind\nIf the array pointer is NULL or dimen is not a valid dimension, -1 is returned')
    getUpper = ASE.Compiler.SIDL.Method.Method()
    getUpper.setIdentifier('getUpper')
    getUpper.setReturnParameter(intReturn)
    getUpper.setChildren([arrayParam, dimenParam])
    getUpper.setExceptions(exc)
    getUpper.setStatic(1)
    getUpper.setComments([comment])
    arrayMethods.append(getUpper)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Get an element of a multi-dimensional array\nThis will use the indices provided up to the actual dimension of the array, ignoring extra indices\nIf the return value is non-NULL, the client owns one reference to the object/interface\nThe client must decrement the reference count when done with the reference')
    getElement = ASE.Compiler.SIDL.Method.Method()
    getElement.setIdentifier('getElement')
    getElement.setReturnParameter(elemReturn)
    getElement.setChildren([arrayParam, indicesParam])
    getElement.setExceptions(exc)
    getElement.setStatic(1)
    getElement.setComments([comment])
    arrayMethods.append(getElement)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Get an element of a multi-dimensional array\nThis will use the indices provided up to the actual dimension of the array, ignoring extra indices\nIf the return value is non-NULL, the client owns one reference to the object/interface\nThe client must decrement the reference count when done with the reference')
    getElement4 = ASE.Compiler.SIDL.Method.Method()
    getElement4.setIdentifier('getElement4')
    getElement4.setReturnParameter(elemReturn)
    getElement4.setChildren([arrayParam, indexParam1, indexParam2, indexParam3, indexParam4])
    getElement4.setExceptions(exc)
    getElement4.setStatic(1)
    getElement4.setComments([comment])
    arrayMethods.append(getElement4)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Set an element of a multi-dimensional array\nThis will use the indices provided up to the actual dimension of the array, ignoring extra indices\nIf the incoming value is non-NULL, this function will increment the reference code of the object/interface\nIf it is overwriting a non-NULL pointer, the reference count of the object/interface being overwritten will be decremented')
    setElement = ASE.Compiler.SIDL.Method.Method()
    setElement.setIdentifier('setElement')
    setElement.setChildren([arrayParam, indicesParam, elemParam])
    setElement.setExceptions(exc)
    setElement.setStatic(1)
    setElement.setComments([comment])
    arrayMethods.append(setElement)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Set an element of a multi-dimensional array\nThis will use the indices provided up to the actual dimension of the array, ignoring extra indices\nIf the incoming value is non-NULL, this function will increment the reference code of the object/interface\nIf it is overwriting a non-NULL pointer, the reference count of the object/interface being overwritten will be decremented')
    setElement4 = ASE.Compiler.SIDL.Method.Method()
    setElement4.setIdentifier('setElement4')
    setElement4.setChildren([arrayParam, indexParam1, indexParam2, indexParam3, indexParam4, elemParam])
    setElement4.setExceptions(exc)
    setElement4.setStatic(1)
    setElement4.setComments([comment])
    arrayMethods.append(setElement4)
    return arrayMethods
  
  def getIterator(self, type):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Struct
    if (type.getBaseType() and type.getIdentifier() == 'void'):
      return ([], [])
    decls = []
    iteratorStruct = ASE.Compiler.Cxx.Struct.Struct()
    iteratorStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__iterator')
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      if not (type.getFullIdentifier() == 'ASE.BaseInterface'):
        decls.append(self.IOR.getGuardedInclude(self.SIDL.getBaseClass(type)))
        baseType = self.SIDL.getBaseClass(type)
        baseDecl = ASE.Compiler.Cxx.Declarator.Declarator()
        baseDecl.setIdentifier(baseType.getFullIdentifier().replace('.', '_').lower())
        baseDecl.setType(self.IOR.getType(baseType).getChildren()[0])
        iteratorStruct.addChildren([self.Cxx.getDecl(baseDecl)])
    iterDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    iterDecl.setIdentifier('iterator')
    iterDecl.setType(self.IOR.typeMap['void pointer'])
    iteratorStruct.addChildren([self.Cxx.getDecl(iterDecl)])
    wrapperDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    wrapperDecl.setIdentifier('objWrapper')
    wrapperDecl.setType(self.IOR.typeMap['void pointer'])
    iteratorStruct.addChildren([self.Cxx.getDecl(wrapperDecl)])
    iteratorDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    iteratorDecl.setType(iteratorStruct)
    decls.append(self.Cxx.getDecl(iteratorDecl, 'The IOR iterator structure for this type'))
    defns = []
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      defns.append(self.getCastIterator(type))
    if not ((type.getBaseType() and type.getIdentifier() == 'void')):
      for method in [self.getCreateIterator(type), self.getDestroyIterator(type)]:
        import ASE.Compiler.Cxx.CompoundStatement
        import ASE.Compiler.Cxx.Function
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        funcDefn = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, method).getChildren()[0])
        funcDefn.setBody(compound)
        funcDefn.setStatic(1)
        getattr(self, 'm_' + method.getIdentifier())(type, method, compound)
        defns.append(funcDefn)
    return (decls, defns)
  
  def getCastIterator(self, type):
    import ASE.Compiler.SIDL.Iterator
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    iterType = ASE.Compiler.SIDL.Iterator.Iterator()
    iterType.setType(type)
    selfVar = self.Cxx.getVar('self')
    nameVar = self.Cxx.getVar('name')
    objVar = self.Cxx.getVar('obj')
    castVar = self.Cxx.getVar('cast')
    baseType = self.SIDL.getBaseClass(type)
    objInit = self.Cxx.getInitializer(None, self.IOR.getTrueIOR(self.Cxx.getGroup(self.Cxx.castToType(selfVar, self.IOR.getType(self.SIDL.getBaseInterface(type))))))
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.addDeclarations([self.Cxx.getDeclaration(objVar, self.IOR.getType(baseType), objInit, 'We are guaranteed here that the first member in any IOR is a ASE.BaseInterface object')])
    compound.addDeclarations([self.Cxx.getDeclaration(castVar, self.IOR.typeMap['opaque'])])
    if not (type.getFullIdentifier() == 'ASE.BaseInterface'):
      foundEntry = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      foundEntry.setChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(castVar, selfVar))])
      checkEntryType = ASE.Compiler.Cxx.If.If()
      checkEntryType.setBranch(self.Cxx.getNegation(self.Cxx.getFunctionCall('strcmp', [nameVar, self.Cxx.getString(type.getFullIdentifier())])))
      checkEntryType.setChildren([foundEntry, self.getCastCheck(baseType, objVar)])
      compound.addChildren([checkEntryType])
    else:
      compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(castVar, objVar))])
    compound.addChildren([self.Cxx.getReturn(castVar)])
    castFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, self.getCast(iterType)).getChildren()[0])
    castFunc.setBody(compound)
    castFunc.setStatic(1)
    return castFunc
  
  def getCreateIterator(self, type):
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Iterator
    arrayMethods = []
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      exc = [self.SIDL.getBaseException(self.SIDL.getASEException(type))]
    else:
      exc = []
    createIterator = ASE.Compiler.SIDL.Method.Method()
    createIterator.setIdentifier('createIterator')
    iterType = ASE.Compiler.SIDL.Iterator.Iterator()
    iterType.setType(type)
    createIterator.setReturnParameter(self.SIDL.getParameter('_return_value', iterType, 'return'))
    createIterator.setChildren([self.SIDL.getParameter('iterator', self.SIDL.typeMap['opaque']), self.SIDL.getParameter('iterNext', self.SIDL.typeMap['opaque'])])
    createIterator.setExceptions(exc)
    createIterator.setStatic(1)
    createIterator.setComments([self.SIDL.getComment('Create an iterator')])
    return createIterator
  
  def getDestroyIterator(self, type):
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Iterator
    arrayMethods = []
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      exc = [self.SIDL.getBaseException(self.SIDL.getASEException(type))]
    else:
      exc = []
    destroyIterator = ASE.Compiler.SIDL.Method.Method()
    destroyIterator.setIdentifier('destroyIterator')
    iterType = ASE.Compiler.SIDL.Iterator.Iterator()
    iterType.setType(type)
    destroyIterator.setChildren([self.SIDL.getParameter('self', iterType)])
    destroyIterator.setExceptions(exc)
    destroyIterator.setStatic(1)
    destroyIterator.setComments([self.SIDL.getComment('Destroy an iterator')])
    return destroyIterator
  
  def m_createIterator(self, interface, method, body):
    import ASE.Compiler.SIDL.Iterator
    if not (interface.isInstanceOf('ASE.Compiler.SIDL.Interface')):
      return 
    iterVar = self.Cxx.getVar('iterator')
    iterNextVar = self.Cxx.getVar('iterNext')
    excVar = self.Cxx.getVar('_exception')
    selfVar = self.Cxx.getVar('self')
    castIteratorVar = self.Cxx.getVar('castIterator')
    baseType = self.SIDL.getBaseClass(interface)
    iterType = ASE.Compiler.SIDL.Iterator.Iterator()
    iterType.setType(interface)
    iterType = self.IOR.getType(iterType)
    baseClass = self.Cxx.getStructRef(selfVar, baseType.getFullIdentifier().replace('.', '_').lower())
    body.addDeclarations([self.Cxx.getDeclaration(selfVar, iterType, self.Cxx.getInitializer(None, self.Cxx.castToType(self.Cxx.getFunctionCall('malloc', [self.Cxx.getSizeof(iterType.getChildren()[0])]), iterType)))])
    if not (interface.getFullIdentifier() == 'ASE.BaseInterface'):
      parentVar = self.Cxx.getVar('parent')
      baseClassEpvVar = self.Cxx.getVar('ASE_BaseClass_epv')
      baseInterfaceEpvVar = self.Cxx.getVar('ASE_BaseInterface_epv')
      body.addDeclarations([self.Cxx.getDeclaration(parentVar, self.IOR.getType(baseType), self.Cxx.getInitializer(None, self.Cxx.getAddress(baseClass)))])
      baseClassEpvType = self.IOR.getEpvType(baseType)
      baseClassInit = self.Cxx.castToType(self.Cxx.getFunctionCall('malloc', [self.Cxx.getSizeof(baseClassEpvType.getChildren()[0])]), baseClassEpvType)
      body.addDeclarations([self.Cxx.getDeclaration(baseClassEpvVar, baseClassEpvType, self.Cxx.getInitializer(None, baseClassInit))])
      baseInterfaceEpvType = self.IOR.getEpvType(self.SIDL.getBaseInterface(interface))
      baseInterfaceInit = self.Cxx.castToType(self.Cxx.getFunctionCall('malloc', [self.Cxx.getSizeof(baseInterfaceEpvType.getChildren()[0])]), baseInterfaceEpvType)
      body.addDeclarations([self.Cxx.getDeclaration(baseInterfaceEpvVar, baseInterfaceEpvType, self.Cxx.getInitializer(None, baseInterfaceInit))])
      initFunc = self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getFunctionCall('ASE_BaseClass__externals'), 'initObject'))
      body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall(self.Cxx.getGroup(self.Cxx.getIndirection(initFunc)), [parentVar, selfVar, excVar]))])
      baseClassEpv = self.Cxx.getStructRef(baseClass, 'd_epv', 0)
      for method in baseType.getMethods():
        name = method.getIdentifier()
        body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(baseClassEpvVar, name), self.Cxx.getStructRef(baseClassEpv, name)))])
      baseInterfaceEpv = self.Cxx.getStructRef(self.Cxx.getStructRef(baseClass, 'ase_baseinterface', 0), 'd_epv', 0)
      for method in self.SIDL.getBaseInterface(interface).getMethods():
        name = method.getIdentifier()
        body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(baseInterfaceEpvVar, name), self.Cxx.getStructRef(baseInterfaceEpv, name)))])
      for parentEpv in [baseClassEpvVar, baseInterfaceEpvVar]:
        body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(parentEpv, 'castObject'), castIteratorVar))])
        body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(parentEpv, 'iterNext'), iterNextVar))])
      body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(baseInterfaceEpv, baseInterfaceEpvVar))])
      body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(baseClassEpv, baseClassEpvVar))])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(selfVar, 'iterator'), iterVar))])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(selfVar, 'objWrapper'), self.Cxx.getNull()))])
    body.addChildren([self.Cxx.getReturn(selfVar)])
    return 
  
  def m_destroyIterator(self, interface, method, body):
    import ASE.Compiler.SIDL.Iterator
    if (not (interface.isInstanceOf('ASE.Compiler.SIDL.Interface')) or interface.getFullIdentifier() == 'ASE.BaseInterface'):
      return 
    selfVar = self.Cxx.getVar('self')
    excVar = self.Cxx.getVar('_exception')
    parentVar = self.Cxx.getVar('parent')
    iterType = ASE.Compiler.SIDL.Iterator.Iterator()
    iterType.setType(interface)
    iterType = self.IOR.getType(iterType)
    baseType = self.SIDL.getBaseClass(interface)
    baseClass = self.Cxx.getStructRef(selfVar, baseType.getFullIdentifier().replace('.', '_').lower())
    body.addDeclarations([self.Cxx.getDeclaration(parentVar, self.IOR.getType(baseType), self.Cxx.getInitializer(None, self.Cxx.getAddress(baseClass)))])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(selfVar, 'iterator'), self.Cxx.getNull()))])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(selfVar, 'objWrapper'), self.Cxx.getNull()))])
    baseClass = self.Cxx.getStructRef(selfVar, 'ase_baseclass')
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('free', [self.Cxx.getStructRef(self.Cxx.getStructRef(baseClass, 'ase_baseinterface', 0), 'd_epv', 0)]))])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('free', [self.Cxx.getStructRef(baseClass, 'd_epv', 0)]))])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(baseClass, 'd_epv', 0), self.Cxx.getStructRef(baseClass, 'd_oldEpv', 0)))])
    finiFunc = self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getFunctionCall('ASE_BaseClass__externals'), 'finiObject'))
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall(self.Cxx.getGroup(self.Cxx.getIndirection(finiFunc)), [parentVar, excVar]))])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('memset', [selfVar, 0, self.Cxx.getSizeof(iterType.getChildren()[0])]))])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('free', [selfVar]))])
    return 
  
  def m_isSame(self, interface, method, body):
    import ASE.Compiler.Cxx.Equality
    import ASE.Compiler.Cxx.Return
    objEqual = ASE.Compiler.Cxx.Equality.Equality()
    objEqual.setChildren([self.IOR.getTrueIOR(self.Cxx.getVar('self')), self.IOR.getTrueIOR(self.Cxx.getVar(method.getChildren()[0].getIdentifier()))])
    returnComp = ASE.Compiler.Cxx.Return.Return()
    returnComp.setChildren([objEqual])
    body.addChildren([returnComp])
    return 
  
  def m_isInstanceOf(self, interface, method, body):
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    selfEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    selfEpv.setChildren([self.Cxx.getVar('self')])
    selfEpv.setMember('d_epv')
    selfEpv.setPointer(1)
    castFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    castFunc.setChildren([selfEpv])
    castFunc.setMember('castObject')
    castFunc.setPointer(1)
    castFuncInd = ASE.Compiler.Cxx.Indirection.Indirection()
    castFuncInd.setChildren([self.Cxx.getGroup(castFunc)])
    castCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castCall.setChildren([self.Cxx.getGroup(castFuncInd)])
    castCall.setArguments([self.Cxx.getVar('self'), self.Cxx.getVar(method.getChildren()[0].getIdentifier()), self.Cxx.getVar('_exception')])
    checkCast = ASE.Compiler.Cxx.If.If()
    checkCast.setBranch(castCall)
    checkCast.setChildren([self.Cxx.getInteger(1), self.Cxx.getInteger(0)])
    checkCast.setTernary(1)
    returnCast = ASE.Compiler.Cxx.Return.Return()
    returnCast.setChildren([checkCast])
    body.addChildren([returnCast])
    return 
  
  def m_getClassName(self, interface, method, body):
    import ASE.Compiler.Cxx.Return
    returnName = ASE.Compiler.Cxx.Return.Return()
    returnName.setChildren([self.Cxx.getFunctionCall('strdup', [self.Cxx.getString(interface.getFullIdentifier())])])
    body.addChildren([returnName])
    return 
  
  def m_getSuper(self, interface, method, body):
    import ASE.Compiler.Cxx.Return
    excVar = self.Cxx.getVar('_exception')
    extFuncVar = self.Cxx.getVar('_extFunc')
    superSelfVar = self.Cxx.getVar('superSelf')
    superSelfInit = self.Cxx.getInitializer(None, self.Cxx.getFunctionCall(self.Cxx.getStructRef(extFuncVar, 'createObject'), [excVar]))
    staticEPVCall = self.Cxx.getFunctionCall(self.Cxx.getStructRef(self.Cxx.getFunctionCall('ASE_Loader__externals'), 'getStaticEPV'), [excVar])
    classMethodsCall = self.Cxx.getFunctionCall(self.Cxx.getStructRef(staticEPVCall, 'getClassMethodTable'), [self.Cxx.getString(interface.getFullIdentifier()), excVar])
    extFuncType = ASE.Compiler.Cxx.Function.Function()
    extFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
    extFuncType.setType(self.IOR.getExternalType(interface))
    extFuncCast = ASE.Compiler.Cxx.Cast.Cast()
    extFuncCast.setTypeName(extFuncType)
    extFuncCast.setChildren([classMethodsCall])
    extFuncInit = self.Cxx.getInitializer(None, self.Cxx.getFunctionCall(self.Cxx.getGroup(extFuncCast)))
    body.addDeclarations([self.Cxx.getDeclaration(extFuncVar, self.IOR.getExternalType(interface), extFuncInit), self.Cxx.getDeclaration(superSelfVar, self.IOR.getType(interface), superSelfInit)])
    selfEpv = self.Cxx.getStructRef(superSelfVar, 'd_epv')
    setEpv = self.Cxx.getAssignment(selfEpv, self.Cxx.getStructRef(superSelfVar, 'd_oldEpv'))
    body.addChildren([self.Cxx.getExpStmt(setEpv)])
    returnCast = ASE.Compiler.Cxx.Return.Return()
    returnCast.setChildren([self.Cxx.getFunctionCall(self.Cxx.getStructRef(selfEpv, 'castObject'), [superSelfVar, self.Cxx.getString('ASE.BaseInterface'), self.Cxx.getVar('_exception')])])
    body.addChildren([returnCast])
    return 
  
  def m_createClass(self, interface, method, body):
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.StructReference
    extFuncVar = self.Cxx.getVar('_extFunc')
    excVar = self.Cxx.getVar('_exception')
    nameVar = self.Cxx.getVar(method.getChildren()[0].getIdentifier())
    genericExtStruct = ASE.Compiler.Cxx.Struct.Struct()
    genericExtStruct.setIdentifier('generic__external')
    genericExtStruct.setChildren([self.IOR.getFunctionDeclaration(interface, self.getConstructor(interface), 1)])
    genericExtDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    genericExtDecl.setType(genericExtStruct)
    body.addDeclarations([self.Cxx.getDecl(genericExtDecl)])
    genericExtStruct = ASE.Compiler.Cxx.Struct.Struct()
    genericExtStruct.setIdentifier('generic__external')
    genericExt = ASE.Compiler.Cxx.Pointer.Pointer()
    genericExt.setChildren([genericExtStruct])
    sidlExtCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    sidlExtCall.setChildren([self.Cxx.getVar('ASE_Loader__externals')])
    staticEPV = ASE.Compiler.Cxx.StructReference.StructReference()
    staticEPV.setChildren([sidlExtCall])
    staticEPV.setMember('getStaticEPV')
    staticEPV.setPointer(1)
    staticEPVCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    staticEPVCall.setChildren([staticEPV])
    staticEPVCall.setArguments([excVar])
    classMethods = ASE.Compiler.Cxx.StructReference.StructReference()
    classMethods.setChildren([staticEPVCall])
    classMethods.setMember('getClassMethodTable')
    classMethods.setPointer(1)
    classMethodsCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    classMethodsCall.setChildren([classMethods])
    classMethodsCall.setArguments([nameVar, excVar])
    extFuncType = ASE.Compiler.Cxx.Function.Function()
    extFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
    extFuncType.setType(genericExt)
    extFunc = ASE.Compiler.Cxx.Cast.Cast()
    extFunc.setTypeName(extFuncType)
    extFunc.setChildren([classMethodsCall])
    extFuncInit = ASE.Compiler.Cxx.Initializer.Initializer()
    extFuncInit.setChildren([extFunc])
    extFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
    extFuncPtr.setIdentifier(extFuncVar.getIdentifier())
    extFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
    extFuncDecl = ASE.Compiler.Cxx.Function.Function()
    extFuncDecl.setChildren([self.Cxx.getDeclGroup(extFuncPtr)])
    extFuncDecl.setType(genericExt)
    extFuncDecl.setInitializer(extFuncInit)
    body.addDeclarations([self.Cxx.getDecl(extFuncDecl)])
    extFuncInd = ASE.Compiler.Cxx.Indirection.Indirection()
    extFuncInd.setChildren([extFuncVar])
    extFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    extFuncCall.setChildren([self.Cxx.getGroup(extFuncInd)])
    createFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    createFuncPtr.setChildren([extFuncCall])
    createFuncPtr.setMember('createObject')
    createFuncPtr.setPointer(1)
    createFunc = ASE.Compiler.Cxx.Indirection.Indirection()
    createFunc.setChildren([self.Cxx.getGroup(createFuncPtr)])
    createFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    createFuncCall.setChildren([self.Cxx.getGroup(createFunc)])
    createFuncCall.setArguments([excVar])
    returnClass = ASE.Compiler.Cxx.Return.Return()
    returnClass.setChildren([self.Cxx.castToType(createFuncCall, self.IOR.getType(self.SIDL.getAncestor(interface, 'ASE.BaseClass')))])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([returnClass])
    checkIor = ASE.Compiler.Cxx.If.If()
    checkIor.setBranch(extFuncVar)
    checkIor.setChildren([compound])
    returnNull = ASE.Compiler.Cxx.Return.Return()
    returnNull.setChildren([self.Cxx.getNull()])
    body.addChildren([checkIor, returnNull])
    return 
  
  def isIORMethod(self, method):
    from ASE.Compiler.CodePurpose import IMPLEMENTATION, IOR, SKELETON
    purpose = method.getPurpose()
    return (not ((SKELETON in purpose or IMPLEMENTATION in purpose)) and IOR in purpose)
  
  def getIORMethods(self, interface):
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      import ASE.Compiler.SIDL.Class
      klass = ASE.Compiler.SIDL.Class.Class(interface)
      implMethods = klass.getImplementedMethods()
      return [method for method in klass.getMethods() if (self.isIORMethod(method) and (method in implMethods or not (method.getVirtual())))]
    return []
  
  def getIORImplementations(self, interface):
    defns = []
    for method in self.getTemplateMethods(self.getIORMethods(interface)):
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      getattr(self, 'm_' + method.getIdentifier())(interface, method, compound)
      funcDefn = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(interface, method).getChildren()[0])
      funcDefn.setBody(compound)
      funcDefn.setStatic(1)
      defns.append(funcDefn)
    return ([], defns)
  
  def getExternal(self, type):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Struct
    if (type.getBaseType() and type.getIdentifier() == 'void'):
      return ([], [])
    comment = 'The entry points into the DLL containing this type'
    externalStruct = ASE.Compiler.Cxx.Struct.Struct()
    externalStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__external')
    if type.isInstanceOf('ASE.Compiler.SIDL.Class'):
      externalStruct.addChildren([self.IOR.getFunctionDeclaration(type, self.getConstructor(type), 1)])
      externalStruct.addChildren([self.IOR.getFunctionDeclaration(type, self.getInit(type), 1)])
      externalStruct.addChildren([self.IOR.getFunctionDeclaration(type, self.getFini(type), 1)])
      if self.hasStaticMethod(type):
        externalStruct.addChildren([self.IOR.getFunctionDeclaration(type, self.getSEPVFunc(type), 1)])
    externalStruct.addChildren([self.IOR.getFunctionDeclaration(type, method, 1) for method in self.getArrayMethods(type)])
    externalStruct.addChildren([self.IOR.getFunctionDeclaration(type, method, 1) for method in [self.getCreateIterator(type), self.getDestroyIterator(type)]])
    externalDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    externalDecl.setType(externalStruct)
    externalStruct = ASE.Compiler.Cxx.Struct.Struct()
    externalStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__external')
    externalInit = ASE.Compiler.Cxx.Initializer.Initializer()
    if type.isInstanceOf('ASE.Compiler.SIDL.Class'):
      externalInit.addChildren([self.Cxx.getVar(self.getConstructor(type).getIdentifier())])
      externalInit.addChildren([self.Cxx.getVar(self.getInit(type).getIdentifier())])
      externalInit.addChildren([self.Cxx.getVar(self.getFini(type).getIdentifier())])
      if self.hasStaticMethod(type):
        externalInit.addChildren([self.Cxx.getVar(self.getSEPVFunc(type).getIdentifier())])
    externalInit.addChildren([self.Cxx.getVar(method.getIdentifier()) for method in self.getArrayMethods(type)])
    externalInit.addChildren([self.Cxx.getVar(method.getIdentifier()) for method in [self.getCreateIterator(type), self.getDestroyIterator(type)]])
    externalDefn = ASE.Compiler.Cxx.Declarator.Declarator()
    externalDefn.setIdentifier('externalEntryPoints')
    externalDefn.setType(externalStruct)
    externalDefn.setStatic(1)
    externalDefn.setInitializer(externalInit)
    return ([self.Cxx.getDecl(externalDecl, comment)], [self.Cxx.getDecl(externalDefn, comment)])
  
  def getExternals(self, type):
    import ASE.Compiler.Cxx.Address
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.Struct
    if (type.getBaseType() and type.getIdentifier() == 'void'):
      return ([], [])
    externalStruct = ASE.Compiler.Cxx.Struct.Struct()
    externalStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__external')
    externalStruct.setConst(1)
    externalType = ASE.Compiler.Cxx.Pointer.Pointer()
    externalType.setChildren([externalStruct])
    externalName = ASE.Compiler.Cxx.Declarator.Declarator()
    externalName.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__externals')
    externalDecl = ASE.Compiler.Cxx.Function.Function()
    externalDecl.setChildren([externalName])
    externalDecl.setType(externalType)
    extAddr = ASE.Compiler.Cxx.Address.Address()
    extAddr.setChildren([self.Cxx.getVar('externalEntryPoints')])
    returnExt = ASE.Compiler.Cxx.Return.Return()
    returnExt.setChildren([extAddr])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([returnExt])
    externalDefn = ASE.Compiler.Cxx.Function.Function()
    externalDefn.setChildren([externalName])
    externalDefn.setType(externalType)
    externalDefn.setBody(compound)
    return ([self.Cxx.getDecl(externalDecl, 'This function, called upon loading the DLL, returns a pointer to the static structure of DLL entry points')], [externalDefn])
  
  def getHeaderFile(self, type, decls):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.Define
    import ASE.Compiler.Cxx.Header
    import ASE.Compiler.Cxx.Include
    import ASE.Compiler.Cxx.PreprocessorConditional
    guard = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    guard.setIdentifier('ifndef')
    guard.setBranchExpression('included_' + type.getFullIdentifier().replace('.', '_'))
    guard.setValue(1)
    define = ASE.Compiler.Cxx.Define.Define()
    define.setIdentifier(guard.getBranchExpression())
    externBegin = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    externBegin.setIdentifier('ifdef')
    externBegin.setBranchExpression('__cplusplus')
    externBegin.setValue(0)
    externBegin.setInactiveCode('extern "C" {')
    externEnd = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    externEnd.setIdentifier('ifdef')
    externEnd.setBranchExpression('__cplusplus')
    externEnd.setValue(0)
    externEnd.setInactiveCode('}')
    typeInclude = ASE.Compiler.Cxx.Include.Include()
    typeInclude.setIdentifier('<sys/types.h>')
    guard.addChildren([define, externBegin, typeInclude])
    guard.addChildren(self.getForwardDeclarations(type)[0])
    for dependency in type.getDependencies():
      guard.addChildren(self.getForwardDeclarations(dependency, noExceptions = 1)[0])
      if (ASE.Compiler.SIDL.Type.Type(dependency).getBaseType() or dependency.isInstanceOf('ASE.Compiler.SIDL.Enumeration')):
        guard.addChildren([self.IOR.getGuardedInclude(dependency)])
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      import ASE.Compiler.SIDL.Interface
      interface = ASE.Compiler.SIDL.Interface.Interface(type)
      if len(interface.getParents()):
        guard.addChildren([self.IOR.getGuardedInclude(parent) for parent in interface.getParents()])
      if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
        import ASE.Compiler.SIDL.Class
        klass = ASE.Compiler.SIDL.Class.Class(interface)
        if len(klass.getInterfaces()):
          guard.addChildren([self.IOR.getGuardedInclude(inf) for inf in klass.getInterfaces()])
    guard.addChildren(decls)
    guard.addChildren([externEnd])
    header = ASE.Compiler.Cxx.Header.Header()
    header.setFilename(self.IOR.getFileBasename(type) + '.h')
    header.setChildren([guard])
    header.setPurpose(ASE.Compiler.CodePurpose.IOR)
    return header
  
  def getSourceFile(self, interface, decls):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.Include
    import ASE.Compiler.Cxx.Source
    stdInclude = ASE.Compiler.Cxx.Include.Include()
    stdInclude.setIdentifier('<stdlib.h>')
    stdInclude.setComments([self.Cxx.getComment('Needed for NULL')])
    strInclude = ASE.Compiler.Cxx.Include.Include()
    strInclude.setIdentifier('<string.h>')
    strInclude.setComments([self.Cxx.getComment('Needed for implementation of getClassName()')])
    source = ASE.Compiler.Cxx.Source.Source()
    source.setFilename(self.IOR.getFileBasename(interface) + '.c')
    source.addChildren([stdInclude, strInclude, self.IOR.getGuardedInclude(interface)])
    source.addChildren(decls)
    source.setPurpose(ASE.Compiler.CodePurpose.IOR)
    return source
  
  def createBaseTypes(self):
    for sidlType in self.SIDL.typeMap.values():
      if not (sidlType.getIdentifier() == 'function'):
        self.visitInterface(sidlType)
    return 
  
  def copyType(self, type):
    if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      newType = ASE.Compiler.SIDL.Array.Array()
      newType.setIdentifier(type.getIdentifier())
      newType.setAttributes(type.getAttributes())
      newType.setLineNumber(type.getLineNumber())
      newType.setComments(type.getComments())
      newType.setChildren([child for child in type.getChildren()])
      newType.setBaseType(type.getBaseType())
      newType.setType(self.copyType(ASE.Compiler.SIDL.Array.Array(type).getType()))
      return newType
    return type
  
  def copyParameter(self, parameter):
    '''
    This will be replaced by parameter.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Parameter
    newParameter = ASE.Compiler.SIDL.Parameter.Parameter()
    newParameter.setIdentifier(parameter.getIdentifier())
    newParameter.setAttributes(parameter.getAttributes())
    newParameter.setParent(parameter.getParent())
    newParameter.setChildren(parameter.getChildren())
    newParameter.setLineNumber(parameter.getLineNumber())
    newParameter.setComments(parameter.getComments())
    newParameter.setDependencies(parameter.getDependencies())
    newParameter.setType(self.copyType(parameter.getType()))
    newParameter.setAccessMode(parameter.getAccessMode())
    newParameter.setRequired(parameter.getRequired())
    return newParameter
  
  def copyMethod(self, method):
    '''
    This will be replaced by method.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    newMethod = ASE.Compiler.SIDL.Method.Method()
    newMethod.setIdentifier(method.getIdentifier())
    newMethod.setAttributes(method.getAttributes())
    newMethod.setParent(method.getParent())
    newMethod.setChildren([self.copyParameter(ASE.Compiler.SIDL.Parameter.Parameter(child)) for child in method.getChildren()])
    newMethod.setLineNumber(method.getLineNumber())
    newMethod.setComments(method.getComments())
    newMethod.setDependencies(method.getDependencies())
    newMethod.setTemplateIdentifiers(method.getTemplateIdentifiers())
    if not (method.getReturnParameter() is None):
      newMethod.setReturnParameter(self.copyParameter(method.getReturnParameter()))
    newMethod.setExceptions(method.getExceptions())
    newMethod.setStatic(method.getStatic())
    newMethod.setInternal(method.getInternal())
    newMethod.setVirtual(method.getVirtual())
    newMethod.setPurpose(method.getPurpose())
    return newMethod
  
  def checkTemplateParameter(self, name, parameter):
    if not (parameter is None):
      type = parameter.getType()
      if (type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name):
        return True
      elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
        import ASE.Compiler.SIDL.Array
        type = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
        if (type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name):
          return True
    return False
  
  def insertType(self, parameter, type):
    if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      self.insertType(ASE.Compiler.SIDL.Array.Array(parameter.getType()), type)
    else:
      parameter.setType(type)
    return 
  
  def insertTypes(self, method, universe, type):
    import ASE.Compiler.SIDL.Parameter
    name = universe.getIdentifier()
    newMethod = self.copyMethod(method)
    newMethod.setIdentifier(newMethod.getIdentifier() + '_' + type.getFullIdentifier())
    if self.checkTemplateParameter(name, method.getReturnParameter()):
      self.insertType(newMethod.getReturnParameter(), type)
    for (child, newChild) in zip(method.getChildren(), newMethod.getChildren()):
      if self.checkTemplateParameter(name, ASE.Compiler.SIDL.Parameter.Parameter(child)):
        self.insertType(ASE.Compiler.SIDL.Parameter.Parameter(newChild), type)
    return newMethod
  
  def template(self, method, name):
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.SIDL.TypeUniverse
    universe = None
    for parameter in [method.getReturnParameter()] + [ASE.Compiler.SIDL.Parameter.Parameter(child) for child in method.getChildren()]:
      if self.checkTemplateParameter(name, parameter):
        type = parameter.getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(ASE.Compiler.SIDL.Array.Array(type).getType())
        else:
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(type)
        break
    if universe is None:
      raise RuntimeError('Could not find type universe for template ' + name + ' in method ' + method.getFullIdentifier())
    self.templateMethods[method] = [self.insertTypes(method, universe, type) for type in universe.getTypes()]
    return 
  
  def createTemplateMethod(self, method):
    if len(method.getTemplateIdentifiers()):
      if len(method.getTemplateIdentifiers()) > 1:
        raise RuntimeError('Can currently only handle a single template identifier')
      if not (method in self.templateMethods):
        self.template(method, method.getTemplateIdentifiers()[0])
    return 
  
  def getTemplateMethods(self, methods):
    for method in methods:
      if method in self.templateMethods:
        for m in self.templateMethods[method]:
          yield m
      else:
        yield method
    return 
