# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.ImplementationGenerator
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.DepthFirstVisitor
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Iterator
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class ImplementationGenerator:
  '''
  This visitor transforms a SIDL AST into a set of C Implementation ASTs
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.source = {}

    self.sourceLanguages = ['Cxx']
    self.constructors = {}

    self.destructors = {}

    self.templateMethods = {}

    import ASE.Compiler.Python.IOR
    self.IOR = ASE.Compiler.Python.IOR.IOR()
    self.SIDL = self.IOR.SIDL
    self.Cxx = self.IOR.Cxx
  
  def getSourceLanguages(self):
    '''
    Retrieve the list of the languages in which ASTs are produced
    '''
    return self.sourceLanguages
  
  def setSourceLanguages(self, outputLanguages):
    '''
    Set the list of the languages in which ASTs are produced
    '''
    self.sourceLanguages = list(sourceLanguages)
  
  def getSource(self, language):
    '''
    Retrieve the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      raise RuntimeError('Invalid source language: ' + str(language))
    if not (language in self.source):
      self.source[language] = []
    return self.source[language]
  
  def setSource(self, language, vertices):
    '''
    Set the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      raise RuntimeError('Invalid source language: ' + str(language))
    self.source[language] = list(vertices)
  
  def addSource(self, language, vertices):
    '''
    Add to the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      raise RuntimeError('Invalid source language: ' + str(language))
    if not (language in self.source):
      self.source[language] = []
    self.source[language].extend(vertices)
  
  def removeSource(self, language, vertices):
    '''
    Remove to the list of source ASTs for the given language
    Throw an exception if a vertex or language does not exist
    '''
    if not (language in self.sourceLanguages):
      raise RuntimeError('Invalid source language: ' + str(language))
    if not (language in self.source):
      self.source[language] = []
    for vertex in vertices:
      if vertex in self.source[language]:
        self.source[language].remove(vertex)
      else:
        e = ASE.Compiler.CompilerException.CompilerException()
        e.setMessage('Source tree does not exist: ' + vertex.getIdentifier())
        raise e
  
  def visitClass(self, node):
    '''
    Each class becomes a source file including the methods and an implementation structure
    '''
    [self.createTemplateMethod(method) for method in node.getImplementedMethods()]
    defns = []
    defns.extend(self.getImplementationStruct(node, withDefinition = 1))
    defns.extend(self.getSetEpvMethod(node, onlyDeclaration = 1))
    for method in self.getTemplateMethods(node.getImplementedMethods()):
      defns.extend(self.getImplMethod(node, method))
    defns.extend(self.getConstructorMethod(node))
    defns.extend(self.getDestructorMethod(node))
    defns.extend(self.getSetEpvMethod(node))
    self.addSource('Cxx', [self.getSourceFile(node, defns)])
  # Ending class methods section
  pass
  
  def getImplementationStruct(self, klass, withDefinition = 0):
    import ASE.Compiler.Cxx.Struct
    implStruct = ASE.Compiler.Cxx.Struct.Struct()
    implStruct.setIdentifier(klass.getFullIdentifier().replace('.', '_'))
    if withDefinition:
      selfStub = ASE.Compiler.Cxx.Declarator.Declarator()
      selfStub.setIdentifier('self')
      selfStub.setType(self.IOR.getType(klass))
      implStruct.setChildren([self.Cxx.getDecl(selfStub, 'Add user specific member variables')])
      implStruct.setComments([self.Cxx.getComment('Implementation Structure')])
      return [self.Cxx.getDecl(implStruct)]
    return implStruct
  
  def getImplementationType(self, klass):
    import ASE.Compiler.Cxx.Pointer
    implType = ASE.Compiler.Cxx.Pointer.Pointer()
    implType.setChildren([self.getImplementationStruct(klass)])
    return implType
  
  def getConstructor(self, klass):
    if not (klass in self.constructors):
      import ASE.Compiler.ParameterAccessMode
      import ASE.Compiler.SIDL.Method
      import ASE.Compiler.SIDL.Parameter
      objVar = self.Cxx.getVar('obj')
      objParam = ASE.Compiler.SIDL.Parameter.Parameter()
      objParam.setIdentifier(objVar.getIdentifier())
      objParam.setType(klass)
      objParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
      constructor = ASE.Compiler.SIDL.Method.Method()
      constructor.setIdentifier(klass.getIdentifier())
      constructor.setChildren([objParam])
      constructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
      constructor.setParent(klass)
      constructor.setStatic(1)
      self.constructors[klass] = constructor
    return self.constructors[klass]
  
  def getConstructorMethod(self, klass):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Sizeof
    import ASE.Compiler.Cxx.StructReference
    methodSkeleton = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(klass, self.getConstructor(klass), prefix = '_impl_').getChildren()[0])
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    data = ASE.Compiler.Cxx.StructReference.StructReference()
    data.setChildren([self.Cxx.getVar(self.getConstructor(klass).getChildren()[0].getIdentifier())])
    data.setMember('d_data')
    data.setPointer(1)
    implStruct = self.getImplementationStruct(klass)
    implDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    implDecl.setType(implStruct)
    sizeofImpl = ASE.Compiler.Cxx.Sizeof.Sizeof()
    sizeofImpl.setTypeName(implDecl)
    mallocCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    mallocCall.setChildren([self.Cxx.getVar('malloc')])
    mallocCall.setArguments([sizeofImpl])
    setData = ASE.Compiler.Cxx.Assignment.Assignment()
    setData.setChildren([data, mallocCall])
    body.addChildren([self.Cxx.getExpStmt(setData)])
    return [methodSkeleton]
  
  def getDestructor(self, klass):
    if not (klass in self.destructors):
      import ASE.Compiler.SIDL.Method
      destructor = ASE.Compiler.SIDL.Method.Method()
      destructor.setIdentifier(klass.getIdentifier() + '_')
      destructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
      destructor.setParent(klass)
      self.destructors[klass] = destructor
    return self.destructors[klass]
  
  def getDestructorMethod(self, klass):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.StructReference
    methodSkeleton = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(klass, self.getDestructor(klass), prefix = '_impl_').getChildren()[0])
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    data = ASE.Compiler.Cxx.StructReference.StructReference()
    data.setChildren([self.Cxx.getVar('self')])
    data.setMember('d_data')
    data.setPointer(1)
    freeCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeCall.setChildren([self.Cxx.getVar('free')])
    freeCall.setArguments([data])
    body.addChildren([self.Cxx.getExpStmt(freeCall)])
    return [methodSkeleton]
  
  def getSetEpv(self, klass):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    epvStruct = ASE.Compiler.Cxx.Struct.Struct()
    epvStruct.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '__epv')
    epvType = ASE.Compiler.Cxx.Pointer.Pointer()
    epvType.setChildren([epvStruct])
    epvParam = ASE.Compiler.Cxx.Parameter.Parameter()
    epvParam.setIdentifier('_epv')
    epvParam.setType(epvType)
    excType = ASE.Compiler.Cxx.Pointer.Pointer()
    excType.setChildren([self.IOR.getType(self.SIDL.getBaseException(self.SIDL.getASEException(klass)))])
    excParam = ASE.Compiler.Cxx.Parameter.Parameter()
    excParam.setIdentifier('_exception')
    excParam.setType(excType)
    name = ASE.Compiler.Cxx.Declarator.Declarator()
    name.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '__set_epv')
    methodDecl = ASE.Compiler.Cxx.Function.Function()
    methodDecl.setType(self.Cxx.typeMap['void'])
    methodDecl.setChildren([name])
    methodDecl.setParameters([epvParam, excParam])
    return methodDecl
  
  def getSetEpvMethod(self, klass, onlyDeclaration = 0):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.StructReference
    if onlyDeclaration:
      return [self.Cxx.getDecl(self.getSetEpv(klass))]
    methodSkeleton = self.getSetEpv(klass)
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    epvVar = self.Cxx.getVar('_epv')
    methods = [self.getConstructor(klass), self.getDestructor(klass)]
    methods.extend([m for m in self.getTemplateMethods(klass.getImplementedMethods())])
    for method in methods:
      funcVar = ASE.Compiler.Cxx.StructReference.StructReference()
      funcVar.setChildren([epvVar])
      funcVar.setMember(method.getIdentifier())
      funcVar.setPointer(1)
      setFunc = ASE.Compiler.Cxx.Assignment.Assignment()
      setFunc.setChildren([funcVar, self.Cxx.getVar('_impl_' + method.getIdentifier())])
      body.addChildren([self.Cxx.getExpStmt(setFunc)])
    return [methodSkeleton]
  
  def getImplMethod(self, klass, method):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.StructReference
    implFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(klass, method, prefix = '_impl_').getChildren()[0])
    implType = self.getImplementationType(klass)
    selfVar = self.Cxx.getVar('self')
    objVar = self.Cxx.getVar('_obj')
    selfData = ASE.Compiler.Cxx.StructReference.StructReference()
    selfData.setChildren([selfVar])
    selfData.setMember('d_data')
    selfData.setPointer(1)
    objInit = ASE.Compiler.Cxx.Initializer.Initializer()
    objInit.setChildren([self.Cxx.castToType(selfData, implType)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.addDeclarations([self.Cxx.getDeclaration(objVar, implType, objInit)])
    implFunc.setBody(compound)
    implFunc.setStatic(1)
    implFunc.setComments(method.getComments())
    return [implFunc]
  
  def getFileBasename(self, interface):
    import os
    idChain = interface.getFullIdentifier().split('.')
    if not (len(idChain[:-(1)])):
      return idChain[-(1)] + '_impl'
    return os.path.join(apply(os.path.join, idChain[:-(1)]), idChain[-(1)] + '_impl')
  
  def getSourceFile(self, klass, defns):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.Include
    import ASE.Compiler.Cxx.Source
    stdInclude = ASE.Compiler.Cxx.Include.Include()
    stdInclude.setIdentifier('<stdlib.h>')
    source = ASE.Compiler.Cxx.Source.Source()
    source.setFilename(self.getFileBasename(klass) + '.c')
    source.addChildren([stdInclude, self.IOR.getGuardedInclude(klass)])
    exceptions = {}

    for method in klass.getImplementedMethods():
      for exception in method.getExceptions():
        exceptions[exception.getFullIdentifier()] = exception
    for exception in exceptions.values():
      source.addChildren([self.IOR.getGuardedInclude(exception)])
    source.addChildren(defns)
    source.setPurpose(ASE.Compiler.CodePurpose.IMPLEMENTATION)
    return source
  
  def copyType(self, type):
    if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      newType = ASE.Compiler.SIDL.Array.Array()
      newType.setIdentifier(type.getIdentifier())
      newType.setAttributes(type.getAttributes())
      newType.setLineNumber(type.getLineNumber())
      newType.setComments(type.getComments())
      newType.setChildren([child for child in type.getChildren()])
      newType.setBaseType(type.getBaseType())
      newType.setType(self.copyType(ASE.Compiler.SIDL.Array.Array(type).getType()))
      return newType
    return type
  
  def copyParameter(self, parameter):
    '''
    This will be replaced by parameter.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Parameter
    newParameter = ASE.Compiler.SIDL.Parameter.Parameter()
    newParameter.setIdentifier(parameter.getIdentifier())
    newParameter.setAttributes(parameter.getAttributes())
    newParameter.setParent(parameter.getParent())
    newParameter.setChildren(parameter.getChildren())
    newParameter.setLineNumber(parameter.getLineNumber())
    newParameter.setComments(parameter.getComments())
    newParameter.setDependencies(parameter.getDependencies())
    newParameter.setType(self.copyType(parameter.getType()))
    newParameter.setAccessMode(parameter.getAccessMode())
    newParameter.setRequired(parameter.getRequired())
    return newParameter
  
  def copyMethod(self, method):
    '''
    This will be replaced by method.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    newMethod = ASE.Compiler.SIDL.Method.Method()
    newMethod.setIdentifier(method.getIdentifier())
    newMethod.setAttributes(method.getAttributes())
    newMethod.setParent(method.getParent())
    newMethod.setChildren([self.copyParameter(ASE.Compiler.SIDL.Parameter.Parameter(child)) for child in method.getChildren()])
    newMethod.setLineNumber(method.getLineNumber())
    newMethod.setComments(method.getComments())
    newMethod.setDependencies(method.getDependencies())
    newMethod.setTemplateIdentifiers(method.getTemplateIdentifiers())
    if not (method.getReturnParameter() is None):
      newMethod.setReturnParameter(self.copyParameter(method.getReturnParameter()))
    newMethod.setExceptions(method.getExceptions())
    newMethod.setStatic(method.getStatic())
    newMethod.setInternal(method.getInternal())
    newMethod.setVirtual(method.getVirtual())
    newMethod.setPurpose(method.getPurpose())
    return newMethod
  
  def checkTemplateParameter(self, name, parameter):
    if not (parameter is None):
      type = parameter.getType()
      if (type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name):
        return True
      elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
        import ASE.Compiler.SIDL.Array
        type = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
        if (type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name):
          return True
    return False
  
  def insertType(self, parameter, type):
    if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      self.insertType(ASE.Compiler.SIDL.Array.Array(parameter.getType()), type)
    else:
      parameter.setType(type)
    return 
  
  def insertTypes(self, method, universe, type):
    import ASE.Compiler.SIDL.Parameter
    name = universe.getIdentifier()
    newMethod = self.copyMethod(method)
    newMethod.setIdentifier(newMethod.getIdentifier() + '_' + type.getFullIdentifier())
    if self.checkTemplateParameter(name, method.getReturnParameter()):
      self.insertType(newMethod.getReturnParameter(), type)
    for (child, newChild) in zip(method.getChildren(), newMethod.getChildren()):
      if self.checkTemplateParameter(name, ASE.Compiler.SIDL.Parameter.Parameter(child)):
        self.insertType(ASE.Compiler.SIDL.Parameter.Parameter(newChild), type)
    return newMethod
  
  def template(self, method, name):
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.SIDL.TypeUniverse
    universe = None
    for parameter in [method.getReturnParameter()] + [ASE.Compiler.SIDL.Parameter.Parameter(child) for child in method.getChildren()]:
      if self.checkTemplateParameter(name, parameter):
        type = parameter.getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(ASE.Compiler.SIDL.Array.Array(type).getType())
        else:
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(type)
        break
    if universe is None:
      raise RuntimeError('Could not find type universe for template ' + name + ' in method ' + method.getFullIdentifier())
    self.templateMethods[method] = [self.insertTypes(method, universe, type) for type in universe.getTypes()]
    return None
  
  def createTemplateMethod(self, method):
    if len(method.getTemplateIdentifiers()):
      if len(method.getTemplateIdentifiers()) > 1:
        raise RuntimeError('Can currently only handle a single template identifier')
      if not (method in self.templateMethods):
        self.template(method, method.getTemplateIdentifiers()[0])
    return None
  
  def getTemplateMethods(self, methods):
    for method in methods:
      if method in self.templateMethods:
        for m in self.templateMethods[method]:
          yield m
      else:
        yield method
    return 
