# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.Cxx.Addition
import ASE.Compiler.Cxx.AdditionAssignment
import ASE.Compiler.Cxx.Address
import ASE.Compiler.Cxx.And
import ASE.Compiler.Cxx.AndAssignment
import ASE.Compiler.Cxx.Array
import ASE.Compiler.Cxx.ArrayReference
import ASE.Compiler.Cxx.Assignment
import ASE.Compiler.Cxx.BitField
import ASE.Compiler.Cxx.Break
import ASE.Compiler.Cxx.Cast
import ASE.Compiler.Cxx.Complement
import ASE.Compiler.Cxx.CompoundStatement
import ASE.Compiler.Cxx.Constant
import ASE.Compiler.Cxx.Continue
import ASE.Compiler.Cxx.Declaration
import ASE.Compiler.Cxx.Declarator
import ASE.Compiler.Cxx.DeclaratorGroup
import ASE.Compiler.Cxx.Define
import ASE.Compiler.Cxx.DepthFirstVisitor
import ASE.Compiler.Cxx.Division
import ASE.Compiler.Cxx.DivisionAssignment
import ASE.Compiler.Cxx.Enumeration
import ASE.Compiler.Cxx.Enumerator
import ASE.Compiler.Cxx.Equality
import ASE.Compiler.Cxx.ExclusiveOr
import ASE.Compiler.Cxx.ExclusiveOrAssignment
import ASE.Compiler.Cxx.Expression
import ASE.Compiler.Cxx.ExpressionStatement
import ASE.Compiler.Cxx.For
import ASE.Compiler.Cxx.Function
import ASE.Compiler.Cxx.FunctionCall
import ASE.Compiler.Cxx.Goto
import ASE.Compiler.Cxx.GreaterThan
import ASE.Compiler.Cxx.GreaterThanOrEqual
import ASE.Compiler.Cxx.Group
import ASE.Compiler.Cxx.Header
import ASE.Compiler.Cxx.If
import ASE.Compiler.Cxx.Include
import ASE.Compiler.Cxx.InclusiveOr
import ASE.Compiler.Cxx.InclusiveOrAssignment
import ASE.Compiler.Cxx.Increment
import ASE.Compiler.Cxx.Indirection
import ASE.Compiler.Cxx.Inequality
import ASE.Compiler.Cxx.Initializer
import ASE.Compiler.Cxx.LeftShift
import ASE.Compiler.Cxx.LeftShiftAssignment
import ASE.Compiler.Cxx.LessThan
import ASE.Compiler.Cxx.LessThanOrEqual
import ASE.Compiler.Cxx.LogicalAnd
import ASE.Compiler.Cxx.LogicalOr
import ASE.Compiler.Cxx.Macro
import ASE.Compiler.Cxx.Modulo
import ASE.Compiler.Cxx.ModuloAssignment
import ASE.Compiler.Cxx.Multiplication
import ASE.Compiler.Cxx.MultiplicationAssignment
import ASE.Compiler.Cxx.Negation
import ASE.Compiler.Cxx.Parameter
import ASE.Compiler.Cxx.Pointer
import ASE.Compiler.Cxx.PreprocessorConditional
import ASE.Compiler.Cxx.PythonModule
import ASE.Compiler.Cxx.Return
import ASE.Compiler.Cxx.RightShift
import ASE.Compiler.Cxx.RightShiftAssignment
import ASE.Compiler.Cxx.Sizeof
import ASE.Compiler.Cxx.Source
import ASE.Compiler.Cxx.Statement
import ASE.Compiler.Cxx.Struct
import ASE.Compiler.Cxx.StructReference
import ASE.Compiler.Cxx.Subtraction
import ASE.Compiler.Cxx.SubtractionAssignment
import ASE.Compiler.Cxx.Switch
import ASE.Compiler.Cxx.Type
import ASE.Compiler.Cxx.Typedef
import ASE.Compiler.Cxx.UnaryMinus
import ASE.Compiler.Cxx.UnaryPlus
import ASE.Compiler.Cxx.Variable
import ASE.Compiler.Cxx.While
import ASE.Compiler.Printer
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class Printer:
  '''
  This class prints a Cxx AST to standard output as Cxx source
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    import sys
    self.tab = '  '
    self.indentSize = 0
    self.output = sys.stdout
  
  def getTab(self):
    '''
    Retrieve the tab string
    '''
    return self.tab
  
  def setTab(self, tab):
    '''
    Set the tab string
    '''
    self.tab = tab
  
  def getIndentSize(self):
    '''
    Retrieve the number of tabs of the current indent
    '''
    return self.indentSize
  
  def setIndentSize(self, indentSize):
    '''
    Set the number of tabs of the current indent
    Throws an exception on a negative indent size
    '''
    if self.indentSize < 0:
      raise RuntimeError('Indent size must be positive: ' + str(self.indentSize))
    self.indentSize = indentSize
  
  def increaseIndent(self):
    '''
    Increase the index size by one
    '''
    self.indentSize += 1
  
  def decreaseIndent(self):
    '''
    Decrease the index size by one
    Throws an exception on a negative indent size
    '''
    self.indentSize -= 1
    if self.indentSize < 0:
      raise RuntimeError('Indent size decreased below zero: ' + str(self.indentSize))
  
  def indent(self):
    '''
    Output the current indent (the indent size in tabs)
    '''
    for i in range(self.getIndentSize()):
      self.__IORself.write(self.getTab())
  
  def write(self, s):
    '''
    Output a string
    '''
    self.output.write(s)
  
  def println(self, s):
    '''
    Output a string, properly indented and followed by a newline
    '''
    self.__IORself.indent()
    self.__IORself.write(s)
    self.__IORself.write('\n')
  
  def visitInclude(self, vertex):
    self.__IORself.println('#include ' + vertex.getIdentifier())
  
  def visitDefine(self, vertex):
    self.indent()
    if vertex.getUndef():
      self.__IORself.write('#undef ')
    else:
      self.__IORself.write('#define ')
    self.__IORself.write(vertex.getIdentifier())
    if len(vertex.getArgumentNames()):
      self.__IORself.write(' (')
      self.__IORself.write(', '.join(vertex.getArgumentNames()))
      self.__IORself.write(')')
    if vertex.getReplacementText():
      self.__IORself.write(' ' + vertex.getReplacementText())
    self.__IORself.write('\n')
  
  def visitMacro(self, vertex):
    self.__IORself.write(vertex.getIdentifier())
  
  def visitPreprocessorConditional(self, vertex):
    self.__IORself.println('#' + vertex.getIdentifier() + ' ' + vertex.getBranchExpression())
    if vertex.getValue():
      num = len(vertex.getChildren())
      for (c, child) in enumerate(vertex.getChildren()):
        child.accept(self.__IORself)
        if c < (num - 1):
          self.__IORself.write('\n')
      if vertex.getInactiveCode():
        self.__IORself.println('#else')
        self.__IORself.write(vertex.getInactiveCode())
        self.__IORself.write('\n')
    else:
      self.__IORself.write(vertex.getInactiveCode())
      self.__IORself.write('\n')
      if len(vertex.getChildren()):
        self.__IORself.println('#else')
        num = len(vertex.getChildren())
        for (c, child) in enumerate(vertex.getChildren()):
          child.accept(self.__IORself)
          if c < (num - 1):
            self.__IORself.write('\n')
    self.__IORself.println('#endif')
  
  def visitType(self, vertex):
    self.writeTypeQualifier(vertex)
    self.__IORself.write(vertex.getIdentifier())
  
  def visitPointer(self, vertex):
    self.__IORself.visitVertex(vertex)
    if (len(vertex.getChildren()) and not (vertex.getChildren()[0].isInstanceOf('ASE.Compiler.Cxx.Pointer'))):
      self.__IORself.write(' ')
    self.__IORself.write('*')
    self.writeTypeQualifier(vertex, 1)
  
  def visitStruct(self, vertex):
    self.writeComments(vertex, newline = 1)
    self.writeTypeQualifier(vertex)
    if vertex.getUnion():
      self.__IORself.write('union')
    else:
      self.__IORself.write('struct')
    if vertex.getIdentifier():
      self.__IORself.write(' ' + vertex.getIdentifier())
    if len(vertex.getChildren()) > 0:
      self.__IORself.write(' {\n')
      self.increaseIndent()
      self.__IORself.visitVertex(vertex)
      self.decreaseIndent()
      self.__IORself.indent()
      self.__IORself.write('}')
  
  def visitBitField(self, vertex):
    self.writeTypeQualifier(vertex)
  
  def visitEnumeration(self, vertex):
    self.writeTypeQualifier(vertex)
    if len(vertex.getChildren()) == 0:
      self.__IORself.write('enum ' + vertex.getIdentifier())
    else:
      self.indent()
      self.__IORself.write('enum ')
      if vertex.getIdentifier():
        self.__IORself.write(vertex.getIdentifier() + ' ')
      self.__IORself.write('{')
      num = len(vertex.getChildren())
      for (i, enumerator) in enumerate(vertex.getChildren()):
        enumerator.accept(self.__IORself)
        if i < (num - 1):
          self.__IORself.write(', ')
      self.__IORself.write('}')
  
  def visitEnumerator(self, vertex):
    self.__IORself.write(vertex.getIdentifier())
    if vertex.getInitialized():
      self.__IORself.write(' = ')
      self.__IORself.write(str(vertex.getValue()))
  
  def visitTypedef(self, vertex):
    self.writeTypeQualifier(vertex)
    self.__IORself.write(vertex.getIdentifier())
  
  def visitGroup(self, vertex):
    self.__IORself.write('(')
    self.__IORself.visitVertex(vertex)
    self.__IORself.write(')')
  
  def visitDeclarator(self, vertex):
    self.writeStorageClassSpecifier(vertex)
    if not (vertex.getType() is None):
      vertex.getType().accept(self.__IORself)
    if vertex.getIdentifier():
      if (not (vertex.getType() is None) and not (vertex.getType().isInstanceOf('ASE.Compiler.Cxx.Pointer'))):
        self.__IORself.write(' ')
      self.__IORself.write(vertex.getIdentifier())
    if vertex.getInitializer():
      self.__IORself.write(' = ')
      vertex.getInitializer().accept(self.__IORself)
  
  def visitDeclaratorGroup(self, vertex):
    self.__IORself.write('(')
    self.__IORself.visitVertex(vertex)
    self.__IORself.write(')')
  
  def visitArray(self, vertex):
    self.writeStorageClassSpecifier(vertex)
    if not (vertex.getType() is None):
      vertex.getType().accept(self.__IORself)
      if not (vertex.getType().isInstanceOf('ASE.Compiler.Cxx.Pointer')):
        self.__IORself.write(' ')
    self.__IORself.visitVertex(vertex)
    self.__IORself.write('[')
    if vertex.getSize():
      vertex.getSize().accept(self.__IORself)
    self.__IORself.write(']')
    if vertex.getInitializer():
      self.__IORself.write(' = ')
      vertex.getInitializer().accept(self.__IORself)
  
  def visitFunction(self, vertex):
    self.writeStorageClassSpecifier(vertex)
    vertex.getType().accept(self.__IORself)
    if not (vertex.getType().isInstanceOf('ASE.Compiler.Cxx.Pointer')):
      self.__IORself.write(' ')
    self.__IORself.visitVertex(vertex)
    num = len(vertex.getParameters())
    if num:
      self.__IORself.write('(')
      for (i, parameter) in enumerate(vertex.getParameters()):
        parameter.accept(self.__IORself)
        if i < (num - 1):
          self.__IORself.write(', ')
      self.__IORself.write(')')
    else:
      self.__IORself.write('(void)')
    if not (vertex.getBody() is None):
      self.__IORself.write('\n')
      vertex.getBody().accept(self.__IORself)
    else:
      if not (vertex.getInitializer() is None):
        self.__IORself.write(' = ')
        vertex.getInitializer().accept(self.__IORself)
  
  def visitParameter(self, vertex):
    self.writeComments(vertex)
    vertex.getType().accept(self.__IORself)
    if vertex.getIdentifier():
      if not (vertex.getType().isInstanceOf('ASE.Compiler.Cxx.Pointer')):
        self.__IORself.write(' ')
      self.__IORself.write(vertex.getIdentifier())
  
  def visitDeclaration(self, vertex):
    self.writeComments(vertex, newline = 1)
    for child in vertex.getChildren():
      self.__IORself.indent()
      child.accept(self.__IORself)
      self.__IORself.write(';\n')
  
  def visitStatement(self, vertex):
    self.__IORself.visitVertex(vertex)
  
  def visitExpressionStatement(self, vertex):
    self.writeComments(vertex, newline = 1)
    self.__IORself.indent()
    if not (vertex.getIdentifier() is None):
      self.__IORself.write(vertex.getIdentifier() + ': ')
    self.__IORself.visitVertex(vertex)
    self.__IORself.write(';\n')
  
  def visitCompoundStatement(self, vertex):
    self.writeComments(vertex, newline = 1)
    self.indent()
    if not (vertex.getIdentifier() is None):
      self.__IORself.write(vertex.getIdentifier() + ': ')
    self.__IORself.write('{\n')
    self.increaseIndent()
    if len(vertex.getDeclarations()):
      for decl in vertex.getDeclarations():
        decl.accept(self.__IORself)
      self.__IORself.println('')
    self.__IORself.visitVertex(vertex)
    self.decreaseIndent()
    self.__IORself.println('}')
  
  def visitIf(self, vertex):
    self.writeComments(vertex, newline = 1)
    if not (vertex.getTernary()):
      self.indent()
      self.__IORself.write('if (')
      vertex.getBranch().accept(self.__IORself)
      self.__IORself.write(')\n')
      vertex.getChildren()[0].accept(self.__IORself)
      if len(vertex.getChildren()) > 1:
        self.__IORself.println('else')
        vertex.getChildren()[1].accept(self.__IORself)
    else:
      vertex.getBranch().accept(self.__IORself)
      self.__IORself.write(' ? ')
      vertex.getChildren()[0].accept(self.__IORself)
      self.__IORself.write(' : ')
      vertex.getChildren()[1].accept(self.__IORself)
  
  def visitSwitch(self, vertex):
    self.writeComments(vertex, newline = 1)
    self.__IORself.indent()
    self.__IORself.write('switch(')
    vertex.getBranch().accept(self.__IORself)
    self.__IORself.write(') {\n')
    self.__IORself.increaseIndent()
    inCase = 0
    for child in vertex.getChildren()[0].getChildren():
      statement = ASE.Compiler.Cxx.Statement.Statement(child)
      if not (statement.getCaseLabel() is None):
        if inCase:
          self.__IORself.decreaseIndent()
          inCase = 0
        self.__IORself.indent()
        self.__IORself.write('case ')
        statement.getCaseLabel().accept(self.__IORself)
        self.__IORself.write(':\n')
        if not (inCase):
          self.__IORself.increaseIndent()
          inCase = 1
      statement.accept(self.__IORself)
    if inCase:
      self.__IORself.decreaseIndent()
    self.__IORself.decreaseIndent()
    self.__IORself.println('}')
  
  def visitWhile(self, vertex):
    self.writeComments(vertex, newline = 1)
    if vertex.getDoWhile():
      self.__IORself.println('do')
      self.__IORself.visitVertex(vertex)
      self.__IORself.indent()
      self.__IORself.write('while(')
      vertex.getBranch().accept(self.__IORself)
      self.__IORself.write(')\n')
    else:
      self.__IORself.indent()
      self.__IORself.write('while(')
      vertex.getBranch().accept(self.__IORself)
      self.__IORself.write(')\n')
      self.__IORself.visitVertex(vertex)
  
  def visitFor(self, vertex):
    self.writeComments(vertex, newline = 1)
    self.__IORself.indent()
    self.__IORself.write('for(')
    if not (vertex.getInitialization() is None):
      vertex.getInitialization().accept(self.__IORself)
    self.__IORself.write(';')
    if not (vertex.getBranch() is None):
      self.__IORself.write(' ')
      vertex.getBranch().accept(self.__IORself)
    self.__IORself.write(';')
    if not (vertex.getIncrement() is None):
      self.__IORself.write(' ')
      vertex.getIncrement().accept(self.__IORself)
    self.__IORself.write(')\n')
    self.__IORself.visitVertex(vertex)
  
  def visitGoto(self, vertex):
    self.writeComments(vertex, newline = 1)
    self.__IORself.println('goto ' + vertex.getIdentifier() + ';')
  
  def visitContinue(self, vertex):
    self.writeComments(vertex, newline = 1)
    self.indent()
    if not (vertex.getIdentifier() is None):
      self.__IORself.write(vertex.getIdentifier() + ': ')
    self.__IORself.write('continue;\n')
  
  def visitBreak(self, vertex):
    self.writeComments(vertex, newline = 1)
    self.indent()
    if not (vertex.getIdentifier() is None):
      self.__IORself.write(vertex.getIdentifier() + ': ')
    self.__IORself.write('break;\n')
  
  def visitReturn(self, vertex):
    self.writeComments(vertex, newline = 1)
    self.indent()
    if not (vertex.getIdentifier() is None):
      self.__IORself.write(vertex.getIdentifier() + ': ')
    self.__IORself.write('return')
    if len(vertex.getChildren()):
      self.__IORself.write(' ')
      self.__IORself.visitVertex(vertex)
    self.__IORself.write(';\n')
  
  def visitInitializer(self, vertex):
    num = len(vertex.getChildren())
    nested = (not (vertex.getParent() is None) and vertex.getParent().isInstanceOf('ASE.Compiler.Cxx.Initializer'))
    if num > 1:
      self.__IORself.write('{')
      self.__IORself.increaseIndent()
      if not (nested):
        self.__IORself.write('\n')
        self.__IORself.indent()
      for (c, child) in enumerate(vertex.getChildren()):
        if not (nested):
          if c > 0:
            self.__IORself.indent()
        child.accept(self.__IORself)
        if nested:
          if c < (num - 1):
            if not (child.isInstanceOf('ASE.Compiler.Cxx.Macro')):
              self.__IORself.write(',')
            self.__IORself.write(' ')
        else:
          if c < (num - 1):
            if not (child.isInstanceOf('ASE.Compiler.Cxx.Macro')):
              self.__IORself.write(',')
            self.__IORself.write('\n')
      self.__IORself.decreaseIndent()
      self.__IORself.write('}')
    elif vertex.getList():
      self.__IORself.write('{')
      vertex.getChildren()[0].accept(self.__IORself)
      self.__IORself.write('}')
    else:
      self.__IORself.visitVertex(vertex)
  
  def visitAssignment(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitInclusiveOrAssignment(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitExclusiveOrAssignment(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitAndAssignment(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitLeftShiftAssignment(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitRightShiftAssignment(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitAdditionAssignment(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitSubtractionAssignment(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitMultiplicationAssignment(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitDivisionAssignment(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitModuloAssignment(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitLogicalOr(self, vertex):
    self.visitNaryExpression(vertex)
  
  def visitLogicalAnd(self, vertex):
    self.visitNaryExpression(vertex)
  
  def visitInclusiveOr(self, vertex):
    self.visitNaryExpression(vertex)
  
  def visitExclusiveOr(self, vertex):
    self.visitNaryExpression(vertex)
  
  def visitAnd(self, vertex):
    self.visitNaryExpression(vertex)
  
  def visitEquality(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitInequality(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitLessThan(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitGreaterThan(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitLessThanOrEqual(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitGreaterThanOrEqual(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitLeftShift(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitRightShift(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitAddition(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitSubtraction(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitMultiplication(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitDivision(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitModulo(self, vertex):
    self.visitBinaryExpression(vertex)
  
  def visitCast(self, vertex):
    self.__IORself.write('(')
    vertex.getTypeName().accept(self.__IORself)
    self.__IORself.write(') ')
    self.__IORself.visitVertex(vertex)
  
  def visitSizeof(self, vertex):
    if len(vertex.getChildren()):
      self.__IORself.write('sizeof ')
      self.__IORself.visitVertex(vertex)
    else:
      self.__IORself.write('sizeof(')
      vertex.getTypeName().accept(self.__IORself)
      self.__IORself.write(')')
  
  def visitAddress(self, vertex):
    self.visitUnaryExpression(vertex)
  
  def visitIndirection(self, vertex):
    self.visitUnaryExpression(vertex)
  
  def visitUnaryPlus(self, vertex):
    self.visitUnaryExpression(vertex)
  
  def visitUnaryMinus(self, vertex):
    self.visitUnaryExpression(vertex)
  
  def visitComplement(self, vertex):
    self.visitUnaryExpression(vertex)
  
  def visitNegation(self, vertex):
    self.visitUnaryExpression(vertex)
  
  def visitIncrement(self, vertex):
    if vertex.getDecrement():
      if vertex.getPostfix():
        self.__IORself.visitVertex(vertex)
        self.__IORself.write('--')
      else:
        self.__IORself.write('--')
        self.__IORself.visitVertex(vertex)
    else:
      if vertex.getPostfix():
        self.__IORself.visitVertex(vertex)
        self.__IORself.write('++')
      else:
        self.__IORself.write('++')
        self.__IORself.visitVertex(vertex)
  
  def visitStructReference(self, vertex):
    self.__IORself.visitVertex(vertex)
    if vertex.getPointer():
      self.__IORself.write('->')
    else:
      self.__IORself.write('.')
    self.__IORself.write(vertex.getMember())
  
  def visitArrayReference(self, vertex):
    self.__IORself.visitVertex(vertex)
    self.__IORself.write('[')
    vertex.getIndex().accept(self.__IORself)
    self.__IORself.write(']')
  
  def visitFunctionCall(self, vertex):
    self.__IORself.visitVertex(vertex)
    self.__IORself.write('(')
    num = len(vertex.getArguments())
    for (a, argument) in enumerate(vertex.getArguments()):
      argument.accept(self.__IORself)
      if a < (num - 1):
        self.__IORself.write(', ')
    self.__IORself.write(')')
  
  def visitVariable(self, vertex):
    self.__IORself.write(vertex.getIdentifier())
  
  def visitConstant(self, vertex):
    if not (vertex.getIdentifier() is None):
      self.__IORself.write('"' + vertex.getIdentifier() + '"')
    else:
      self.__IORself.write(str(vertex.getValue_string()))
  # Ending class methods section
  pass
  
  def shitMethod1(self, x, y):
    '''
    Should be lambda x, y: x or y
    '''
    return (x or y)
  
  def getDeclAbstract(self, decl):
    '''
    Determine whether a declaration is abstract
       - The argument can be either a declaration or a type
    '''
    if reduce(self.shitMethod1, [decl.isInstanceOf('ASE.Compiler.Cxx.' + interface) for interface in ['Array', 'Function', 'DeclaratorGroup']], 0):
      isAbstract = self.getDeclAbstract(decl.getChildren()[0])
    else:
      isAbstract = decl.getIdentifier() is None
    return isAbstract
  
  def visitUnaryExpression(self, vertex):
    children = vertex.getChildren()
    if not (len(children) == 1):
      self.raiseException('Unary expressions must have exactly one child')
    self.__IORself.write(vertex.getIdentifier())
    children[0].accept(self.__IORself)
    return None
  
  def visitBinaryExpression(self, vertex):
    children = vertex.getChildren()
    if not (len(children) == 2):
      self.raiseException('Binary expressions must have exactly two children')
    children[0].accept(self.__IORself)
    self.__IORself.write(' ' + vertex.getIdentifier() + ' ')
    children[1].accept(self.__IORself)
    return None
  
  def visitNaryExpression(self, vertex):
    children = vertex.getChildren()
    if len(children) < 2:
      e = ASE.Compiler.CompilerException()
      e.setMessage('Nary expressions must have at least two children')
      raise e
    children[0].accept(self.__IORself)
    for child in children[1:]:
      self.__IORself.write(' ' + vertex.getIdentifier() + ' ')
      child.accept(self.__IORself)
    return None
  
  def writeComments(self, vertex, newline = 0):
    for comment in vertex.getComments():
      if newline:
        self.__IORself.println('/* ' + comment.getComment() + ' */')
      else:
        self.__IORself.write('/*' + comment.getComment() + '*/ ')
    return None
  
  def writeTypeQualifier(self, type, isPointer = 0):
    if type.getConst():
      if isPointer:
        self.__IORself.write(' ')
      self.__IORself.write('const ')
    if type.getVolatile():
      if isPointer:
        self.__IORself.write(' ')
      self.__IORself.write('volatile ')
    return None
  
  def writeStorageClassSpecifier(self, declarator):
    if declarator.getExtern():
      self.__IORself.write('extern ')
    if declarator.getStatic():
      self.__IORself.write('static ')
    if declarator.getTypedef():
      self.__IORself.write('typedef ')
    return None
