# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.DepthFirstVisitor
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Iterator
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.StubGenerator
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class StubGenerator:
  '''
  This visitor transforms a SIDL AST into a set of C Stub ASTs
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.source = {}

    self.sourceLanguages = ['Cxx']
    self.templateMethods = {}

    import ASE.Compiler.Python.IOR
    self.IOR = ASE.Compiler.Python.IOR.IOR()
    self.SIDL = self.IOR.SIDL
    self.Cxx = self.IOR.Cxx
  
  def getSourceLanguages(self):
    '''
    Retrieve the list of the languages in which ASTs are produced
    '''
    return self.sourceLanguages
  
  def setSourceLanguages(self, outputLanguages):
    '''
    Set the list of the languages in which ASTs are produced
    '''
    self.sourceLanguages = list(sourceLanguages)
  
  def getSource(self, language):
    '''
    Retrieve the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      raise RuntimeError('Invalid source language: ' + str(language))
    if not (language in self.source):
      self.source[language] = []
    return self.source[language]
  
  def setSource(self, language, vertices):
    '''
    Set the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      raise RuntimeError('Invalid source language: ' + str(language))
    self.source[language] = list(vertices)
  
  def addSource(self, language, vertices):
    '''
    Add to the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      raise RuntimeError('Invalid source language: ' + str(language))
    if not (language in self.source):
      self.source[language] = []
    self.source[language].extend(vertices)
  
  def removeSource(self, language, vertices):
    '''
    Remove to the list of source ASTs for the given language
    Throw an exception if a vertex or language does not exist
    '''
    if not (language in self.sourceLanguages):
      raise RuntimeError('Invalid source language: ' + str(language))
    if not (language in self.source):
      self.source[language] = []
    for vertex in vertices:
      if vertex in self.source[language]:
        self.source[language].remove(vertex)
      else:
        e = ASE.Compiler.CompilerException.CompilerException()
        e.setMessage('Source tree does not exist: ' + vertex.getIdentifier())
        raise e
  
  def visitInterface(self, node):
    '''
    Each interface becomes a source and header file including the methods and cast method
    '''
    decls = []
    defns = []
    
    def extend(d):
      decls.extend(d[0])
      defns.extend(d[1])
      return None
    [self.createTemplateMethod(method) for method in node.getMethods()]
    extend(self.getConstructorMethod(node))
    for method in self.getTemplateMethods(node.getMethods()):
      extend(self.getStubMethod(node, method))
    self.addSource('Cxx', [self.getHeaderFile(node, decls)])
    self.addSource('Cxx', [self.getSourceFile(node, defns)])
  
  def visitClass(self, node):
    '''
    Each class becomes a source and header file including the methods and constructor method
    '''
    self.visitInterface(node)
  # Ending class methods section
  pass
  
  def getFileBasename(self, interface):
    import os
    idChain = interface.getFullIdentifier().split('.')
    if not (len(idChain[:-(1)])):
      return idChain[-(1)] + '_Stub'
    return os.path.join(apply(os.path.join, idChain[:-(1)]), idChain[-(1)] + '_Stub')
  
  def getGuardedInclude(self, interface):
    import ASE.Compiler.Cxx.PreprocessorConditional
    guard = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    guard.setIdentifier('ifndef')
    guard.setBranchExpression('included_' + interface.getFullIdentifier().replace('.', '_') + '_stub')
    guard.setValue(1)
    guard.addChildren([self.Cxx.getInclude(self.getFileBasename(interface) + '.h')])
    return guard
  
  def getConstructor(self, interface):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Comment
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Constructor and cast method')
    iorParam = ASE.Compiler.SIDL.Parameter.Parameter()
    iorParam.setIdentifier('obj')
    iorParam.setType(self.SIDL.getBaseInterface(interface))
    iorParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    iorReturn = ASE.Compiler.SIDL.Parameter.Parameter()
    iorReturn.setIdentifier('_result')
    iorReturn.setType(self.SIDL.typeMap['opaque'])
    iorReturn.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    constructor = ASE.Compiler.SIDL.Method.Method()
    constructor.setIdentifier(interface.getIdentifier())
    constructor.setReturnParameter(iorReturn)
    constructor.setChildren([iorParam])
    constructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(interface))])
    constructor.setParent(interface)
    constructor.setComments([comment])
    constructor.setStatic(1)
    return constructor
  
  def getConstructorMethod(self, interface):
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    method = self.getConstructor(interface)
    stubDecl = self.IOR.getFunctionDeclaration(interface, method, qualifyName = 1)
    stubFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(interface, method, qualifyName = 1).getChildren()[0])
    objVar = self.Cxx.getVar(stubFunc.getParameters()[0].getIdentifier())
    extFuncVar = self.Cxx.getVar('_extFunc')
    excVar = self.Cxx.getVar(stubFunc.getParameters()[-(1)].getIdentifier())
    selfEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    selfEpv.setChildren([objVar])
    selfEpv.setMember('d_epv')
    selfEpv.setPointer(1)
    castPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    castPtr.setChildren([selfEpv])
    castPtr.setMember('castObject')
    castPtr.setPointer(1)
    castFunc = ASE.Compiler.Cxx.Indirection.Indirection()
    castFunc.setChildren([self.Cxx.getGroup(castPtr)])
    castCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castCall.setChildren([self.Cxx.getGroup(castFunc)])
    castCall.setArguments([objVar, self.Cxx.getString(interface.getFullIdentifier()), excVar])
    castReturn = ASE.Compiler.Cxx.Return.Return()
    castReturn.setChildren([castCall])
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      trueBranch.setChildren([castReturn])
      falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      extFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
      extFuncPtr.setIdentifier(extFuncVar.getIdentifier())
      extFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
      extFuncDecl = ASE.Compiler.Cxx.Function.Function()
      extFuncDecl.setChildren([self.Cxx.getDeclGroup(extFuncPtr)])
      extFuncDecl.setType(self.IOR.getExternalType(interface))
      falseBranch.addDeclarations([self.Cxx.getDecl(extFuncDecl)])
      extFuncType = ASE.Compiler.Cxx.Function.Function()
      extFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
      extFuncType.setType(self.IOR.getExternalType(interface))
      sidlExtCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      sidlExtCall.setChildren([self.Cxx.getVar('ASE_Loader__externals')])
      staticEPV = ASE.Compiler.Cxx.StructReference.StructReference()
      staticEPV.setChildren([sidlExtCall])
      staticEPV.setMember('getStaticEPV')
      staticEPV.setPointer(1)
      staticEPVCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      staticEPVCall.setChildren([staticEPV])
      staticEPVCall.setArguments([self.Cxx.getAddress(excVar)])
      classMethods = ASE.Compiler.Cxx.StructReference.StructReference()
      classMethods.setChildren([staticEPVCall])
      classMethods.setMember('getClassMethodTable')
      classMethods.setPointer(1)
      classMethodsCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      classMethodsCall.setChildren([classMethods])
      classMethodsCall.setArguments([self.Cxx.getString(interface.getFullIdentifier()), self.Cxx.getAddress(excVar)])
      extFunc = ASE.Compiler.Cxx.Cast.Cast()
      extFunc.setTypeName(extFuncType)
      extFunc.setChildren([classMethodsCall])
      setExternals = ASE.Compiler.Cxx.Assignment.Assignment()
      setExternals.setChildren([extFuncVar, extFunc])
      extFuncInd = ASE.Compiler.Cxx.Indirection.Indirection()
      extFuncInd.setChildren([extFuncVar])
      extFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      extFuncCall.setChildren([self.Cxx.getGroup(extFuncInd)])
      createPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      createPtr.setChildren([extFuncCall])
      createPtr.setMember('createObject')
      createPtr.setPointer(1)
      createFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      createFunc.setChildren([self.Cxx.getGroup(createPtr)])
      createCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      createCall.setChildren([self.Cxx.getGroup(createFunc)])
      createCall.setArguments([excVar])
      createReturn = ASE.Compiler.Cxx.Return.Return()
      createReturn.setChildren([createCall])
      falseBranch.setChildren([self.Cxx.getExpStmt(setExternals), createReturn])
      checkObj = ASE.Compiler.Cxx.If.If()
      checkObj.setBranch(objVar)
      checkObj.setChildren([trueBranch, falseBranch])
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound.setChildren([checkObj])
    else:
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound.setDeclarations([self.Cxx.getDeclaration(self.Cxx.getVar('_result'), self.IOR.getType(interface))])
      compound.setChildren([self.IOR.getErrorCheck(interface, method, objVar, 'Interfaces require an argument for the constructor'), castReturn])
    stubFunc.setBody(compound)
    return ([stubDecl], [stubFunc])
  
  def getStubMethod(self, interface, method):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    stubDecl = self.IOR.getFunctionDeclaration(interface, method, qualifyName = 1)
    stubFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(interface, method, qualifyName = 1).getChildren()[0])
    if not (method.getStatic()):
      epv = ASE.Compiler.Cxx.StructReference.StructReference()
      epv.setChildren([self.Cxx.getVar('self')])
      epv.setMember('d_epv')
      epv.setPointer(1)
    else:
      extFunc = ASE.Compiler.Cxx.Declarator.Declarator()
      extFunc.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__externals')
      extFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      extFuncCall.setChildren([extFunc])
      sepvFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      sepvFuncPtr.setChildren([extFuncCall])
      sepvFuncPtr.setMember('getStaticEPV')
      sepvFuncPtr.setPointer(1)
      epv = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      epv.setChildren([sepvFuncPtr])
      epv.setArguments([self.Cxx.getVar('_exception')])
    epvFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    epvFuncPtr.setChildren([epv])
    epvFuncPtr.setMember(method.getIdentifier())
    epvFuncPtr.setPointer(1)
    epvFunc = ASE.Compiler.Cxx.Indirection.Indirection()
    epvFunc.setChildren([self.Cxx.getGroup(epvFuncPtr)])
    epvFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    epvFuncCall.setChildren([self.Cxx.getGroup(epvFunc)])
    args = []
    for (p, param) in enumerate(stubFunc.getParameters()):
      if (p == 0 and not (method.getStatic())):
        selfObj = ASE.Compiler.Cxx.StructReference.StructReference()
        selfObj.setChildren([self.Cxx.getVar('self')])
        selfObj.setMember('d_object')
        selfObj.setPointer(1)
        args.append(selfObj)
      else:
        args.append(self.Cxx.getVar(param.getIdentifier()))
    epvFuncCall.setArguments(args)
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    if method.getReturnParameter() is None:
      compound.setChildren([self.Cxx.getExpStmt(epvFuncCall)])
    else:
      epvReturn = ASE.Compiler.Cxx.Return.Return()
      epvReturn.setChildren([epvFuncCall])
      compound.setChildren([epvReturn])
    stubFunc.setBody(compound)
    return ([stubDecl], [stubFunc])
  
  def getHeaderFile(self, interface, decls):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.Define
    import ASE.Compiler.Cxx.Header
    import ASE.Compiler.Cxx.PreprocessorConditional
    guard = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    guard.setIdentifier('ifndef')
    guard.setBranchExpression('included_' + interface.getFullIdentifier().replace('.', '_') + '_stub')
    guard.setValue(1)
    define = ASE.Compiler.Cxx.Define.Define()
    define.setIdentifier(guard.getBranchExpression())
    externBegin = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    externBegin.setIdentifier('ifdef')
    externBegin.setBranchExpression('__cplusplus')
    externBegin.setValue(0)
    externBegin.setInactiveCode('extern "C" {')
    externEnd = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    externEnd.setIdentifier('ifdef')
    externEnd.setBranchExpression('__cplusplus')
    externEnd.setValue(0)
    externEnd.setInactiveCode('}')
    guard.addChildren([define, externBegin])
    guard.addChildren([self.IOR.getGuardedInclude(interface)])
    guard.addChildren(decls)
    guard.addChildren([externEnd])
    header = ASE.Compiler.Cxx.Header.Header()
    header.setFilename(self.getFileBasename(interface) + '.h')
    header.setChildren([guard])
    header.setPurpose(ASE.Compiler.CodePurpose.STUB)
    return header
  
  def getSourceFile(self, interface, defns):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.Include
    import ASE.Compiler.Cxx.Source
    stdInclude = ASE.Compiler.Cxx.Include.Include()
    stdInclude.setIdentifier('<stdlib.h>')
    source = ASE.Compiler.Cxx.Source.Source()
    source.setFilename(self.getFileBasename(interface) + '.c')
    source.addChildren([stdInclude, self.getGuardedInclude(interface), self.IOR.getGuardedInclude(self.SIDL.getASEException(interface)), self.IOR.getGuardedInclude(self.SIDL.getLoader(interface))])
    source.addChildren(defns)
    source.setPurpose(ASE.Compiler.CodePurpose.STUB)
    return source
  
  def copyType(self, type):
    if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      newType = ASE.Compiler.SIDL.Array.Array()
      newType.setIdentifier(type.getIdentifier())
      newType.setAttributes(type.getAttributes())
      newType.setLineNumber(type.getLineNumber())
      newType.setComments(type.getComments())
      newType.setChildren([child for child in type.getChildren()])
      newType.setBaseType(type.getBaseType())
      newType.setType(self.copyType(ASE.Compiler.SIDL.Array.Array(type).getType()))
      return newType
    return type
  
  def copyParameter(self, parameter):
    '''
    This will be replaced by parameter.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Parameter
    newParameter = ASE.Compiler.SIDL.Parameter.Parameter()
    newParameter.setIdentifier(parameter.getIdentifier())
    newParameter.setAttributes(parameter.getAttributes())
    newParameter.setParent(parameter.getParent())
    newParameter.setChildren(parameter.getChildren())
    newParameter.setLineNumber(parameter.getLineNumber())
    newParameter.setComments(parameter.getComments())
    newParameter.setDependencies(parameter.getDependencies())
    newParameter.setType(self.copyType(parameter.getType()))
    newParameter.setAccessMode(parameter.getAccessMode())
    newParameter.setRequired(parameter.getRequired())
    return newParameter
  
  def copyMethod(self, method):
    '''
    This will be replaced by method.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    newMethod = ASE.Compiler.SIDL.Method.Method()
    newMethod.setIdentifier(method.getIdentifier())
    newMethod.setAttributes(method.getAttributes())
    newMethod.setParent(method.getParent())
    newMethod.setChildren([self.copyParameter(ASE.Compiler.SIDL.Parameter.Parameter(child)) for child in method.getChildren()])
    newMethod.setLineNumber(method.getLineNumber())
    newMethod.setComments(method.getComments())
    newMethod.setDependencies(method.getDependencies())
    newMethod.setTemplateIdentifiers(method.getTemplateIdentifiers())
    if not (method.getReturnParameter() is None):
      newMethod.setReturnParameter(self.copyParameter(method.getReturnParameter()))
    newMethod.setExceptions(method.getExceptions())
    newMethod.setStatic(method.getStatic())
    newMethod.setInternal(method.getInternal())
    newMethod.setVirtual(method.getVirtual())
    newMethod.setPurpose(method.getPurpose())
    return newMethod
  
  def checkTemplateParameter(self, name, parameter):
    if not (parameter is None):
      type = parameter.getType()
      if (type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name):
        return True
      elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
        import ASE.Compiler.SIDL.Array
        type = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
        if (type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name):
          return True
    return False
  
  def insertType(self, parameter, type):
    if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      self.insertType(ASE.Compiler.SIDL.Array.Array(parameter.getType()), type)
    else:
      parameter.setType(type)
    return 
  
  def insertTypes(self, method, universe, type):
    import ASE.Compiler.SIDL.Parameter
    name = universe.getIdentifier()
    newMethod = self.copyMethod(method)
    newMethod.setIdentifier(newMethod.getIdentifier() + '_' + type.getFullIdentifier())
    if self.checkTemplateParameter(name, method.getReturnParameter()):
      self.insertType(newMethod.getReturnParameter(), type)
    for (child, newChild) in zip(method.getChildren(), newMethod.getChildren()):
      if self.checkTemplateParameter(name, ASE.Compiler.SIDL.Parameter.Parameter(child)):
        self.insertType(ASE.Compiler.SIDL.Parameter.Parameter(newChild), type)
    return newMethod
  
  def template(self, method, name):
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.SIDL.TypeUniverse
    universe = None
    for parameter in [method.getReturnParameter()] + [ASE.Compiler.SIDL.Parameter.Parameter(child) for child in method.getChildren()]:
      if self.checkTemplateParameter(name, parameter):
        type = parameter.getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(ASE.Compiler.SIDL.Array.Array(type).getType())
        else:
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(type)
        break
    if universe is None:
      raise RuntimeError('Could not find type universe for template ' + name + ' in method ' + method.getFullIdentifier())
    self.templateMethods[method] = [self.insertTypes(method, universe, type) for type in universe.getTypes()]
    return None
  
  def createTemplateMethod(self, method):
    if len(method.getTemplateIdentifiers()):
      if len(method.getTemplateIdentifiers()) > 1:
        raise RuntimeError('Can currently only handle a single template identifier')
      if not (method in self.templateMethods):
        self.template(method, method.getTemplateIdentifiers()[0])
    return None
  
  def getTemplateMethods(self, methods):
    for method in methods:
      if method in self.templateMethods:
        for m in self.templateMethods[method]:
          yield m
      else:
        yield method
    return 
