# Starting imports section
pass
import ASE.BaseClass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.Parser
import ASE.Compiler.ParserException
import ASE.Compiler.Vertex
# Ending imports section
pass
import compiler
# Starting static methods section
pass
# Ending static methods section
pass

class Parser:
  '''
  A Python parser
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
  
  def parse(self, buf):
    '''
    Parse the buffer and return an AST object
    '''
    self.ast = self.createTree(compiler.parse(buf))
    return self.ast
  
  def parseFile(self, filename):
    '''
    Parse the file and return an AST object
    '''
    self.ast = self.createTree(compiler.parseFile(filename))
    return self.ast
  # Ending class methods section
  pass
  
  def createTree(self, ast):
    return Parser.builder().preorder(ast)
  
  class builder(compiler.visitor.ASTVisitor):
    
    def __init__(self):
      compiler.visitor.ASTVisitor.__init__(self)
      self.visitor = self
      return None
    
    def preorder(self, tree, visitor = None, *args):
      '''
      Do preorder walk of tree using visitor
         - For some reason, the default visitor does not return the result of preorder
      '''
      if visitor:
        self.visitor = visitor
      self.visitor.visit = self.dispatch
      return self.dispatch(tree, *args)
    
    def visitComment(self, comment):
      import ASE.Compiler.Python.Comment
      c = ASE.Compiler.Python.Comment.Comment()
      c.setComment(comment)
      return c
    
    def visitAdd(self, node):
      import ASE.Compiler.Python.Add
      a = ASE.Compiler.Python.Add.Add()
      a.setChildren([self.visit(node.left), self.visit(node.right)])
      return a
    
    def visitAnd(self, node):
      import ASE.Compiler.Python.And
      a = ASE.Compiler.Python.And.And()
      a.setChildren([self.visit(n) for n in node.nodes])
      return a
    
    def visitAssAttr(self, node):
      import ASE.Compiler.Python.AssAttr
      a = ASE.Compiler.Python.AssAttr.AssAttr()
      a.setIdentifier(node.attrname)
      a.setChildren([self.visit(node.expr)])
      a.setFlags(str(node.flags))
      return a
    
    def visitAssList(self, node):
      import ASE.Compiler.Python.AssList
      a = ASE.Compiler.Python.AssList.AssList()
      a.setChildren([self.visit(n) for n in node.nodes])
      return a
    
    def visitAssName(self, node):
      import ASE.Compiler.Python.AssName
      a = ASE.Compiler.Python.AssName.AssName()
      a.setIdentifier(node.name)
      a.setFlags(str(node.flags))
      return a
    
    def visitAssTuple(self, node):
      import ASE.Compiler.Python.AssTuple
      a = ASE.Compiler.Python.AssTuple.AssTuple()
      a.setChildren([self.visit(n) for n in node.nodes])
      return a
    
    def visitAssign(self, node):
      import ASE.Compiler.Python.Assign
      a = ASE.Compiler.Python.Assign.Assign()
      a.setValue(self.visit(node.expr))
      a.setChildren([self.visit(n) for n in node.nodes])
      return a
    
    def visitAugAssign(self, node):
      import ASE.Compiler.Python.AugAssign
      a = ASE.Compiler.Python.AugAssign.AugAssign()
      a.setValue(self.visit(node.expr))
      a.setOp(node.op)
      a.setChildren([self.visit(node.node)])
      return a
    
    def visitBitand(self, node):
      import ASE.Compiler.Python.Bitand
      b = ASE.Compiler.Python.Bitand.Bitand()
      b.setChildren([self.visit(n) for n in node.nodes])
      return b
    
    def visitBitor(self, node):
      import ASE.Compiler.Python.Bitor
      b = ASE.Compiler.Python.Bitor.Bitor()
      b.setChildren([self.visit(n) for n in node.nodes])
      return b
    
    def visitBitxor(self, node):
      import ASE.Compiler.Python.Bitxor
      b = ASE.Compiler.Python.Bitxor.Bitxor()
      b.setChildren([self.visit(n) for n in node.nodes])
      return b
    
    def visitBreak(self, node):
      import ASE.Compiler.Python.Break
      b = ASE.Compiler.Python.Break.Break()
      return b
    
    def visitCallFunc(self, node):
      import ASE.Compiler.Python.CallFunc
      c = ASE.Compiler.Python.CallFunc.CallFunc()
      c.setChildren([self.visit(node.node)])
      c.setArguments([self.visit(arg) for arg in node.args])
      if not (node.star_args is None):
        c.setOptionalArguments(self.visit(node.star_args))
      if not (node.dstar_args is None):
        c.setKeywordsArguments(self.visit(node.dstar_args))
      return c
    
    def toName(self, klass):
      import compiler.ast
      if isinstance(klass, str):
        return klass
      name = ''
      while isinstance(klass, compiler.ast.Getattr):
        if name:
          name = '.' + name
        name = klass.attrname + name
        klass = klass.expr
      if not (isinstance(klass, compiler.ast.Name)):
        raise RuntimeError('Base class must end in a Name: ' + str(klass))
      if name:
        name = '.' + name
      return klass.name + name
    
    def visitClass(self, node):
      import ASE.Compiler.Python.Class
      c = ASE.Compiler.Python.Class.Class()
      parents = []
      for base in node.bases:
        b = ASE.Compiler.Python.Class.Class()
        b.setIdentifier(self.toName(base))
        parents.append(b)
      c.setIdentifier(node.name)
      c.setParents(parents)
      if node.doc:
        c.setComments([self.visitComment(node.doc)])
      c.setChildren([self.visit(node.code)])
      return c
    
    def visitCompare(self, node):
      import ASE.Compiler.Python.Compare
      c = ASE.Compiler.Python.Compare.Compare()
      c.setChildren([self.visit(node.expr)])
      for (op, expr) in node.ops:
        c.setTestExpr(op, self.visit(expr))
      return c
    
    def escapeString(self, s):
      '''
      We only escape single quotes, since we are always single-quoting and double quotes are harmless
      '''
      return s.replace('\\', '\\\\').replace('\'', '\\\'').replace('\n', '\\n').replace('\t', '\\t')
    
    def visitConst(self, node):
      if node.value is None:
        import ASE.Compiler.Python.PyNone
        c = ASE.Compiler.Python.PyNone.PyNone()
      else:
        import ASE.Compiler.Python.Const
        c = ASE.Compiler.Python.Const.Const()
        if isinstance(node.value, str):
          c.setValue_string('\'' + self.escapeString(node.value) + '\'')
        elif isinstance(node.value, complex):
          c.setValue_dcomplex(node.value)
        elif isinstance(node.value, float):
          c.setValue_double(node.value)
        elif isinstance(node.value, int):
          c.setValue_int(node.value)
        else:
          c.setValue_string(str(node.value))
      return c
    
    def visitContinue(self, node):
      import ASE.Compiler.Python.Continue
      c = ASE.Compiler.Python.Continue.Continue()
      return c
    
    def visitDict(self, node):
      import ASE.Compiler.Python.Dict
      d = ASE.Compiler.Python.Dict.Dict()
      for item in node.items:
        d.addChildren([self.visit(n) for n in item])
      return d
    
    def visitDiscard(self, node):
      import ASE.Compiler.Python.Discard
      d = ASE.Compiler.Python.Discard.Discard()
      d.setChildren([self.visit(node.expr)])
      return d
    
    def visitDiv(self, node):
      import ASE.Compiler.Python.Div
      d = ASE.Compiler.Python.Div.Div()
      d.setChildren([self.visit(node.left), self.visit(node.right)])
      return d
    
    def visitEllipsis(self, node):
      import ASE.Compiler.Python.Ellipsis
      e = ASE.Compiler.Python.Ellipsis.Ellipsis()
      return e
    
    def visitFor(self, node):
      import ASE.Compiler.Python.For
      f = ASE.Compiler.Python.For.For()
      f.setChildren([self.visit(node.body)])
      f.setVar(self.visit(node.assign))
      f.setList(self.visit(node.list))
      return f
    
    def visitFrom(self, node):
      import ASE.Compiler.Python.From
      f = ASE.Compiler.Python.From.From()
      f.setModuleNames([node.modname])
      f.setImportedNames([name[0] for name in node.names])
      for (name, alias) in node.names:
        if not (alias is None):
          f.setAlias(name, alias)
      return f
    
    def visitFunction(self, node):
      import ASE.Compiler.Python.Function
      f = ASE.Compiler.Python.Function.Function()
      f.setIdentifier(node.name)
      f.setArgumentNames(node.argnames)
      f.setDefaults([self.visit(default) for default in node.defaults])
      if node.doc:
        f.setComments([self.visitComment(node.doc)])
      f.setChildren([self.visit(node.code)])
      f.setFlags(str(node.flags))
      return f
    
    def visitGetattr(self, node):
      import ASE.Compiler.Python.GetAttr
      g = ASE.Compiler.Python.GetAttr.GetAttr()
      g.setChildren([self.visit(node.expr)])
      g.setName(node.attrname)
      return g
    
    def visitGlobal(self, node):
      import ASE.Compiler.Python.Global
      g = ASE.Compiler.Python.Global.Global()
      g.setNames(node.names)
      return g
    
    def visitIf(self, node):
      import ASE.Compiler.Python.If
      i = ASE.Compiler.Python.If.If()
      for test in node.tests:
        i.addChildren([self.visit(t) for t in test])
      if not (node.else_ is None):
        i.addChildren([self.visit(node.else_)])
      return i
    
    def visitImport(self, node):
      import ASE.Compiler.Python.Import
      i = ASE.Compiler.Python.Import.Import()
      i.setModuleNames([t[0] for t in node.names])
      for (name, alias) in node.names:
        if not (alias is None):
          i.setAlias(name, alias)
      return i
    
    def visitInvert(self, node):
      import ASE.Compiler.Python.Invert
      i = ASE.Compiler.Python.Invert.Invert()
      i.setChildren([self.visit(n) for n in node.nodes])
      return i
    
    def visitKeyword(self, node):
      import ASE.Compiler.Python.Keyword
      k = ASE.Compiler.Python.Keyword.Keyword()
      k.setIdentifier(node.name)
      k.setChildren([self.visit(node.expr)])
      return k
    
    def visitLambda(self, node):
      import ASE.Compiler.Python.Lambda
      l = ASE.Compiler.Python.Lambda.Lambda()
      l.setArgumentNames(node.argnames)
      l.setDefaults(node.defaults)
      l.setChildren([self.visit(node.code)])
      l.setFlags(str(node.flags))
      return l
    
    def visitLeftShift(self, node):
      import ASE.Compiler.Python.LeftShift
      l = ASE.Compiler.Python.LeftShift.LeftShift()
      l.setChildren([self.visit(node.left), self.visit(node.right)])
      return l
    
    def visitList(self, node):
      import ASE.Compiler.Python.List
      l = ASE.Compiler.Python.List.List()
      l.setChildren([self.visit(n) for n in node.nodes])
      return l
    
    def visitListComp(self, node):
      import ASE.Compiler.Python.ListComp
      l = ASE.Compiler.Python.ListComp.ListComp()
      l.setChildren([self.visit(node.expr)])
      l.setQualifiers([self.visit(q) for q in node.quals])
      return l
    
    def visitListCompFor(self, node):
      import ASE.Compiler.Python.ListCompFor
      l = ASE.Compiler.Python.ListCompFor.ListCompFor()
      l.setChildren([self.visit(ifNode) for ifNode in node.ifs])
      l.setVar(self.visit(node.assign))
      l.setList(self.visit(node.list))
      return l
    
    def visitListCompIf(self, node):
      import ASE.Compiler.Python.ListCompIf
      l = ASE.Compiler.Python.ListCompIf.ListCompIf()
      l.setChildren([self.visit(node.test)])
      return l
    
    def visitMod(self, node):
      import ASE.Compiler.Python.Mod
      m = ASE.Compiler.Python.Mod.Mod()
      m.setChildren([self.visit(node.left), self.visit(node.right)])
      return m
    
    def visitModule(self, node):
      import ASE.Compiler.Python.Module
      m = ASE.Compiler.Python.Module.Module()
      if node.doc:
        m.setComments([self.visitComment(node.doc)])
      m.setChildren([self.visit(node.node)])
      return m
    
    def visitMul(self, node):
      import ASE.Compiler.Python.Mul
      m = ASE.Compiler.Python.Mul.Mul()
      m.setChildren([self.visit(node.left), self.visit(node.right)])
      return m
    
    def visitName(self, node):
      import ASE.Compiler.Python.Name
      n = ASE.Compiler.Python.Name.Name()
      n.setIdentifier(node.name)
      return n
    
    def visitNot(self, node):
      import ASE.Compiler.Python.Not
      n = ASE.Compiler.Python.Not.Not()
      n.setChildren([self.visit(node.expr)])
      return n
    
    def visitOr(self, node):
      import ASE.Compiler.Python.Or
      o = ASE.Compiler.Python.Or.Or()
      o.setChildren([self.visit(n) for n in node.nodes])
      return o
    
    def visitPass(self, node):
      import ASE.Compiler.Python.Pass
      p = ASE.Compiler.Python.Pass.Pass()
      return p
    
    def visitPower(self, node):
      import ASE.Compiler.Python.Power
      p = ASE.Compiler.Python.Power.Power()
      p.setChildren([self.visit(node.left), self.visit(node.right)])
      return p
    
    def visitPrint(self, node):
      import ASE.Compiler.Python.Print
      p = ASE.Compiler.Python.Print.Print()
      p.setChildren([self.visit(n) for n in node.nodes])
      if not (node.dest is None):
        p.setDest(self.visit(node.dest))
      return p
    
    def visitPrintnl(self, node):
      import ASE.Compiler.Python.Printnl
      p = ASE.Compiler.Python.Printnl.Printnl()
      p.setChildren([self.visit(n) for n in node.nodes])
      if not (node.dest is None):
        p.setDest(self.visit(node.dest))
      return p
    
    def visitRaise(self, node):
      import ASE.Compiler.Python.Raise
      r = ASE.Compiler.Python.Raise.Raise()
      if not (node.expr1 is None):
        r.addChildren([self.visit(node.expr1)])
      if not (node.expr2 is None):
        r.addChildren([self.visit(node.expr2)])
      if not (node.expr3 is None):
        r.addChildren([self.visit(node.expr3)])
      return r
    
    def visitReturn(self, node):
      import ASE.Compiler.Python.Return
      r = ASE.Compiler.Python.Return.Return()
      if not (node.value is None):
        r.setChildren([self.visit(node.value)])
      return r
    
    def visitRightShift(self, node):
      import ASE.Compiler.Python.RightShift
      r = ASE.Compiler.Python.RightShift.RightShift()
      r.setChildren([self.visit(node.left), self.visit(node.right)])
      return r
    
    def visitSlice(self, node):
      import ASE.Compiler.Python.Slice
      s = ASE.Compiler.Python.Slice.Slice()
      s.setChildren([self.visit(node.expr)])
      if not (node.lower is None):
        s.setLower(self.visit(node.lower))
      if not (node.upper is None):
        s.setUpper(self.visit(node.upper))
      s.setFlags(str(node.flags))
      return s
    
    def visitStmt(self, node):
      import ASE.Compiler.Python.Stmt
      s = ASE.Compiler.Python.Stmt.Stmt()
      s.setChildren(map(self.visit, node.nodes))
      return s
    
    def visitSub(self, node):
      import ASE.Compiler.Python.Sub
      s = ASE.Compiler.Python.Sub.Sub()
      s.setChildren([self.visit(node.left), self.visit(node.right)])
      return s
    
    def visitSubscript(self, node):
      import ASE.Compiler.Python.Subscript
      s = ASE.Compiler.Python.Subscript.Subscript()
      s.setChildren([self.visit(node.expr)])
      s.setSubscripts([self.visit(sub) for sub in node.subs])
      s.setFlags(str(node.flags))
      return s
    
    def visitTryExcept(self, node):
      import ASE.Compiler.Python.TryExcept
      import ASE.Compiler.Python.ExceptionHandler
      t = ASE.Compiler.Python.TryExcept.TryExcept()
      t.setChildren([self.visit(node.body)])
      for handler in node.handlers:
        h = ASE.Compiler.Python.ExceptionHandler.ExceptionHandler()
        h.setType(self.visit(handler[0]))
        if not (handler[1] is None):
          h.setVariable(self.visit(handler[1]))
        h.setChildren([self.visit(handler[2])])
        t.addHandlers([h])
      return t
    
    def visitTryFinally(self, node):
      import ASE.Compiler.Python.TryFinally
      t = ASE.Compiler.Python.TryFinally.TryFinally()
      t.setChildren([self.visit(node.body)])
      t.setFinal(node.final)
      return t
    
    def visitTuple(self, node):
      import ASE.Compiler.Python.Tuple
      t = ASE.Compiler.Python.Tuple.Tuple()
      t.setChildren([self.visit(n) for n in node.nodes])
      return t
    
    def visitUnaryAdd(self, node):
      import ASE.Compiler.Python.UnaryAdd
      u = ASE.Compiler.Python.UnaryAdd.UnaryAdd()
      u.setChildren([self.visit(node.expr)])
      return u
    
    def visitUnarySub(self, node):
      import ASE.Compiler.Python.UnarySub
      u = ASE.Compiler.Python.UnarySub.UnarySub()
      u.setChildren([self.visit(node.expr)])
      return u
    
    def visitWhile(self, node):
      import ASE.Compiler.Python.While
      w = ASE.Compiler.Python.While.While()
      w.setChildren([self.visit(node.body)])
      w.setTest(self.visit(node.test))
      return w
    
    def visitYield(self, node):
      import ASE.Compiler.Python.Yield
      y = ASE.Compiler.Python.Yield.Yield()
      y.setChildren([self.visit(node.value)])
      return y
