class Cxx(object):
  def __init__(self):
    self.integers = {}

    return

  def clear(self):
    del self.typeMap
    del self.nullVar
    self.integers.clear()
    return

  def getTypeMap(self):
    if not hasattr(self, '_typeMap'):
      self._typeMap = self.createTypeMap()
    return self._typeMap
  def setTypeMap(self, typeMap):
    self._typeMap = typeMap
    return
  def delTypeMap(self):
    if hasattr(self, '_typeMap'):
      del self._typeMap
    return
  typeMap = property(getTypeMap, setTypeMap, delTypeMap, doc = 'A map from typenames to Cxx type objects')

  def getNullVar(self):
    if not hasattr(self, '_nullVar'):
      self._nullVar = self.getVar('NULL')
    return self._nullVar
  def setNullVar(self, nullVar):
    self._nullVar = nullVar
    return
  def delNullVar(self):
    if hasattr(self, '_nullVar'):
      del self._nullVar
    return
  nullVar = property(getNullVar, setNullVar, delNullVar, doc = 'The C variable NULL')
  
  def createTypeMap(self):
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.Type
    typeMap = {}

    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('bool')
    cxxType.setBaseType(1)
    typeMap['bool'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('char')
    cxxType.setBaseType(1)
    typeMap['char'] = cxxType
    cxxType = ASE.Compiler.Cxx.Pointer.Pointer()
    cxxType.setChildren([typeMap['char']])
    typeMap['char pointer'] = cxxType
    cxxType = ASE.Compiler.Cxx.Struct.Struct()
    cxxType.setIdentifier('complex<double>')
    typeMap['dcomplex'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('double')
    cxxType.setBaseType(1)
    typeMap['double'] = cxxType
    cxxType = ASE.Compiler.Cxx.Pointer.Pointer()
    cxxType.setChildren([typeMap['double']])
    typeMap['double pointer'] = cxxType
    cxxType = ASE.Compiler.Cxx.Struct.Struct()
    cxxType.setIdentifier('complex<float>')
    typeMap['fcomplex'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('float')
    cxxType.setBaseType(1)
    typeMap['float'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('int')
    cxxType.setBaseType(1)
    typeMap['int'] = cxxType
    cxxType = ASE.Compiler.Cxx.Pointer.Pointer()
    cxxType.setChildren([typeMap['int']])
    typeMap['int pointer'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('long long')
    cxxType.setBaseType(1)
    typeMap['long'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('void')
    cxxType.setBaseType(1)
    typeMap['void'] = cxxType
    cxxType = ASE.Compiler.Cxx.Pointer.Pointer()
    cxxType.setChildren([typeMap['void']])
    typeMap['void pointer'] = cxxType
    typeMap['opaque'] = typeMap['void pointer']
    cxxType = ASE.Compiler.Cxx.Pointer.Pointer()
    cxxType.setChildren([typeMap['void pointer']])
    typeMap['void pointer pointer'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('std::str')
    cxxType.setBaseType(1)
    typeMap['string'] = cxxType
    return typeMap
  
  def getType(self, sidlType):
    '''
    Return the C type corresponding to this sidl type
    '''
    cType = None
    if sidlType.getBaseType():
      try:
        cType = self.typeMap[sidlType.getIdentifier()]
      except KeyError:
        self.raiseException('No base type: ' + str(sidlType.getIdentifier()))
    elif sidlType.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      import ASE.Compiler.Cxx.Pointer
      cType = ASE.Compiler.Cxx.Pointer.Pointer()
      cType.setChildren([self.getType(ASE.Compiler.SIDL.Array.Array(sidlType).getType())])
    else:
      import ASE.Compiler.Cxx.Type
      cType = ASE.Compiler.Cxx.Type.Type()
      cType.setIdentifier(sidlType.getIdentifier())
    return cType
  
  def getNull(self):
    if self.nullVar is None:
      raise RuntimeError('Object prematurely cleared')
    return self.nullVar
  
  def getInteger(self, num):
    if not num in self.integers:
      import ASE.Compiler.Cxx.Constant
      integer = ASE.Compiler.Cxx.Constant.Constant()
      integer.setValue_int(num)
      self.integers[num] = integer
    return self.integers[num]
  
  def getDouble(self, num):
    import ASE.Compiler.Cxx.Constant
    double = ASE.Compiler.Cxx.Constant.Constant()
    double.setValue_double(num)
    return double

  def getInclude(self, filename):
    import ASE.Compiler.Cxx.Include
    include = ASE.Compiler.Cxx.Include.Include()
    if filename[0] == '<' and filename[-1] == '>':
      include.setIdentifier(filename)
    else:
      include.setIdentifier('"' + filename + '"')
    return include
  
  def getString(self, s):
    import ASE.Compiler.Cxx.Constant
    string = ASE.Compiler.Cxx.Constant.Constant()
    string.setIdentifier(s)
    string.setValue_string(s)
    return string
  
  def getComment(self, comment):
    import ASE.Compiler.Cxx.Comment
    c = ASE.Compiler.Cxx.Comment.Comment()
    c.setComment(comment)
    return c
  
  def getVar(self, name):
    import ASE.Compiler.Cxx.Variable
    var = ASE.Compiler.Cxx.Variable.Variable()
    var.setIdentifier(name)
    return var
  
  def getAddress(self, exp):
    import ASE.Compiler.Cxx.Address
    addr = ASE.Compiler.Cxx.Address.Address()
    addr.setChildren([exp])
    return addr
  
  def getIndirection(self, exp):
    import ASE.Compiler.Cxx.Indirection
    ind = ASE.Compiler.Cxx.Indirection.Indirection()
    ind.setChildren([exp])
    return ind

  def getMinus(self, exp):
    import ASE.Compiler.Cxx.UnaryMinus
    minus = ASE.Compiler.Cxx.UnaryMinus.UnaryMinus()
    minus.setChildren([exp])
    return minus
  
  def getGroup(self, exp):
    import ASE.Compiler.Cxx.Group
    group = ASE.Compiler.Cxx.Group.Group()
    group.setChildren([exp])
    return group
  
  def getDeclGroup(self, exp):
    import ASE.Compiler.Cxx.DeclaratorGroup
    group = ASE.Compiler.Cxx.DeclaratorGroup.DeclaratorGroup()
    group.setChildren([exp])
    return group

  def getPointer(self, exp):
    import ASE.Compiler.Cxx.Pointer
    pointer = ASE.Compiler.Cxx.Pointer.Pointer()
    pointer.setChildren([exp])
    return pointer
  
  def getDecl(self, declarator, comment = None):
    '''
    Make a declaration from a declarator
    '''
    import ASE.Compiler.Cxx.Declaration
    decl = ASE.Compiler.Cxx.Declaration.Declaration()
    decl.setChildren([declarator])
    if not comment is None:
      decl.setComments([self.getComment(comment)])
    return decl
  
  def getInitializer(self, type, value = None, isList = 0):
    import ASE.Compiler.Cxx.Constant
    import ASE.Compiler.Cxx.Initializer
    init = ASE.Compiler.Cxx.Initializer.Initializer()
    if not value is None:
      if not isinstance(value, list):
        value = [value]
      init.setChildren(value)
    elif type.getBaseType():
      typeName = type.getIdentifier()
      if typeName in ['bool', 'char', 'int', 'long', 'long long', 'int32_t', 'int64_t']:
        init.setChildren([self.getInteger(0)])
      elif typeName in ['double', 'float']:
        init.setChildren([self.getDouble(0.0)])
      elif typeName in ['dcomplex', 'fcomplex']:
        init.setChildren([self.getDouble(0.0), self.getDouble(0.0)])
      elif typeName in ['string']:
        init.setChildren([self.getNull()])
    elif type.isInstanceOf('ASE.Compiler.Cxx.Pointer'):
      init.setChildren([self.getNull()])
    elif type.isInstanceOf('ASE.Compiler.Cxx.Struct'):
      if type.getIdentifier() in ['SIDL_dcomplex', 'SIDL_fcomplex']:
        init.setChildren([self.getDouble(0.0), self.getDouble(0.0)])
      else:
        return None
    elif type.isInstanceOf('ASE.Compiler.Cxx.Type'):
      if type.getIdentifier() in ['Py_complex']:
        init.setChildren([self.getDouble(0.0), self.getDouble(0.0)])
      else:
        init.setChildren([self.getInteger(0)])
    else:
      init.setChildren([self.getInteger(0)])
    if not isList is None:
      init.setList(isList)
    return init
  
  def getDeclaration(self, var, type, initializer = None, comment = None, isStatic = 0, isForward = 0):
    import ASE.Compiler.Cxx.Declarator
    decl = ASE.Compiler.Cxx.Declarator.Declarator()
    decl.setIdentifier(var.getIdentifier())
    decl.setType(type)
    decl.setStatic(isStatic)
    if initializer is None:
      if not isForward:
        init = self.getInitializer(type)
        if not init is None:
          decl.setInitializer(init)
    else:
      decl.setInitializer(initializer)
    return self.getDecl(decl, comment)

  def getParameter(self, name, paramType):
    import ASE.Compiler.Cxx.Parameter
    param = ASE.Compiler.Cxx.Parameter.Parameter()
    param.setIdentifier(name)
    param.setType(paramType)
    return param

  def getArray(self, var, entryType, size = None, initializer = None, isStatic = 0):
    import ASE.Compiler.Cxx.Array
    arrayDecl = ASE.Compiler.Cxx.Array.Array()
    arrayDecl.setChildren([self.getValue(var)])
    arrayDecl.setType(entryType)
    if not size is None:
      arrayDecl.setSize(self.getValue(size))
    if not initializer is None:
      arrayDecl.setInitializer(initializer)
    arrayDecl.setStatic(isStatic)
    return self.getDecl(arrayDecl)

  def getSizeof(self, type):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Sizeof
    typeDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    typeDecl.setType(type)
    sizeof = ASE.Compiler.Cxx.Sizeof.Sizeof()
    sizeof.setTypeName(typeDecl)
    return sizeof
  
  def getExpStmt(self, exp, comment = None):
    '''
    Make a statement from an expression
    '''
    import ASE.Compiler.Cxx.ExpressionStatement
    stmt = ASE.Compiler.Cxx.ExpressionStatement.ExpressionStatement()
    stmt.setChildren([exp])
    if not comment is None:
      stmt.setComments([self.getComment(comment)])
    return stmt

  def getNegation(self, exp):
    import ASE.Compiler.Cxx.Negation
    neg = ASE.Compiler.Cxx.Negation.Negation()
    neg.setChildren([exp])
    return neg

  def getAddition(self, a, b):
    import ASE.Compiler.Cxx.Addition
    exp = ASE.Compiler.Cxx.Addition.Addition()
    exp.setChildren([a, b])
    return exp

  def getSubtraction(self, a, b):
    import ASE.Compiler.Cxx.Subtraction
    exp = ASE.Compiler.Cxx.Subtraction.Subtraction()
    exp.setChildren([a, b])
    return exp

  def getMultiplication(self, a, b):
    import ASE.Compiler.Cxx.Multiplication
    exp = ASE.Compiler.Cxx.Multiplication.Multiplication()
    exp.setChildren([a, b])
    return exp

  def getDivision(self, a, b):
    import ASE.Compiler.Cxx.Division
    exp = ASE.Compiler.Cxx.Division.Division()
    exp.setChildren([a, b])
    return exp

  def getArrayRef(self, arrayName, index):
    '''arrayName is a Variable, and index is processed by getValue()'''
    import ASE.Compiler.Cxx.ArrayReference
    arrayRef = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    arrayRef.setChildren([arrayName])
    arrayRef.setIndex(self.getValue(index))
    return arrayRef

  def getStructRef(self, structName, member, isPointer = 1):
    '''structName is a Variable, and member is a string'''
    import ASE.Compiler.Cxx.StructReference
    structRef = ASE.Compiler.Cxx.StructReference.StructReference()
    structRef.setChildren([structName])
    structRef.setMember(member)
    structRef.setPointer(isPointer)
    return structRef

  def getAssignment(self, target, value):
    import ASE.Compiler.Cxx.Assignment
    assignment = ASE.Compiler.Cxx.Assignment.Assignment()
    assignment.setChildren([target, value])
    return assignment

  def castToType(self, exp, type, numPointers = 0):
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Pointer
    decl = ASE.Compiler.Cxx.Declarator.Declarator()
    for n in range(numPointers):
      pointer = ASE.Compiler.Cxx.Pointer.Pointer()
      pointer.setChildren([type])
      type = pointer
    decl.setType(type)
    cast = ASE.Compiler.Cxx.Cast.Cast()
    cast.setChildren([exp])
    cast.setTypeName(decl)
    return cast
  
  def castToNamedType(self, exp, typeName, numPointers = 0, isConst = 0, isStruct = 0):
    if isStruct:
      import ASE.Compiler.Cxx.Struct
      type = ASE.Compiler.Cxx.Struct.Struct()
      type.setIdentifier(typeName)
    else:
      if typeName in self.typeMap and not isConst:
        type = self.typeMap[typeName]
      else:
        import ASE.Compiler.Cxx.Type
        type = ASE.Compiler.Cxx.Type.Type()
        type.setIdentifier(typeName)
        type.setBaseType(typeName in self.typeMap.keys())
    if isConst:
      type.setConst(1)
    return self.castToType(exp, type, numPointers)
  
  def castToVoidPointer(self, exp):
    return self.castToNamedType(exp, 'void', 1)
  
  def castToString(self, constString):
    return self.castToNamedType(self.getString(constString), 'char', 1)

  def getValue(self, value):
    '''Return a constant or variable which best reflect the Python object'''
    if isinstance(value, str):
      cxxValue = self.getVar(value)
    elif isinstance(value, int):
      cxxValue = self.getInteger(value)
    elif isinstance(value, float):
      cxxValue = self.getReal(value)
    else:
      cxxValue = value
    return cxxValue

  def getFunctionCall(self, name, args = []):
    import ASE.Compiler.Cxx.FunctionCall
    funcCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    funcCall.setChildren([self.getValue(name)])
    funcCall.addArguments([self.getValue(arg) for arg in args])
    return funcCall

  def getSimpleLoop(self, loopVar, lowerBound, upperBound, increment = 1, allowEquality = 0):
    import ASE.Compiler.Cxx.For

    if allowEquality:
      import ASE.Compiler.Cxx.LessThanOrEqual
      comp = ASE.Compiler.Cxx.LessThanOrEqual.LessThanOrEqual()
    else:
      import ASE.Compiler.Cxx.LessThan
      comp = ASE.Compiler.Cxx.LessThan.LessThan()
    comp.setChildren([loopVar, self.getValue(upperBound)])
    if increment == 1:
      import ASE.Compiler.Cxx.Increment
      inc  = ASE.Compiler.Cxx.Increment.Increment()
      inc.setChildren([loopVar])
      inc.setPostfix(1)
    elif increment == -1:
      import ASE.Compiler.Cxx.Increment
      inc  = ASE.Compiler.Cxx.Increment.Increment()
      inc.setChildren([loopVar])
      inc.setDecrement(1)
      inc.setPostfix(1)
    else:
      import ASE.Compiler.Cxx.AdditionAssignment
      inc = ASE.Compiler.Cxx.AdditionAssignment.AdditionAssignment()
      inc.setChildren([loopVar, self.getValue(increment)])
    loop = ASE.Compiler.Cxx.For.For()
    loop.setInitialization(self.getAssignment(loopVar, self.getValue(lowerBound)))
    loop.setBranch(comp)
    loop.setIncrement(inc)
    return loop

  def getReturn(self, exp):
    import ASE.Compiler.Cxx.Return
    returnStmt = ASE.Compiler.Cxx.Return.Return()
    returnStmt.setChildren([exp])
    return returnStmt
