class IOR(object):
  def __init__(self, Cxx = None, SIDL = None):
    if Cxx is None:
      import ASE.Compiler.Python.Cxx
      self.Cxx = ASE.Compiler.Python.Cxx.Cxx()
    else:
      self.Cxx = Cxx
    if SIDL is None:
      import ASE.Compiler.Python.SIDL
      self.SIDL = ASE.Compiler.Python.SIDL.SIDL()
    else:
      self.SIDL = SIDL
    self.external = {}

    self.epv = {}

    self.sepv = {}

    return

  def clear(self):
    self.Cxx.clear()
    self.SIDL.clear()
    del self.typeMap
    self.external.clear()
    self.epv.clear()
    self.sepv.clear()
    return

  def getTypeMap(self):
    if not hasattr(self, '_typeMap'):
      self._typeMap = self.createTypeMap()
    return self._typeMap
  def setTypeMap(self, typeMap):
    self._typeMap = typeMap
    return
  def delTypeMap(self):
    if hasattr(self, '_typeMap'):
      del self._typeMap
    return
  typeMap = property(getTypeMap, setTypeMap, delTypeMap, doc = 'A map from typenames to IOR type objects')
  
  def raiseException(self, message, trace = ''):
    import ASE.Compiler.CompilerException
    import sys
    e = ASE.Compiler.CompilerException.CompilerException()
    e.setMessage(message)
    if isinstance(trace, list):
      for line in trace:
        e.addStackLine(line.strip())
    elif trace:
      e.addStackLine(trace)
    e.addStackEntry(__file__, sys._getframe(1).f_lineno, sys._getframe(1).f_code.co_name)
    raise ASE.Compiler.CompilerException.Exception, e
  
  def createTypeMap(self):
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.Type
    typeMap = {}

    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('int')
    iorType.setBaseType(1)
    typeMap['bool'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('char')
    iorType.setBaseType(1)
    typeMap['char'] = iorType
    iorType = ASE.Compiler.Cxx.Struct.Struct()
    iorType.setIdentifier('SIDL_dcomplex')
    typeMap['dcomplex'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('double')
    iorType.setBaseType(1)
    typeMap['double'] = iorType
    iorType = ASE.Compiler.Cxx.Struct.Struct()
    iorType.setIdentifier('SIDL_fcomplex')
    typeMap['fcomplex'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('float')
    iorType.setBaseType(1)
    typeMap['float'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('int32_t')
    iorType.setBaseType(1)
    typeMap['int'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('int64_t')
    iorType.setBaseType(1)
    typeMap['long'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('void')
    iorType.setBaseType(1)
    typeMap['void'] = iorType
    iorType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorType.setChildren([typeMap['void']])
    typeMap['void pointer'] = iorType
    typeMap['opaque'] = typeMap['void pointer']
    methodType = ASE.Compiler.Cxx.Type.Type()
    methodType.setIdentifier('PyObject')
    iorType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorType.setChildren([methodType])
    typeMap['function'] = iorType
    constChar = ASE.Compiler.Cxx.Type.Type()
    constChar.setIdentifier('char')
    constChar.setBaseType(1)
    constChar.setConst(1)
    iorType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorType.setChildren([constChar])
    typeMap['string'] = iorType
    return typeMap
  
  def getType(self, type):
    '''
    Return the C IOR type corresponding to this sidl type
    '''
    import ASE.Compiler.Cxx.Enumeration
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    iorType = None
    if type.getBaseType():
      try:
        iorType = self.typeMap[type.getIdentifier()]
      except KeyError:
        self.raiseException('No base type: ' + str(type.getIdentifier()))
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      import ASE.Compiler.Cxx.Enumeration
      iorType = ASE.Compiler.Cxx.Enumeration.Enumeration()
      iorType.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__enum')
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      entryType = ASE.Compiler.SIDL.Array.Array(type).getType()
      iorStruct = ASE.Compiler.Cxx.Struct.Struct()
      iorStruct.setIdentifier(entryType.getFullIdentifier().replace('.', '_') + '__array')
      iorType = ASE.Compiler.Cxx.Pointer.Pointer()
      iorType.setChildren([iorStruct])
    elif type.isInstanceOf('ASE.Compiler.SIDL.Iterator'):
      import ASE.Compiler.SIDL.Iterator
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      entryType = ASE.Compiler.SIDL.Iterator.Iterator(type).getType()
      iorStruct = ASE.Compiler.Cxx.Struct.Struct()
      iorStruct.setIdentifier(entryType.getFullIdentifier().replace('.', '_') + '__iterator')
      iorType = ASE.Compiler.Cxx.Pointer.Pointer()
      iorType.setChildren([iorStruct])
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      iorStruct = ASE.Compiler.Cxx.Struct.Struct()
      iorStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__object')
      iorType = ASE.Compiler.Cxx.Pointer.Pointer()
      iorType.setChildren([iorStruct])
    else:
      import ASE.Compiler.Cxx.Type
      iorType = ASE.Compiler.Cxx.Type.Type()
      iorType.setIdentifier(type.getIdentifier())
    return iorType

  def getBaseType(self, interface):
    return self.getType(self.SIDL.getBaseInterface(interface))

  def getEpvType(self, type):
    if not type in self.epv:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct

      epvStruct = ASE.Compiler.Cxx.Struct.Struct()
      epvStruct.setIdentifier(type.getFullIdentifier().replace('.', '_')+'__epv')
      epvType   = ASE.Compiler.Cxx.Pointer.Pointer()
      epvType.setChildren([epvStruct])
      self.epv[type] = epvType
    return self.epv[type]
  
  def getExternalType(self, type):
    if not type in self.external:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      extStruct = ASE.Compiler.Cxx.Struct.Struct()
      extStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__external')
      extStruct.setConst(1)
      extType = ASE.Compiler.Cxx.Pointer.Pointer()
      extType.setChildren([extStruct])
      self.external[type] = extType
    return self.external[type]
  
  def getStaticExternalType(self, type):
    if not type in self.sepv:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      sepvStruct = ASE.Compiler.Cxx.Struct.Struct()
      sepvStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__sepv')
      sepvStruct.setConst(1)
      sepvType = ASE.Compiler.Cxx.Pointer.Pointer()
      sepvType.setChildren([sepvStruct])
      self.sepv[type] = sepvType
    return self.sepv[type]

  def getTrueIOR(self, ior, isPointer = 1):
    return self.Cxx.getStructRef(ior, 'd_object', isPointer)
  
  def getParameter(self, parameter, mode = None):
    '''
    Return the C IOR parameter corresponding to this sidl parameter
    '''
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.Cxx.Comment
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    if mode is None:
      type = parameter.getType()
      mode = parameter.getAccessMode()
    else:
      type = parameter
    comment = ASE.Compiler.Cxx.Comment.Comment()
    iorType = self.getType(type)
    if mode == ASE.Compiler.ParameterAccessMode.IN:
      comment.setComment('in')
    elif mode == ASE.Compiler.ParameterAccessMode.OUT:
      comment.setComment('out')
      newType = ASE.Compiler.Cxx.Pointer.Pointer()
      newType.setChildren([iorType])
      iorType = newType
    elif mode == ASE.Compiler.ParameterAccessMode.INOUT:
      comment.setComment('inout')
      newType = ASE.Compiler.Cxx.Pointer.Pointer()
      newType.setChildren([iorType])
      iorType = newType
    iorParam = ASE.Compiler.Cxx.Parameter.Parameter()
    iorParam.setIdentifier(parameter.getIdentifier())
    iorParam.setType(iorType)
    iorParam.setComments([comment])
    return iorParam

  def getArrayDeclaration(self, name, sidlType, upper, lower = None, borrow = 0):
    '''Declare a fully specified static IOR array
       - name is a Cxx Variable
       - sidlType is the SIDL type of the entries
       - lower is an array of lower bounds
       - upper is an array of inclusive upper bounds
       - borrow is the flag for an array which does not allocate its own memory'''
    import ASE.Compiler.SIDL.Array
    import ASE.Compiler.Cxx.Array
    import ASE.Compiler.Cxx.Initializer

    if lower is None:
      lower = [0] * len(upper)
    if not len(lower) == len(upper):
      raise RuntimeError('Lengths of lower and upper arrays must match: '+str(len(lower))+' <> '+str(len(upper)))
    decls = []
    dim = len(upper)
    cxxLower = [self.Cxx.getValue(size) for size in lower]
    cxxUpper = [self.Cxx.getValue(size) for size in upper]
    arrayType = ASE.Compiler.SIDL.Array.Array()
    arrayType.setType(sidlType)

    if borrow:
      borrowed = 1
      if isinstance(borrow, int) or isinstance(borrow, bool):
        data = self.Cxx.getNull()
      else:
        data = borrow
    else:
      borrowed = 0
      import ASE.Compiler.Cxx.If
      import ASE.Compiler.Cxx.Sizeof

      logicalSizeVar = self.Cxx.getVar(name.getIdentifier()+'_size')
      for t, (l, u) in enumerate(zip(cxxLower, cxxUpper)):
        term = self.Cxx.getGroup(self.Cxx.getAddition(self.Cxx.getSubtraction(u, l), self.Cxx.getInteger(1)))
        if t == 0:
          logicalSize = term
        else:
          logicalSize = self.Cxx.getMultiplication(term, logicalSize)
      logicalSizeInit = ASE.Compiler.Cxx.Initializer.Initializer()
      logicalSizeInit.setChildren([logicalSize])
      decls.append(self.Cxx.getDeclaration(logicalSizeVar, self.Cxx.typeMap['int'], logicalSizeInit))
      safeDimen = ASE.Compiler.Cxx.If.If()
      safeDimen.setBranch(logicalSizeVar)
      safeDimen.setChildren([logicalSizeVar, self.Cxx.getInteger(1)])
      safeDimen.setTernary(1)
      typeDecl = ASE.Compiler.Cxx.Declarator.Declarator()
      typeDecl.setType(self.getType(sidlType))
      sizeofType = ASE.Compiler.Cxx.Sizeof.Sizeof()
      sizeofType.setTypeName(typeDecl)
      data = self.Cxx.getFunctionCall('malloc', [self.Cxx.getMultiplication(self.Cxx.getGroup(safeDimen), sizeofType)])

    lowerVar = self.Cxx.getVar(name.getIdentifier()+'_lower')
    lowerArray = ASE.Compiler.Cxx.Array.Array()
    lowerArray.setChildren([lowerVar])
    lowerArray.setType(self.typeMap['int'])
    lowerArray.setSize(self.Cxx.getInteger(dim))
    lowerInit = ASE.Compiler.Cxx.Initializer.Initializer()
    lowerInit.setChildren(cxxLower)
    lowerInit.setList(1)
    lowerArray.setInitializer(lowerInit)
    decls.append(self.Cxx.getDecl(lowerArray))

    upperVar = self.Cxx.getVar(name.getIdentifier()+'_upper')
    upperArray = ASE.Compiler.Cxx.Array.Array()
    upperArray.setChildren([upperVar])
    upperArray.setType(self.typeMap['int'])
    upperArray.setSize(self.Cxx.getInteger(dim))
    upperInit = ASE.Compiler.Cxx.Initializer.Initializer()
    upperInit.setChildren(cxxUpper)
    upperInit.setList(1)
    upperArray.setInitializer(upperInit)
    decls.append(self.Cxx.getDecl(upperArray))

    strideVar = self.Cxx.getVar(name.getIdentifier()+'_stride')
    strideArray = ASE.Compiler.Cxx.Array.Array()
    strideArray.setChildren([strideVar])
    strideArray.setType(self.typeMap['int'])
    strideArray.setSize(self.Cxx.getInteger(dim))
    strideInit = ASE.Compiler.Cxx.Initializer.Initializer()
    strideInit.setChildren([self.Cxx.getInteger(1) for d in range(dim)])
    strideInit.setList(1)
    strideArray.setInitializer(strideInit)
    decls.append(self.Cxx.getDecl(strideArray))

    arrayInit = ASE.Compiler.Cxx.Initializer.Initializer()
    arrayInit.setChildren([data, lowerVar, upperVar, strideVar, self.Cxx.getInteger(borrowed), self.Cxx.getInteger(dim)])
    arrayDecl = self.Cxx.getDeclaration(name, self.getType(arrayType).getChildren()[0], arrayInit)
    decls.append(arrayDecl)
    return decls

  def getArrayCleanup(self, name, borrow = 0, check = 0):
    '''Free a static IOR array
       - name is a Cxx Variable
       - borrow is the flag for an array which does not allocate its own memory'''
    if borrow:
      return []
    array = self.Cxx.getStructRef(name, 'd_firstElement', 0)
    freeCall = self.Cxx.getExpStmt(self.Cxx.getFunctionCall('free', [array]))
    if check:
      import ASE.Compiler.Cxx.CompoundStatement
      import ASE.Compiler.Cxx.If
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound.setChildren([freeCall])
      checkArray = ASE.Compiler.Cxx.If.If()
      checkArray.setBranch(array)
      checkArray.setChildren([compound])
      return [checkArray]
    return [freeCall]

  def getArrayLoop(self, loopVar, arrayVar, entryVar, entryType):
    '''Return a loop over the IOR array arrayVar'''
    import ASE.Compiler.Cxx.CompoundStatement
    loop = self.Cxx.getSimpleLoop(loopVar, self.Cxx.getArrayRef(self.Cxx.getStructRef(arrayVar, 'd_lower'), 0), self.Cxx.getArrayRef(self.Cxx.getStructRef(arrayVar, 'd_upper'), 0), allowEquality = 1)
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    loop.addChildren([compound])
    compound.addDeclarations([self.Cxx.getDeclaration(entryVar, entryType, self.Cxx.getInitializer(None, self.Cxx.getArrayRef(self.Cxx.getStructRef(arrayVar, 'd_firstElement'), loopVar)))])
    return loop

  def getFunctionDeclaration(self, type, method, isPointer = 0, qualifyName = 0, prefix = None, onlyTypes = 0):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    func = ASE.Compiler.Cxx.Function.Function()
    if qualifyName:
      name = type.getFullIdentifier().replace('.', '_') + '_' + method.getIdentifier()
    else:
      name = method.getIdentifier()
    if not (prefix is None):
      name = prefix + name
    if isPointer:
      funcPtr = ASE.Compiler.Cxx.Declarator.Declarator()
      funcPtr.setIdentifier(name)
      funcPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
      func.setChildren([self.Cxx.getDeclGroup(funcPtr)])
    else:
      decl = ASE.Compiler.Cxx.Declarator.Declarator()
      decl.setIdentifier(name)
      func.setChildren([decl])
    if method.getReturnParameter() is None:
      func.setType(self.typeMap['void'])
    else:
      func.setType(self.getType(method.getReturnParameter().getType()))
    if not method.getStatic():
      selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
      if not onlyTypes:
        selfParam.setIdentifier('self')
      selfParam.setType(self.getType(type))
      func.addParameters([selfParam])
    for child in method.getChildren():
      param = self.getParameter(ASE.Compiler.SIDL.Parameter.Parameter(child))
      if onlyTypes:
        param.setIdentifier('')
      func.addParameters([param])
    excParam = ASE.Compiler.SIDL.Parameter.Parameter()
    excParam.setIdentifier('_exception')
    excParam.setAccessMode(ASE.Compiler.ParameterAccessMode.OUT)
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface') and not type.getFullIdentifier() == 'ASE.BaseInterface':
      excParam.setType(self.SIDL.getBaseException(method.getExceptions()[0]))
    else:
      excParam.setType(self.SIDL.typeMap['opaque'])
    func.addParameters([self.getParameter(excParam)])
    if onlyTypes:
      comment = None
    else:
      comment = method.getFullIdentifier() + ': '
      comment += '\n'.join([c.getComment() for c in method.getComments()])
    return self.Cxx.getDecl(func, comment)

  def castToPrototype(self, exp, interface, method):
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Pointer

    funcPtr = ASE.Compiler.Cxx.Declarator.Declarator()
    funcPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
    funcPtrDecl = ASE.Compiler.Cxx.Function.Function(self.getFunctionDeclaration(interface, method).getChildren()[0])
    funcPtrDecl.setChildren([self.Cxx.getDeclGroup(funcPtr)])
    cast = ASE.Compiler.Cxx.Cast.Cast()
    cast.setChildren([exp])
    cast.setTypeName(funcPtrDecl)
    return cast
  
  def getFileBasename(self, interface):
    import os
    idChain = interface.getFullIdentifier().split('.')
    if not len(idChain[:-1]):
      return idChain[-1] + '_IOR'
    return os.path.join(apply(os.path.join, idChain[:-1]), idChain[-1] + '_IOR')
  
  def getGuardedInclude(self, interface):
    import ASE.Compiler.Cxx.PreprocessorConditional
    guard = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    guard.setIdentifier('ifndef')
    guard.setBranchExpression('included_' + interface.getFullIdentifier().replace('.', '_'))
    guard.setValue(1)
    guard.addChildren([self.Cxx.getInclude('<'+self.getFileBasename(interface) + '.h>')])
    return guard

  def getErrorCheck(self, interface, method, branchVar, errorMessage, branchExp = None, returnVar = None):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Negation

    excType      = self.SIDL.getASEException(interface)
    excVar       = self.Cxx.getVar('_exception')
    localExcVar  = self.Cxx.getVar('_local_exception')
    extFunc      = self.Cxx.getFunctionCall(excType.getFullIdentifier().replace('.', '_')+'__externals')
    createFunc   = self.Cxx.getStructRef(extFunc, 'createObject')
    createExc    = self.Cxx.getFunctionCall(createFunc, [self.Cxx.getNull()])
    localExcInit = ASE.Compiler.Cxx.Initializer.Initializer()
    localExcInit.setChildren([createExc])
    excEpv       = self.Cxx.getStructRef(localExcVar, 'd_epv')
    messageFunc  = self.Cxx.getStructRef(excEpv, 'setMessage')
    setMessage   = self.Cxx.getFunctionCall(messageFunc, [localExcVar, self.Cxx.getString(errorMessage), excVar])
    excInd       = ASE.Compiler.Cxx.Indirection.Indirection()
    excInd.setChildren([excVar])
    setExc       = self.Cxx.getAssignment(excInd, self.Cxx.castToType(localExcVar, self.getType(self.SIDL.getBaseException(excType))))
    compound     = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(setExc)])
    checkExc     = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(excVar)
    checkExc.setChildren([compound])
    returnStmt   = ASE.Compiler.Cxx.Return.Return()
    if not method is None and not method.getReturnParameter() is None:
      if returnVar is None:
        returnVar = self.Cxx.getVar(method.getReturnParameter().getIdentifier())
      returnStmt.setChildren([returnVar])
    compound     = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setDeclarations([self.Cxx.getDeclaration(localExcVar, self.getType(excType), localExcInit)])
    compound.setChildren([self.Cxx.getExpStmt(setMessage), checkExc, returnStmt])
    checkVar     = ASE.Compiler.Cxx.If.If()
    if branchExp is None:
      negVar     = ASE.Compiler.Cxx.Negation.Negation()
      negVar.setChildren([branchVar])
      checkVar.setBranch(negVar)
    else:
      checkVar.setBranch(branchExp)
    checkVar.setChildren([compound])
    return checkVar

  def getClassMethodTable(self, klass):
    excVar        = self.Cxx.getVar('_exception')
    loaderExtCall = self.Cxx.getFunctionCall('ASE_Loader__externals')
    staticEPVFunc = self.Cxx.getStructRef(loaderExtCall, 'getStaticEPV')
    staticEPVCall = self.Cxx.getFunctionCall(staticEPVFunc, [excVar])
    tableFunc     = self.Cxx.getStructRef(staticEPVCall, 'getClassMethodTable')
    tableFuncCall = self.Cxx.getFunctionCall(tableFunc, [self.Cxx.getString(klass.getFullIdentifier()), excVar])
    return tableFuncCall
  
  def getMethodSignature(self, klass, method):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    methodName = ASE.Compiler.Cxx.Declarator.Declarator()
    methodName.setIdentifier(method.getFullIdentifier().replace('.', '_'))
    methodSkeleton = ASE.Compiler.Cxx.Function.Function()
    methodSkeleton.setChildren([methodName])
    if method.getReturnParameter() is None:
      methodSkeleton.setType(self.typeMap['void'])
    else:
      methodSkeleton.setType(self.getType(method.getReturnParameter().getType()))
    methodSkeleton.setStatic(1)
    if not method.getStatic():
      selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
      selfParam.setIdentifier('_self')
      selfParam.setType(self.getType(klass))
      methodSkeleton.addParameters([selfParam])
    for child in method.getChildren():
      parameter = ASE.Compiler.SIDL.Parameter.Parameter(child)
      param = ASE.Compiler.Cxx.Parameter.Parameter()
      param.setIdentifier(parameter.getIdentifier())
      if self.SIDL.isOut(parameter):
        paramType = ASE.Compiler.Cxx.Pointer.Pointer()
        paramType.setChildren([self.getType(parameter.getType())])
        param.setType(paramType)
      else:
        param.setType(self.getType(parameter.getType()))
      methodSkeleton.addParameters([param])
    if len(method.getExceptions()):
      excType = ASE.Compiler.Cxx.Pointer.Pointer()
      excType.setChildren([self.getType(self.SIDL.getBaseException(method.getExceptions()[0]))])
      excParam = ASE.Compiler.Cxx.Parameter.Parameter()
      excParam.setIdentifier('_exception')
      excParam.setType(excType)
      methodSkeleton.addParameters([excParam])
    return methodSkeleton

  def addReference(self, ior):
    addRefFunc = self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getStructRef(self.Cxx.getGroup(ior), 'd_epv'), 'addReference'))))
    return self.Cxx.getExpStmt(self.Cxx.getFunctionCall(addRefFunc, [self.getTrueIOR(ior), self.Cxx.getNull()]))
  
  def dispatch(self, interface, method, body):
    import ASE.Compiler.SIDL.Parameter
    args = []
    if interface is None:
      iorCall = self.Cxx.getFunctionCall(method.getIdentifier() + '_impl')
    elif method.getStatic():
      iorMethod = self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getVar('_sepv'), method.getIdentifier())))
      iorCall = self.Cxx.getFunctionCall(self.Cxx.getGroup(iorMethod))
    else:
      selfVar = self.Cxx.getVar('_self_ior')
      iorMethod = self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getStructRef(selfVar, 'd_epv'), method.getIdentifier())))
      iorCall = self.Cxx.getFunctionCall(self.Cxx.getGroup(iorMethod))
      if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
        args.append(selfVar)
      else:
        args.append(self.Cxx.getStructRef(selfVar, 'd_object'))
    for parameter in [ASE.Compiler.SIDL.Parameter.Parameter(p) for p in method.getChildren()]:
      if self.SIDL.isOut(parameter):
        args.append(self.Cxx.getAddress(self.Cxx.getVar(parameter.getIdentifier())))
      else:
        args.append(self.Cxx.getVar(parameter.getIdentifier()))
    if not interface is None and interface.getFullIdentifier() == 'ASE.BaseInterface':
      args.append(self.Cxx.getNull())
    else:
      args.append(self.Cxx.getAddress(self.Cxx.getVar('_exception')))
    iorCall.setArguments(args)
    if not method.getReturnParameter() is None:
      body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getVar(method.getReturnParameter().getIdentifier()), iorCall), 'Dispatch to IOR')])
    else:
      body.addChildren([self.Cxx.getExpStmt(iorCall, 'Dispatch to IOR')])
    return
