# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.ImplementationGenerator
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.DepthFirstVisitor
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Iterator
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class ImplementationGenerator:
  '''
  This visitor transforms a SIDL AST into a set of Python Implementation ASTs
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    import ASE.Compiler.Python.Python
    self.Python = ASE.Compiler.Python.Python.Python()
    self.source = []
    self.packages = [[]]
    self.classes = [[]]
    self.sourceLanguages = ['Python']
    self.templateMethods = {}

    self.packageChildren = {}

  
  def getSourceLanguages(self):
    '''
    Retrieve the list of the languages in which ASTs are produced
    '''
    return self.sourceLanguages
  
  def setSourceLanguages(self, outputLanguages):
    '''
    Set the list of the languages in which ASTs are produced
    '''
    self.sourceLanguages = sourceLanguages
  
  def getSource(self, language):
    '''
    Retrieve the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    return self.source
  
  def setSource(self, language, vertices):
    '''
    Set the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    self.source = list(vertices)
  
  def addSource(self, language, vertices):
    '''
    Add to the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    self.source.extend(vertices)
  
  def removeSource(self, language, vertices):
    '''
    Remove to the list of source ASTs for the given language
    Throw an exception if a vertex or language does not exist
    '''
    if not (language in self.sourceLanguages):
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    for vertex in vertices:
      if vertex in self.source:
        self.source.remove(vertex)
      else:
        e = ASE.Compiler.CompilerException()
        e.setMessage('Source tree does not exist: ' + vertex.getIdentifier())
        raise e
  
  def visitPackage(self, node):
    '''
    Each package has an associated __init__.py module
    '''
    import ASE.Compiler.Python.Module
    import ASE.Compiler.Python.Stmt
    import ASE.Compiler.Python.Assign
    import ASE.Compiler.Python.AssName
    import ASE.Compiler.Python.List
    self.addChild(node)
    self.__IORself.visitVertex(node)
    target = ASE.Compiler.Python.AssName.AssName()
    target.setIdentifier('all')
    value = ASE.Compiler.Python.List.List()
    value.setChildren([self.Python.getString(name) for name in self.packageChildren[node.getFullIdentifier()]])
    assign = ASE.Compiler.Python.Assign.Assign()
    assign.setValue(value)
    assign.setChildren([target])
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    stmt.setChildren([assign])
    module = ASE.Compiler.Python.Module.Module()
    module.setIdentifier(node.getFullIdentifier() + '.__init__')
    module.setComments(node.getComments())
    module.setChildren([stmt])
    del self.packageChildren[node.getFullIdentifier()]
    self.addSource('Python', [module])
  
  def visitClass(self, node):
    '''
    Each class becomes an implementation module
    '''
    self.addChild(node)
    from ASE.Compiler.CodePurpose import IMPLEMENTATION
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.SIDL.Type
    import ASE.Compiler.Python.Class
    import ASE.Compiler.Python.Import
    import ASE.Compiler.Python.Module
    import ASE.Compiler.Python.Stmt
    module = ASE.Compiler.Python.Module.Module()
    module.setIdentifier(node.getFullIdentifier() + '_impl')
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    module.setChildren([stmt])
    stmt.addChildren([self.getCommentedPass('Starting imports section')])
    if not (node.getFullIdentifier() == 'ASE.Loader'):
      moduleNames = []
      for dep in node.getDependencies():
        if (dep.isInstanceOf('ASE.Compiler.SIDL.Type') and ASE.Compiler.SIDL.Type.Type(dep).getBaseType()):
          continue
        moduleNames.append(dep.getFullIdentifier())
      moduleNames.sort()
      for moduleName in moduleNames:
        i = ASE.Compiler.Python.Import.Import()
        i.setModuleNames([moduleName])
        stmt.addChildren([i])
    stmt.addChildren([self.getCommentedPass('Ending imports section')])
    stmt.addChildren([self.getCommentedPass('Starting static methods section')])
    self.pushClass()
    for method in [m for m in node.getImplementedMethods() if (m.getStatic() and IMPLEMENTATION in m.getPurpose())]:
      method.accept(self.__IORself)
    stmt.addChildren(self.popClass())
    stmt.addChildren([self.getCommentedPass('Ending static methods section')])
    c = ASE.Compiler.Python.Class.Class()
    c.setIdentifier(node.getIdentifier())
    c.setComments(node.getComments())
    stmt.addChildren([c])
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    c.setChildren([stmt])
    iorParam = ASE.Compiler.SIDL.Parameter.Parameter()
    iorParam.setIdentifier('IORself')
    constructor = ASE.Compiler.SIDL.Method.Method()
    constructor.setIdentifier('__init__')
    constructor.setChildren([iorParam])
    stmt.addChildren([self.getCommentedPass('Starting class methods section')])
    self.pushClass()
    [self.createTemplateMethod(method) for method in node.getImplementedMethods()]
    for method in [constructor] + [m for m in self.getTemplateMethods(node.getImplementedMethods()) if (not (m.getStatic()) and IMPLEMENTATION in m.getPurpose())]:
      method.accept(self.__IORself)
    stmt.addChildren(self.popClass())
    stmt.addChildren([self.getCommentedPass('Ending class methods section')])
    self.addSource('Python', [module])
  
  def visitMethod(self, node):
    import ASE.Compiler.Python.Function
    import ASE.Compiler.Python.Stmt
    f = ASE.Compiler.Python.Function.Function()
    f.setIdentifier(node.getIdentifier())
    argNames = [parameter.getIdentifier() for parameter in node.getInParameters()]
    if not (node.getStatic()):
      argNames.insert(0, 'self')
    f.setArgumentNames(argNames)
    f.setComments(node.getComments())
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    if node.getIdentifier() == '__init__':
      import ASE.Compiler.Python.Assign
      import ASE.Compiler.Python.AssAttr
      import ASE.Compiler.Python.Name
      assign = ASE.Compiler.Python.Assign.Assign()
      target = ASE.Compiler.Python.AssAttr.AssAttr()
      obj = ASE.Compiler.Python.Name.Name()
      value = ASE.Compiler.Python.Name.Name()
      obj.setIdentifier('self')
      target.setIdentifier('__IORself')
      target.setChildren([obj])
      value.setIdentifier('IORself')
      assign.setValue(value)
      assign.setChildren([target])
      stmt.setChildren([assign])
    else:
      import ASE.Compiler.Python.Pass
      p = ASE.Compiler.Python.Pass.Pass()
      stmt.setChildren([p])
    f.setChildren([stmt])
    self.classAppend(f)
  # Ending class methods section
  pass
  
  def addChild(self, node):
    if node.isInstanceOf('ASE.Compiler.SIDL.Package'):
      package = node.getFullIdentifier()
      if not (package in self.packageChildren):
        self.packageChildren[package] = []
    parent = node.getParent()
    if not (parent.isInstanceOf('ASE.Compiler.SIDL.Package')):
      return 
    import ASE.Compiler.SIDL.Package
    package = ASE.Compiler.SIDL.Package.Package(parent).getFullIdentifier()
    if not (package in self.packageChildren):
      self.packageChildren[package] = []
    if node.isInstanceOf('ASE.Compiler.SIDL.Class'):
      self.packageChildren[package].append(node.getIdentifier() + '_impl')
    else:
      self.packageChildren[package].append(node.getIdentifier())
    return 
  
  def pushClass(self):
    return self.classes.append([])
  
  def popClass(self):
    return self.classes.pop()
  
  def classAppend(self, name):
    return self.classes[-(1)].append(name)
  
  def getCommentedPass(self, comment):
    import ASE.Compiler.Python.Comment
    import ASE.Compiler.Python.Pass
    c = ASE.Compiler.Python.Comment.Comment()
    c.setComment(comment)
    p = ASE.Compiler.Python.Pass.Pass()
    p.setComments([c])
    return p
  
  def copyType(self, type):
    if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      newType = ASE.Compiler.SIDL.Array.Array()
      newType.setIdentifier(type.getIdentifier())
      newType.setAttributes(type.getAttributes())
      newType.setLineNumber(type.getLineNumber())
      newType.setComments(type.getComments())
      newType.setChildren([child for child in type.getChildren()])
      newType.setBaseType(type.getBaseType())
      newType.setType(self.copyType(ASE.Compiler.SIDL.Array.Array(type).getType()))
      return newType
    return type
  
  def copyParameter(self, parameter):
    '''
    This will be replaced by parameter.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Parameter
    newParameter = ASE.Compiler.SIDL.Parameter.Parameter()
    newParameter.setIdentifier(parameter.getIdentifier())
    newParameter.setAttributes(parameter.getAttributes())
    newParameter.setParent(parameter.getParent())
    newParameter.setChildren(parameter.getChildren())
    newParameter.setLineNumber(parameter.getLineNumber())
    newParameter.setComments(parameter.getComments())
    newParameter.setDependencies(parameter.getDependencies())
    newParameter.setType(self.copyType(parameter.getType()))
    newParameter.setAccessMode(parameter.getAccessMode())
    newParameter.setRequired(parameter.getRequired())
    return newParameter
  
  def copyMethod(self, method):
    '''
    This will be replaced by method.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    newMethod = ASE.Compiler.SIDL.Method.Method()
    newMethod.setIdentifier(method.getIdentifier())
    newMethod.setAttributes(method.getAttributes())
    newMethod.setParent(method.getParent())
    newMethod.setChildren([self.copyParameter(ASE.Compiler.SIDL.Parameter.Parameter(child)) for child in method.getChildren()])
    newMethod.setLineNumber(method.getLineNumber())
    newMethod.setComments(method.getComments())
    newMethod.setDependencies(method.getDependencies())
    newMethod.setTemplateIdentifiers(method.getTemplateIdentifiers())
    if not (method.getReturnParameter() is None):
      newMethod.setReturnParameter(self.copyParameter(method.getReturnParameter()))
    newMethod.setExceptions(method.getExceptions())
    newMethod.setStatic(method.getStatic())
    newMethod.setInternal(method.getInternal())
    newMethod.setVirtual(method.getVirtual())
    newMethod.setPurpose(method.getPurpose())
    return newMethod
  
  def checkTemplateParameter(self, name, parameter):
    if not (parameter is None):
      type = parameter.getType()
      if (type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name):
        return True
      elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
        import ASE.Compiler.SIDL.Array
        type = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
        if (type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name):
          return True
    return False
  
  def insertType(self, parameter, type):
    if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      self.insertType(ASE.Compiler.SIDL.Array.Array(parameter.getType()), type)
    else:
      parameter.setType(type)
    return 
  
  def insertTypes(self, method, universe, type):
    import ASE.Compiler.SIDL.Parameter
    name = universe.getIdentifier()
    newMethod = self.copyMethod(method)
    newMethod.setIdentifier(newMethod.getIdentifier() + '_' + type.getFullIdentifier())
    if self.checkTemplateParameter(name, method.getReturnParameter()):
      self.insertType(newMethod.getReturnParameter(), type)
    for (child, newChild) in zip(method.getChildren(), newMethod.getChildren()):
      if self.checkTemplateParameter(name, ASE.Compiler.SIDL.Parameter.Parameter(child)):
        self.insertType(ASE.Compiler.SIDL.Parameter.Parameter(newChild), type)
    return newMethod
  
  def template(self, method, name):
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.SIDL.TypeUniverse
    universe = None
    for parameter in [method.getReturnParameter()] + [ASE.Compiler.SIDL.Parameter.Parameter(child) for child in method.getChildren()]:
      if self.checkTemplateParameter(name, parameter):
        type = parameter.getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(ASE.Compiler.SIDL.Array.Array(type).getType())
        else:
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(type)
        break
    if universe is None:
      raise RuntimeError('Could not find type universe for template ' + name + ' in method ' + method.getFullIdentifier())
    self.templateMethods[method] = [self.insertTypes(method, universe, type) for type in universe.getTypes()]
    return None
  
  def createTemplateMethod(self, method):
    if len(method.getTemplateIdentifiers()):
      if len(method.getTemplateIdentifiers()) > 1:
        raise RuntimeError('Can currently only handle a single template identifier')
      if not (method in self.templateMethods):
        self.template(method, method.getTemplateIdentifiers()[0])
    return None
  
  def getTemplateMethods(self, methods):
    for method in methods:
      if method in self.templateMethods:
        for m in self.templateMethods[method]:
          yield m
      else:
        yield method
    return 
