# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CodePurpose
import ASE.Compiler.CompilerException
import ASE.Compiler.Output
import ASE.Compiler.Python.Add
import ASE.Compiler.Python.And
import ASE.Compiler.Python.AssAttr
import ASE.Compiler.Python.AssList
import ASE.Compiler.Python.AssName
import ASE.Compiler.Python.AssTuple
import ASE.Compiler.Python.Assign
import ASE.Compiler.Python.AugAssign
import ASE.Compiler.Python.Bitand
import ASE.Compiler.Python.Bitor
import ASE.Compiler.Python.Bitxor
import ASE.Compiler.Python.Break
import ASE.Compiler.Python.CallFunc
import ASE.Compiler.Python.Class
import ASE.Compiler.Python.Compare
import ASE.Compiler.Python.Const
import ASE.Compiler.Python.Continue
import ASE.Compiler.Python.Dict
import ASE.Compiler.Python.Discard
import ASE.Compiler.Python.Div
import ASE.Compiler.Python.Ellipsis
import ASE.Compiler.Python.For
import ASE.Compiler.Python.From
import ASE.Compiler.Python.Function
import ASE.Compiler.Python.GetAttr
import ASE.Compiler.Python.Global
import ASE.Compiler.Python.If
import ASE.Compiler.Python.Import
import ASE.Compiler.Python.Invert
import ASE.Compiler.Python.Keyword
import ASE.Compiler.Python.Lambda
import ASE.Compiler.Python.LeftShift
import ASE.Compiler.Python.List
import ASE.Compiler.Python.ListComp
import ASE.Compiler.Python.ListCompFor
import ASE.Compiler.Python.ListCompIf
import ASE.Compiler.Python.Mod
import ASE.Compiler.Python.Module
import ASE.Compiler.Python.Mul
import ASE.Compiler.Python.Name
import ASE.Compiler.Python.Not
import ASE.Compiler.Python.Or
import ASE.Compiler.Python.Pass
import ASE.Compiler.Python.Power
import ASE.Compiler.Python.Print
import ASE.Compiler.Python.Printer
import ASE.Compiler.Python.Printnl
import ASE.Compiler.Python.PyNone
import ASE.Compiler.Python.Raise
import ASE.Compiler.Python.Return
import ASE.Compiler.Python.RightShift
import ASE.Compiler.Python.Slice
import ASE.Compiler.Python.Stmt
import ASE.Compiler.Python.Sub
import ASE.Compiler.Python.Subscript
import ASE.Compiler.Python.TryExcept
import ASE.Compiler.Python.TryFinally
import ASE.Compiler.Python.Tuple
import ASE.Compiler.Python.UnaryAdd
import ASE.Compiler.Python.UnarySub
import ASE.Compiler.Python.Vertex
import ASE.Compiler.Python.While
import ASE.Compiler.Python.Yield
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class Output:
  '''
  This class prints a Python AST to the file system as Python source
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.disableOutput = 0
    self.disableSource = 0
    self.roots = {}

    self.files = []
    self.f = None
    self.includes = []
    self.systemIncludes = []
  
  def getDisableOutput(self):
    '''
    Retrieve the flag to disable writing to disk
    This is useful for generating the list of files which would have been output
    '''
    return self.disableOutput
  
  def setDisableOutput(self, disableOutput):
    '''
    Set the flag to disable writing to disk
    This is useful for generating the list of files which would have been output
    '''
    self.disableOutput = disableOutput
  
  def getDisableSource(self):
    '''
    Retrieve the flag to disable writing source
    Headers and other support files will still be output
    '''
    return self.disableSource
  
  def setDisableSource(self, disableSource):
    '''
    Set the flag to disable writing source
    Headers and other support files will still be output
    '''
    self.disableSource = disableSource
  
  def getRoot(self, purpose):
    '''
    Retrieve the directory root for files generated for the given purpose
    The purpose can be stub, cartilage, IOR, skeleton, or implementation
    '''
    return self.roots[purpose]
  
  def setRoot(self, purpose, root):
    '''
    Retrieve the directory root for files generated for the given purpose
    The purpose can be stub, cartilage, IOR, skeleton, or implementation
    '''
    self.roots[purpose] = root
  
  def getIncludeDirectories(self):
    '''
    Retrieve the list of include directories
    This seems general enough to include for all languages
    '''
    return self.includes
  
  def setIncludeDirectories(self, dirs):
    '''
    Set the list of include directories
    This seems general enough to include for all languages
    '''
    self.includes = list(dirs)
  
  def getSystemIncludeDirectories(self):
    '''
    Retrieve the list of system include directories
    This seems general enough to include for all languages
    '''
    return self.systemIncludes
  
  def setSystemIncludeDirectories(self, dirs):
    '''
    Set the list of system include directories
    This seems general enough to include for all languages
    '''
    self.systemIncludes = list(dirs)
  
  def getFiles(self):
    '''
    Retrieve the list of output files
    '''
    return self.files
  
  def setFiles(self, files):
    '''
    Set the list of output files
    '''
    self.files = list(files)
  
  def visitModule(self, node):
    import ASE.Compiler.CodePurpose
    import os
    if ASE.Compiler.CodePurpose.STUB in self.roots:
      purpose = ASE.Compiler.CodePurpose.STUB
      baseDir = self.roots[purpose]
    elif ASE.Compiler.CodePurpose.IMPLEMENTATION in self.roots:
      purpose = ASE.Compiler.CodePurpose.IMPLEMENTATION
      baseDir = self.roots[purpose]
    else:
      raise RuntimeError('Base directory not set for module type')
    filename = apply(os.path.join, [baseDir] + node.getIdentifier().split('.')) + '.py'
    if (os.path.dirname(filename) and not (os.path.exists(os.path.dirname(filename)))):
      os.makedirs(os.path.dirname(filename))
    if (not ((self.disableOutput or self.disableSource)) and os.path.isfile(filename) and purpose == ASE.Compiler.CodePurpose.IMPLEMENTATION and not (os.path.basename(filename) == '__init__.py')):
      import ASE.Compiler.ParserException
      import ASE.Compiler.Python.Parser
      try:
        print 'Parsing', filename
        userModule = ASE.Compiler.Python.Parser.Parser().parseFile(filename)
      except ASE.Compiler.ParserException:
        pass
      children = list(node.getChildren()[0].getChildren())
      userChildren = list(userModule.getChildren()[0].getChildren())
      userImports = self.checkImports(children, userChildren)
      userMethods = self.checkStaticMethods(children, userChildren)
      userClass = self.checkClass(children, userChildren)
      node.setChildren(userImports + userMethods + userClass)
    print 'Creating', filename
    self.files.append(filename)
    if (self.disableOutput or self.disableSource):
      if not (os.path.isfile(filename)):
        raise RuntimeError('SIDL file ' + filename + ' has not been generated')
    else:
      self.f = file(filename, 'w')
      self.__IORself.printDocComment(node)
      self.__IORself.visitVertex(node)
      self.f.close()
      self.f = None
  
  def write(self, s):
    '''
    This is overridden to redirect output to a file
    '''
    self.f.write(s)
  # Ending class methods section
  pass
  
  def getCodeSection(self, nodes):
    '''
    Returns a tuple of nodes preceeding a pass block together with the pass block
       - The block itself contains the pass statements
       - These two blocks are deleted from nodes
    '''
    prefixBlock = []
    passBlock = []
    for node in nodes:
      if node.isInstanceOf('ASE.Compiler.Python.Pass'):
        break
      prefixBlock.append(node)
    if len(prefixBlock) == len(nodes):
      raise RuntimeError('Missing pass block')
    passBlock.append(nodes[len(prefixBlock)])
    for node in nodes[len(prefixBlock) + 1:]:
      passBlock.append(node)
      if node.isInstanceOf('ASE.Compiler.Python.Pass'):
        break
    if not (passBlock[0].isInstanceOf('ASE.Compiler.Python.Pass')):
      raise RuntimeError('Missing pass at the beginning of a pass block')
    if not (passBlock[-(1)].isInstanceOf('ASE.Compiler.Python.Pass')):
      raise RuntimeError('Missing pass at the end of a pass block')
    del nodes[:len(prefixBlock) + len(passBlock)]
    return (prefixBlock, passBlock)
  
  def checkImports(self, genChildren, userChildren):
    '''
    The first section of generated children is a list of imports, followed by the static methods
       - In order to simplify the merge, the block of generated imports is bracketed by pass statements
       - The user is not allowed to use top-level pass statements in this module
    '''
    (genPrefix, genImports) = self.getCodeSection(genChildren)
    if len(genPrefix):
      raise RuntimeError('Invalid generated module: imports pass block cannot be preceeded by code')
    (userPrefix, userImports) = self.getCodeSection(userChildren)
    return userPrefix + genImports
  
  def checkMethods(self, genChildren, userChildren):
    '''
    Build a method list, taking the bodies from the user code
       - Remove all utilized code from children and userChildren
    '''
    (genPrefix, genMethods) = self.getCodeSection(genChildren)
    if len(genPrefix):
      raise RuntimeError('Invalid generated module: methods pass block cannot be preceeded by code')
    (userPrefix, userMethods) = self.getCodeSection(userChildren)
    methodBodies = {}

    for method in userMethods[1:-(1)]:
      methodBodies[method.getIdentifier()] = method.getChildren()[0].getChildren()
    for method in genMethods[1:-(1)]:
      if method.getIdentifier() in methodBodies:
        method.getChildren()[0].setChildren(methodBodies[method.getIdentifier()])
    return userPrefix + genMethods
  
  def checkStaticMethods(self, genChildren, userChildren):
    '''
    The second section of generated children is a list of static methods, followed by the class definition
       - In order to simplify the merge, the block of generated static methods is bracketed by pass statements
       - The user is not allowed to use top-level pass statements in this module
    '''
    return self.checkMethods(genChildren, userChildren)
  
  def checkClass(self, genChildren, userChildren):
    '''
    The third section of generated children is the class declaration, followed by the class definition
       - The user is not allowed to use top-level pass statements in this module and class
    '''
    genClass = genChildren[0]
    if not (genClass.isInstanceOf('ASE.Compiler.Python.Class')):
      raise RuntimeError('Invalid generated module: class declaration section must start with a class')
    del genChildren[0]
    if len(genChildren):
      raise RuntimeError('Invalid generated module: no code may follow the class declaration section')
    userPrefix = []
    for child in userChildren:
      if child.isInstanceOf('ASE.Compiler.Python.Class'):
        break
      userPrefix.append(child)
    if len(userPrefix) == len(userChildren):
      raise RuntimeError('Invalid user module: class declaration section must contain a class')
    del userChildren[:len(userPrefix)]
    userClass = userChildren[0]
    del userChildren[0]
    userPostfix = userChildren[:]
    del userChildren[:]
    genMethods = list(genClass.getChildren()[0].getChildren())
    userMethods = list(userClass.getChildren()[0].getChildren())
    genClass.getChildren()[0].setChildren(self.checkMethods(genMethods, userMethods))
    genClass.getChildren()[0].addChildren(userMethods)
    return userPrefix + [genClass] + userPostfix
