# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.DepthFirstVisitor
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Iterator
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.SkeletonGenerator
import ASE.Compiler.Vertex
# Ending imports section
pass
import ASE.Compiler.Python.Python
import ASE.Compiler.Python.SIDL
# Starting static methods section
pass
# Ending static methods section
pass

class SkeletonGenerator:
  '''
  This visitor transforms a SIDL AST into a set of Python Stub ASTs
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.source = {}

    self.sourceLanguages = ['Cxx']
    self.Skeleton = Skeleton()
    self.Python = ASE.Compiler.Python.Python.Python()
    self.IOR = self.Python.IOR
    self.SIDL = self.Python.SIDL
    self.Cxx = self.Python.Cxx
    self.pyObjType = self.Python.objType
    self.constructors = {}

    self.destructors = {}

    self.templateMethods = {}

  
  def getSourceLanguages(self):
    '''
    Retrieve the list of the languages in which ASTs are produced
    '''
    return self.sourceLanguages
  
  def setSourceLanguages(self, outputLanguages):
    '''
    Set the list of the languages in which ASTs are produced
    '''
    self.sourceLanguages = sourceLanguages
  
  def getSource(self, language):
    '''
    Retrieve the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not (language in self.source):
      self.source[language] = []
    return self.source[language]
  
  def setSource(self, language, vertices):
    '''
    Set the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not (language in self.source):
      self.source[language] = []
    self.source[language] = list(vertices)
  
  def addSource(self, language, vertices):
    '''
    Add to the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not (language in self.source):
      self.source[language] = []
    self.source[language].extend(vertices)
  
  def removeSource(self, language, vertices):
    '''
    Remove to the list of source ASTs for the given language
    Throw an exception if a vertex or language does not exist
    '''
    if not (language in self.sourceLanguages):
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not (language in self.source):
      self.source[language] = []
    for vertex in vertices:
      if vertex in self.source[language]:
        self.source[language].remove(vertex)
      else:
        e = ASE.Compiler.CompilerException()
        e.setMessage('Source tree does not exist: ' + vertex.getIdentifier())
        raise e
  
  def visitClass(self, node):
    defns = []
    defns.extend([self.Python.getRefTypeEnumeration(), self.Python.getSIDLObject(node)])
    defns.extend([self.Python.getNew(node)])
    methods = list(node.getImplementedMethods()) + [self.getConstructorMethod(node), self.getDestructorMethod(node)]
    (wrapIORFuncs, extractIORFuncs, wrapArrayFuncs, extractArrayFuncs, extractIteratorFuncs) = self.getParameterProcessors(node, methods)
    'Every class now can return an iterator automatically'
    extractIORFuncs[node] = self.Python.getExtractIOR(node, node)
    wrapIORFuncs[(node, 1)] = self.Python.getWrapIOR(node, isWeak = 1)
    defns.extend(wrapIORFuncs.values())
    defns.extend(extractIORFuncs.values())
    for ((entryType, borrow), func) in wrapArrayFuncs.items():
      defns.extend([self.IOR.getGuardedInclude(entryType), func])
    for ((entryType, borrow), func) in extractArrayFuncs.items():
      defns.extend([self.IOR.getGuardedInclude(entryType), func])
    outputIterNext = 0
    for (entryType, func) in extractIteratorFuncs.items():
      if entryType == node:
        outputIterNext = 1
      defns.append(self.IOR.getGuardedInclude(entryType))
      defns.extend(self.getIterNextSkeleton(entryType, node))
      defns.append(func)
    if not (outputIterNext):
      defns.extend(self.getIterNextSkeleton(node, node))
    [self.createTemplateMethod(method) for method in node.getImplementedMethods()]
    for method in self.getTemplateMethods(node.getImplementedMethods()):
      defns.extend(self.getMethodSkeleton(node, method))
    defns.extend(self.getConstructorSkeleton(node))
    defns.extend(self.getDestructorSkeleton(node))
    defns.extend(self.getSetEpvSkeleton(node))
    defns.extend(self.getSetSepvSkeleton(node))
    self.addSource('Cxx', [self.getSourceFile(node, defns)])
  # Ending class methods section
  pass
  
  def hasStaticMethod(self, type):
    '''
    We could use  if method.getStatic() and not method.getInternal():
    '''
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      for method in type.getMethods():
        if method.getStatic():
          return 1
    return 0
  
  def getParameterProcessors(self, klass, methods):
    wrapIORFuncs = {}

    extractIORFuncs = {}

    wrapArrayFuncs = {}

    extractArrayFuncs = {}

    extractIteratorFuncs = {}

    for method in methods:
      for parameter in method.getInParameters():
        paramType = parameter.getType()
        if paramType.getBaseType():
          if paramType.getIdentifier() == 'opaque':
            wrapIORFuncs[(paramType, 0)] = self.Python.getWrapOpaque(paramType)
        elif paramType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
          wrapIORFuncs[(paramType, 0)] = self.Python.getWrapIOR(paramType)
        elif paramType.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          entryType = ASE.Compiler.SIDL.Array.Array(paramType).getType()
          if entryType.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse'):
            import ASE.Compiler.SIDL.TypeUniverse
            for t in ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(entryType).getTypes():
              borrow = self.Python.canBorrowArray(t)
              wrapArrayFuncs[(t, borrow)] = self.Python.getWrapArray(t, borrow)
          else:
            borrow = self.Python.canBorrowArray(entryType)
            wrapArrayFuncs[(entryType, borrow)] = self.Python.getWrapArray(entryType, borrow)
          if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
            wrapIORFuncs[(entryType, 0)] = self.Python.getWrapIOR(entryType)
      for parameter in method.getOutParameters():
        paramType = parameter.getType()
        if paramType.getBaseType():
          if paramType.getIdentifier() == 'opaque':
            extractIORFuncs[paramType] = self.Python.getExtractOpaque(paramType)
        elif paramType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
          extractIORFuncs[paramType] = self.Python.getExtractIOR(paramType, klass)
        elif paramType.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          entryType = ASE.Compiler.SIDL.Array.Array(paramType).getType()
          if entryType.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse'):
            import ASE.Compiler.SIDL.TypeUniverse
            for t in ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(entryType).getTypes():
              borrow = self.Python.canBorrowArray(t)
              extractArrayFuncs[(t, borrow)] = self.Python.getExtractArray(t, borrow, klass)
          else:
            borrow = self.Python.canBorrowArray(entryType)
            extractArrayFuncs[(entryType, borrow)] = self.Python.getExtractArray(entryType, borrow, klass)
          if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
            extractIORFuncs[entryType] = self.Python.getExtractIOR(entryType, klass)
        elif paramType.isInstanceOf('ASE.Compiler.SIDL.Iterator'):
          import ASE.Compiler.SIDL.Iterator
          entryType = ASE.Compiler.SIDL.Iterator.Iterator(paramType).getType()
          if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
            extractIteratorFuncs[entryType] = self.Python.getExtractIterator(entryType, klass)
            extractIORFuncs[entryType] = self.Python.getExtractIOR(entryType, klass)
            wrapIORFuncs[(entryType, 1)] = self.Python.getWrapIOR(entryType, isWeak = 1)
    return (wrapIORFuncs, extractIORFuncs, wrapArrayFuncs, extractArrayFuncs, extractIteratorFuncs)
  
  def getSourceFile(self, klass, decls):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.Include
    import ASE.Compiler.Cxx.Source
    import operator
    import sets
    source = ASE.Compiler.Cxx.Source.Source()
    source.setFilename(self.Skeleton.getFileBasename(klass) + '.c')
    pyInclude = ASE.Compiler.Cxx.Include.Include()
    pyInclude.setIdentifier('<Python.h>')
    numericInclude = ASE.Compiler.Cxx.Include.Include()
    numericInclude.setIdentifier('<Numeric/arrayobject.h>')
    source.addChildren([pyInclude, numericInclude, self.IOR.getGuardedInclude(klass)])
    source.addChildren([self.IOR.getGuardedInclude(self.SIDL.getLoader(klass)), self.IOR.getGuardedInclude(self.SIDL.getASEException(klass))])
    source.addChildren([self.IOR.getGuardedInclude(exc) for exc in sets.Set(reduce(operator.concat, [list(method.getExceptions()) for method in klass.getImplementedMethods()], []))])
    source.addChildren(decls)
    source.setPurpose(ASE.Compiler.CodePurpose.SKELETON)
    return source
  
  def copyPythonTraceback(self, excVar, tbVar):
    import ASE.Compiler.Cxx.And
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Inequality
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.While
    codeVar = self.Cxx.getVar('code')
    getAttrVar = self.Cxx.getVar('PyObject_GetAttrString')
    getStringVar = self.Cxx.getVar('PyString_AsString')
    tbNotNone = ASE.Compiler.Cxx.Inequality.Inequality()
    tbNotNone.setChildren([tbVar, self.Cxx.getVar('Py_None')])
    tbValid = ASE.Compiler.Cxx.LogicalAnd.LogicalAnd()
    tbValid.setChildren([tbVar, self.Cxx.getGroup(tbNotNone)])
    getFrame = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getFrame.setChildren([getAttrVar])
    getFrame.setArguments([tbVar, self.Cxx.castToString('tb_frame')])
    getCode = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getCode.setChildren([getAttrVar])
    getCode.setArguments([getFrame, self.Cxx.castToString('f_code')])
    codeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    codeInit.setChildren([getCode])
    excVarInd = ASE.Compiler.Cxx.Indirection.Indirection()
    excVarInd.setChildren([excVar])
    excVarObj = ASE.Compiler.Cxx.StructReference.StructReference()
    excVarObj.setChildren([self.Cxx.getGroup(excVarInd)])
    excVarObj.setMember('d_object')
    excVarObj.setPointer(1)
    excEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    excEpv.setChildren([self.Cxx.getGroup(excVarInd)])
    excEpv.setMember('d_epv')
    excEpv.setPointer(1)
    addTraceFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    addTraceFunc.setChildren([excEpv])
    addTraceFunc.setMember('addStackEntry')
    addTraceFunc.setPointer(1)
    filenameCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    filenameCall.setChildren([getAttrVar])
    filenameCall.setArguments([codeVar, self.Cxx.castToString('co_filename')])
    getFilename = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getFilename.setChildren([getStringVar])
    getFilename.setArguments([filenameCall])
    linenoCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    linenoCall.setChildren([getAttrVar])
    linenoCall.setArguments([tbVar, self.Cxx.castToString('tb_lineno')])
    getLineno = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getLineno.setChildren([self.Cxx.getVar('PyInt_AsLong')])
    getLineno.setArguments([linenoCall])
    nameCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    nameCall.setChildren([getAttrVar])
    nameCall.setArguments([codeVar, self.Cxx.castToString('co_name')])
    getName = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getName.setChildren([getStringVar])
    getName.setArguments([nameCall])
    addTrace = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    addTrace.setChildren([addTraceFunc])
    addTrace.setArguments([excVarObj, getFilename, getLineno, getName, self.Cxx.getNull()])
    getNextTb = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getNextTb.setChildren([getAttrVar])
    getNextTb.setArguments([tbVar, self.Cxx.castToString('tb_next')])
    incTrace = ASE.Compiler.Cxx.Assignment.Assignment()
    incTrace.setChildren([tbVar, getNextTb])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setDeclarations([self.Cxx.getDeclaration(codeVar, self.pyObjType, codeInit)])
    compound.setChildren([self.Cxx.getExpStmt(addTrace), self.Cxx.getExpStmt(incTrace)])
    tbLoop = ASE.Compiler.Cxx.While.While()
    tbLoop.setBranch(tbValid)
    tbLoop.setChildren([compound])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([tbLoop])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(excVar)
    checkExc.setChildren([compound])
    return checkExc
  
  def setException(self, excVar, localExcVar, localExcType):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    excInd = ASE.Compiler.Cxx.Indirection.Indirection()
    excInd.setChildren([excVar])
    setExc = ASE.Compiler.Cxx.Assignment.Assignment()
    setExc.setChildren([excInd, self.Cxx.castToType(localExcVar, self.Python.getType(self.SIDL.getBaseException(localExcType)))])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(setExc)])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(excVar)
    checkExc.setChildren([compound])
    return checkExc
  
  def throwDefaultException(self, klass, excVar, valVar):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.StructReference
    localExcVar = self.Cxx.getVar('_local_exception')
    excType = self.SIDL.getASEException(klass)
    extFunc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    extFunc.setChildren([self.Cxx.getVar(excType.getFullIdentifier().replace('.', '_') + '__externals')])
    createFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    createFunc.setChildren([extFunc])
    createFunc.setMember('createObject')
    createFunc.setPointer(1)
    createExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    createExc.setChildren([createFunc])
    createExc.setArguments([self.Cxx.getNull()])
    setLocalExc = ASE.Compiler.Cxx.Assignment.Assignment()
    setLocalExc.setChildren([localExcVar, createExc])
    excEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    excEpv.setChildren([localExcVar])
    excEpv.setMember('d_epv')
    excEpv.setPointer(1)
    messageFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    messageFunc.setChildren([excEpv])
    messageFunc.setMember('setMessage')
    messageFunc.setPointer(1)
    excPyStr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    excPyStr.setChildren([self.Cxx.getVar('PyObject_Str')])
    excPyStr.setArguments([valVar])
    excStr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    excStr.setChildren([self.Cxx.getVar('PyString_AsString')])
    excStr.setArguments([excPyStr])
    setMessage = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    setMessage.setChildren([messageFunc])
    setMessage.setArguments([localExcVar, excStr, self.Cxx.getNull()])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setDeclarations([self.Cxx.getDeclaration(localExcVar, self.Python.getType(excType))])
    compound.setChildren([self.Cxx.getExpStmt(setLocalExc), self.Cxx.getExpStmt(setMessage), self.setException(excVar, localExcVar, excType)])
    return compound
  
  def throwException(self, klass, exception, excVar, typeVar, valVar, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.StructReference
    moduleVar = self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_module')
    pythonTypeVar = self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_type')
    localExcVar = self.Cxx.getVar('_local_exception')
    sidlExcVar = self.Cxx.getVar('_sidlExc')
    isClassVar = self.Cxx.getVar('isClass')
    body.addDeclarations([self.Cxx.getDeclaration(moduleVar, self.pyObjType), self.Cxx.getDeclaration(pythonTypeVar, self.pyObjType)])
    moduleCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    moduleCall.setChildren([self.Cxx.getVar('PyImport_ImportModule')])
    moduleCall.setArguments([self.Cxx.castToString(exception.getFullIdentifier())])
    setModule = ASE.Compiler.Cxx.Assignment.Assignment()
    setModule.setChildren([moduleVar, moduleCall])
    moduleDict = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    moduleDict.setChildren([self.Cxx.getVar('PyModule_GetDict')])
    moduleDict.setArguments([moduleVar])
    typeCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    typeCall.setChildren([self.Cxx.getVar('PyMapping_GetItemString')])
    typeCall.setArguments([moduleDict, self.Cxx.castToString('Exception')])
    setType = ASE.Compiler.Cxx.Assignment.Assignment()
    setType.setChildren([pythonTypeVar, typeCall])
    decrefModule = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefModule.setChildren([self.Cxx.getVar('Py_DECREF')])
    decrefModule.setArguments([moduleVar])
    body.addChildren(map(self.Cxx.getExpStmt, [setModule, setType, decrefModule]))
    checkInstance = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    checkInstance.setChildren([self.Cxx.getVar('PyObject_IsInstance')])
    checkInstance.setArguments([valVar, pythonTypeVar])
    isClassInit = ASE.Compiler.Cxx.Initializer.Initializer()
    isClassInit.setChildren([checkInstance])
    checkType = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    checkType.setChildren([self.Cxx.getVar('PyErr_GivenExceptionMatches')])
    checkType.setArguments([typeVar, pythonTypeVar])
    getExcArgs = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getExcArgs.setChildren([self.Cxx.getVar('PyObject_GetAttrString')])
    getExcArgs.setArguments([valVar, self.Cxx.castToString('args')])
    getSidlExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getSidlExc.setChildren([self.Cxx.getVar('PySequence_GetItem')])
    getSidlExc.setArguments([getExcArgs, self.Cxx.getInteger(0)])
    setSidlExc = ASE.Compiler.Cxx.Assignment.Assignment()
    setSidlExc.setChildren([sidlExcVar, self.Cxx.castToType(getSidlExc, self.Python.getSIDLObjectType(klass))])
    trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    trueBranch.setChildren([self.Cxx.getExpStmt(setSidlExc)])
    setSidlExc = ASE.Compiler.Cxx.Assignment.Assignment()
    setSidlExc.setChildren([sidlExcVar, self.Cxx.castToType(valVar, self.Python.getSIDLObjectType(klass))])
    falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    falseBranch.setChildren([self.Cxx.getExpStmt(setSidlExc)])
    checkValue = ASE.Compiler.Cxx.If.If()
    checkValue.setBranch(isClassVar)
    checkValue.setChildren([trueBranch, falseBranch])
    increfValue = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    increfValue.setChildren([self.Cxx.getVar('PyObject_CallMethod')])
    increfValue.setArguments([self.Cxx.castToType(sidlExcVar, self.pyObjType), self.Cxx.castToString('addReference'), self.Cxx.castToString('')])
    valIor = ASE.Compiler.Cxx.StructReference.StructReference()
    valIor.setChildren([sidlExcVar])
    valIor.setMember('ior')
    valIor.setPointer(1)
    setExc = ASE.Compiler.Cxx.Assignment.Assignment()
    setExc.setChildren([localExcVar, self.Cxx.castToType(valIor, self.Python.getType(exception))])
    localEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    localEpv.setChildren([localExcVar])
    localEpv.setMember('d_epv')
    localEpv.setPointer(1)
    castFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    castFunc.setChildren([localEpv])
    castFunc.setMember('castObject')
    castFunc.setPointer(1)
    castFuncInd = ASE.Compiler.Cxx.Indirection.Indirection()
    castFuncInd.setChildren([self.Cxx.getGroup(castFunc)])
    localObj = ASE.Compiler.Cxx.StructReference.StructReference()
    localObj.setChildren([localExcVar])
    localObj.setMember('d_object')
    localObj.setPointer(1)
    castCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castCall.setChildren([self.Cxx.getGroup(castFuncInd)])
    castCall.setArguments([localObj, self.Cxx.getString(exception.getFullIdentifier()), self.Cxx.getNull()])
    castExc = ASE.Compiler.Cxx.Assignment.Assignment()
    castExc.setChildren([localExcVar, castCall])
    decrefSidlExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefSidlExc.setChildren([self.Cxx.getVar('Py_DECREF')])
    decrefSidlExc.setArguments([sidlExcVar])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(decrefSidlExc)])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(isClassVar)
    checkExc.setChildren([compound])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.addDeclarations([self.Cxx.getDeclaration(localExcVar, self.Python.getType(exception)), self.Cxx.getDeclaration(sidlExcVar, self.Python.getSIDLObjectType(klass)), self.Cxx.getDeclaration(isClassVar, self.Python.typeMap['bool'], isClassInit)])
    compound.addChildren([checkValue])
    compound.addChildren(map(self.Cxx.getExpStmt, [increfValue, setExc, castExc]))
    compound.addChildren([checkExc, self.setException(excVar, localExcVar, exception)])
    checkExcType = ASE.Compiler.Cxx.If.If()
    checkExcType.setBranch(checkType)
    checkExcType.setChildren([compound])
    return checkExcType
  
  def getMethodHandleExceptions(self, klass, method, body):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Inequality
    import ASE.Compiler.Cxx.Return
    excVar = self.Cxx.getVar('_exception')
    typeVar = self.Cxx.getVar('_excType')
    valVar = self.Cxx.getVar('_excValue')
    tbVar = self.Cxx.getVar('_excTraceback')
    excBody = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    excBody.addDeclarations([self.Cxx.getDeclaration(typeVar, self.pyObjType), self.Cxx.getDeclaration(valVar, self.pyObjType), self.Cxx.getDeclaration(tbVar, self.pyObjType)])
    fetchExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    fetchExc.setChildren([self.Cxx.getVar('PyErr_Fetch')])
    fetchExc.setArguments([self.Cxx.getAddress(typeVar), self.Cxx.getAddress(valVar), self.Cxx.getAddress(tbVar)])
    excBody.addChildren([self.Cxx.getExpStmt(fetchExc)])
    checkExcType = None
    for exception in method.getExceptions():
      check = self.throwException(klass, exception, excVar, typeVar, valVar, excBody)
      if checkExcType is None:
        checkExcType = check
        fullExcCheck = checkExcType
      else:
        checkExcType.addChildren([check])
        checkExcType = check
    checkExcType.addChildren([self.throwDefaultException(klass, excVar, valVar)])
    excBody.addChildren([fullExcCheck, self.copyPythonTraceback(excVar, tbVar)])
    for exception in method.getExceptions():
      decrefType = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefType.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefType.setArguments([self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_type')])
      excBody.addChildren([self.Cxx.getExpStmt(decrefType)])
    decrefType = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefType.setChildren([self.Cxx.getVar('Py_DECREF')])
    decrefType.setArguments([typeVar])
    decrefVal = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefVal.setChildren([self.Cxx.getVar('Py_DECREF')])
    decrefVal.setArguments([valVar])
    decrefTb = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefTb.setChildren([self.Cxx.getVar('Py_XDECREF')])
    decrefTb.setArguments([tbVar])
    tbNotNone = ASE.Compiler.Cxx.Inequality.Inequality()
    tbNotNone.setChildren([tbVar, self.Cxx.getVar('Py_None')])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(decrefTb)])
    checkTb = ASE.Compiler.Cxx.If.If()
    checkTb.setBranch(tbNotNone)
    checkTb.setChildren([compound])
    returnStmt = ASE.Compiler.Cxx.Return.Return()
    if (not (method is None) and not (method.getReturnParameter() is None)):
      returnStmt.setChildren([self.Cxx.getVar('_return_value')])
    excBody.addChildren([self.Cxx.getExpStmt(decrefType), self.Cxx.getExpStmt(decrefVal), checkTb, returnStmt])
    isErr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    isErr.setChildren([self.Cxx.getVar('PyErr_Occurred')])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(isErr)
    checkExc.setChildren([excBody])
    body.addChildren([checkExc])
    return 
  
  def getMethodDummyArguments(self, klass, method, body):
    argsVar = self.Cxx.getVar('_args')
    body.addDeclarations([self.Cxx.getDeclaration(argsVar, self.pyObjType)])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(argsVar, self.Cxx.getFunctionCall('Py_BuildValue', [self.Cxx.castToString('()')])), 'Dummy arguments for dispatch to Python')])
    body.addChildren([self.IOR.getErrorCheck(klass, method, argsVar, 'Failed to create dummy arguments for ' + klass.getFullIdentifier() + '.' + method.getIdentifier())])
    return 
  
  def getMethodImplMethod(self, klass, method, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.FunctionCall
    contextVar = self.Cxx.getVar('_context')
    methodVar = self.Cxx.getVar('_method')
    getMethod = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getMethod.setChildren([self.Cxx.getVar('PyObject_GetAttrString')])
    getMethod.setArguments([contextVar, self.Cxx.castToString(method.getIdentifier())])
    setMethod = ASE.Compiler.Cxx.Assignment.Assignment()
    setMethod.setChildren([methodVar, getMethod])
    errorMsg = 'Null implementation method for ' + klass.getFullIdentifier() + '.' + method.getIdentifier()
    body.addDeclarations([self.Cxx.getDeclaration(methodVar, self.pyObjType)])
    body.addChildren([self.Cxx.getExpStmt(setMethod, 'Get method object'), self.Python.getClearError(), self.IOR.getErrorCheck(klass, method, methodVar, errorMsg)])
    return None
  
  def getMethodImplObject(self, klass, method, body, forceStatic = 0):
    import ASE.Compiler.Cxx.If
    contextVar = self.Cxx.getVar('_context')
    if (not (method.getStatic()) and not (forceStatic)):
      selfVar = self.Cxx.getVar('_self')
      dataVar = self.Cxx.getStructRef(selfVar, 'd_data')
      checkSelf = ASE.Compiler.Cxx.If.If()
      checkSelf.setBranch(selfVar)
      checkSelf.setChildren([dataVar, self.Cxx.getNull()])
      checkSelf.setTernary(1)
      setContext = self.Cxx.getAssignment(contextVar, checkSelf)
    else:
      importMod = self.Cxx.getFunctionCall('PyImport_ImportModule', [self.Cxx.castToString(klass.getFullIdentifier() + '_impl')])
      setContext = self.Cxx.getAssignment(contextVar, importMod)
    body.addDeclarations([self.Cxx.getDeclaration(contextVar, self.pyObjType)])
    errorMsg = 'Null implementation object for ' + klass.getFullIdentifier() + '.' + method.getIdentifier()
    body.addChildren([self.Cxx.getExpStmt(setContext, 'Get implementation object'), self.Python.getClearError(), self.IOR.getErrorCheck(klass, method, contextVar, errorMsg)])
    return 
  
  def getMethodSkeleton(self, klass, method):
    import ASE.Compiler.Cxx.CompoundStatement
    methodSkeleton = self.IOR.getMethodSignature(klass, method)
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    if (klass.getFullIdentifier() == 'ASE.BaseClass' and method.getFullIdentifier() == 'ASE.BaseInterface.deleteReference'):
      typeVar = self.Cxx.getVar('_saveExcType')
      valVar = self.Cxx.getVar('_saveExcValue')
      tbVar = self.Cxx.getVar('_saveExcTraceback')
      body.addDeclarations([self.Cxx.getDeclaration(typeVar, self.pyObjType), self.Cxx.getDeclaration(valVar, self.pyObjType), self.Cxx.getDeclaration(tbVar, self.pyObjType)])
      body.addChildren([self.Python.getSuspendError()])
    self.getMethodImplObject(klass, method, body)
    self.getMethodImplMethod(klass, method, body)
    self.Python.marshalCToPythonCall(klass, method, body)
    self.Python.dispatch(method, body)
    self.getMethodHandleExceptions(klass, method, body)
    self.Python.marshalPythonToCReturn(klass, method, body)
    return [methodSkeleton]
  
  def getConstructorMethod(self, klass):
    if not (klass in self.constructors):
      import ASE.Compiler.ParameterAccessMode
      import ASE.Compiler.SIDL.Method
      import ASE.Compiler.SIDL.Parameter
      objVar = self.Cxx.getVar('obj')
      objParam = ASE.Compiler.SIDL.Parameter.Parameter()
      objParam.setIdentifier(objVar.getIdentifier())
      objParam.setType(klass)
      objParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
      constructor = ASE.Compiler.SIDL.Method.Method()
      constructor.setIdentifier(klass.getIdentifier())
      constructor.setChildren([objParam])
      constructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
      constructor.setParent(klass)
      constructor.setStatic(1)
      self.constructors[klass] = constructor
    return self.constructors[klass]
  
  def getDestructorMethod(self, klass):
    if not (klass in self.destructors):
      import ASE.Compiler.SIDL.Method
      destructor = ASE.Compiler.SIDL.Method.Method()
      destructor.setIdentifier(klass.getIdentifier() + '_')
      destructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
      destructor.setParent(klass)
      self.destructors[klass] = destructor
    return self.destructors[klass]
  
  def getIterNextMethod(self, interface):
    '''
    The iterator looks like an entry object, but its d_data member is actually a Python iterator pointer
    '''
    import ASE.Compiler.SIDL.Iterator
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Type
    iterType = ASE.Compiler.SIDL.Iterator.Iterator()
    iterType.setType(interface)
    iterNext = ASE.Compiler.SIDL.Method.Method()
    iterNext.setIdentifier('iterNext')
    iterNext.setReturnParameter(self.SIDL.getParameter('_return_value', interface, 'return'))
    iterNext.setChildren([self.SIDL.getParameter('_self', iterType)])
    iterNext.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(interface))])
    iterNext.setParent(interface)
    iterNext.setComments([self.SIDL.getComment('Builtin method')])
    iterNext.setStatic(1)
    return iterNext
  
  def getConstructorSkeleton(self, klass):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.StructReference
    method = self.getConstructorMethod(klass)
    methodSkeleton = self.IOR.getMethodSignature(klass, method)
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    self.getMethodImplObject(klass, method, body)
    self.getMethodImplMethod(klass, method, body)
    self.Python.marshalCToPythonCall(klass, method, body, isConstructor = 1)
    methodVar = self.Cxx.getVar('_method')
    argsVar = self.Cxx.getVar('_args')
    returnVar = self.Cxx.getVar('_return_value')
    pyReturnVar = self.Cxx.getVar('_pyReturn')
    objVar = self.Cxx.getVar(method.getChildren()[0].getIdentifier())
    if klass.getFullIdentifier() == 'ASE.BaseClass':
      longType = ASE.Compiler.Cxx.Type.Type()
      longType.setIdentifier('long')
      longType.setBaseType(1)
      objectsVar = self.Cxx.getVar('objects')
      addrVar = self.Cxx.getVar('addr')
      oneVar = self.Cxx.getVar('one')
      createOne = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      createOne.setChildren([self.Cxx.getVar('PyInt_FromLong')])
      createOne.setArguments([self.Cxx.getInteger(1)])
      oneInit = ASE.Compiler.Cxx.Initializer.Initializer()
      oneInit.setChildren([createOne])
      createAddr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      createAddr.setChildren([self.Cxx.getVar('PyInt_FromLong')])
      createAddr.setArguments([self.Cxx.castToType(objVar, longType)])
      addrInit = ASE.Compiler.Cxx.Initializer.Initializer()
      addrInit.setChildren([createAddr])
      getObjects = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      getObjects.setChildren([self.Cxx.getVar('PyObject_GetAttrString')])
      getObjects.setArguments([self.Cxx.getVar('_context'), self.Cxx.getString('objects')])
      setObjects = ASE.Compiler.Cxx.Assignment.Assignment()
      setObjects.setChildren([objectsVar, getObjects])
      body.addChildren([self.Cxx.getExpStmt(setObjects)])
      branchExp = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      branchExp.setChildren([self.Cxx.getVar('PyMapping_HasKey')])
      branchExp.setArguments([objectsVar, addrVar])
      body.addChildren([self.IOR.getErrorCheck(klass, method, None, 'Redundant object creation', branchExp)])
      setItem = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      setItem.setChildren([self.Cxx.getVar('PyDict_SetItem')])
      setItem.setArguments([objectsVar, addrVar, oneVar])
      decrefAddr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefAddr.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefAddr.setArguments([addrVar])
      decrefOne = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefOne.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefOne.setArguments([oneVar])
      decrefObjects = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefObjects.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefObjects.setArguments([objectsVar])
      body.addDeclarations([self.Cxx.getDeclaration(objectsVar, self.pyObjType), self.Cxx.getDeclaration(addrVar, self.pyObjType, addrInit), self.Cxx.getDeclaration(oneVar, self.pyObjType, oneInit)])
      body.addChildren([self.Cxx.getExpStmt(s) for s in [setItem, decrefAddr, decrefOne, decrefObjects]])
    createObj = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    createObj.setChildren([self.Cxx.getVar('PyInstance_New')])
    createObj.setArguments([methodVar, argsVar, self.Cxx.getNull()])
    setReturn = ASE.Compiler.Cxx.Assignment.Assignment()
    setReturn.setChildren([returnVar, createObj])
    setPyReturn = ASE.Compiler.Cxx.Assignment.Assignment()
    setPyReturn.setChildren([pyReturnVar, self.Cxx.getVar('Py_None')])
    decrefMethod = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefMethod.setChildren([self.Cxx.getVar('Py_DECREF')])
    decrefMethod.setArguments([methodVar])
    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.pyObjType)])
    body.addChildren([self.Cxx.getExpStmt(setReturn, 'Create a new instance'), self.Cxx.getExpStmt(setPyReturn), self.Cxx.getExpStmt(decrefMethod)])
    errorMsg = 'Could not create instance of ' + klass.getFullIdentifier() + '_impl.' + klass.getIdentifier()
    body.addChildren([self.Python.getClearError(), self.IOR.getErrorCheck(klass, method, returnVar, errorMsg)])
    dataVar = ASE.Compiler.Cxx.StructReference.StructReference()
    dataVar.setChildren([objVar])
    dataVar.setMember('d_data')
    dataVar.setPointer(1)
    setObj = ASE.Compiler.Cxx.Assignment.Assignment()
    setObj.setChildren([dataVar, returnVar])
    body.addChildren([self.Cxx.getExpStmt(setObj)])
    self.getMethodHandleExceptions(klass, method, body)
    self.Python.marshalPythonToCReturn(klass, method, body)
    return [methodSkeleton]
  
  def getDestructorSkeleton(self, klass):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Inequality
    import ASE.Compiler.Cxx.Type
    method = self.getDestructorMethod(klass)
    methodSkeleton = self.IOR.getMethodSignature(klass, method)
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    selfVar = self.Cxx.getVar('_self')
    if klass.getFullIdentifier() == 'ASE.BaseClass':
      self.getMethodImplObject(klass, method, body, forceStatic = 1)
      longType = ASE.Compiler.Cxx.Type.Type()
      longType.setIdentifier('long')
      longType.setBaseType(1)
      twoVar = self.Cxx.getVar('two')
      addrVar = self.Cxx.getVar('addr')
      objectsVar = self.Cxx.getVar('objects')
      twoInit = self.Cxx.getInitializer(None, self.Cxx.getFunctionCall('PyInt_FromLong', [self.Cxx.getInteger(2)]))
      addrInit = self.Cxx.getInitializer(None, self.Cxx.getFunctionCall('PyInt_FromLong', [self.Cxx.castToType(selfVar, longType)]))
      body.addDeclarations([self.Cxx.getDeclaration(objectsVar, self.pyObjType)])
      body.addDeclarations([self.Cxx.getDeclaration(addrVar, self.pyObjType, addrInit), self.Cxx.getDeclaration(twoVar, self.pyObjType, twoInit)])
      setObjects = self.Cxx.getAssignment(objectsVar, self.Cxx.getFunctionCall('PyObject_GetAttrString', [self.Cxx.getVar('_context'), self.Cxx.getString('objects')]))
      body.addChildren([self.Cxx.getExpStmt(setObjects)])
      hasKey = self.Cxx.getFunctionCall('PyMapping_HasKey', [objectsVar, addrVar])
      body.addChildren([self.IOR.getErrorCheck(klass, method, None, 'Destruction of an object which was never created', self.Cxx.getNegation(hasKey))])
      branchExp = ASE.Compiler.Cxx.Inequality.Inequality()
      branchExp.setChildren([self.Cxx.getFunctionCall('PyInt_AsLong', [self.Cxx.getFunctionCall('PyDict_GetItem', [objectsVar, addrVar])]), self.Cxx.getInteger(1)])
      body.addChildren([self.IOR.getErrorCheck(klass, method, None, 'Destruction of an object already destroyed', branchExp)])
      body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('PyDict_SetItem', [objectsVar, addrVar, twoVar]))])
      body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_DECREF', [addrVar])), self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_DECREF', [twoVar]))])
      body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_DECREF', [objectsVar]))])
    dataVar = self.Cxx.getVar('data')
    body.addDeclarations([self.Cxx.getDeclaration(dataVar, self.pyObjType, self.Cxx.getInitializer(None, self.Cxx.castToType(self.Cxx.getStructRef(selfVar, 'd_data'), self.pyObjType)))])
    setData = self.Cxx.getAssignment(self.Cxx.getStructRef(selfVar, 'd_data'), self.Cxx.getNull())
    body.addChildren([self.Cxx.getExpStmt(setData, 'If we do not set d_data to NULL, we get a SEGV on the recursive call to deleteReference() coming from self.__IORself')])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_XDECREF', [dataVar]), 'Remove only reference to Python object')])
    return [methodSkeleton]
  
  def getIterNextSkeleton(self, klass, containerClass = None):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    if containerClass is None:
      containerClass = klass
    method = self.getIterNextMethod(klass)
    methodSkeleton = self.IOR.getMethodSignature(klass, method)
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    selfVar = self.Cxx.getVar('_self')
    iteratorVar = self.Cxx.getVar('iterator')
    returnVar = self.Cxx.getVar('_pyReturn')
    body.addDeclarations([self.Cxx.getDeclaration(iteratorVar, self.pyObjType, self.Cxx.getInitializer(None, self.Cxx.castToType(self.Cxx.getStructRef(selfVar, 'iterator'), self.pyObjType)))])
    errorMsg = 'Invalid iterator for ' + klass.getFullIdentifier() + '.' + method.getIdentifier()
    body.addChildren([self.IOR.getErrorCheck(klass, method, None, errorMsg, self.Cxx.getNegation(self.Cxx.getFunctionCall('PyIter_Check', [iteratorVar])))])
    self.Python.marshalCToPythonCall(klass, method, body)
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(returnVar, self.Cxx.getFunctionCall('PyIter_Next', [iteratorVar])))])
    self.getMethodHandleExceptions(containerClass, method, body)
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(returnVar, self.Cxx.getVar('Py_None'))), self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_INCREF', [returnVar]))])
    checkReturn = ASE.Compiler.Cxx.If.If()
    checkReturn.setBranch(self.Cxx.getNegation(returnVar))
    checkReturn.setChildren([compound])
    body.addChildren([checkReturn])
    self.Python.marshalPythonToCReturn(klass, method, body)
    return [methodSkeleton]
  
  def getInitialization(self, klass, body, imports = []):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Negation
    initVar = self.Cxx.getVar('initialized')
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(initVar, self.Cxx.getInteger(1))), self.Cxx.getExpStmt(self.Cxx.getFunctionCall('SIDL_Python_Init'))])
    for moduleName in ['user', 'importer'] + imports:
      negMod = ASE.Compiler.Cxx.Negation.Negation()
      negMod.setChildren([self.Cxx.getFunctionCall('PyImport_ImportModule', [self.Cxx.castToString(moduleName)])])
      compound.addChildren([self.IOR.getErrorCheck(klass, None, None, 'Could not import ' + moduleName, negMod)])
    compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('import_array'))])
    notInitVar = ASE.Compiler.Cxx.Negation.Negation()
    notInitVar.setChildren([initVar])
    checkInit = ASE.Compiler.Cxx.If.If()
    checkInit.setBranch(notInitVar)
    checkInit.setChildren([compound])
    body.addDeclarations([self.Cxx.getDeclaration(initVar, self.Cxx.typeMap['int'], isStatic = 1)])
    body.addChildren([checkInit])
    return None
  
  def getSetEpvSignature(self, klass):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    epvStruct = ASE.Compiler.Cxx.Struct.Struct()
    epvStruct.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '__epv')
    epvType = ASE.Compiler.Cxx.Pointer.Pointer()
    epvType.setChildren([epvStruct])
    epvParam = ASE.Compiler.Cxx.Parameter.Parameter()
    epvParam.setIdentifier('_epv')
    epvParam.setType(epvType)
    excType = ASE.Compiler.Cxx.Pointer.Pointer()
    excType.setChildren([self.IOR.getType(self.SIDL.getBaseException(self.SIDL.getASEException(klass)))])
    excParam = ASE.Compiler.Cxx.Parameter.Parameter()
    excParam.setIdentifier('_exception')
    excParam.setType(excType)
    methodName = ASE.Compiler.Cxx.Declarator.Declarator()
    methodName.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '__set_epv')
    methodDecl = ASE.Compiler.Cxx.Function.Function()
    methodDecl.setType(self.Python.typeMap['void'])
    methodDecl.setChildren([methodName])
    methodDecl.setParameters([epvParam, excParam])
    return methodDecl
  
  def getSetEpvSkeleton(self, klass):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.CompoundStatement
    methodDecl = self.Cxx.getDecl(self.getSetEpvSignature(klass))
    methodSkeleton = self.getSetEpvSignature(klass)
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    self.getInitialization(klass, body, [klass.getFullIdentifier()])
    epvVar = self.Cxx.getVar('_epv')
    methods = [self.getConstructorMethod(klass), self.getDestructorMethod(klass), self.getIterNextMethod(klass)]
    methods.extend([m for m in self.getTemplateMethods(klass.getImplementedMethods()) if (not (m.getStatic()) and ASE.Compiler.CodePurpose.SKELETON in m.getPurpose())])
    for method in methods:
      body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(epvVar, method.getIdentifier()), self.Cxx.getVar(method.getFullIdentifier().replace('.', '_'))))])
    return [methodDecl, methodSkeleton]
  
  def getSetSepvSignature(self, klass):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    sepvStruct = ASE.Compiler.Cxx.Struct.Struct()
    sepvStruct.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '__sepv')
    sepvType = ASE.Compiler.Cxx.Pointer.Pointer()
    sepvType.setChildren([sepvStruct])
    sepvParam = ASE.Compiler.Cxx.Parameter.Parameter()
    sepvParam.setIdentifier('_sepv')
    sepvParam.setType(sepvType)
    excType = ASE.Compiler.Cxx.Pointer.Pointer()
    excType.setChildren([self.IOR.getType(self.SIDL.getBaseException(self.SIDL.getASEException(klass)))])
    excParam = ASE.Compiler.Cxx.Parameter.Parameter()
    excParam.setIdentifier('_exception')
    excParam.setType(excType)
    methodName = ASE.Compiler.Cxx.Declarator.Declarator()
    methodName.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '__set_sepv')
    methodDecl = ASE.Compiler.Cxx.Function.Function()
    methodDecl.setType(self.Python.typeMap['void'])
    methodDecl.setChildren([methodName])
    methodDecl.setParameters([sepvParam, excParam])
    return methodDecl
  
  def getSetSepvSkeleton(self, klass):
    if not (self.hasStaticMethod(klass)):
      return []
    import ASE.Compiler.Cxx.CompoundStatement
    methodDecl = self.Cxx.getDecl(self.getSetSepvSignature(klass))
    methodSkeleton = self.getSetSepvSignature(klass)
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    self.getInitialization(klass, body, [klass.getFullIdentifier()])
    sepvVar = self.Cxx.getVar('_sepv')
    methods = [m for m in self.getTemplateMethods(klass.getImplementedMethods()) if (m.getStatic() and ASE.Compiler.CodePurpose.SKELETON in m.getPurpose())]
    for method in methods:
      body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(sepvVar, method.getIdentifier()), self.Cxx.getVar(method.getFullIdentifier().replace('.', '_'))))])
    return [methodDecl, methodSkeleton]
  
  def copyType(self, type):
    if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      newType = ASE.Compiler.SIDL.Array.Array()
      newType.setIdentifier(type.getIdentifier())
      newType.setAttributes(type.getAttributes())
      newType.setLineNumber(type.getLineNumber())
      newType.setComments(type.getComments())
      newType.setChildren([child for child in type.getChildren()])
      newType.setBaseType(type.getBaseType())
      newType.setType(self.copyType(ASE.Compiler.SIDL.Array.Array(type).getType()))
      return newType
    return type
  
  def copyParameter(self, parameter):
    '''
    This will be replaced by parameter.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Parameter
    newParameter = ASE.Compiler.SIDL.Parameter.Parameter()
    newParameter.setIdentifier(parameter.getIdentifier())
    newParameter.setAttributes(parameter.getAttributes())
    newParameter.setParent(parameter.getParent())
    newParameter.setChildren(parameter.getChildren())
    newParameter.setLineNumber(parameter.getLineNumber())
    newParameter.setComments(parameter.getComments())
    newParameter.setDependencies(parameter.getDependencies())
    newParameter.setType(self.copyType(parameter.getType()))
    newParameter.setAccessMode(parameter.getAccessMode())
    newParameter.setRequired(parameter.getRequired())
    return newParameter
  
  def copyMethod(self, method):
    '''
    This will be replaced by method.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    newMethod = ASE.Compiler.SIDL.Method.Method()
    newMethod.setIdentifier(method.getIdentifier())
    newMethod.setAttributes(method.getAttributes())
    newMethod.setParent(method.getParent())
    newMethod.setChildren([self.copyParameter(ASE.Compiler.SIDL.Parameter.Parameter(child)) for child in method.getChildren()])
    newMethod.setLineNumber(method.getLineNumber())
    newMethod.setComments(method.getComments())
    newMethod.setDependencies(method.getDependencies())
    newMethod.setTemplateIdentifiers(method.getTemplateIdentifiers())
    if not (method.getReturnParameter() is None):
      newMethod.setReturnParameter(self.copyParameter(method.getReturnParameter()))
    newMethod.setExceptions(method.getExceptions())
    newMethod.setStatic(method.getStatic())
    newMethod.setInternal(method.getInternal())
    newMethod.setVirtual(method.getVirtual())
    newMethod.setPurpose(method.getPurpose())
    return newMethod
  
  def checkTemplateParameter(self, name, parameter):
    if not (parameter is None):
      type = parameter.getType()
      if (type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name):
        return True
      elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
        import ASE.Compiler.SIDL.Array
        type = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
        if (type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name):
          return True
    return False
  
  def insertType(self, parameter, type):
    if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      self.insertType(ASE.Compiler.SIDL.Array.Array(parameter.getType()), type)
    else:
      parameter.setType(type)
    return 
  
  def insertTypes(self, method, universe, type):
    import ASE.Compiler.SIDL.Parameter
    name = universe.getIdentifier()
    newMethod = self.copyMethod(method)
    newMethod.setIdentifier(newMethod.getIdentifier() + '_' + type.getFullIdentifier())
    if self.checkTemplateParameter(name, method.getReturnParameter()):
      self.insertType(newMethod.getReturnParameter(), type)
    for (child, newChild) in zip(method.getChildren(), newMethod.getChildren()):
      if self.checkTemplateParameter(name, ASE.Compiler.SIDL.Parameter.Parameter(child)):
        self.insertType(ASE.Compiler.SIDL.Parameter.Parameter(newChild), type)
    return newMethod
  
  def template(self, method, name):
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.SIDL.TypeUniverse
    universe = None
    for parameter in [method.getReturnParameter()] + [ASE.Compiler.SIDL.Parameter.Parameter(child) for child in method.getChildren()]:
      if self.checkTemplateParameter(name, parameter):
        type = parameter.getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(ASE.Compiler.SIDL.Array.Array(type).getType())
        else:
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(type)
        break
    if universe is None:
      raise RuntimeError('Could not find type universe for template ' + name + ' in method ' + method.getFullIdentifier())
    self.templateMethods[method] = [self.insertTypes(method, universe, type) for type in universe.getTypes()]
    return None
  
  def createTemplateMethod(self, method):
    if len(method.getTemplateIdentifiers()):
      if len(method.getTemplateIdentifiers()) > 1:
        raise RuntimeError('Can currently only handle a single template identifier')
      if not (method in self.templateMethods):
        self.template(method, method.getTemplateIdentifiers()[0])
    return None
  
  def getTemplateMethods(self, methods):
    for method in methods:
      if method in self.templateMethods:
        for m in self.templateMethods[method]:
          yield m
      else:
        yield method
    return 

class Skeleton(object):
  
  def getFileBasename(self, klass):
    import os
    idChain = klass.getFullIdentifier().split('.')
    if not (len(idChain[:-(1)])):
      return idChain[-(1)] + '_Skeleton'
    return os.path.join(apply(os.path.join, idChain[:-(1)]), idChain[-(1)] + '_Skeleton')
