# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.DepthFirstVisitor
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Iterator
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.StubGenerator
import ASE.Compiler.Vertex
# Ending imports section
pass
import ASE.Compiler.Python.Python
# Starting static methods section
pass
# Ending static methods section
pass

class StubGenerator:
  '''
  This visitor transforms a SIDL AST into a set of Python Stub ASTs
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.source = {}

    self.sourceLanguages = ['Python', 'Cxx']
    self.Stub = Stub()
    self.Python = ASE.Compiler.Python.Python.Python()
    self.IOR = self.Python.IOR
    self.Cxx = self.Python.Cxx
    self.SIDL = self.Python.SIDL
    self.refTypeType = self.Python.refTypeType
    self.pyObjType = self.Python.objType
    self.pyArrayType = self.Python.arrayType
    self.pyTypeType = self.Python.typeType
    self.pyTypeStruct = ASE.Compiler.Cxx.Type.Type()
    self.pyTypeStruct.setIdentifier('PyTypeObject')
    self.templateMethods = {}

    self.packageChildren = {}

  
  def getSourceLanguages(self):
    '''
    Retrieve the list of the languages in which ASTs are produced
    '''
    return self.sourceLanguages
  
  def setSourceLanguages(self, outputLanguages):
    '''
    Set the list of the languages in which ASTs are produced
    '''
    self.sourceLanguages = sourceLanguages
  
  def getSource(self, language):
    '''
    Retrieve the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not (language in self.source):
      self.source[language] = []
    return self.source[language]
  
  def setSource(self, language, vertices):
    '''
    Set the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not (language in self.source):
      self.source[language] = []
    if len(vertices) == 0:
      self.templateMethods = {}

      self.Python.clear()
    self.source[language] = list(vertices)
  
  def addSource(self, language, vertices):
    '''
    Add to the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not (language in self.sourceLanguages):
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not (language in self.source):
      self.source[language] = []
    self.source[language].extend(vertices)
  
  def removeSource(self, language, vertices):
    '''
    Remove to the list of source ASTs for the given language
    Throw an exception if a vertex or language does not exist
    '''
    if not (language in self.sourceLanguages):
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    for vertex in vertices:
      if vertex in self.source[language]:
        self.source[language].remove(vertex)
      else:
        e = ASE.Compiler.CompilerException()
        e.setMessage('Source tree does not exist: ' + vertex.getIdentifier())
        raise e
  
  def visitPackage(self, node):
    '''
    Each package has an associated __init__.py module
    '''
    import ASE.Compiler.Python.Module
    import ASE.Compiler.Python.Stmt
    import ASE.Compiler.Python.Assign
    import ASE.Compiler.Python.AssName
    import ASE.Compiler.Python.List
    self.addChild(node)
    self.__IORself.visitVertex(node)
    target = ASE.Compiler.Python.AssName.AssName()
    target.setIdentifier('all')
    value = ASE.Compiler.Python.List.List()
    value.setChildren([self.Python.getString(name) for name in self.packageChildren[node.getFullIdentifier()]])
    assign = ASE.Compiler.Python.Assign.Assign()
    assign.setValue(value)
    assign.setChildren([target])
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    stmt.setChildren([assign])
    module = ASE.Compiler.Python.Module.Module()
    module.setIdentifier(node.getFullIdentifier() + '.__init__')
    module.setComments(node.getComments())
    module.setChildren([stmt])
    del self.packageChildren[node.getFullIdentifier()]
    self.addSource('Python', [module])
  
  def visitInterface(self, node):
    '''
    Each interface becomes a client module
    '''
    self.addChild(node)
    import ASE.Compiler.SIDL.TypeUniverse
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Initializer
    module = self.Python.getClientModule(node)
    constructorMethod = self.getConstructorMethod(node)
    node.addChildren([constructorMethod])
    [self.createTemplateMethod(method) for method in node.getMethods()]
    module.addChildren(self.getEPV(node))
    module.addChildren([self.Python.getRefTypeEnumeration(), self.Python.getSIDLObject(node)])
    module.addChildren([self.Cxx.getDeclaration(self.Cxx.getVar('SIDLPythonObjectType'), self.pyTypeStruct, isStatic = 1, isForward = 1, comment = 'Need to forward declare the type structure')])
    module.addChildren([self.Python.getNew(node)])
    (wrapIORFuncs, extractIORFuncs, wrapArrayFuncs, extractArrayFuncs) = self.getParameterProcessors(node, list(node.getMethods()) + [constructorMethod])
    module.addChildren([self.Python.getWrapIOR(node, isWeak = 1)])
    module.addChildren(wrapIORFuncs.values())
    module.addChildren(extractIORFuncs.values())
    for ((entryType, borrow), func) in wrapArrayFuncs.items():
      module.addChildren([self.IOR.getGuardedInclude(entryType), func])
    for ((entryType, borrow), func) in extractArrayFuncs.items():
      module.addChildren([self.IOR.getGuardedInclude(entryType), func])
    for method in self.getTemplateMethods(node.getMethods()):
      module.addChildren(self.getMethodStub(node, method))
    weakReferenceMethod = self.getWeakReferenceMethod(node)
    module.addChildren(self.getWeakReference(node, weakReferenceMethod))
    extraMethods = [weakReferenceMethod]
    if node.getFullIdentifier() == 'ASE.BaseInterface':
      iorClassNameMethod = self.getIORClassNameMethod(node)
      module.addChildren(self.getIORClassName(node, iorClassNameMethod))
      extraMethods.append(iorClassNameMethod)
    module.addChildren(self.getConstructor(node, constructorMethod))
    module.addChildren([self.getObjectMethods(node, extraMethods)])
    module.addChildren([self.getModuleMethods(node)])
    module.addChildren(self.getPythonType(node))
    module.addChildren(self.getModuleInit(node))
    self.addSource('Cxx', [module])
  
  def visitClass(self, node):
    '''
    Each class becomes a client module
    '''
    self.visitInterface(node)
  
  def visitMethod(self, node):
    '''
    Generates method stub code available from getSource()
    Not called by interface or class generation
    '''
    method = node
    wrapIORFuncs = {}

    extractIORFuncs = {}

    wrapArrayFuncs = {}

    extractArrayFuncs = {}

    for type in [parameter.getType() for parameter in method.getOutParameters()]:
      if type.getIdentifier() == 'opaque':
        wrapIORFuncs[type] = self.Python.getWrapOpaque(type)
    for type in [parameter.getType() for parameter in method.getOutParameters()]:
      wrapIORFuncs[type] = self.Python.getWrapIOR(type)
    for type in [parameter.getType() for parameter in method.getInParameters()]:
      if type.getIdentifier() == 'opaque':
        extractIORFuncs[type] = self.Python.getExtractOpaque(type)
    for type in [parameter.getType() for parameter in method.getInParameters()]:
      extractIORFuncs[type] = self.Python.getExtractIOR(type, node)
    for type in [parameter.getType() for parameter in method.getOutParameters()]:
      entryType = ASE.Compiler.SIDL.Array.Array(type).getType()
      wrapArrayFuncs[entryType] = self.Python.getWrapArray(entryType)
      if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
        wrapIORFuncs[entryType] = self.Python.getWrapIOR(entryType)
    for type in [parameter.getType() for parameter in method.getInParameters()]:
      entryType = ASE.Compiler.SIDL.Array.Array(type).getType()
      extractArrayFuncs[entryType] = self.Python.getExtractArray(entryType, not (self.SIDL.isOut(parameter)), node)
      if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
        extractIORFuncs[entryType] = self.Python.getExtractIOR(entryType, node)
    self.addSource('Cxx', wrapIORFuncs.values())
    self.addSource('Cxx', extractIORFuncs.values())
    self.addSource('Cxx', wrapArrayFuncs.values())
    self.addSource('Cxx', extractArrayFuncs.values())
    self.addSource('Cxx', self.getMethodStub(None, method))
  
  def visitEnumeration(self, node):
    '''
    Each enumeration becomes a client module
    '''
    self.addChild(node)
    import ASE.Compiler.Python.Module
    import ASE.Compiler.Python.Stmt
    module = ASE.Compiler.Python.Module.Module()
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    module.setIdentifier(node.getFullIdentifier())
    module.setComments(node.getComments())
    module.setChildren([stmt])
    self.enumAssigns = []
    self.__IORself.visitVertex(node)
    stmt.setChildren(self.enumAssigns)
    self.addSource('Python', [module])
  
  def visitEnumerator(self, node):
    '''
    Used only internally
    '''
    import ASE.Compiler.Python.Assign
    import ASE.Compiler.Python.AssName
    import ASE.Compiler.Python.Const
    assign = ASE.Compiler.Python.Assign.Assign()
    target = ASE.Compiler.Python.AssName.AssName()
    value = ASE.Compiler.Python.Const.Const()
    target.setIdentifier(node.getIdentifier())
    value.setValue_int(node.getValue())
    assign.setValue(value)
    assign.setChildren([target])
    self.enumAssigns.append(assign)
  # Ending class methods section
  pass
  
  def addChild(self, node):
    parent = node.getParent()
    if (parent is None or not (parent.isInstanceOf('ASE.Compiler.SIDL.Package'))):
      return 
    import ASE.Compiler.SIDL.Package
    package = ASE.Compiler.SIDL.Package.Package(parent).getFullIdentifier()
    if not (package in self.packageChildren):
      self.packageChildren[package] = []
    self.packageChildren[package].append(node.getIdentifier())
    return 
  
  def hasStaticMethod(self, type):
    '''
    We could use  if method.getStatic() and not method.getInternal():
    '''
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      for method in type.getMethods():
        if method.getStatic():
          return 1
    return 0
  
  def getParameterProcessors(self, interface, methods):
    wrapIORFuncs = {}

    extractIORFuncs = {}

    wrapArrayFuncs = {}

    extractArrayFuncs = {}

    for method in methods:
      for parameter in method.getOutParameters():
        paramType = parameter.getType()
        if paramType.getBaseType():
          if paramType.getIdentifier() == 'opaque':
            wrapIORFuncs[(paramType, 0)] = self.Python.getWrapOpaque(paramType)
        elif paramType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
          wrapIORFuncs[(paramType, 0)] = self.Python.getWrapIOR(paramType)
        elif paramType.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          entryType = ASE.Compiler.SIDL.Array.Array(paramType).getType()
          if entryType.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse'):
            import ASE.Compiler.SIDL.TypeUniverse
            for t in ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(entryType).getTypes():
              borrow = self.Python.canBorrowArray(t)
              wrapArrayFuncs[(t, borrow)] = self.Python.getWrapArray(t, borrow)
          else:
            borrow = self.Python.canBorrowArray(entryType)
            wrapArrayFuncs[(entryType, borrow)] = self.Python.getWrapArray(entryType, borrow)
          if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
            wrapIORFuncs[(entryType, 0)] = self.Python.getWrapIOR(entryType)
        elif paramType.isInstanceOf('ASE.Compiler.SIDL.Iterator'):
          import ASE.Compiler.SIDL.Iterator
          entryType = ASE.Compiler.SIDL.Iterator.Iterator(paramType).getType()
          if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
            wrapIORFuncs[(entryType, 1)] = self.Python.getWrapIOR(entryType, isWeak = 1)
      for parameter in method.getInParameters():
        paramType = parameter.getType()
        if paramType.getBaseType():
          if paramType.getIdentifier() == 'opaque':
            extractIORFuncs[paramType] = self.Python.getExtractOpaque(paramType)
        elif paramType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
          extractIORFuncs[paramType] = self.Python.getExtractIOR(paramType, interface)
        elif paramType.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          entryType = ASE.Compiler.SIDL.Array.Array(paramType).getType()
          if entryType.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse'):
            import ASE.Compiler.SIDL.TypeUniverse
            for t in ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(entryType).getTypes():
              borrow = self.Python.canBorrowArray(t)
              extractArrayFuncs[(t, borrow)] = self.Python.getExtractArray(t, borrow, interface)
          else:
            borrow = self.Python.canBorrowArray(entryType)
            extractArrayFuncs[(entryType, borrow)] = self.Python.getExtractArray(entryType, borrow, interface)
          if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
            extractIORFuncs[entryType] = self.Python.getExtractIOR(entryType, interface)
      for exception in method.getExceptions():
        wrapIORFuncs[(exception, 0)] = self.Python.getWrapIOR(exception)
    return (wrapIORFuncs, extractIORFuncs, wrapArrayFuncs, extractArrayFuncs)
  
  def getEPV(self, type):
    '''
    Generate epv pointer
    '''
    epv = [self.Cxx.getDeclaration(self.Cxx.getVar('_epv'), self.IOR.getExternalType(type), isStatic = 1)]
    if self.hasStaticMethod(type):
      epv.append(self.Cxx.getDeclaration(self.Cxx.getVar('_sepv'), self.IOR.getStaticExternalType(type), isStatic = 1))
    return epv
  
  def getPythonDeallocFunc(self, klass):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Inequality
    import ASE.Compiler.Cxx.LogicalAnd
    import ASE.Compiler.Cxx.Parameter
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    selfVar = self.Cxx.getVar('self')
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('PyObject_ClearWeakRefs', [self.Cxx.castToType(selfVar, self.pyObjType)]))])
    handleWeakRef = ASE.Compiler.Cxx.If.If()
    handleWeakRef.setBranch(self.Cxx.getStructRef(selfVar, 'weakRefList'))
    handleWeakRef.setChildren([compound])
    objIOR = self.Cxx.getStructRef(selfVar, 'ior')
    checkRef = ASE.Compiler.Cxx.Inequality.Inequality()
    checkRef.setChildren([self.Cxx.getStructRef(selfVar, 'refType'), self.Cxx.getVar('WEAK_REF')])
    checkSelf = ASE.Compiler.Cxx.LogicalAnd.LogicalAnd()
    checkSelf.setChildren([self.Cxx.getGroup(checkRef), self.Cxx.getGroup(self.Cxx.getStructRef(objIOR, 'd_data'))])
    objEpv = self.Cxx.getStructRef(self.Cxx.getGroup(self.Cxx.castToType(objIOR, self.IOR.getBaseType(klass))), 'd_epv')
    derefFunc = self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(objEpv, 'deleteReference'))))
    derefCall = self.Cxx.getFunctionCall(derefFunc, [self.Cxx.getStructRef(objIOR, 'd_object'), self.Cxx.getNull()])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(derefCall)])
    handleRef = ASE.Compiler.Cxx.If.If()
    if klass.isInstanceOf('ASE.Compiler.SIDL.Class'):
      handleRef.setBranch(checkSelf)
    else:
      handleRef.setBranch(checkRef)
    handleRef.setChildren([compound])
    freeCall = self.Cxx.getFunctionCall(self.Cxx.getStructRef(self.Cxx.getStructRef(selfVar, 'ob_type'), 'tp_free'), [selfVar])
    body.addChildren([handleWeakRef, handleRef, self.Cxx.getExpStmt(freeCall)])
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    selfParam.setIdentifier(selfVar.getIdentifier())
    selfParam.setType(self.Python.getSIDLObjectType(klass))
    deallocName = ASE.Compiler.Cxx.Declarator.Declarator()
    deallocName.setIdentifier('sidl_dealloc')
    deallocFunc = ASE.Compiler.Cxx.Function.Function()
    deallocFunc.setChildren([deallocName])
    deallocFunc.setType(self.Cxx.typeMap['void'])
    deallocFunc.setParameters([selfParam])
    deallocFunc.setBody(body)
    deallocFunc.setStatic(1)
    return deallocFunc
  
  def getPythonCmpFunc(self, klass):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    selfVar = self.Cxx.getVar('self')
    argVar = self.Cxx.getVar('arg')
    ior1Var = self.Cxx.getVar('ior1')
    ior2Var = self.Cxx.getVar('ior2')
    o1Var = self.Cxx.getVar('o1')
    o2Var = self.Cxx.getVar('o2')
    resultVar = self.Cxx.getVar('_return_value')
    objIOR = self.Cxx.getStructRef(self.Cxx.getGroup(self.Cxx.castToType(argVar, self.Python.getSIDLObjectType(klass))), 'ior')
    baseInterface = self.IOR.getBaseType(klass)
    body.addDeclarations([self.Cxx.getDeclaration(ior1Var, self.IOR.getType(klass), self.Cxx.getInitializer(None, self.Cxx.getStructRef(selfVar, 'ior')))])
    body.addDeclarations([self.Cxx.getDeclaration(ior2Var, baseInterface, self.Cxx.getInitializer(None, self.Cxx.castToType(objIOR, baseInterface)))])
    body.addDeclarations([self.Cxx.getDeclaration(o1Var, self.pyObjType), self.Cxx.getDeclaration(o2Var, self.pyObjType), self.Cxx.getDeclaration(resultVar, self.Cxx.typeMap['int'])])
    setO1 = self.Cxx.getAssignment(o1Var, self.Cxx.getFunctionCall('Py_BuildValue', [self.Cxx.castToString('i'), self.Cxx.getStructRef(ior1Var, 'd_object')]))
    setO2 = self.Cxx.getAssignment(o2Var, self.Cxx.getFunctionCall('Py_BuildValue', [self.Cxx.castToString('i'), self.Cxx.getStructRef(ior2Var, 'd_object')]))
    setResult = self.Cxx.getAssignment(resultVar, self.Cxx.getFunctionCall('PyObject_Compare', [o1Var, o2Var]))
    derefO1 = self.Cxx.getFunctionCall('Py_DECREF', [o1Var])
    derefO2 = self.Cxx.getFunctionCall('Py_DECREF', [o2Var])
    returnResult = ASE.Compiler.Cxx.Return.Return()
    returnResult.setChildren([resultVar])
    self.getMethodSIDLObjectCheck('sidl_cmp', body, resultVar, argVar)
    body.addChildren([self.Python.getErrorCheck('PyExc_ValueError', 'Null IOR pointer in sidl_cmp()', ior1Var, resultVar)])
    body.addChildren([self.Python.getErrorCheck('PyExc_ValueError', 'Null IOR pointer in sidl_cmp()', ior2Var, resultVar)])
    body.addChildren([self.Cxx.getExpStmt(setO1), self.Python.getErrorCheck('PyExc_RuntimeError', 'Could not build integer from IOR pointer in sidl_cmp()', o1Var, resultVar)])
    body.addChildren([self.Cxx.getExpStmt(setO2), self.Python.getErrorCheck('PyExc_RuntimeError', 'Could not build integer from IOR pointer in sidl_cmp()', o2Var, resultVar)])
    body.addChildren([self.Cxx.getExpStmt(setResult), self.Cxx.getExpStmt(derefO1), self.Cxx.getExpStmt(derefO2), returnResult])
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    selfParam.setIdentifier(selfVar.getIdentifier())
    selfParam.setType(self.Python.getSIDLObjectType(klass))
    argParam = ASE.Compiler.Cxx.Parameter.Parameter()
    argParam.setIdentifier(argVar.getIdentifier())
    argParam.setType(self.pyObjType)
    cmpName = ASE.Compiler.Cxx.Declarator.Declarator()
    cmpName.setIdentifier('sidl_cmp')
    cmpFunc = ASE.Compiler.Cxx.Function.Function()
    cmpFunc.setChildren([cmpName])
    cmpFunc.setType(self.Cxx.typeMap['int'])
    cmpFunc.setParameters([selfParam, argParam])
    cmpFunc.setBody(body)
    cmpFunc.setStatic(1)
    return cmpFunc
  
  def getPythonHashFunc(self, klass):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.Type
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    longType = ASE.Compiler.Cxx.Type.Type()
    longType.setIdentifier('long')
    longType.setBaseType(1)
    selfVar = self.Cxx.getVar('self')
    iorVar = self.Cxx.getVar('ior')
    hashVar = self.Cxx.getVar('_hash_arg')
    resultVar = self.Cxx.getVar('_return_value')
    body.addDeclarations([self.Cxx.getDeclaration(iorVar, self.IOR.getType(klass), self.Cxx.getInitializer(None, self.Cxx.getStructRef(selfVar, 'ior')))])
    body.addDeclarations([self.Cxx.getDeclaration(hashVar, self.pyObjType), self.Cxx.getDeclaration(resultVar, longType)])
    setHash = self.Cxx.getAssignment(hashVar, self.Cxx.getFunctionCall('Py_BuildValue', [self.Cxx.castToString('i'), self.Cxx.getStructRef(iorVar, 'd_object')]))
    setResult = self.Cxx.getAssignment(resultVar, self.Cxx.getFunctionCall('PyObject_Hash', [hashVar]))
    derefHash = self.Cxx.getFunctionCall('Py_DECREF', [hashVar])
    returnResult = ASE.Compiler.Cxx.Return.Return()
    returnResult.setChildren([resultVar])
    body.addChildren([self.Python.getErrorCheck('PyExc_ValueError', 'Null IOR pointer in sidl_hash()', iorVar, resultVar)])
    body.addChildren([self.Cxx.getExpStmt(setHash), self.Python.getErrorCheck('PyExc_RuntimeError', 'Could not hash IOR pointer in sidl_hash()', hashVar, resultVar)])
    body.addChildren([self.Cxx.getExpStmt(setResult), self.Cxx.getExpStmt(derefHash), returnResult])
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    selfParam.setIdentifier(selfVar.getIdentifier())
    selfParam.setType(self.Python.getSIDLObjectType(klass))
    hashName = ASE.Compiler.Cxx.Declarator.Declarator()
    hashName.setIdentifier('sidl_hash')
    hashFunc = ASE.Compiler.Cxx.Function.Function()
    hashFunc.setChildren([hashName])
    hashFunc.setType(longType)
    hashFunc.setParameters([selfParam])
    hashFunc.setBody(body)
    hashFunc.setStatic(1)
    return hashFunc
  
  def getPythonStrFunc(self, klass):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    selfVar = self.Cxx.getVar('self')
    iorVar = self.Cxx.getVar('ior')
    strVar = self.Cxx.getVar('_str')
    resultVar = self.Cxx.getVar('_return_value')
    body.addDeclarations([self.Cxx.getDeclaration(iorVar, self.IOR.getType(klass), self.Cxx.getInitializer(None, self.Cxx.getStructRef(selfVar, 'ior')))])
    body.addDeclarations([self.Cxx.getDeclaration(strVar, self.IOR.typeMap['string']), self.Cxx.getDeclaration(resultVar, self.pyObjType)])
    iorMethod = self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getStructRef(iorVar, 'd_epv'), 'strObject'))))
    setStr = self.Cxx.getAssignment(strVar, self.Cxx.getFunctionCall(iorMethod, [iorVar, self.Cxx.getNull()]))
    setResult = self.Cxx.getAssignment(resultVar, self.Cxx.getFunctionCall('PyString_FromString', [strVar]))
    increfResult = self.Cxx.getFunctionCall('Py_INCREF', [resultVar])
    returnResult = ASE.Compiler.Cxx.Return.Return()
    returnResult.setChildren([resultVar])
    body.addChildren([self.Cxx.getExpStmt(setStr), self.Cxx.getExpStmt(setResult), self.Cxx.getExpStmt(increfResult), returnResult])
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    selfParam.setIdentifier(selfVar.getIdentifier())
    selfParam.setType(self.Python.getSIDLObjectType(klass))
    strName = ASE.Compiler.Cxx.Declarator.Declarator()
    strName.setIdentifier('sidl_str')
    strFunc = ASE.Compiler.Cxx.Function.Function()
    strFunc.setChildren([strName])
    strFunc.setType(self.pyObjType)
    strFunc.setParameters([selfParam])
    strFunc.setBody(body)
    strFunc.setStatic(1)
    return strFunc
  
  def getPythonRichCmpFunc(self, klass):
    import ASE.Compiler.Cxx.Break
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Switch
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    selfVar = self.Cxx.getVar('self')
    argVar = self.Cxx.getVar('arg')
    opVar = self.Cxx.getVar('op')
    ior1Var = self.Cxx.getVar('ior1')
    ior2Var = self.Cxx.getVar('ior2')
    o1Var = self.Cxx.getVar('o1')
    o2Var = self.Cxx.getVar('o2')
    resultVar = self.Cxx.getVar('_return_value')
    objIOR = self.Cxx.getStructRef(self.Cxx.getGroup(self.Cxx.castToType(argVar, self.Python.getSIDLObjectType(klass))), 'ior')
    baseInterface = self.IOR.getBaseType(klass)
    body.addDeclarations([self.Cxx.getDeclaration(ior1Var, self.IOR.getType(klass), self.Cxx.getInitializer(None, self.Cxx.getStructRef(selfVar, 'ior')))])
    body.addDeclarations([self.Cxx.getDeclaration(ior2Var, baseInterface, self.Cxx.getInitializer(None, self.Cxx.castToType(objIOR, baseInterface)))])
    body.addDeclarations([self.Cxx.getDeclaration(o1Var, self.pyObjType), self.Cxx.getDeclaration(o2Var, self.pyObjType), self.Cxx.getDeclaration(resultVar, self.pyObjType)])
    switchBody = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    increfResult = self.Cxx.getFunctionCall('Py_INCREF', [resultVar])
    eqCase = self.Cxx.getExpStmt(self.Cxx.getAssignment(resultVar, self.Cxx.getVar('Py_False')))
    eqCase.setCaseLabel(self.Cxx.getVar('Py_EQ'))
    switchBody.addChildren([eqCase, self.Cxx.getExpStmt(increfResult), ASE.Compiler.Cxx.Break.Break()])
    neqCase = self.Cxx.getExpStmt(self.Cxx.getAssignment(resultVar, self.Cxx.getVar('Py_True')))
    neqCase.setCaseLabel(self.Cxx.getVar('Py_NE'))
    switchBody.addChildren([neqCase, self.Cxx.getExpStmt(increfResult), ASE.Compiler.Cxx.Break.Break()])
    ltCase = ASE.Compiler.Cxx.Statement.Statement()
    ltCase.setCaseLabel(self.Cxx.getVar('Py_LT'))
    leqCase = ASE.Compiler.Cxx.Statement.Statement()
    leqCase.setCaseLabel(self.Cxx.getVar('Py_LE'))
    gtCase = ASE.Compiler.Cxx.Statement.Statement()
    gtCase.setCaseLabel(self.Cxx.getVar('Py_GT'))
    geqCase = ASE.Compiler.Cxx.Statement.Statement()
    geqCase.setCaseLabel(self.Cxx.getVar('Py_GE'))
    errorCall = self.Cxx.getFunctionCall('PyErr_SetString', ['PyExc_TypeError', self.Cxx.castToString('sidl_richcmp: Cannot order non-SIDL object')])
    switchBody.addChildren([ltCase, leqCase, gtCase, geqCase, self.Cxx.getExpStmt(errorCall)])
    switchOp = ASE.Compiler.Cxx.Switch.Switch()
    switchOp.setBranch(opVar)
    switchOp.setChildren([switchBody])
    returnResult = ASE.Compiler.Cxx.Return.Return()
    returnResult.setChildren([resultVar])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([switchOp, returnResult])
    self.getMethodSIDLObjectCheck('sidl_richcmp', body, resultVar, argVar, errorHandler = compound, anySIDLObject = 1)
    setO1 = self.Cxx.getAssignment(o1Var, self.Cxx.getFunctionCall('Py_BuildValue', [self.Cxx.castToString('i'), self.Cxx.getStructRef(ior1Var, 'd_object')]))
    setO2 = self.Cxx.getAssignment(o2Var, self.Cxx.getFunctionCall('Py_BuildValue', [self.Cxx.castToString('i'), self.Cxx.getStructRef(ior2Var, 'd_object')]))
    setResult = self.Cxx.getAssignment(resultVar, self.Cxx.getFunctionCall('PyObject_RichCompare', [o1Var, o2Var, opVar]))
    derefO1 = self.Cxx.getFunctionCall('Py_DECREF', [o1Var])
    derefO2 = self.Cxx.getFunctionCall('Py_DECREF', [o2Var])
    body.addChildren([self.Python.getErrorCheck('PyExc_ValueError', 'Null IOR pointer in sidl_richcmp()', ior1Var, resultVar)])
    body.addChildren([self.Python.getErrorCheck('PyExc_ValueError', 'Null IOR pointer in sidl_richcmp()', ior2Var, resultVar)])
    body.addChildren([self.Cxx.getExpStmt(setO1), self.Python.getErrorCheck('PyExc_RuntimeError', 'Could not build integer from IOR pointer in sidl_richcmp()', o1Var, resultVar)])
    body.addChildren([self.Cxx.getExpStmt(setO2), self.Python.getErrorCheck('PyExc_RuntimeError', 'Could not build integer from IOR pointer in sidl_richcmp()', o2Var, resultVar)])
    body.addChildren([self.Cxx.getExpStmt(setResult), self.Cxx.getExpStmt(derefO1), self.Cxx.getExpStmt(derefO2), self.Cxx.getExpStmt(increfResult), returnResult])
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    selfParam.setIdentifier(selfVar.getIdentifier())
    selfParam.setType(self.Python.getSIDLObjectType(klass))
    argParam = ASE.Compiler.Cxx.Parameter.Parameter()
    argParam.setIdentifier(argVar.getIdentifier())
    argParam.setType(self.pyObjType)
    opParam = ASE.Compiler.Cxx.Parameter.Parameter()
    opParam.setIdentifier(opVar.getIdentifier())
    opParam.setType(self.Cxx.typeMap['int'])
    cmpName = ASE.Compiler.Cxx.Declarator.Declarator()
    cmpName.setIdentifier('sidl_richcmp')
    cmpFunc = ASE.Compiler.Cxx.Function.Function()
    cmpFunc.setChildren([cmpName])
    cmpFunc.setType(self.pyObjType)
    cmpFunc.setParameters([selfParam, argParam, opParam])
    cmpFunc.setBody(body)
    cmpFunc.setStatic(1)
    return cmpFunc
  
  def getPythonGetIterFunc(self, klass):
    '''
    This method merely returns the object itself
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    selfVar = self.Cxx.getVar('self')
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_INCREF', [selfVar])), self.Cxx.getReturn(self.Cxx.castToType(selfVar, self.pyObjType))])
    iterName = ASE.Compiler.Cxx.Declarator.Declarator()
    iterName.setIdentifier('sidl_getiter')
    iterFunc = ASE.Compiler.Cxx.Function.Function()
    iterFunc.setChildren([iterName])
    iterFunc.setType(self.pyObjType)
    iterFunc.setParameters([self.Cxx.getParameter(selfVar.getIdentifier(), self.Python.getSIDLObjectType(klass))])
    iterFunc.setBody(body)
    iterFunc.setStatic(1)
    return iterFunc
  
  def getPythonIterNextFunc(self, klass):
    '''
    This method calls the IOR iterNext() method with the input IOR pointer as an argument
    '''
    import ASE.Compiler.SIDL.Iterator
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    selfVar = self.Cxx.getVar('self')
    iorVar = self.Cxx.getVar('ior')
    iorObjVar = self.Cxx.getVar('nextIorObj')
    pyObjVar = self.Cxx.getVar('nextPyObj')
    iterType = ASE.Compiler.SIDL.Iterator.Iterator()
    iterType.setType(klass)
    body.addDeclarations([self.Cxx.getDeclaration(iorVar, self.IOR.getType(iterType), self.Cxx.getInitializer(None, self.Cxx.getStructRef(selfVar, 'ior')))])
    baseIor = self.Cxx.getGroup(self.Cxx.castToType(iorVar, self.IOR.getType(self.SIDL.getBaseInterface(klass))))
    iorNextFunc = self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getStructRef(baseIor, 'd_epv'), 'iterNext'))))
    body.addDeclarations([self.Cxx.getDeclaration(iorObjVar, self.IOR.getType(klass), self.Cxx.getInitializer(None, self.Cxx.getFunctionCall(iorNextFunc, [iorVar, self.Cxx.getNull()])))])
    body.addDeclarations([self.Cxx.getDeclaration(pyObjVar, self.Python.getSIDLObjectType(klass), isStatic = 1)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_DECREF', [pyObjVar])), self.Cxx.getExpStmt(self.Cxx.getAssignment(pyObjVar, self.Cxx.getNull()))])
    destroyFunc = self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getVar('_epv'), 'destroyIterator'))))
    compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall(destroyFunc, [iorVar, self.Cxx.getNull()]))])
    compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(selfVar, 'ior'), self.Cxx.getNull()))])
    checkPyObj = ASE.Compiler.Cxx.If.If()
    checkPyObj.setBranch(pyObjVar)
    checkPyObj.setChildren([compound])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([checkPyObj, self.Cxx.getReturn(self.Cxx.getNull())])
    checkIorObj = ASE.Compiler.Cxx.If.If()
    checkIorObj.setBranch(self.Cxx.getNegation(iorObjVar))
    checkIorObj.setChildren([compound])
    objWrapper = self.Cxx.getStructRef(iorVar, 'objWrapper')
    baseObjVar = self.Cxx.castToType(iorObjVar, self.IOR.getType(self.SIDL.getBaseInterface(klass)))
    pyNewFunc = self.Cxx.getFunctionCall(self.Cxx.getVar('sidl_new'), [self.Cxx.getAddress(self.Cxx.getVar('SIDLPythonObjectType')), baseObjVar, self.Cxx.getVar('WEAK_REF')])
    trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    trueBranch.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(objWrapper, pyNewFunc))])
    trueBranch.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(pyObjVar, self.Cxx.castToType(objWrapper, self.Python.getSIDLObjectType(klass))))])
    falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    falseBranch.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(pyObjVar, self.Cxx.castToType(objWrapper, self.Python.getSIDLObjectType(klass))))])
    falseBranch.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getStructRef(pyObjVar, 'ior'), iorObjVar))])
    checkPyObj = ASE.Compiler.Cxx.If.If()
    checkPyObj.setBranch(self.Cxx.getNegation(objWrapper))
    checkPyObj.setChildren([trueBranch, falseBranch])
    body.addChildren([checkIorObj, checkPyObj, self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_INCREF', [pyObjVar])), self.Cxx.getReturn(self.Cxx.castToType(pyObjVar, self.pyObjType))])
    iterName = ASE.Compiler.Cxx.Declarator.Declarator()
    iterName.setIdentifier('sidl_iternext')
    iterFunc = ASE.Compiler.Cxx.Function.Function()
    iterFunc.setChildren([iterName])
    iterFunc.setType(self.pyObjType)
    iterFunc.setParameters([self.Cxx.getParameter(selfVar.getIdentifier(), self.Python.getSIDLObjectType(klass))])
    iterFunc.setBody(body)
    iterFunc.setStatic(1)
    return iterFunc
  
  def getPythonType(self, klass):
    import ASE.Compiler.Cxx.InclusiveOr
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Macro
    import ASE.Compiler.Cxx.Struct
    constructor = klass.getFullIdentifier().replace('.', '_') + '_' + klass.getIdentifier()
    functions = [self.getPythonDeallocFunc(klass), self.getPythonCmpFunc(klass), self.getPythonHashFunc(klass), self.getPythonStrFunc(klass), self.getPythonRichCmpFunc(klass), self.getPythonGetIterFunc(klass), self.getPythonIterNextFunc(klass)]
    flags = ASE.Compiler.Cxx.InclusiveOr.InclusiveOr()
    flags.setChildren([self.Cxx.getVar('Py_TPFLAGS_DEFAULT'), self.Cxx.getVar('Py_TPFLAGS_BASETYPE'), self.Cxx.getVar('Py_TPFLAGS_HAVE_WEAKREFS')])
    headInit = ASE.Compiler.Cxx.Macro.Macro()
    headInit.setIdentifier('PyObject_HEAD_INIT(NULL)')
    objStruct = ASE.Compiler.Cxx.Struct.Struct()
    objStruct.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '_PythonObject')
    weakRefList = self.Cxx.getStructRef(self.Cxx.getGroup(self.Cxx.castToType(self.Cxx.getInteger(0), self.Python.getSIDLObjectType(klass))), 'weakRefList')
    weakRefListAddr = self.Cxx.getAddress(weakRefList)
    typeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    typeInit.setList(1)
    typeInit.setChildren([headInit, self.Cxx.getInteger(0), self.Cxx.castToString(klass.getFullIdentifier()), self.Cxx.getSizeof(objStruct), self.Cxx.getInteger(0), self.Cxx.castToNamedType(self.Cxx.getVar('sidl_dealloc'), 'destructor'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'printfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'getattrfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'setattrfunc'), self.Cxx.castToNamedType(self.Cxx.getVar('sidl_cmp'), 'cmpfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'reprfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'PyNumberMethods', 1), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'PySequenceMethods', 1), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'PyMappingMethods', 1), self.Cxx.castToNamedType(self.Cxx.getVar('sidl_hash'), 'hashfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'ternaryfunc'), self.Cxx.castToNamedType(self.Cxx.getVar('sidl_str'), 'reprfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'getattrofunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'setattrofunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'PyBufferProcs', 1), flags, self.Cxx.castToString('This is a Python wrapper for a SIDL object of type ' + klass.getFullIdentifier()), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'traverseproc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'inquiry'), self.Cxx.castToNamedType(self.Cxx.getVar('sidl_richcmp'), 'richcmpfunc'), self.Cxx.castToType(weakRefListAddr, self.Cxx.typeMap['int']), self.Cxx.castToNamedType(self.Cxx.getVar('sidl_getiter'), 'getiterfunc'), self.Cxx.castToNamedType(self.Cxx.getVar('sidl_iternext'), 'iternextfunc'), self.Cxx.getVar('_' + klass.getIdentifier() + 'ObjectMethods'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'PyMemberDef', 1, isStruct = 1), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'PyGetSetDef', 1, isStruct = 1), self.Cxx.getInteger(0), self.Cxx.getInteger(0), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'descrgetfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'descrsetfunc'), self.Cxx.getInteger(0), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'initproc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'allocfunc'), self.Cxx.castToNamedType(self.Cxx.getVar(constructor), 'newfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'freefunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'inquiry')])
    return functions + [self.Cxx.getDeclaration(self.Cxx.getVar('SIDLPythonObjectType'), self.pyTypeStruct, typeInit, isStatic = 1)]
  
  def getConstructorMethod(self, interface):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    objParam = ASE.Compiler.SIDL.Parameter.Parameter()
    objParam.setIdentifier('obj')
    objParam.setType(interface)
    objParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    objParam.setRequired(0)
    retParam = ASE.Compiler.SIDL.Parameter.Parameter()
    retParam.setIdentifier('_return_value')
    retParam.setType(interface)
    retParam.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    retParam.setRequired(1)
    constructor = ASE.Compiler.SIDL.Method.Method()
    constructor.setIdentifier(interface.getIdentifier())
    constructor.setReturnParameter(retParam)
    retParam.setParent(constructor)
    constructor.setChildren([objParam])
    constructor.setExceptions([self.SIDL.getBaseException(interface, 1)])
    return constructor
  
  def getConstructor(self, interface, method):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.If
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    objVar = self.Cxx.getVar(method.getChildren()[0].getIdentifier())
    self.Python.marshalPythonToCCall(method, body)
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      createdVar = self.Cxx.getVar('created')
      body.addDeclarations([self.Cxx.getDeclaration(createdVar, self.IOR.typeMap['bool'])])
      compound1 = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound1.setChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(createdVar, self.Cxx.getInteger(0)))])
      createFunc = self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getVar('_epv'), 'createObject')))
      createCall = self.Cxx.getFunctionCall(self.Cxx.getGroup(createFunc), [self.Cxx.getNull()])
      compound2 = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound2.setChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(createdVar, self.Cxx.getInteger(1))), self.Cxx.getExpStmt(self.Cxx.getAssignment(objVar, createCall))])
      checkObj = ASE.Compiler.Cxx.If.If()
      checkObj.setBranch(objVar)
      checkObj.addChildren([compound1, compound2])
      body.addChildren([checkObj])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getVar(method.getReturnParameter().getIdentifier()), objVar))])
    self.Python.marshalCToPythonReturn(method, body)
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      delRefFunc = self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getStructRef(objVar, 'd_epv'), 'deleteReference'))))
      delRefCall = self.Cxx.getFunctionCall(delRefFunc, [self.Cxx.getStructRef(objVar, 'd_object'), self.Cxx.getNull()])
      compound1 = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound1.setChildren([self.Cxx.getExpStmt(delRefCall)])
      checkRef = ASE.Compiler.Cxx.If.If()
      checkRef.setBranch(createdVar)
      checkRef.addChildren([compound1])
      body.addChildren([checkRef])
    returnVar = self.Cxx.getVar('_pyReturn')
    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.Python.objType)])
    returnVal = ASE.Compiler.Cxx.Return.Return()
    returnVal.setChildren([returnVar])
    body.addChildren([returnVal])
    typeParam = ASE.Compiler.Cxx.Parameter.Parameter()
    typeParam.setIdentifier('_type')
    typeParam.setType(self.pyObjType)
    argsParam = ASE.Compiler.Cxx.Parameter.Parameter()
    argsParam.setIdentifier('_args')
    argsParam.setType(self.pyObjType)
    kwdictParam = ASE.Compiler.Cxx.Parameter.Parameter()
    kwdictParam.setIdentifier('_kwdict')
    kwdictParam.setType(self.pyObjType)
    name = ASE.Compiler.Cxx.Declarator.Declarator()
    name.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '_' + interface.getIdentifier())
    constructor = ASE.Compiler.Cxx.Function.Function()
    constructor.setChildren([name])
    constructor.setType(self.pyObjType)
    constructor.setParameters([typeParam, argsParam, kwdictParam])
    constructor.setStatic(1)
    constructor.setBody(body)
    return [constructor]
  
  def getWeakReferenceMethod(self, interface):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    retParam = ASE.Compiler.SIDL.Parameter.Parameter()
    retParam.setIdentifier('_return_value')
    retParam.setType(interface)
    retParam.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    retParam.setRequired(1)
    method = ASE.Compiler.SIDL.Method.Method()
    method.setIdentifier('weakReference')
    method.setReturnParameter(retParam)
    method.setExceptions([self.SIDL.getBaseException(interface, 1)])
    method.setParent(interface)
    return method
  
  def getWeakReference(self, interface, method):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    returnVar = self.Cxx.getVar('_pyReturn')
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    self.getMethodIORPointer(interface, method, body)
    self.getMethodSIDLObjectCheck(method.getIdentifier(), body, returnVar)
    body.addChildren([self.Python.getErrorCheck('PyExc_TypeError', 'Null IOR pointer in ' + method.getIdentifier(), self.Cxx.getVar('_self_ior'), returnVar)])
    self.Python.marshalPythonToCCall(method, body)
    setReturn = ASE.Compiler.Cxx.Assignment.Assignment()
    setReturn.setChildren([self.Cxx.getVar('_return_value'), self.Cxx.getVar('_self_ior')])
    body.addChildren([self.Cxx.getExpStmt(setReturn)])
    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.pyObjType)])
    parameter = method.getReturnParameter()
    format = self.Python.getParseFormat(parameter)
    localArgs = self.Python.buildArgumentInterface(parameter, isArgument = 0, isWeak = 1)
    self.Python.getConvertBuildParameter(parameter, body, isArgument = 0)
    buildValueCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    buildValueCall.setChildren([self.Cxx.getVar('Py_BuildValue')])
    buildValueCall.setArguments([self.Cxx.castToString(format)] + localArgs)
    buildValue = ASE.Compiler.Cxx.Assignment.Assignment()
    buildValue.setChildren([returnVar, buildValueCall])
    body.addChildren([self.Cxx.getExpStmt(buildValue, 'Build return values')])
    returnVal = ASE.Compiler.Cxx.Return.Return()
    returnVal.setChildren([returnVar])
    body.addChildren([returnVal])
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    selfParam.setIdentifier('_self')
    selfParam.setType(self.pyObjType)
    argsParam = ASE.Compiler.Cxx.Parameter.Parameter()
    argsParam.setIdentifier('_args')
    argsParam.setType(self.pyObjType)
    kwdictParam = ASE.Compiler.Cxx.Parameter.Parameter()
    kwdictParam.setIdentifier('_kwdict')
    kwdictParam.setType(self.pyObjType)
    name = ASE.Compiler.Cxx.Declarator.Declarator()
    name.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '_' + method.getIdentifier())
    weakReference = ASE.Compiler.Cxx.Function.Function()
    weakReference.setChildren([name])
    weakReference.setType(self.pyObjType)
    weakReference.setStatic(1)
    weakReference.setParameters([selfParam, argsParam, kwdictParam])
    weakReference.setBody(body)
    return [weakReference]
  
  def getIORClassNameMethod(self, interface):
    import ASE.Compiler.SIDL.Method
    method = ASE.Compiler.SIDL.Method.Method()
    method.setIdentifier('getIORClassName')
    method.setReturnParameter(self.SIDL.getParameter('_return_value', self.SIDL.typeMap['string'], 'return'))
    method.setChildren([self.SIDL.getParameter('ior', self.SIDL.typeMap['long'])])
    method.setExceptions([self.SIDL.getBaseException(interface, 1)])
    method.setParent(interface)
    method.setStatic(1)
    return method
  
  def getIORClassName(self, interface, method):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    self.Python.marshalPythonToCCall(method, body)
    iorVar = self.Cxx.getVar('ior')
    iorReturnVar = self.Cxx.getVar('_return_value')
    baseIor = self.Cxx.getGroup(self.Cxx.castToType(iorVar, self.IOR.getType(self.SIDL.getBaseInterface(interface))))
    iorFunc = self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getStructRef(baseIor, 'd_epv'), 'getClassName'))))
    iorCall = self.Cxx.getFunctionCall(iorFunc, [self.IOR.getTrueIOR(baseIor), self.Cxx.castToType(self.Cxx.getAddress(self.Cxx.getVar('_exception')), self.Cxx.typeMap['void pointer pointer'])])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(iorReturnVar, iorCall), 'Dispatch to IOR')])
    self.getMethodHandleExceptions(interface, method, body)
    self.Python.marshalCToPythonReturn(method, body)
    name = ASE.Compiler.Cxx.Declarator.Declarator()
    name.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '_' + method.getIdentifier())
    methodStub = self.Python.getClientFunction(method)
    methodBody = methodStub.getBody()
    methodBody.setDeclarations(list(methodBody.getDeclarations()) + list(body.getDeclarations()))
    methodBody.setChildren(list(body.getChildren()) + list(methodBody.getChildren()))
    return [methodStub]
  
  def getMethodIORPointer(self, interface, method, body):
    '''
    We should not need to cast here since the pointer should be cast on creation of the stub itself
    '''
    import ASE.Compiler.Cxx.Initializer
    init = ASE.Compiler.Cxx.Initializer.Initializer()
    init.setChildren([self.Cxx.getStructRef(self.Cxx.getGroup(self.Cxx.castToType(self.Cxx.getVar('_self'), self.Python.getSIDLObjectType(interface))), 'ior')])
    body.addDeclarations([self.Cxx.getDeclaration(self.Cxx.getVar('_self_ior'), self.IOR.getType(interface), init, 'Retrieve the IOR pointer')])
    return 
  
  def getMethodSIDLObjectCheck(self, methodName, body, returnVar, var = None, errorHandler = None, anySIDLObject = 0):
    import ASE.Compiler.Cxx.If
    if var is None:
      var = self.Cxx.getVar('_self')
    typeCheck = ASE.Compiler.Cxx.If.If()
    if not (anySIDLObject):
      import ASE.Compiler.Cxx.Inequality
      branch = ASE.Compiler.Cxx.Inequality.Inequality()
      branch.setChildren([self.Cxx.getStructRef(var, 'ob_type'), self.Cxx.getAddress(self.Cxx.getVar('SIDLPythonObjectType'))])
    else:
      import ASE.Compiler.Cxx.Negation
      branch = ASE.Compiler.Cxx.Negation.Negation()
      branch.setChildren([self.Cxx.getFunctionCall('PyObject_HasAttrString', [var, self.Cxx.castToString('getClassName')])])
    if not (errorHandler is None):
      typeCheck.setChildren([errorHandler])
    else:
      import ASE.Compiler.Cxx.CompoundStatement
      import ASE.Compiler.Cxx.Return
      returnVal = ASE.Compiler.Cxx.Return.Return()
      if not (returnVar is None):
        returnVal.setChildren([returnVar])
      exception = self.Cxx.getFunctionCall('PyErr_SetString', [self.Cxx.getVar('PyExc_TypeError'), self.Cxx.getString('Could not parse arguments for ' + methodName)])
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound.setChildren([self.Cxx.getExpStmt(exception), returnVal])
      typeCheck.setChildren([compound])
    typeCheck.setBranch(branch)
    typeCheck.setComments([self.Cxx.getComment('Check for a valid SIDL Object')])
    body.addChildren([typeCheck])
    return 
  
  def throwDefaultException(self, interface, method, excVar):
    import ASE.Compiler.Cxx.CompoundStatement
    excObj = self.Cxx.getStructRef(excVar, 'd_object')
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    baseEpv = self.Cxx.getStructRef(self.Cxx.getGroup(self.Cxx.castToType(excObj, self.IOR.getBaseType(interface))), 'd_epv')
    castFunc = self.Cxx.getIndirection(self.Cxx.getStructRef(baseEpv, 'castObject'))
    baseExcType = self.SIDL.getBaseException(method.getExceptions()[0])
    castCall = self.Cxx.getFunctionCall(self.Cxx.getGroup(castFunc), [excObj, self.Cxx.getString(baseExcType.getFullIdentifier()), self.Cxx.getNull()])
    resetExc = self.Cxx.getAssignment(excVar, castCall)
    excEpv = self.Cxx.getStructRef(excVar, 'd_epv')
    messageFunc = self.Cxx.getStructRef(excEpv, 'getMessage')
    getMessage = self.Cxx.getFunctionCall(messageFunc, [excObj, self.Cxx.getNull()])
    setExc = self.Cxx.getFunctionCall('PyErr_SetString', [self.Cxx.getVar('PyExc_RuntimeError'), getMessage])
    compound.addChildren(map(self.Cxx.getExpStmt, [resetExc, setExc]))
    return compound
  
  def throwException(self, interface, exception, excVar, body):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.StructReference
    moduleVar = self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_module')
    pythonTypeVar = self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_type')
    objVar = self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_obj')
    pyObjVar = self.Cxx.getVar('_obj')
    moduleCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    moduleCall.setChildren([self.Cxx.getVar('PyImport_ImportModule')])
    moduleCall.setArguments([self.Cxx.castToString(exception.getFullIdentifier())])
    moduleInit = ASE.Compiler.Cxx.Initializer.Initializer()
    moduleInit.setChildren([moduleCall])
    moduleDict = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    moduleDict.setChildren([self.Cxx.getVar('PyModule_GetDict')])
    moduleDict.setArguments([moduleVar])
    typeCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    typeCall.setChildren([self.Cxx.getVar('PyMapping_GetItemString')])
    typeCall.setArguments([moduleDict, self.Cxx.castToString('Exception')])
    typeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    typeInit.setChildren([typeCall])
    excEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    excEpv.setChildren([excVar])
    excEpv.setMember('d_epv')
    excEpv.setPointer(1)
    castFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    castFunc.setChildren([excEpv])
    castFunc.setMember('castObject')
    castFunc.setPointer(1)
    castFuncInd = ASE.Compiler.Cxx.Indirection.Indirection()
    castFuncInd.setChildren([self.Cxx.getGroup(castFunc)])
    excObj = ASE.Compiler.Cxx.StructReference.StructReference()
    excObj.setChildren([excVar])
    excObj.setMember('d_object')
    excObj.setPointer(1)
    castCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castCall.setChildren([self.Cxx.getGroup(castFuncInd)])
    castCall.setArguments([excObj, self.Cxx.getString(exception.getFullIdentifier()), self.Cxx.getNull()])
    objInit = ASE.Compiler.Cxx.Initializer.Initializer()
    objInit.setChildren([castCall])
    body.addDeclarations([self.Cxx.getDeclaration(moduleVar, self.pyObjType, moduleInit), self.Cxx.getDeclaration(pythonTypeVar, self.pyObjType, typeInit), self.Cxx.getDeclaration(objVar, self.IOR.getType(exception), objInit)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    createObj = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    createObj.setChildren([self.Python.getWrapIOR(exception).getChildren()[0]])
    createObj.setArguments([objVar])
    pyObjInit = ASE.Compiler.Cxx.Initializer.Initializer()
    pyObjInit.setChildren([createObj])
    compound.addDeclarations([self.Cxx.getDeclaration(pyObjVar, self.pyObjType, pyObjInit)])
    setExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    setExc.setChildren([self.Cxx.getVar('PyErr_SetObject')])
    setExc.setArguments([pythonTypeVar, pyObjVar])
    compound.addChildren([self.Cxx.getExpStmt(setExc)])
    checkExcType = ASE.Compiler.Cxx.If.If()
    checkExcType.setBranch(objVar)
    checkExcType.setChildren([compound])
    return checkExcType
  
  def getMethodHandleExceptions(self, interface, method, body):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Return
    if interface is None:
      return None
    excVar = self.Cxx.getVar('_exception')
    body.addDeclarations([self.Cxx.getDeclaration(excVar, self.IOR.getType(self.SIDL.getBaseException(method.getExceptions()[0])))])
    excBody = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    checkExcType = None
    for exception in method.getExceptions():
      check = self.throwException(interface, exception, excVar, excBody)
      if checkExcType is None:
        checkExcType = check
        fullExcCheck = checkExcType
      else:
        checkExcType.addChildren([check])
        checkExcType = check
    checkExcType.addChildren([self.throwDefaultException(interface, method, excVar)])
    excBody.addChildren([fullExcCheck])
    for exception in method.getExceptions():
      decrefModule = self.Cxx.getFunctionCall('Py_DECREF', [self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_module')])
      decrefType = self.Cxx.getFunctionCall('Py_DECREF', [self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_type')])
      excBody.addChildren(map(self.Cxx.getExpStmt, [decrefModule, decrefType]))
    returnExc = ASE.Compiler.Cxx.Return.Return()
    returnExc.setChildren([self.Cxx.getVar('_pyReturn')])
    excBody.addChildren([returnExc])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(excVar)
    checkExc.setChildren([excBody])
    body.addChildren([checkExc])
    return 
  
  def getMethodStub(self, interface, method):
    import ASE.Compiler.Cxx.CompoundStatement
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    if not (method.getStatic()):
      returnVar = self.Cxx.getVar('_pyReturn')
      self.getMethodIORPointer(interface, method, body)
      self.getMethodSIDLObjectCheck(method.getIdentifier(), body, returnVar)
      body.addChildren([self.Python.getErrorCheck('PyExc_TypeError', 'Null IOR pointer in ' + method.getIdentifier(), self.Cxx.getVar('_self_ior'), returnVar)])
    self.Python.marshalPythonToCCall(method, body)
    self.IOR.dispatch(interface, method, body)
    self.getMethodHandleExceptions(interface, method, body)
    self.Python.marshalCToPythonReturn(method, body)
    if method.getFullIdentifier() == 'ASE.Loader.createClass':
      parameter = method.getReturnParameter()
      objVar = self.Cxx.getGroup(self.Cxx.castToType(self.Cxx.getVar(parameter.getIdentifier()), self.IOR.getBaseType(parameter.getType())))
      delRefFunc = self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getStructRef(objVar, 'd_epv'), 'deleteReference'))))
      delRefCall = self.Cxx.getFunctionCall(delRefFunc, [self.Cxx.getStructRef(objVar, 'd_object'), self.Cxx.getNull()])
      body.addChildren([self.Cxx.getExpStmt(delRefCall)])
    methodStub = self.Python.getClientFunction(method)
    methodBody = methodStub.getBody()
    methodBody.setDeclarations(list(methodBody.getDeclarations()) + list(body.getDeclarations()))
    methodBody.setChildren(list(body.getChildren()) + list(methodBody.getChildren()))
    return [methodStub]
  
  def getMethodDescriptor(self, method):
    import ASE.Compiler.Cxx.InclusiveOr
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Type
    funcType = ASE.Compiler.Cxx.Type.Type()
    funcType.setIdentifier('PyCFunction')
    cast = self.Cxx.castToType(self.Cxx.getVar(method.getFullIdentifier().replace('.', '_')), funcType)
    orExp = ASE.Compiler.Cxx.InclusiveOr.InclusiveOr()
    orExp.setChildren([self.Cxx.getVar('METH_VARARGS'), self.Cxx.getVar('METH_KEYWORDS')])
    if method.getStatic():
      orExp2 = ASE.Compiler.Cxx.InclusiveOr.InclusiveOr()
      orExp2.setChildren([orExp, self.Cxx.getVar('METH_STATIC')])
      orExp = orExp2
    methodDesc = ASE.Compiler.Cxx.Initializer.Initializer()
    methodDesc.setChildren([self.Cxx.castToString(method.getIdentifier()), cast, self.Cxx.getGroup(orExp), self.Cxx.castToString(self.Python.getMethodComment(method))])
    methodDesc.setList(1)
    return [methodDesc]
  
  def getObjectMethods(self, interface, extraMethods = []):
    import ASE.Compiler.Cxx.Array
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Struct
    nullMethod = ASE.Compiler.Cxx.Initializer.Initializer()
    nullMethod.setChildren([self.Cxx.getVar('NULL'), self.Cxx.getVar('NULL')])
    nullMethod.setList(1)
    methodType = ASE.Compiler.Cxx.Struct.Struct()
    methodType.setIdentifier('PyMethodDef')
    methodTable = ASE.Compiler.Cxx.Initializer.Initializer()
    for method in self.getTemplateMethods(list(interface.getMethods()) + extraMethods):
      methodTable.addChildren(self.getMethodDescriptor(method))
    methodTable.addChildren([nullMethod])
    methodTable.setList(1)
    objectMethods = ASE.Compiler.Cxx.Array.Array()
    objectMethods.setChildren([self.Cxx.getVar('_' + interface.getIdentifier() + 'ObjectMethods')])
    objectMethods.setType(methodType)
    objectMethods.setStatic(1)
    objectMethods.setInitializer(methodTable)
    return self.Cxx.getDecl(objectMethods)
  
  def getModuleMethods(self, interface, extraMethods = []):
    import ASE.Compiler.Cxx.Array
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Struct
    nullMethod = ASE.Compiler.Cxx.Initializer.Initializer()
    nullMethod.setChildren([self.Cxx.getVar('NULL'), self.Cxx.getVar('NULL')])
    nullMethod.setList(1)
    methodType = ASE.Compiler.Cxx.Struct.Struct()
    methodType.setIdentifier('PyMethodDef')
    methodTable = ASE.Compiler.Cxx.Initializer.Initializer()
    for method in extraMethods:
      methodTable.addChildren(self.getMethodDescriptor(method))
    methodTable.addChildren([nullMethod])
    methodTable.setList(1)
    moduleMethods = ASE.Compiler.Cxx.Array.Array()
    moduleMethods.setChildren([self.Cxx.getVar('_' + interface.getIdentifier() + 'ModuleMethods')])
    moduleMethods.setType(methodType)
    moduleMethods.setStatic(1)
    moduleMethods.setInitializer(methodTable)
    return self.Cxx.getDecl(moduleMethods)
  
  def setupPythonType(self, body):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.LessThan
    import ASE.Compiler.Cxx.Return
    pythonType = self.Cxx.getAddress(self.Cxx.getVar('SIDLPythonObjectType'))
    readyTypeCall = self.Cxx.getFunctionCall('PyType_Ready', [pythonType])
    readyTypeCheck = ASE.Compiler.Cxx.LessThan.LessThan()
    readyTypeCheck.setChildren([readyTypeCall, self.Cxx.getInteger(0)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([ASE.Compiler.Cxx.Return.Return()])
    readyType = ASE.Compiler.Cxx.If.If()
    readyType.setBranch(readyTypeCheck)
    readyType.setChildren([compound])
    readyType.setComments([self.Cxx.getComment('Setup the python type')])
    body.addChildren([readyType])
    return pythonType
  
  def initializeModule(self, klass, body):
    name = klass.getIdentifier()
    modVar = self.Cxx.getVar('module')
    body.addDeclarations([self.Cxx.getDeclaration(modVar, self.pyObjType)])
    pythonType = self.setupPythonType(body)
    moduleInit = self.Cxx.getFunctionCall('Py_InitModule3', [self.Cxx.castToString(name), '_' + name + 'ModuleMethods', self.Cxx.castToString(self.Python.getObjectComment(klass))])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(modVar, moduleInit), 'Initialize the module'), self.Python.getErrorCheck('PyExc_ImportError', 'Could not initialize module ' + klass.getFullIdentifier())])
    incRef = self.Cxx.getFunctionCall('Py_INCREF', [pythonType])
    addType = self.Cxx.getFunctionCall('PyModule_AddObject', [modVar, self.Cxx.castToString(klass.getIdentifier()), self.Cxx.castToType(pythonType, self.pyObjType)])
    body.addChildren([self.Cxx.getExpStmt(incRef, 'Add the type to the module and increment the ref count'), self.Cxx.getExpStmt(addType)])
    return 
  
  def getIORExternals(self, klass, body):
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Pointer
    excVar = self.Cxx.getVar('_exception')
    extFuncVar = self.Cxx.getVar('_extFunc')
    extFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
    extFuncPtr.setIdentifier(extFuncVar.getIdentifier())
    extFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
    extFuncDecl = ASE.Compiler.Cxx.Function.Function()
    extFuncDecl.setChildren([self.Cxx.getDeclGroup(extFuncPtr)])
    extFuncDecl.setType(self.IOR.getExternalType(klass))
    body.addDeclarations([self.Cxx.getDecl(extFuncDecl)])
    extFuncType = ASE.Compiler.Cxx.Function.Function()
    extFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
    extFuncType.setType(self.IOR.getExternalType(klass))
    sidlExtCall = self.Cxx.getFunctionCall('ASE_Loader__externals')
    staticEPV = self.Cxx.getStructRef(sidlExtCall, 'getStaticEPV')
    staticEPVCall = self.Cxx.getFunctionCall(staticEPV, [self.Cxx.getAddress(excVar)])
    classMethods = self.Cxx.getStructRef(staticEPVCall, 'getClassMethodTable')
    classMethodsCall = self.Cxx.getFunctionCall(classMethods, [self.Cxx.getString(klass.getFullIdentifier()), self.Cxx.getAddress(excVar)])
    extFunc = ASE.Compiler.Cxx.Cast.Cast()
    extFunc.setTypeName(extFuncType)
    extFunc.setChildren([classMethodsCall])
    setExternals = self.Cxx.getAssignment(extFuncVar, extFunc)
    body.addDeclarations([self.Cxx.getDeclaration(excVar, self.IOR.getType(self.SIDL.getBaseException(klass, 1)))])
    body.addChildren([self.Cxx.getExpStmt(setExternals, 'Set externals function')])
    return None
  
  def checkException(self, body):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    excVar = self.Cxx.getVar('_exception')
    excEpv = self.Cxx.getStructRef(excVar, 'd_epv')
    resetExc = self.Cxx.getAssignment(excVar, self.Cxx.getFunctionCall(self.Cxx.getStructRef(excEpv, 'castObject'), [excVar, self.Cxx.getString('ASE.BaseException'), self.Cxx.getNull()]))
    getMessage = self.Cxx.getFunctionCall(self.Cxx.getStructRef(excEpv, 'getMessage'), [self.Cxx.getStructRef(excVar, 'd_object'), self.Cxx.getNull()])
    setExc = self.Cxx.getFunctionCall('PyErr_SetString', [self.Cxx.getVar('PyExc_RuntimeError'), getMessage])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(resetExc), self.Cxx.getExpStmt(setExc), ASE.Compiler.Cxx.Return.Return()])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(excVar)
    checkExc.setChildren([compound])
    checkExc.setComments([self.Cxx.getComment('Check exception')])
    body.addChildren([checkExc])
    return None
  
  def setEpvPointer(self, klass, body):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    epvVar = self.Cxx.getVar('_epv')
    extFuncVar = self.Cxx.getVar('_extFunc')
    extFuncCall = self.Cxx.getFunctionCall(self.Cxx.getGroup(self.Cxx.getIndirection(extFuncVar)))
    assignEpvSt = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    assignEpvSt.setChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(epvVar, extFuncCall))])
    setEpv = ASE.Compiler.Cxx.If.If()
    setEpv.setBranch(extFuncVar)
    setEpv.setChildren([assignEpvSt])
    setEpv.setComments([self.Cxx.getComment('Set epv pointer')])
    body.addChildren([setEpv, self.Python.getErrorCheck('PyExc_ImportError', 'Cannot load SIDL implementation for ' + klass.getFullIdentifier(), epvVar)])
    return 
  
  def setSepvPointer(self, klass, body):
    if self.hasStaticMethod(klass):
      sepvVar = self.Cxx.getVar('_sepv')
      sepvFunc = self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getVar('_epv'), 'getStaticEPV')))
      sepvCall = self.Cxx.getFunctionCall(self.Cxx.getGroup(sepvFunc), [self.Cxx.getNull()])
      setSepv = self.Cxx.getAssignment(sepvVar, sepvCall)
      body.addChildren([self.Cxx.getExpStmt(setSepv, 'Set sepv pointer'), self.Python.getErrorCheck('PyExc_ImportError', 'Cannot load static methods for ' + klass.getFullIdentifier(), sepvVar)])
    return 
  
  def getModuleInit(self, klass):
    '''
    Generate module initializer
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    self.initializeModule(klass, body)
    self.getIORExternals(klass, body)
    self.checkException(body)
    self.setEpvPointer(klass, body)
    self.setSepvPointer(klass, body)
    if not (self.SIDL.getBaseException(klass) is None):
      newExcCall = self.Cxx.getFunctionCall('PyErr_NewException', [self.Cxx.castToString(klass.getFullIdentifier() + '.Exception'), self.Cxx.getNull(), self.Cxx.getNull()])
      setExc = self.Cxx.getFunctionCall('PyModule_AddObject', [self.Cxx.getVar('module'), self.Cxx.castToString('Exception'), self.Cxx.castToType(newExcCall, self.pyObjType)])
      body.addChildren([self.Cxx.getExpStmt(setExc, 'Create an Exception class')])
    importNumeric = self.Cxx.getFunctionCall('import_array')
    body.addChildren([self.Cxx.getExpStmt(importNumeric, 'Import Numeric'), self.Python.getErrorCheck('PyExc_ImportError', 'Cannot import Numeric Python array module')])
    initName = ASE.Compiler.Cxx.Declarator.Declarator()
    initName.setIdentifier('init' + klass.getIdentifier())
    initFunction = ASE.Compiler.Cxx.Function.Function()
    initFunction.setChildren([initName])
    initFunction.setType(self.Python.typeMap['void'])
    initFunction.setBody(body)
    return [initFunction]
  
  def copyType(self, type):
    if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      newType = ASE.Compiler.SIDL.Array.Array()
      newType.setIdentifier(type.getIdentifier())
      newType.setAttributes(type.getAttributes())
      newType.setLineNumber(type.getLineNumber())
      newType.setComments(type.getComments())
      newType.setChildren([child for child in type.getChildren()])
      newType.setBaseType(type.getBaseType())
      newType.setType(self.copyType(ASE.Compiler.SIDL.Array.Array(type).getType()))
      return newType
    return type
  
  def copyParameter(self, parameter):
    '''
    This will be replaced by parameter.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Parameter
    newParameter = ASE.Compiler.SIDL.Parameter.Parameter()
    newParameter.setIdentifier(parameter.getIdentifier())
    newParameter.setAttributes(parameter.getAttributes())
    newParameter.setParent(parameter.getParent())
    newParameter.setChildren(parameter.getChildren())
    newParameter.setLineNumber(parameter.getLineNumber())
    newParameter.setComments(parameter.getComments())
    newParameter.setDependencies(parameter.getDependencies())
    newParameter.setType(self.copyType(parameter.getType()))
    newParameter.setAccessMode(parameter.getAccessMode())
    newParameter.setRequired(parameter.getRequired())
    return newParameter
  
  def copyMethod(self, method):
    '''
    This will be replaced by method.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    newMethod = ASE.Compiler.SIDL.Method.Method()
    newMethod.setIdentifier(method.getIdentifier())
    newMethod.setAttributes(method.getAttributes())
    newMethod.setParent(method.getParent())
    newMethod.setChildren([self.copyParameter(ASE.Compiler.SIDL.Parameter.Parameter(child)) for child in method.getChildren()])
    newMethod.setLineNumber(method.getLineNumber())
    newMethod.setComments(method.getComments())
    newMethod.setDependencies(method.getDependencies())
    newMethod.setTemplateIdentifiers(method.getTemplateIdentifiers())
    if not (method.getReturnParameter() is None):
      newMethod.setReturnParameter(self.copyParameter(method.getReturnParameter()))
    newMethod.setExceptions(method.getExceptions())
    newMethod.setStatic(method.getStatic())
    newMethod.setInternal(method.getInternal())
    newMethod.setVirtual(method.getVirtual())
    newMethod.setPurpose(method.getPurpose())
    return newMethod
  
  def checkTemplateParameter(self, name, parameter):
    if not (parameter is None):
      type = parameter.getType()
      if (type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name):
        return True
      elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
        import ASE.Compiler.SIDL.Array
        type = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
        if (type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name):
          return True
    return False
  
  def insertType(self, parameter, type):
    if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      self.insertType(ASE.Compiler.SIDL.Array.Array(parameter.getType()), type)
    else:
      parameter.setType(type)
    return 
  
  def insertTypes(self, method, universe, type):
    import ASE.Compiler.SIDL.Parameter
    name = universe.getIdentifier()
    newMethod = self.copyMethod(method)
    newMethod.setIdentifier(newMethod.getIdentifier() + '_' + type.getFullIdentifier())
    if self.checkTemplateParameter(name, method.getReturnParameter()):
      self.insertType(newMethod.getReturnParameter(), type)
    for (child, newChild) in zip(method.getChildren(), newMethod.getChildren()):
      if self.checkTemplateParameter(name, ASE.Compiler.SIDL.Parameter.Parameter(child)):
        self.insertType(ASE.Compiler.SIDL.Parameter.Parameter(newChild), type)
    return newMethod
  
  def template(self, method, name):
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.SIDL.TypeUniverse
    universe = None
    for parameter in [method.getReturnParameter()] + [ASE.Compiler.SIDL.Parameter.Parameter(child) for child in method.getChildren()]:
      if self.checkTemplateParameter(name, parameter):
        type = parameter.getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(ASE.Compiler.SIDL.Array.Array(type).getType())
        else:
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(type)
        break
    if universe is None:
      raise RuntimeError('Could not find type universe for template ' + name + ' in method ' + method.getFullIdentifier())
    self.templateMethods[method] = [self.insertTypes(method, universe, type) for type in universe.getTypes()]
    return None
  
  def createTemplateMethod(self, method):
    if len(method.getTemplateIdentifiers()):
      if len(method.getTemplateIdentifiers()) > 1:
        raise RuntimeError('Can currently only handle a single template identifier')
      if not (method in self.templateMethods):
        self.template(method, method.getTemplateIdentifiers()[0])
    return None
  
  def getTemplateMethods(self, methods):
    for method in methods:
      if method in self.templateMethods:
        for m in self.templateMethods[method]:
          yield m
      else:
        yield method
    return 

class Stub(object):
  
  def __init__(self):
    return None
