# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.Attribute
import ASE.Compiler.Comment
import ASE.Compiler.CompilerException
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.Vertex
import ASE.Compiler.SIDL.Visitor
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class Interface:
  '''
  A SIDL interface
  The children of this node are methods defined in the interface
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.parents = []
    self.methods = []
  
  def accept(self, visitor):
    '''
    Overridden to dispatch correctly
    '''
    visitor.visitInterface(self.__IORself)
  
  def getParents(self):
    '''
    Retrieve the superinterfaces of this interface
    '''
    return self.parents
  
  def setParents(self, parents):
    '''
    Set the superinterfaces of this interface
    '''
    self.parents = list(parents)
  
  def addParents(self, parents):
    '''
    Add superinterfaces to this interface
    '''
    self.parents.extend(parents)
  
  def removeParents(self, parents):
    '''
    Remove superinterfaces from this interface
    Throws an exception if a parent does not exist
    '''
    for parent in parents:
      if parent in self.parents:
        self.parents.remove(parent)
      else:
        e = ASE.Compiler.CompilerException()
        e.setMessage('Parent does not exist: ' + parent.getIdentifier())
        raise e
  
  def getMethods(self):
    '''
    Retrieve the union of all methods supported by this interface
    '''
    return self.methods
  
  def setMethods(self, methods):
    '''
    Set the union of all methods supported by this interface
    '''
    self.methods = list(methods)
  
  def addMethods(self, methods):
    '''
    Add to the union of all methods supported by this interface
    Method uniqueness is decided only the the method name, keeping the most recent one
    '''
    for method in methods:
      for m in self.methods:
        if m.getIdentifier() == method.getIdentifier():
          self.methods.remove(m)
          break
      self.methods.append(method)
  
  def removeMethods(self, methods):
    '''
    Remove from the union of all methods supported by this interface
    Throws an exception if a method does not exist
    '''
    for method in methods:
      for m in self.methods:
        if m.getIdentifier() == method.getIdentifier():
          self.methods.remove(m)
          break
  # Ending class methods section
  pass
