#include <Python.h>

#include <Numeric/arrayobject.h>

#ifndef included_ASE_Compiler_SIDL_Type
#include <ASE/Compiler/SIDL/Type_IOR.h>
#endif

#ifndef included_ASE_Loader
#include <ASE/Loader_IOR.h>
#endif

#ifndef included_ASE_ASEException
#include <ASE/ASEException_IOR.h>
#endif

#ifndef included_ASE_Compiler_CompilerException
#include <ASE/Compiler/CompilerException_IOR.h>
#endif

#ifndef included_ASE_BaseException
#include <ASE/BaseException_IOR.h>
#endif

enum ReferenceType {STOLEN_REF, WEAK_REF, NEW_REF};

struct ASE_Compiler_SIDL_Type_PythonObject {
  PyObject_HEAD;
  struct ASE_Compiler_SIDL_Type__object *ior;
  enum ReferenceType refType;
  PyObject *weakRefList;
};

static PyObject *sidl_new(PyTypeObject *type, struct ASE_BaseInterface__object *ior, enum ReferenceType refType)
{
  PyObject *return_value = NULL;
  
  if (ior)
  {
    struct ASE_Compiler_SIDL_Type_PythonObject *self = (struct ASE_Compiler_SIDL_Type_PythonObject *) type->tp_alloc(type, 0);
    
    self->weakRefList = NULL;
    if (self)
    {
      struct ASE_BaseClass__object *baseIor = (*(ior->d_epv->castObject))(ior->d_object, "ASE.BaseClass", NULL);
      
      self->ior = ((*(ior->d_epv->castObject)))(ior->d_object, type->tp_name, NULL);
      self->refType = refType;
      /* Allow objects without an attached implementation (used in the skeleton constructor) */
      if (baseIor->d_data)
      {
        if (refType == NEW_REF)
        {
          (*(baseIor->d_epv->addReference))(baseIor->d_object, NULL);
        }
      }
      return_value = (PyObject *) self;
    }
    else
    {
      PyErr_SetString(PyExc_MemoryError, (char *) "Unable to allocate interface object");
    }
  }
  else
  {
    return_value = Py_None;
    Py_INCREF(return_value);
  }
  return return_value;
}

static PyObject *ASE_Compiler_SIDL_Type__wrap_ior_weak(struct ASE_Compiler_SIDL_Type__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.Type");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Type");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.SIDL.Type");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.SIDL.Type");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, WEAK_REF);
}

static PyObject *ASE_Compiler_SIDL_Type__wrap_ior(struct ASE_Compiler_SIDL_Type__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.Type");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Type");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.SIDL.Type");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.SIDL.Type");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_SIDL_Visitor__wrap_ior(struct ASE_Compiler_SIDL_Visitor__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.Visitor");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Visitor");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.SIDL.Visitor");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.SIDL.Visitor");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static long ASE_Compiler_SIDL_Vertex__extract_ior(struct ASE_Compiler_SIDL_Type_PythonObject *obj, struct ASE_Compiler_SIDL_Vertex__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.SIDL.Vertex", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.SIDL.Vertex");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_SIDL_Type__extract_ior(struct ASE_Compiler_SIDL_Type_PythonObject *obj, struct ASE_Compiler_SIDL_Type__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.SIDL.Type", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.SIDL.Type");
    return 0;
  }
  return 1;
}

static struct ASE_Compiler_SIDL_Type__object *ASE_Compiler_SIDL_Type_iterNext(struct ASE_Compiler_SIDL_Type__iterator *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *iterator = (PyObject *) _self->iterator;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  PyObject *returnTuple = NULL;
  struct ASE_Compiler_SIDL_Type__object *_return_value = NULL;
  
  if (!PyIter_Check(iterator))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid iterator for ASE.Compiler.SIDL.Type.iterNext", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_SIDL_Type__wrap_ior_weak, _self);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.Type.iterNext", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  _pyReturn = PyIter_Next(iterator);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_Type_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return _return_value;
  }
  if (!_pyReturn)
  {
    _pyReturn = Py_None;
    Py_INCREF(_pyReturn);
  }
  returnTuple = PyTuple_New(1);
  PyTuple_SetItem(returnTuple, 0, _pyReturn);
  /* Parse arguments */
  _okay = PyArg_ParseTuple(returnTuple, (char *) "O&", (void *) ASE_Compiler_SIDL_Type__extract_ior, &_return_value);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_okay)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Could not parse return value for ASE.Compiler.SIDL.Type.iterNext", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Cleanup local return value */
  Py_DECREF(returnTuple);
  /* Cleanup arguments */
  Py_DECREF(_args);
  return _return_value;
}

static void ASE_Compiler_SIDL_Type_accept(struct ASE_Compiler_SIDL_Type__object *_self, struct ASE_Compiler_SIDL_Visitor__object *visitor, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.Type.accept", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "accept");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.Type.accept", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  if (visitor)
  {
    (*((((struct ASE_BaseInterface__object *) visitor))->d_epv->addReference))(((struct ASE_BaseInterface__object *) visitor)->d_object, NULL);
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_SIDL_Visitor__wrap_ior, visitor);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.Type.accept", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return;
  }
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_Type_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_Type_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.SIDL.Type.accept", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static struct ASE_Compiler_SIDL_Vertex__object *ASE_Compiler_SIDL_Type_copy(struct ASE_Compiler_SIDL_Type__object *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  PyObject *returnTuple = NULL;
  struct ASE_Compiler_SIDL_Vertex__object *_return_value = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.Type.copy", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "copy");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.Type.copy", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "()");
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.Type.copy", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return _return_value;
  }
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_Type_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return _return_value;
  }
  returnTuple = PyTuple_New(1);
  PyTuple_SetItem(returnTuple, 0, _pyReturn);
  /* Parse arguments */
  _okay = PyArg_ParseTuple(returnTuple, (char *) "O&", (void *) ASE_Compiler_SIDL_Vertex__extract_ior, &_return_value);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_okay)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Could not parse return value for ASE.Compiler.SIDL.Type.copy", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Cleanup local return value */
  Py_DECREF(returnTuple);
  /* Cleanup arguments */
  Py_DECREF(_args);
  return _return_value;
}

static const char *ASE_Compiler_SIDL_Type_getFullIdentifier(struct ASE_Compiler_SIDL_Type__object *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  PyObject *returnTuple = NULL;
  const char *_return_value = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.Type.getFullIdentifier", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "getFullIdentifier");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.Type.getFullIdentifier", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "()");
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.Type.getFullIdentifier", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return _return_value;
  }
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_Type_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return _return_value;
  }
  returnTuple = PyTuple_New(1);
  PyTuple_SetItem(returnTuple, 0, _pyReturn);
  /* Parse arguments */
  _okay = PyArg_ParseTuple(returnTuple, (char *) "z", &_return_value);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_okay)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Could not parse return value for ASE.Compiler.SIDL.Type.getFullIdentifier", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Copying the Python string to IOR controlled memory */
  _return_value = _return_value ? strcpy(malloc(strlen(_return_value) + 1), _return_value) : NULL;
  /* Cleanup local return value */
  Py_DECREF(returnTuple);
  /* Cleanup arguments */
  Py_DECREF(_args);
  return _return_value;
}

static int ASE_Compiler_SIDL_Type_getBaseType(struct ASE_Compiler_SIDL_Type__object *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  PyObject *returnTuple = NULL;
  int _return_value = 0;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.Type.getBaseType", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "getBaseType");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.Type.getBaseType", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "()");
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.Type.getBaseType", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return _return_value;
  }
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_Type_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return _return_value;
  }
  returnTuple = PyTuple_New(1);
  PyTuple_SetItem(returnTuple, 0, _pyReturn);
  /* Parse arguments */
  _okay = PyArg_ParseTuple(returnTuple, (char *) "i", &_return_value);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_okay)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Could not parse return value for ASE.Compiler.SIDL.Type.getBaseType", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Cleanup local return value */
  Py_DECREF(returnTuple);
  /* Cleanup arguments */
  Py_DECREF(_args);
  return _return_value;
}

static void ASE_Compiler_SIDL_Type_setBaseType(struct ASE_Compiler_SIDL_Type__object *_self, int baseType, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.Type.setBaseType", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "setBaseType");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.Type.setBaseType", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(i)", baseType);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.Type.setBaseType", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return;
  }
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_Type_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.SIDL.Type.setBaseType", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_SIDL_Type_Type(struct ASE_Compiler_SIDL_Type__object *obj, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_return_value = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.Type_impl");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.Type.Type", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "Type");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.Type.Type", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_SIDL_Type__wrap_ior, obj);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.Type.Type", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Create a new instance */
  _return_value = PyInstance_New(_method, _args, NULL);
  _pyReturn = Py_None;
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_return_value)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Could not create instance of ASE.Compiler.SIDL.Type_impl.Type", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  obj->d_data = _return_value;
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_Type_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_Type_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.SIDL.Type.Type", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_SIDL_Type_Type_(struct ASE_Compiler_SIDL_Type__object *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *data = (PyObject *) _self->d_data;
  
  /* If we do not set d_data to NULL, we get a SEGV on the recursive call to deleteReference() coming from self.__IORself */
  _self->d_data = NULL;
  /* Remove only reference to Python object */
  Py_XDECREF(data);
}

void ASE_Compiler_SIDL_Type__set_epv(struct ASE_Compiler_SIDL_Type__epv *_epv, struct ASE_BaseException__object **_exception);

void ASE_Compiler_SIDL_Type__set_epv(struct ASE_Compiler_SIDL_Type__epv *_epv, struct ASE_BaseException__object **_exception)
{
  static int initialized = 0;
  
  if (!initialized)
  {
    initialized = 1;
    SIDL_Python_Init();
    if (!PyImport_ImportModule((char *) "user"))
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Could not import user", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    if (!PyImport_ImportModule((char *) "importer"))
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Could not import importer", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    if (!PyImport_ImportModule((char *) "ASE.Compiler.SIDL.Type"))
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Could not import ASE.Compiler.SIDL.Type", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    import_array();
  }
  _epv->Type = ASE_Compiler_SIDL_Type_Type;
  _epv->Type_ = ASE_Compiler_SIDL_Type_Type_;
  _epv->iterNext = ASE_Compiler_SIDL_Type_iterNext;
  _epv->accept = ASE_Compiler_SIDL_Type_accept;
  _epv->copy = ASE_Compiler_SIDL_Type_copy;
  _epv->getFullIdentifier = ASE_Compiler_SIDL_Type_getFullIdentifier;
  _epv->getBaseType = ASE_Compiler_SIDL_Type_getBaseType;
  _epv->setBaseType = ASE_Compiler_SIDL_Type_setBaseType;
}
