# Starting imports section
pass
import ASE.BaseClass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.Attribute
import ASE.Compiler.Comment
import ASE.Compiler.CompilerException
import ASE.Compiler.SIDL.Vertex
import ASE.Compiler.Vertex
import ASE.Compiler.Visitor
# Ending imports section
pass
import sets
# Starting static methods section
pass
# Ending static methods section
pass

class Vertex:
  '''
  A generic node in the SIDL AST
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.identifier = None
    self.dependencies = sets.Set()
    self.attributes = []
    self.parent = None
    self.children = []
    self.lineNumber = -(1)
    self.comments = []
  
  def clone(self):
    '''
    Return an empty object of the same type
    '''
    return ASE.Compiler.SIDL.Vertex.Vertex()
  
  def copy(self):
    '''
    Return a deep copy of this subtree
    '''
    root = ASE.Compiler.SIDL.Vertex.Vertex()
    root.setIdentifier(self.__IORself.getIdentifier())
    root.setAttributes(self.__IORself.getAttributes())
    root.setLineNumber(self.__IORself.getLineNumber())
    root.setComments(self.__IORself.getComments())
    root.setChildren([child.copy() for child in self.__IORself.getChildren()])
    return root
  
  def accept(self, visitor):
    '''
    Support the visitor pattern for traversal.
    '''
    visitor.visitVertex(self.__IORself)
  
  def getIdentifier(self):
    '''
    Retrieve the identifier associated with this vertex
    '''
    return self.identifier
  
  def setIdentifier(self, identifier):
    '''
    Assign the identifier associated with this vertex
    '''
    self.identifier = identifier
  
  def getAttributes(self):
    '''
    Get attribute list for this element
    '''
    return self.attributes
  
  def setAttributes(self, attributes):
    '''
    Set attribute list for this element
    '''
    self.attributes = attributes
  
  def hasAttribute(self, name):
    '''
    Determine whether an attribute is present on this vertex
    '''
    for attribute in self.attributes:
      if attribute.getName() == name:
        return 1
    return 0
  
  def addAttribute(self, attribute):
    '''
    Add attribute to this vertex
    '''
    for attr in self.attributes:
      if attr.getName() == attribute.getName():
        self.attributes.remove(attribute)
    self.attributes.append(attribute)
  
  def deleteAttribute(self, name):
    '''
    Remove an attribute on this vertex
    Throws an exception if the attribute does not exist
    '''
    for attribute in self.attributes:
      if attribute.getName() == name:
        return self.attributes.remove(attribute)
    e = ASE.Compiler.CompilerException.CompilerException()
    e.setMessage('Attribute does not exist: ' + name)
    raise e
  
  def getAttribute(self, name):
    '''
    Get an attribute on this vertex
    Throws an exception if the attribute does not exist
    '''
    for attribute in self.attributes:
      if attribute.getName() == name:
        return attribute
    e = ASE.Compiler.CompilerException.CompilerException()
    e.setMessage('Attribute does not exist: ' + name)
    raise e
  
  def getParent(self):
    '''
    Retrieve the parent of this vertex
    '''
    return self.parent
  
  def setParent(self, parent):
    '''
    Assign the parent of this vertex
    '''
    self.parent = parent.weakReference()
  
  def getChildren(self):
    '''
    Retrieve the children of this vertex
    '''
    return self.children
  
  def setChildren(self, children):
    '''
    Set the children of this vertex
    '''
    self.children = list(children)
    for child in children:
      child.setParent(self.__IORself)
  
  def addChildren(self, children):
    '''
    Add children to this vertex (duplicates allowed)
    '''
    self.children.extend(children)
    for child in children:
      child.setParent(self.__IORself)
  
  def removeChildren(self, children):
    '''
    Remove children from this vertex
    Throws an exception if any child does not exist
    '''
    for child in children:
      if child in self.children:
        self.children.remove(child)
        child.setParent(None)
      else:
        e = ASE.Compiler.CompilerException()
        e.setMessage('Child does not exist: ' + str(child))
        raise e
  
  def getLineNumber(self):
    '''
    Retrieve the line number associated with this vertex
    '''
    return self.lineNumber
  
  def setLineNumber(self, lineNumber):
    '''
    Set the line number associated with this vertex
    '''
    self.lineNumber = lineNumber
  
  def getComments(self):
    '''
    Retrieve the comments of this vertex
    '''
    return self.comments
  
  def setComments(self, comments):
    '''
    Set the children of this vertex
    '''
    self.comments = list(comments)
  
  def addComments(self, comments):
    '''
    Add comments to this vertex (duplicates allowed)
    '''
    self.comments.extend(comments)
  
  def removeComments(self, comments):
    '''
    Remove comments from this vertex
    Throws an exception if a comment does not exist
    '''
    for comment in comments:
      if comment in self.comments:
        self.comments.remove(comment)
      else:
        e = ASE.Compiler.CompilerException()
        e.setMessage('Comment does not exist: ' + str(comment))
        raise e
  
  def getFullIdentifier(self):
    '''
    Retrieve the fully qualified identifier associated with this node
    '''
    return self.identifier
  
  def getDependencies(self):
    '''
    Retrieve nodes on which this node depends
    '''
    return list(self.dependencies)
  
  def setDependencies(self, dependencies):
    '''
    Set nodes on which this node depends
    '''
    self.dependencies = sets.Set([dep.weakReference() for dep in dependencies])
  
  def addDependencies(self, dependencies):
    '''
    Add nodes on which this node depends
    '''
    self.dependencies.update([dep.weakReference() for dep in dependencies])
  
  def removeDependencies(self, dependencies):
    '''
    Remove nodes on which this node depends
    Throws an exception if a dependency does not exist
    '''
    for dependency in dependencies:
      if dependency in self.dependencies:
        self.dependencies.remove(dependency)
      else:
        e = ASE.Compiler.CompilerException()
        e.setMessage('Dependency does not exist: ' + dependency.getIdentifier())
        raise e
  # Ending class methods section
  pass
