#include <Python.h>

#include <Numeric/arrayobject.h>

#ifndef included_ASE_Compiler_SIDL_DependencyAnalyzer
#include <ASE/Compiler/SIDL/DependencyAnalyzer_IOR.h>
#endif

#ifndef included_ASE_Loader
#include <ASE/Loader_IOR.h>
#endif

#ifndef included_ASE_ASEException
#include <ASE/ASEException_IOR.h>
#endif

#ifndef included_ASE_Compiler_CompilerException
#include <ASE/Compiler/CompilerException_IOR.h>
#endif

#ifndef included_ASE_BaseException
#include <ASE/BaseException_IOR.h>
#endif

enum ReferenceType {STOLEN_REF, WEAK_REF, NEW_REF};

struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject {
  PyObject_HEAD;
  struct ASE_Compiler_SIDL_DependencyAnalyzer__object *ior;
  enum ReferenceType refType;
  PyObject *weakRefList;
};

static PyObject *sidl_new(PyTypeObject *type, struct ASE_BaseInterface__object *ior, enum ReferenceType refType)
{
  PyObject *return_value = NULL;
  
  if (ior)
  {
    struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *self = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) type->tp_alloc(type, 0);
    
    self->weakRefList = NULL;
    if (self)
    {
      struct ASE_BaseClass__object *baseIor = (*(ior->d_epv->castObject))(ior->d_object, "ASE.BaseClass", NULL);
      
      self->ior = ((*(ior->d_epv->castObject)))(ior->d_object, type->tp_name, NULL);
      self->refType = refType;
      /* Allow objects without an attached implementation (used in the skeleton constructor) */
      if (baseIor->d_data)
      {
        if (refType == NEW_REF)
        {
          (*(baseIor->d_epv->addReference))(baseIor->d_object, NULL);
        }
      }
      return_value = (PyObject *) self;
    }
    else
    {
      PyErr_SetString(PyExc_MemoryError, (char *) "Unable to allocate interface object");
    }
  }
  else
  {
    return_value = Py_None;
    Py_INCREF(return_value);
  }
  return return_value;
}

static PyObject *ASE_Compiler_SIDL_Interface__wrap_ior(struct ASE_Compiler_SIDL_Interface__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.Interface");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Interface");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.SIDL.Interface");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.SIDL.Interface");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_SIDL_DependencyAnalyzer__wrap_ior(struct ASE_Compiler_SIDL_DependencyAnalyzer__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.DependencyAnalyzer");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "DependencyAnalyzer");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.SIDL.DependencyAnalyzer");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.SIDL.DependencyAnalyzer");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_SIDL_Parameter__wrap_ior(struct ASE_Compiler_SIDL_Parameter__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.Parameter");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Parameter");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.SIDL.Parameter");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.SIDL.Parameter");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_SIDL_DependencyAnalyzer__wrap_ior_weak(struct ASE_Compiler_SIDL_DependencyAnalyzer__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.DependencyAnalyzer");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "DependencyAnalyzer");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.SIDL.DependencyAnalyzer");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.SIDL.DependencyAnalyzer");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, WEAK_REF);
}

static PyObject *ASE_Compiler_SIDL_TypeUniverse__wrap_ior(struct ASE_Compiler_SIDL_TypeUniverse__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.TypeUniverse");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "TypeUniverse");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.SIDL.TypeUniverse");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.SIDL.TypeUniverse");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_SIDL_Array__wrap_ior(struct ASE_Compiler_SIDL_Array__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.Array");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Array");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.SIDL.Array");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.SIDL.Array");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_SIDL_Iterator__wrap_ior(struct ASE_Compiler_SIDL_Iterator__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.Iterator");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Iterator");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.SIDL.Iterator");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.SIDL.Iterator");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_SIDL_Class__wrap_ior(struct ASE_Compiler_SIDL_Class__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.Class");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Class");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.SIDL.Class");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.SIDL.Class");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_SIDL_Method__wrap_ior(struct ASE_Compiler_SIDL_Method__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.Method");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Method");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.SIDL.Method");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.SIDL.Method");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static long ASE_Compiler_SIDL_DependencyAnalyzer__extract_ior(struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *obj, struct ASE_Compiler_SIDL_DependencyAnalyzer__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.SIDL.DependencyAnalyzer", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.SIDL.DependencyAnalyzer");
    return 0;
  }
  return 1;
}

static struct ASE_Compiler_SIDL_DependencyAnalyzer__object *ASE_Compiler_SIDL_DependencyAnalyzer_iterNext(struct ASE_Compiler_SIDL_DependencyAnalyzer__iterator *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *iterator = (PyObject *) _self->iterator;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  PyObject *returnTuple = NULL;
  struct ASE_Compiler_SIDL_DependencyAnalyzer__object *_return_value = NULL;
  
  if (!PyIter_Check(iterator))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid iterator for ASE.Compiler.SIDL.DependencyAnalyzer.iterNext", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_SIDL_DependencyAnalyzer__wrap_ior_weak, _self);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.DependencyAnalyzer.iterNext", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  _pyReturn = PyIter_Next(iterator);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return _return_value;
  }
  if (!_pyReturn)
  {
    _pyReturn = Py_None;
    Py_INCREF(_pyReturn);
  }
  returnTuple = PyTuple_New(1);
  PyTuple_SetItem(returnTuple, 0, _pyReturn);
  /* Parse arguments */
  _okay = PyArg_ParseTuple(returnTuple, (char *) "O&", (void *) ASE_Compiler_SIDL_DependencyAnalyzer__extract_ior, &_return_value);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_okay)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Could not parse return value for ASE.Compiler.SIDL.DependencyAnalyzer.iterNext", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Cleanup local return value */
  Py_DECREF(returnTuple);
  /* Cleanup arguments */
  Py_DECREF(_args);
  return _return_value;
}

static void ASE_Compiler_SIDL_DependencyAnalyzer_visitTypeUniverse(struct ASE_Compiler_SIDL_DependencyAnalyzer__object *_self, struct ASE_Compiler_SIDL_TypeUniverse__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.DependencyAnalyzer.visitTypeUniverse", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitTypeUniverse");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.DependencyAnalyzer.visitTypeUniverse", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  if (node)
  {
    (*((((struct ASE_BaseInterface__object *) node))->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_SIDL_TypeUniverse__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.DependencyAnalyzer.visitTypeUniverse", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return;
  }
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.SIDL.DependencyAnalyzer.visitTypeUniverse", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_SIDL_DependencyAnalyzer_visitArray(struct ASE_Compiler_SIDL_DependencyAnalyzer__object *_self, struct ASE_Compiler_SIDL_Array__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.DependencyAnalyzer.visitArray", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitArray");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.DependencyAnalyzer.visitArray", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  if (node)
  {
    (*((((struct ASE_BaseInterface__object *) node))->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_SIDL_Array__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.DependencyAnalyzer.visitArray", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return;
  }
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.SIDL.DependencyAnalyzer.visitArray", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_SIDL_DependencyAnalyzer_visitIterator(struct ASE_Compiler_SIDL_DependencyAnalyzer__object *_self, struct ASE_Compiler_SIDL_Iterator__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.DependencyAnalyzer.visitIterator", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitIterator");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.DependencyAnalyzer.visitIterator", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  if (node)
  {
    (*((((struct ASE_BaseInterface__object *) node))->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_SIDL_Iterator__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.DependencyAnalyzer.visitIterator", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return;
  }
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.SIDL.DependencyAnalyzer.visitIterator", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_SIDL_DependencyAnalyzer_visitInterface(struct ASE_Compiler_SIDL_DependencyAnalyzer__object *_self, struct ASE_Compiler_SIDL_Interface__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.DependencyAnalyzer.visitInterface", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitInterface");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.DependencyAnalyzer.visitInterface", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  if (node)
  {
    (*((((struct ASE_BaseInterface__object *) node))->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_SIDL_Interface__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.DependencyAnalyzer.visitInterface", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return;
  }
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.SIDL.DependencyAnalyzer.visitInterface", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_SIDL_DependencyAnalyzer_visitClass(struct ASE_Compiler_SIDL_DependencyAnalyzer__object *_self, struct ASE_Compiler_SIDL_Class__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.DependencyAnalyzer.visitClass", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitClass");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.DependencyAnalyzer.visitClass", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  if (node)
  {
    (*((((struct ASE_BaseInterface__object *) node))->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_SIDL_Class__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.DependencyAnalyzer.visitClass", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return;
  }
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.SIDL.DependencyAnalyzer.visitClass", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_SIDL_DependencyAnalyzer_visitParameter(struct ASE_Compiler_SIDL_DependencyAnalyzer__object *_self, struct ASE_Compiler_SIDL_Parameter__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.DependencyAnalyzer.visitParameter", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitParameter");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.DependencyAnalyzer.visitParameter", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  if (node)
  {
    (*((((struct ASE_BaseInterface__object *) node))->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_SIDL_Parameter__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.DependencyAnalyzer.visitParameter", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return;
  }
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.SIDL.DependencyAnalyzer.visitParameter", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_SIDL_DependencyAnalyzer_visitMethod(struct ASE_Compiler_SIDL_DependencyAnalyzer__object *_self, struct ASE_Compiler_SIDL_Method__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.DependencyAnalyzer.visitMethod", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitMethod");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.DependencyAnalyzer.visitMethod", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  if (node)
  {
    (*((((struct ASE_BaseInterface__object *) node))->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_SIDL_Method__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.DependencyAnalyzer.visitMethod", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return;
  }
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.SIDL.DependencyAnalyzer.visitMethod", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_SIDL_DependencyAnalyzer_DependencyAnalyzer(struct ASE_Compiler_SIDL_DependencyAnalyzer__object *obj, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_return_value = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = PyImport_ImportModule((char *) "ASE.Compiler.SIDL.DependencyAnalyzer_impl");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.SIDL.DependencyAnalyzer.DependencyAnalyzer", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "DependencyAnalyzer");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.SIDL.DependencyAnalyzer.DependencyAnalyzer", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_SIDL_DependencyAnalyzer__wrap_ior, obj);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.SIDL.DependencyAnalyzer.DependencyAnalyzer", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Create a new instance */
  _return_value = PyInstance_New(_method, _args, NULL);
  _pyReturn = Py_None;
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_return_value)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Could not create instance of ASE.Compiler.SIDL.DependencyAnalyzer_impl.DependencyAnalyzer", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  obj->d_data = _return_value;
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_SIDL_DependencyAnalyzer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, (char *) "tb_frame"), (char *) "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, (char *) "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, (char *) "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, (char *) "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, (char *) "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    if (_excTraceback != Py_None)
    {
      Py_XDECREF(_excTraceback);
    }
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.SIDL.DependencyAnalyzer.DependencyAnalyzer", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_SIDL_DependencyAnalyzer_DependencyAnalyzer_(struct ASE_Compiler_SIDL_DependencyAnalyzer__object *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *data = (PyObject *) _self->d_data;
  
  /* If we do not set d_data to NULL, we get a SEGV on the recursive call to deleteReference() coming from self.__IORself */
  _self->d_data = NULL;
  /* Remove only reference to Python object */
  Py_XDECREF(data);
}

void ASE_Compiler_SIDL_DependencyAnalyzer__set_epv(struct ASE_Compiler_SIDL_DependencyAnalyzer__epv *_epv, struct ASE_BaseException__object **_exception);

void ASE_Compiler_SIDL_DependencyAnalyzer__set_epv(struct ASE_Compiler_SIDL_DependencyAnalyzer__epv *_epv, struct ASE_BaseException__object **_exception)
{
  static int initialized = 0;
  
  if (!initialized)
  {
    initialized = 1;
    SIDL_Python_Init();
    if (!PyImport_ImportModule((char *) "user"))
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Could not import user", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    if (!PyImport_ImportModule((char *) "importer"))
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Could not import importer", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    if (!PyImport_ImportModule((char *) "ASE.Compiler.SIDL.DependencyAnalyzer"))
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Could not import ASE.Compiler.SIDL.DependencyAnalyzer", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    import_array();
  }
  _epv->DependencyAnalyzer = ASE_Compiler_SIDL_DependencyAnalyzer_DependencyAnalyzer;
  _epv->DependencyAnalyzer_ = ASE_Compiler_SIDL_DependencyAnalyzer_DependencyAnalyzer_;
  _epv->iterNext = ASE_Compiler_SIDL_DependencyAnalyzer_iterNext;
  _epv->visitTypeUniverse = ASE_Compiler_SIDL_DependencyAnalyzer_visitTypeUniverse;
  _epv->visitArray = ASE_Compiler_SIDL_DependencyAnalyzer_visitArray;
  _epv->visitIterator = ASE_Compiler_SIDL_DependencyAnalyzer_visitIterator;
  _epv->visitInterface = ASE_Compiler_SIDL_DependencyAnalyzer_visitInterface;
  _epv->visitClass = ASE_Compiler_SIDL_DependencyAnalyzer_visitClass;
  _epv->visitParameter = ASE_Compiler_SIDL_DependencyAnalyzer_visitParameter;
  _epv->visitMethod = ASE_Compiler_SIDL_DependencyAnalyzer_visitMethod;
}
