# Starting imports section
pass
import ASE.BaseClass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Iterator
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.SIDL.Visitor
import ASE.Compiler.Search
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class DepthFirstVisitor:
  '''
  Traverses the AST in a depth first manner
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.repository = []
    self.seen = []
    self.reverseEdges = 0
    self.vertexNumber = 0
  
  def getRepository(self):
    '''
    Retrieve auxilliary ASTs
    '''
    return self.repository
  
  def setRepository(self, repository):
    '''
    Set auxilliary ASTs
    '''
    self.repository = repository
  
  def visitVertex(self, vertex):
    self.discoverVertex(vertex)
    self.__IORself.traverseVertex(vertex)
    self.finishVertex(vertex)
  
  def visitSpecification(self, node):
    self.__IORself.visitVertex(node)
  
  def visitPackage(self, node):
    self.__IORself.visitVertex(node)
  
  def visitType(self, node):
    self.__IORself.visitVertex(node)
  
  def visitTypeUniverse(self, node):
    self.__IORself.visitVertex(node)
  
  def visitEnumeration(self, node):
    self.__IORself.visitVertex(node)
  
  def visitEnumerator(self, node):
    self.__IORself.visitVertex(node)
  
  def visitArray(self, node):
    self.__IORself.visitVertex(node)
  
  def visitIterator(self, node):
    self.__IORself.visitVertex(node)
  
  def visitInterface(self, node):
    self.__IORself.visitVertex(node)
  
  def visitClass(self, node):
    self.__IORself.visitVertex(node)
  
  def visitParameter(self, node):
    self.__IORself.visitVertex(node)
  
  def visitMethod(self, node):
    self.__IORself.visitVertex(node)
  
  def getReverseEdges(self):
    '''
    Retrieve the flag for reversing the orientation of edges in the traversal
    '''
    return self.reverseEdges
  
  def setReverseEdges(self, reverseEdges):
    '''
    Set the flag for reversing the orientation of edges in the traversal
    '''
    self.reverseEdges = reverseEdges
  
  def traverseVertex(self, vertex):
    '''
    Traverse each edge of this vertex
    '''
    self.__IORself.setVertexSeen(vertex, 1)
    if self.__IORself.getReverseEdges():
      neighbors = [vertex.getParent()]
    else:
      neighbors = vertex.getChildren()
    for neighbor in neighbors:
      neighbor.accept(self.__IORself)
  
  def resetVertexNumbering(self):
    '''
    Reset the vertex discovery and finish numbering
    '''
    self.vertexNumber = 0
  
  def discoverVertex(self, vertex):
    '''
    Mark a vertex as discovered and increment the count
    '''
    self.vertexNumber += 1
  
  def finishVertex(self, vertex):
    '''
    Mark a vertex as finished and increment the count
    '''
    self.vertexNumber += 1
  
  def getVerticesSeen(self):
    '''
    Retrieve the list of vertices discovered
    '''
    return self.seen
  
  def setVerticesSeen(self, seen):
    '''
    Set the list of vertices discovered
    '''
    self.seen = list(seen)
  
  def getVertexSeen(self, vertex):
    '''
    Retrieve the vertex discovery state
    '''
    return vertex in self.seen
  
  def setVertexSeen(self, vertex, state):
    '''
    Set the vertex discovery state
    '''
    if state:
      if not (vertex in self.seen):
        self.seen.append(vertex)
    else:
      if vertex in self.seen:
        self.seen.remove(vertex)
  # Ending class methods section
  pass
