# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.Printer
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.DepthFirstVisitor
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Iterator
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class Printer:
  '''
  Prints an AST to stdout as SIDL source
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    import sys
    self.tab = '  '
    self.indentSize = 0
    self.output = sys.stdout
    self.packages = []
    self.declOnly = [0]
  
  def getTab(self):
    '''
    Retrieve the tab string
    '''
    return self.tab
  
  def setTab(self, tab):
    '''
    Set the tab string
    '''
    self.tab = tab
  
  def getIndentSize(self):
    '''
    Retrieve the number of tabs of the current indent
    '''
    return self.indentSize
  
  def setIndentSize(self, indentSize):
    '''
    Set the number of tabs of the current indent
    Throws an exception on a negative indent size
    '''
    self.indentSize = indentSize
  
  def increaseIndent(self):
    '''
    Increase the index size by one
    '''
    self.indentSize += 1
  
  def decreaseIndent(self):
    '''
    Decrease the index size by one
    Throws an exception on a negative indent size
    '''
    self.indentSize -= 1
    if self.indentSize < 0:
      raise RuntimeError('Indent size decreased below zero: ' + str(self.indentSize))
  
  def indent(self):
    '''
    Output the current indent (the indent size in tabs)
    '''
    for i in range(self.getIndentSize()):
      self.__IORself.write(self.getTab())
  
  def write(self, s):
    '''
    Output a string
    '''
    self.output.write(s)
  
  def println(self, s):
    '''
    Output a string, properly indented and followed by a newline
    '''
    self.__IORself.indent()
    self.__IORself.write(s)
    self.__IORself.write('\n')
  
  def visitPackage(self, node):
    self.packages.append(node)
    templateIds = node.getTemplateIdentifiers()
    self.printComments(node)
    self.indent()
    self.write('package ' + node.getIdentifier())
    if len(templateIds):
      self.write('[')
      for (t, id) in enumerate(templateIds):
        if t > 0:
          self.__IORself.write(', ')
        self.__IORself.write(id)
      self.__IORself.write(']')
    self.__IORself.write(' {\n')
    self.__IORself.increaseIndent()
    children = node.getChildren()
    for (e, child) in enumerate(children):
      if e > 0:
        self.write('\n')
      child.accept(self.__IORself)
    self.__IORself.decreaseIndent()
    self.__IORself.println('}')
    self.packages.pop()
  
  def visitType(self, node):
    if node.getBaseType():
      self.__IORself.write(node.getIdentifier())
    else:
      self.__IORself.write(self.getQualifiedName(node))
  
  def visitTypeUniverse(self, node):
    if self.declOnly[-(1)]:
      self.__IORself.write(self.getQualifiedName(node))
      return None
    self.__IORself.indent()
    self.__IORself.write('universe ' + node.getIdentifier() + ' {')
    for (t, type) in enumerate(node.getTypes()):
      if t > 0:
        self.write(', ')
      type.accept(self.__IORself)
    self.__IORself.write('};\n')
  
  def visitEnumeration(self, node):
    if self.declOnly[-(1)]:
      self.__IORself.write(self.getQualifiedName(node))
      return None
    self.printComments(node)
    self.__IORself.indent()
    self.__IORself.write('enum ' + node.getIdentifier())
    self.write(' {')
    for (e, child) in enumerate(node.getChildren()):
      if e > 0:
        self.write(', ')
      child.accept(self.__IORself)
    self.__IORself.write('}')
    self.__IORself.write(';\n')
  
  def visitEnumerator(self, node):
    if node.getInitialized():
      self.__IORself.write(node.getIdentifier() + ' = ' + str(node.getValue()))
    else:
      self.__IORself.write(node.getIdentifier())
  
  def visitArray(self, node):
    self.__IORself.write('array<')
    self.pushDeclOnly()
    node.getType().accept(self.__IORself)
    self.popDeclOnly()
    if node.getDimension() > 0:
      self.__IORself.write(', ' + str(node.getDimension()))
    self.__IORself.write('>')
  
  def visitIterator(self, node):
    self.__IORself.write('iterator<')
    self.pushDeclOnly()
    node.getType().accept(self.__IORself)
    self.popDeclOnly()
    self.__IORself.write('>')
  
  def visitInterface(self, node):
    if self.declOnly[-(1)]:
      self.__IORself.write(self.getQualifiedName(node))
      return None
    self.printComments(node)
    self.__IORself.indent()
    self.__IORself.write('interface ' + node.getIdentifier())
    parents = node.getParents()
    if len(parents):
      self.__IORself.write(' extends ')
      for i in range(len(parents)):
        if i > 0:
          self.write(', ')
        self.__IORself.write(self.getQualifiedName(parents[i]))
    self.__IORself.write(' {\n')
    self.__IORself.increaseIndent()
    self.__IORself.visitVertex(node)
    self.__IORself.decreaseIndent()
    self.__IORself.println('}')
  
  def visitClass(self, node):
    if self.declOnly[-(1)]:
      self.__IORself.write(self.getQualifiedName(node))
      return None
    self.printComments(node)
    self.__IORself.indent()
    self.__IORself.write('class ' + node.getIdentifier())
    parents = node.getParents()
    if (len(parents) and not (parents[0].getFullIdentifier() == 'ASE.BaseClass')):
      self.__IORself.write(' extends ')
      for i in range(len(parents)):
        if i > 0:
          self.write(', ')
        self.__IORself.write(self.getQualifiedName(parents[i]))
    interfaces = node.getInterfaces()
    if len(interfaces):
      self.__IORself.write(' implements ')
      for i in range(len(interfaces)):
        if i > 0:
          self.write(', ')
        self.__IORself.write(self.getQualifiedName(interfaces[i]))
    self.__IORself.write(' {\n')
    self.__IORself.increaseIndent()
    self.__IORself.visitVertex(node)
    self.__IORself.decreaseIndent()
    self.__IORself.println('}')
  
  def visitParameter(self, node):
    type = node.getType()
    mode = node.getAccessMode()
    if mode == ASE.Compiler.ParameterAccessMode.IN:
      self.__IORself.write('in ')
    elif mode == ASE.Compiler.ParameterAccessMode.INOUT:
      self.__IORself.write('inout ')
    elif mode == ASE.Compiler.ParameterAccessMode.OUT:
      self.__IORself.write('out ')
    elif mode == ASE.Compiler.ParameterAccessMode.RETURN:
      pass
    else:
      raise RuntimeError('Invalid parameter access mode: ' + str(mode))
    self.pushDeclOnly()
    type.accept(self.__IORself)
    self.popDeclOnly()
    if (not (type.getBaseType()) and type.getIdentifier() == 'void'):
      if mode == ASE.Compiler.ParameterAccessMode.RETURN:
        if not (node.getIdentifier() == '_return_value'):
          self.__IORself.write(':' + node.getIdentifier())
      else:
        self.__IORself.write(' ' + node.getIdentifier())
  
  def visitMethod(self, node):
    templateIds = node.getTemplateIdentifiers()
    self.printComments(node)
    self.__IORself.indent()
    if node.getStatic():
      self.__IORself.write('static ')
    if not (node.getReturnParameter() is None):
      self.pushDeclOnly()
      node.getReturnParameter().accept(self.__IORself)
      self.popDeclOnly()
    else:
      self.__IORself.write('void')
    self.__IORself.write(' ' + node.getIdentifier())
    if len(templateIds):
      self.__IORself.write('[')
      for (t, id) in enumerate(node.getTemplateIdentifiers()):
        if t > 0:
          self.__IORself.write(', ')
        self.__IORself.write(id)
      self.__IORself.write(']')
    self.__IORself.write('(')
    for (c, child) in enumerate(node.getChildren()):
      if c > 0:
        self.__IORself.write(', ')
      child.accept(self.__IORself)
    self.__IORself.write(')')
    exceptions = self.getDeclaredExceptions(node)
    if len(exceptions) > 0:
      self.__IORself.write(' throws ')
      for (e, exc) in enumerate(exceptions):
        if e > 0:
          self.write(', ')
        self.__IORself.write(self.getQualifiedName(exc))
    self.__IORself.write(';\n')
  # Ending class methods section
  pass
  
  def pushDeclOnly(self, declOnly = 1):
    self.declOnly.append(declOnly)
    return 
  
  def popDeclOnly(self):
    return self.declOnly.pop()
  
  def getQualifiedName(self, vertex, packageName = None):
    '''
    Return the full identifier, unless the type is defined in the given package, when we return just the identifier
    '''
    if packageName is None:
      packageName = self.packages[-(1)].getFullIdentifier()
    name = vertex.getIdentifier()
    qualifiedName = vertex.getFullIdentifier()
    if qualifiedName == packageName + '.' + name:
      qualifiedName = name
    return qualifiedName
  
  def getDeclaredExceptions(self, method):
    '''
    Return all declared exceptions, i.e ignore NullIORException and BaseException
    '''
    exceptions = []
    for exception in method.getExceptions():
      if not (exception.getFullIdentifier() in ['ASE.BaseException', 'ASE.NullIORException', 'SIDL.NullIORException']):
        exceptions.append(exception)
    return exceptions
  
  def printComments(self, vertex):
    for comment in vertex.getComments():
      [self.println('//' + c) for c in comment.getComment().split('\n')]
    return 
